@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c,v 1.22 1994/12/30 05:30:47 master Exp $";
+ *   "$Id: arch.c,v 1.28 1997/03/09 04:03:48 master Exp $";
  */
 
 /*
@@ -103,6 +103,24 @@ void dump_all_archetypes() {
     dump_arch(at);
 }
 
+void free_all_archs()
+{
+    archetype *at, *next;
+    int i=0,f=0;
+
+    for (at=first_archetype; at!=NULL; at=next) {
+	if (at->faces) {
+		free(at->faces);
+		f++;
+	}
+	if (at->more) next=at->more;
+	else next=at->next;
+	free(at);
+	i++;
+    }
+    LOG(llevDebug,"Freed %d archetypes, %d faces\n", i, f);
+}
+
 /*
  * Allocates, initialises and returns the pointer to an archetype structure.
  */
@@ -116,7 +134,7 @@ archetype *get_archetype_struct() {
   new->faces=NULL;
   new->next=NULL;
   new->name=NULL;
-  new->randomitems=NULL;
+/*  new->randomitems=NULL;*/
   new->clone.other_arch=NULL;
   new->clone.name=NULL;
   new->clone.title=NULL;
@@ -171,6 +189,7 @@ void first_arch_pass(FILE *fp) {
     clear_object(op);
     op->arch=at;
   }
+  free(at);
   free_object(op);
 }
 
@@ -207,17 +226,17 @@ void second_arch_pass(FILE *fp) {
         if(tl==NULL)
           LOG(llevError,"Failed to link treasure to arch: %s\n",variable);
         else
-          at->randomitems=tl;
+          at->clone.randomitems=tl;
       }
     }
   }
 }
 
-#if 0
+#ifdef DEBUG
 void check_generators() {
   archetype *at;
   for(at=first_archetype;at!=NULL;at=at->next)
-    if(IS_GENERATOR((&at->clone))&&at->clone.other_arch==NULL)
+    if(QUERY_FLAG(&at->clone,FLAG_GENERATOR)&&at->clone.other_arch==NULL)
       LOG(llevError,"Warning: %s is generator but lacks other_arch.\n",
               at->name);
 }
@@ -235,7 +254,8 @@ void load_archetypes() {
   char filename[MAX_BUF];
   int comp;
 
-  sprintf(filename,"%s/%s",LibDir,ArcheTypes);
+  sprintf(filename,"%s/%s",settings.libdir,settings.archetypes);
+  LOG(llevDebug,"Reading archetypes from %s...\n",filename);
   if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
     LOG(llevError,"Can't open archetype file.\n");
     return;
@@ -253,7 +273,7 @@ void load_archetypes() {
   LOG(llevDebug,"done\narch-pass 2...");
   second_arch_pass(fp);
   LOG(llevDebug,"done\n");
-#if 0
+#ifdef DEBUG
   check_generators();
 #endif
   close_and_delete(fp, comp);
