@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_button_c =
- *   "$Id: button.c,v 1.25 1995/04/15 03:07:25 master Exp $";
+ *   "$Id: button.c,v 1.29 1996/12/01 07:11:21 master Exp $";
  */
 
 /*
@@ -46,7 +46,7 @@ void push_button(object *op) {
 /*   LOG(llevDebug, "push_button: %s (%d)\n", op->name, op->count); */
   for (ol = get_button_links(op); ol; ol = ol->next) {
     if (!ol->ob || ol->ob->count != ol->id) {
-      LOG(llevError, "Internal error in push_button.\n");
+      LOG(llevError, "Internal error in push_button (%s).\n", op->name);
       continue;
     }
     /* a button link object can become freed when the map is saving.  As
@@ -126,7 +126,7 @@ void update_button(object *op) {
   /* LOG(llevDebug, "update_button: %s (%d)\n", op->name, op->count); */
   for (ol = get_button_links(op); ol; ol = ol->next) {
     if (!ol->ob || ol->ob->count != ol->id) {
-      LOG(llevError, "Internal error in update_button.\n");
+      LOG(llevError, "Internal error in update_button (%s).\n", op->name);
       continue;
     }
     tmp = ol->ob;
@@ -168,7 +168,12 @@ void update_buttons(mapstruct *m) {
   for (obp = m->buttons; obp; obp = obp->next)
     for (ol = obp->link; ol; ol = ol->next) {
       if (!ol->ob || ol->ob->count != ol->id) {
-        LOG(llevError, "Internal error in update_button.\n");
+        LOG(llevError, "Internal error in update_button (%s (%dx%d):%d, connected %d ).\n",
+	    ol->ob?ol->ob->name:"null",
+	    ol->ob?ol->ob->x:-1,
+	    ol->ob?ol->ob->y:-1,
+	    ol->id,
+	    obp->value);
         continue;
       }
       if (ol->ob->type==BUTTON || ol->ob->type==PEDESTAL)
@@ -201,13 +206,46 @@ void animate_turning(object *op) /* only one part objects */
     update_object(op);
 }
 
-/*
- * check_altar checks if sacrifice was accepted and removes sacrificed
- * objects. If sacrifice was succeed return 1 else 0
- */
 #define ARCH_SACRIFICE(xyz) ((xyz)->slaying)
 #define NROF_SACRIFICE(xyz) ((xyz)->stats.food)
 
+/* Returns that object if it meets the sacrifice needs of the altar.
+ * If no object meets the needs, return NULL.
+ * Function put in (0.92.1) so that identify altars won't grab money
+ * unnecessarily - we can see if there is sufficient money, see if something
+ * needs to be identified, and then remove money if needed. (via check_altar)
+ * Check_altar uses this function also.
+ */
+ 
+object *check_altar_sacrifice(object *altar)
+{
+  object *op;
+
+  for (op=altar->above; op; op=op->above)
+    if (!QUERY_FLAG(op,FLAG_ALIVE) && op->type != PLAYER) {
+      if ((ARCH_SACRIFICE(altar) == op->arch->name  ||
+          ARCH_SACRIFICE(altar) == op->name ||
+	  ARCH_SACRIFICE(altar) == op->slaying) 
+	  && NROF_SACRIFICE(altar) <= (op->nrof?op->nrof:1))
+		return op;
+      if (!strcmp(ARCH_SACRIFICE(altar), "money") &&
+	  op->type==MONEY && op->nrof*op->value>=NROF_SACRIFICE(altar))
+		return op;
+    }
+    return NULL;
+}
+
+
+/*
+ * check_altar checks if sacrifice was accepted and removes sacrificed
+ * objects. If sacrifice was succeed return 1 else 0  Might be better to
+ * call check_altar_sacrifice (above) than depend on the return value,
+ * since check_altar will remove the sacrifice also.
+ *
+ * Don't really like the name of this functin, operate_altar might be better -
+ * we aren't really checking it, since it will remove the sacrifice
+ */
+
 int check_altar (object *altar)
 {
   object *op;
@@ -215,18 +253,22 @@ int check_altar (object *altar)
   if (!altar->slaying || altar->value)
     return 0;
 
-  for (op=altar->above; op; op=op->above)
-    if (!QUERY_FLAG(op,FLAG_ALIVE) &&
-        (op->type != PLAYER && (ARCH_SACRIFICE(altar) == op->arch->name  ||
-         ARCH_SACRIFICE(altar) == op->name ||
-	 op->slaying == ARCH_SACRIFICE(altar))) &&
-        NROF_SACRIFICE(altar) <= (op->nrof?op->nrof:1))
-      break;
-
+  op=check_altar_sacrifice(altar);
   if (!op)
     return 0;
 
-  decrease_ob_nr (op, NROF_SACRIFICE(altar));
+  /* get_altar_sacrifice should have already verified that enough money
+   * has been dropped.
+   */
+  if (!strcmp(ARCH_SACRIFICE(altar), "money")) {
+	int number=NROF_SACRIFICE(altar)/op->value;
+
+	/* Round up any sacrifices.  Altars don't make change either */
+	if (NROF_SACRIFICE(altar)%op->value) number++;
+	decrease_ob_nr (op, number);
+  }
+  else
+    decrease_ob_nr (op, NROF_SACRIFICE(altar));
  
   for (op = get_map_ob(altar->map,altar->x,altar->y);
        op && op->type != PLAYER; op=op->above);
@@ -307,6 +349,7 @@ void add_button_link(object *button, mapstruct *map, int connected) {
     LOG(llevError, "Tried to add button-link without map.\n");
     return;
   }
+/*  LOG(llevDebug,"adding button %s (%d)\n", button->name, connected);*/
 
   SET_FLAG(button,FLAG_IS_LINKED);
 
@@ -457,36 +500,49 @@ void do_mood_floor(object *op, object *op2) {
         return;
 }
 
+/* this function returns the object it matches, or NULL if non.
+ * It will descend through containers to find the object.
+ *		slaying = match object slaying flag
+ * 		race = match object archetype name flag
+ *		hp = match object type (excpt type '0'== PLAYER)
+ */
+
+object * check_inv_recursive(object *op, object *trig)
+{
+    object *tmp,*ret=NULL;
+
+    for(tmp=op->inv; tmp; tmp=tmp->below) {
+	if (tmp->inv) {
+	    ret=check_inv_recursive(tmp, trig);
+	    if (ret) return ret;
+	}
+	else if((trig->stats.hp && tmp->type == trig->stats.hp) 
+		|| tmp->slaying == trig->slaying 
+		|| tmp->arch->name == trig->race) 
+		    return tmp;
+    }
+    return NULL;
+}
+
 /* check_inv(), a function to search the inventory, 
  * of a player and then based on a set of conditions,
  * the square will activate connected items. 
  * Monsters can't trigger this square (for now)
  * Values are:	last_sp = 1/0 obj/no obj triggers 
  * 		last_heal = 1/0  remove/dont remove obj if triggered
- *		slaying = match object slaying flag
- * 		race = match object archetype name flag
  * -b.t. (thomas@nomad.astro.psu.edu 
  */
 
 void check_inv (object *op, object *trig) {
-object *tmp;
+object *match;
 
-	if(op->type != PLAYER) return;
-	if(trig->last_sp) { 			/* obj in inv will trigger  */ 
-	   for(tmp=op->inv; tmp; tmp=tmp->below)
-	       if(tmp->slaying == trig->slaying || tmp->arch->name == trig->race) { 
-		   if(trig->last_heal) 
-		       decrease_ob(tmp);
-		   use_trigger(trig);
-		   break;
-	       } 
-	} else {				/* no obj in inv triggers */ 
-	   for(tmp=op->inv; tmp; tmp=tmp->below)
-	       if(tmp->slaying == trig->slaying || tmp->arch->name == trig->race) { 
-		   break; 
-	       } else if(!tmp->below) {  
-		   use_trigger(trig);
-	       }
-	}
-	return;
-}          
+    if(op->type != PLAYER) return;
+    match = check_inv_recursive(op,trig);
+    if (match && trig->last_sp) {
+	if(trig->last_heal) 
+	    decrease_ob(match);
+	use_trigger(trig);
+    }
+    else if (!match && !trig->last_sp)
+	use_trigger(trig);
+}
