@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_glue_c =
- *   "$Id: glue.c,v 1.16 1994/12/30 05:32:10 master Exp $";
+ *   "$Id: glue.c,v 1.18 1996/10/29 09:57:42 master Exp $";
  */
 
 /*
@@ -55,7 +55,8 @@ type_func_int_map_char	info_map_func;
 type_func_ob	move_teleporter_func;
 type_func_ob_int  trap_adjust_func;
 type_func_ob    move_creator_func;
-type_func_int_ob_ob esrv_send_item_func;
+type_func_ob_ob esrv_send_item_func;
+type_func_int_int esrv_del_item_func;
 
 static char *fatalmsgs[80]={
   "Failed to allocate memory",
@@ -94,7 +95,8 @@ void init_function_pointers() {
   move_teleporter_func = dummy_function_ob;
   trap_adjust_func = dummy_function_ob_int;
   move_creator_func = dummy_function_ob;
-  esrv_send_item_func = dummy_function_int_ob_ob;
+  esrv_send_item_func = dummy_function_ob2;
+  esrv_del_item_func = dummy_function_int_int;
 
 }
 
@@ -255,10 +257,13 @@ void set_move_creator(type_func_ob addr) {
   move_creator_func = addr;
 }
 
-void set_esrv_send_item (type_func_int_ob_ob addr) {
+void set_esrv_send_item (type_func_ob_ob addr) {
   esrv_send_item_func = addr;
 }
 
+void set_esrv_del_item(type_func_int_int addr) {
+  esrv_del_item_func = addr;
+}
 
 /*
  * fatal() is meant to be called whenever a fatal signal is intercepted.
@@ -269,12 +274,8 @@ void fatal(int err) {
   fprintf(logfile,"Fatal: %s\n",fatalmsgs[err]);
   (*emergency_save_func)(0);
   (*clean_tmp_files_func)();
-#ifdef GOD
-  pray();
-#else
   fprintf(logfile,"Exiting...\n");
   exit(err);
-#endif
 }
 
 #ifndef __Making_docs__ /* Don't want documentation on these */
@@ -289,6 +290,9 @@ void fatal(int err) {
 void dummy_function_int(int i) {
 }
 
+void dummy_function_int_int(int i, int j) {
+}
+
 
 void dummy_function() {
 }
