@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.19 1994/11/09 02:59:20 master Exp $";
+ *   "$Id: init.c,v 1.20 1997/03/09 04:06:13 master Exp $";
  */
 
 /*
@@ -29,6 +29,31 @@
 #include <global.h>
 #include <object.h>
 
+/* You unforunately need to looking in include/structs.h to see what these
+ * correspond to.
+ */
+struct Settings settings = {
+NULL, "logfile", PORT, CSPORT,	/*chrfont, logfile, port, new cs port */
+SERVER_OFF, 1, Dm_Font,		/* -server option, fixfontpath, displaymode */
+1, 0, 0,			/* synchronize, bw only, noadd */
+#ifdef DEBUG
+  llevDebug,
+#else
+  llevError,
+#endif
+0, 0, 0,    /* split, dumpvalues, daemonmode */
+0, /* argc */
+NULL, /* argv */
+LIBDIR, FONTDIR, PLAYERDIR, MAPDIR, ARCHETYPES,TREASURES, 
+#ifdef ITEMS_DIR
+ITEMS_DIR ,
+#else
+"",
+#endif
+TMPDIR
+};
+
+
 /*
  * It is vital that init_library() is called by any functions
  * using this library.
@@ -39,17 +64,48 @@
  */
 
 void init_library() {
-  init_globals();
-  init_function_pointers();
-  init_hash_table();
-  init_objects();
-  init_vars();
-  init_block();
-  ReadBmapNames ();
-  init_archetypes();	/* Reads all archetypes from file */
-  init_dynamic ();
+    init_environ();
+    init_globals();
+    init_function_pointers();
+    init_hash_table();
+    init_objects();
+    init_vars();
+    init_block();
+    ReadBmapNames ();
+    init_archetypes();	/* Reads all archetypes from file */
+    init_dynamic ();
 }
 
+/* init_environ initializes values from the environmental variables.
+ * it needs to be called very early, since command line options should
+ * overwrite these if specified.
+ */
+void init_environ() {
+#ifndef SECURE
+    char *cp;
+
+    cp=getenv("CROSSFIRE_LIBDIR");
+    if (cp) settings.libdir=cp;
+    cp=getenv("CROSSFIRE_FONTDIR");
+    if (cp) settings.fontdir=cp;
+    cp=getenv("CROSSFIRE_PLAYERDIR");
+    if (cp) settings.playerdir=cp;
+    cp=getenv("CROSSFIRE_MAPDIR");
+    if (cp) settings.mapdir=cp;
+    cp=getenv("CROSSFIRE_ARCHETYPES");
+    if (cp) settings.archetypes=cp;
+    cp=getenv("CROSSFIRE_TREASURES");
+    if (cp) settings.treasures=cp;
+#ifdef UNIQUE_ITEMS
+    cp=getenv("CROSSFIRE_ITEMSDIR");
+    if (cp) settings.uniquedir=cp;
+#endif
+#endif
+    cp=getenv("CROSSFIRE_TMPDIR");
+    if (cp) settings.tmpdir=cp;
+}
+    
+
 /*
  * Initialises all global variables.
  * Might use environment-variables as default for some of them.
@@ -58,65 +114,27 @@ void init_library() {
 void init_globals() {
   logfile = stderr;
   exiting = 0;
-#ifndef SECURE
-  LibDir=getenv("CROSSFIRE_LIBDIR");
-  if (!LibDir)
-#endif
-    LibDir=LIBDIR;
-#ifndef SECURE
-  FontDir=getenv("CROSSFIRE_FONTDIR");
-  if (!FontDir)
-#endif
-    FontDir=FONTDIR;
-#ifndef SECURE
-  PlayerDir=getenv("CROSSFIRE_PLAYERDIR");
-  if (!PlayerDir)
-#endif
-    PlayerDir=PLAYERDIR;
-#ifndef SECURE
-  MapDir=getenv("CROSSFIRE_MAPDIR");
-  if (!MapDir)
-#endif
-    MapDir=MAPDIR;
-#ifndef SECURE
-  ArcheTypes=getenv("CROSSFIRE_ARCHETYPES");
-  if (!ArcheTypes)
-#endif
-    ArcheTypes=ARCHETYPES;
-#ifndef SECURE
-  Treasures=getenv("CROSSFIRE_TREASURES");
-  if (!Treasures)
-#endif
-    Treasures=TREASURES;
-#ifdef UNIQUE_ITEMS
-#ifndef SECURE
-  ItemsDir=getenv("CROSSFIRE_ITEMSDIR");
-  if (!ItemsDir)
-#endif
-    ItemsDir=ITEMS_DIR;
-#endif
   delete_last_file=0;
   first_player=NULL;
   first_friendly_object=NULL;
   first_map=NULL;
   first_treasurelist=NULL;
   first_artifactlist=NULL;
-  chrfont=NULL;
   first_archetype=NULL;
   warn_archetypes=0;
   first_map=NULL;
+#if 0
   write_state=0;
   write_pos=0;
   write_d=0;
   button_flag=0;
   last_bard=0;
   fix_walls=0;
-  default_split_window=0;
-  dump_monsters=0;
   monsters = 1;
   generators = 1;
   hidden = 0;
   others = 1;
+#endif
   last_micro = 0;
   nroftreasures = 0;
   nrofartifacts = 0;
@@ -125,9 +143,7 @@ void init_globals() {
   amulet_arch = NULL;
   staff_arch = NULL;
   undead_name = add_string("undead");
-  use_pixmaps = 0;
   name = strdup_local("crossfire");
-  server_mode = SERVER_OFF;
   active_socket = (sockets *) NULL;
   first_socket = (sockets *) NULL;
   font_graphic = FONTNAME;
@@ -170,14 +186,6 @@ void init_objects() {
  */
 
 void init_defaults() {
-  fix_fontpath=1;
-  synchronize=1;
-#ifdef DEBUG
-  debug= llevDebug;
-#else
-  debug= llevError;
-#endif
-  no_color=0;
   editor=0;
   maplevel=0; /* What level the map we're editing has */
   nroferrors=0;
