@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.30 1995/07/24 22:01:33 master Exp $";
+ *   "$Id: item.c,v 1.40 1997/01/05 09:38:44 master Exp $";
  */
 
 /*
@@ -44,23 +44,6 @@
       strcat(retbuf,")"); \
     }
 
-extern char *spellpathnames[NRSPELLPATHS];
-
-#define	DESCRIBE_PATH(retbuf, variable, name) \
-    if(variable) { \
-      int i,j=0; \
-      strcat(retbuf,"(" name ": "); \
-      for(i=0; i<NRSPELLPATHS; i++) \
-        if(variable & (1<<i)) { \
-          if (j) \
-            strcat(retbuf,", "); \
-          else \
-            j = 1; \
-          strcat(retbuf, spellpathnames[i]); \
-        } \
-      strcat(retbuf,")"); \
-    }
-
 static char numbers[21][20] = {
   "no","a","two","three","four","five","six","seven","eight","nine","ten",
   "eleven","twelve","thirteen","fourteen","fifteen","sixteen","seventeen",
@@ -152,6 +135,9 @@ char *get_number(int i) {
  *  Or make FULL_RING_DESCRIPTION standard part of a game and let
  *  client handle names.
  */
+/* Aug 95 modified this slightly so that Skill tools don't have magic bonus
+ * from stats.sp - b.t.
+ */
 char *ring_desc (object *op) 
 {
     static char buf[MAX_BUF];
@@ -162,13 +148,13 @@ char *ring_desc (object *op)
     if (! QUERY_FLAG(op, FLAG_IDENTIFIED))
 	return buf;
 
-    for (attr=0; attr<6; attr++) {
+    for (attr=0; attr<7; attr++) {
 	if ((val=get_attr_value(&(op->stats),attr))!=0) {
 	    sprintf (buf+strlen(buf), "(%s%+d)", short_stat_name[attr], val);
 	}
     }
     if(op->stats.exp)
-	sprintf(buf+strlen(buf), "(speed %+ld)", op->stats.exp);
+	sprintf(buf+strlen(buf), "(speed %+d)", op->stats.exp);
     if(op->stats.wc)
 	sprintf(buf+strlen(buf), "(wc%+d)", op->stats.wc);
     if(op->stats.dam)
@@ -181,7 +167,9 @@ char *ring_desc (object *op)
 	sprintf(buf+strlen(buf), "(sustenance%+d)", op->stats.food);
     else if (op->stats.food < 0)
 	sprintf(buf+strlen(buf), "(hunger%+d)", op->stats.food);
-    if(op->stats.sp)
+    if(op->stats.grace)
+	sprintf(buf+strlen(buf), "(grace%+d)", op->stats.grace);
+    if(op->stats.sp && op->type!=SKILL)
 	sprintf(buf+strlen(buf), "(magic%+d)", op->stats.sp);
     if(op->stats.hp)
 	sprintf(buf+strlen(buf), "(regeneration%+d)", op->stats.hp);
@@ -202,8 +190,8 @@ char *ring_desc (object *op)
     DESCRIBE_PATH(buf, op->path_attuned, "Attuned");
     DESCRIBE_PATH(buf, op->path_repelled, "Repelled");
     DESCRIBE_PATH(buf, op->path_denied, "Denied");
-    if(buf[0] == 0)
-	strcpy(buf,"of adorment");
+    if(buf[0] == 0 && op->type!=SKILL)
+	strcpy(buf,"of adornment");
     return buf;
 }
 
@@ -282,6 +270,7 @@ char *query_short_name(object *op)
 	}
 	break;
 
+      case SKILL:
       case AMULET:
       case RING:
 #ifdef FULL_RING_DESCRIPTION
@@ -307,25 +296,34 @@ char *query_short_name(object *op)
 /*
  * query_name(object) returns a character pointer pointing to a static
  * buffer which contains a verbose textual representation of the name
- * of the given object.  The buffer will be overwritten at the next
- * call to query_name().
+ * of the given object.
+ * cf 0.92.6:  Put in 5 buffers that it will cycle through.  In this way,
+ * you can make several calls to query_name before the bufs start getting
+ * overwritten.  This may be a bad thing (it may be easier to assume the value
+ * returned is good forever.)  However, it makes printing statements that
+ * use several names much easier (don't need to store them to temp variables.)
+ *
  */
 char *query_name(object *op) {
-    static char buf[MAX_BUF];
+    static char buf[5][MAX_BUF];
+    static int use_buf=0;
 
-    strcpy (buf, query_short_name(op));
+    use_buf++;
+    use_buf %=5;
+
+    strcpy (buf[use_buf], query_short_name(op));
 
     if (QUERY_FLAG(op,FLAG_INV_LOCKED))
-	strcat(buf, " *");
+	strcat(buf[use_buf], " *");
     if (op->type == CONTAINER && ((op->env && op->env->container == op) || 
 	(!op->env && QUERY_FLAG(op,FLAG_APPLIED))))
-	strcat(buf," (open)");
+	strcat(buf[use_buf]," (open)");
 
     if (QUERY_FLAG(op,FLAG_KNOWN_CURSED)) {
 	if(QUERY_FLAG(op,FLAG_DAMNED))
-	    strcat(buf, " (damned)");
+	    strcat(buf[use_buf], " (damned)");
 	else if(QUERY_FLAG(op,FLAG_CURSED))
-	    strcat(buf, " (cursed)");
+	    strcat(buf[use_buf], " (cursed)");
     }
     /* Basically, if the object is known magical (detect magic spell on it),
      * and it isn't identified,  print out the fact that
@@ -337,17 +335,17 @@ char *query_name(object *op) {
      * abilities, especially for artifact items.
      */
     if (QUERY_FLAG(op,FLAG_KNOWN_MAGICAL) && !QUERY_FLAG(op,FLAG_IDENTIFIED))
-	strcat(buf, " (magic)");
+	strcat(buf[use_buf], " (magic)");
     if(QUERY_FLAG(op,FLAG_APPLIED)) {
 	switch(op->type) {
 	  case BOW:
 	  case WAND:
 	  case ROD:
 	  case HORN:
-	    strcat(buf," (readied)");
+	    strcat(buf[use_buf]," (readied)");
 	    break;
 	  case WEAPON:
-	    strcat(buf," (wielded)");
+	    strcat(buf[use_buf]," (wielded)");
 	    break;
 	  case ARMOUR:
 	  case HELMET:
@@ -359,22 +357,80 @@ char *query_name(object *op) {
 	  case GIRDLE:
 	  case BRACERS:
 	  case CLOAK:
-	    strcat(buf," (worn)");
+	    strcat(buf[use_buf]," (worn)");
 	    break;
 	  case CONTAINER:
-	    strcat(buf," (active)");
+	    strcat(buf[use_buf]," (active)");
 	    break;
 	  case SKILL:
 	  default:
-	    strcat(buf," (applied)");
+	    strcat(buf[use_buf]," (applied)");
 	}
     }
     if(QUERY_FLAG(op, FLAG_UNPAID))
-	strcat(buf," (unpaid)");
+	strcat(buf[use_buf]," (unpaid)");
 
-    return buf;
+    return buf[use_buf];
 }
 
+/*
+ * query_base_name(object) returns a character pointer pointing to a static
+ * buffer which contains a verbose textual representation of the name
+ * of the given object.  The buffer will be overwritten at the next
+ * call to query_base_name().   This is a lot like query_name, but we
+ * don't include the item count or item status.  Used for inventory sorting.
+ */
+char *query_base_name(object *op) {
+    static char buf[MAX_BUF];
+
+    if(op->name == NULL)
+	return "(null)";
+    if(!op->nrof && !op->weight && !op->title && !is_magical(op)) 
+	return op->name; /* To speed things up (or make things slower?) */
+
+    strcpy(buf,op->name);
+    if (op->title && QUERY_FLAG(op,FLAG_IDENTIFIED)) {
+	strcat(buf, " ");
+	strcat(buf, op->title);
+    }
+
+    switch(op->type) {
+      case SCROLL:
+      case WAND:
+      case SPELLBOOK:
+      case ROD:
+	if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
+	    if(!op->title) {
+		strcat(buf," of ");
+		strcat(buf,spells[op->stats.sp].name);
+		if(op->type != SPELLBOOK)
+		    sprintf(buf+strlen(buf), " (lvl %d)", op->level);
+	    }
+	}
+	break;
+
+      case SKILL:
+      case AMULET:
+      case RING:
+#ifdef FULL_RING_DESCRIPTION
+	if (!op->title) {
+	    /* If ring has a title, full description isn't so useful */ 
+	    char *s = ring_desc(op);
+	    if (s[0]) {
+		strcat (buf, " ");
+		strcat (buf, s);
+	    }
+	}
+#endif
+	break;
+      default:
+	if(op->magic && ((QUERY_FLAG(op,FLAG_BEEN_APPLIED) && 
+	   need_identify(op)) || QUERY_FLAG(op,FLAG_IDENTIFIED))) {
+	    sprintf(buf + strlen(buf), " %+d", op->magic);
+	}
+    }
+    return buf;
+}
 /*
  * Returns a pointer to a static buffer which contains a
  * description of the given object.
@@ -439,7 +495,31 @@ char *describe_item(object *op) {
     case GIRDLE:
     case BRACERS:
     case CLOAK:
-      break;
+	break;
+
+    case FOOD:
+    case FLESH:
+    case DRINK:
+      if(!need_identify(op) || QUERY_FLAG(op,FLAG_IDENTIFIED) || QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
+	sprintf(buf,"(food+%d)", op->stats.food);
+	strcat(retbuf, buf);
+	if (!QUERY_FLAG(op,FLAG_CURSED)) {
+	    if (op->stats.hp)
+		strcat(retbuf,"(heals)");
+	    if (op->stats.sp)
+		strcat(retbuf,"(spellpoint regen)");
+	}
+	else {
+	    if (op->stats.hp)
+		strcat(retbuf,"(damages)");
+	    if (op->stats.sp)
+		strcat(retbuf,"(spellpoint depletion)");
+	}
+      }
+	break;
+
+
+    case SKILL:
     case RING:
     case AMULET:
 #ifdef FULL_RING_DESCRIPTION
@@ -449,7 +529,7 @@ char *describe_item(object *op) {
       strcat (retbuf, ring_desc(op));
 #endif
       return retbuf;
-      break;
+
     default:
       return retbuf;
     }
@@ -458,7 +538,7 @@ char *describe_item(object *op) {
 
       int attr,val;
 
-      for (attr=0; attr<6; attr++) {
+      for (attr=0; attr<7; attr++) {
         if ((val=get_attr_value(&(op->stats),attr))!=0) {
           sprintf(buf, "(%s%+d)", short_stat_name[attr], val);
           strcat(retbuf,buf);
@@ -466,7 +546,7 @@ char *describe_item(object *op) {
       }
 
       if(op->stats.exp) {
-        sprintf(buf,"(speed %+ld)",op->stats.exp);
+        sprintf(buf,"(speed %+d)",op->stats.exp);
         strcat(retbuf,buf);
       }
       switch(op->type) {
@@ -478,6 +558,7 @@ char *describe_item(object *op) {
       case BOOTS:
       case GLOVES:
       case WEAPON:
+      case SKILL:
       case RING:
       case AMULET:
       case ARMOUR:
@@ -508,6 +589,9 @@ char *describe_item(object *op) {
         case BOW:
         case ARROW:
         case WAND:
+	case FOOD:
+	case FLESH:
+	case DRINK:
           break;
         default:
           if(op->stats.food) {
@@ -517,6 +601,10 @@ char *describe_item(object *op) {
               sprintf(buf,"(hunger%+d)",op->stats.food);
             strcat(retbuf,buf);
           }
+          if(op->stats.grace) {
+            sprintf(buf,"(grace%+d)",op->stats.grace);
+            strcat(retbuf,buf);
+          }
           if(op->stats.sp) {
             sprintf(buf,"(magic%+d)",op->stats.sp);
             strcat(retbuf,buf);
@@ -535,6 +623,10 @@ char *describe_item(object *op) {
         sprintf(buf,"(hunger%+d)",op->contr->digestion);
       strcat(retbuf,buf);
     }
+    if(op->contr->gen_grace) {
+      sprintf(buf,"(grace%+d)",op->contr->gen_grace);
+      strcat(retbuf,buf);
+    }
     if(op->contr->gen_sp) {
       sprintf(buf,"(magic%+d)",op->contr->gen_sp);
       strcat(retbuf,buf);
@@ -553,6 +645,8 @@ char *describe_item(object *op) {
       strcat(retbuf,"(xray-vision)");
     if(QUERY_FLAG(op,FLAG_FLYING))
       strcat(retbuf,"(levitate)");
+    if(QUERY_FLAG(op,FLAG_SEE_IN_DARK))
+      strcat(retbuf,"(infravision)");
   }
   if(!need_identify(op) || QUERY_FLAG(op,FLAG_IDENTIFIED)) {
     if(op->stats.luck) {
@@ -596,6 +690,8 @@ char *describe_item(object *op) {
       strcat(retbuf,"(use rod)");
     if(QUERY_FLAG(op,FLAG_USE_HORN))
       strcat(retbuf,"(use horn)");
+    if(QUERY_FLAG(op,FLAG_CAN_USE_SKILL))
+      strcat(retbuf,"(skill user)");
     if(QUERY_FLAG(op,FLAG_CAST_SPELL))
       strcat(retbuf,"(spellcaster)");
     if(QUERY_FLAG(op,FLAG_FRIENDLY))
@@ -604,11 +700,11 @@ char *describe_item(object *op) {
       strcat(retbuf,"(unaggressive)");
     if(QUERY_FLAG(op,FLAG_HITBACK))
       strcat(retbuf,"(hitback)");
-    if(op->arch->randomitems != NULL) {
+    if(op->randomitems != NULL) {
       treasure *t;
       int first = 1;
-      for(t=op->arch->randomitems->items; t != NULL; t=t->next)
-        if(t->item->clone.type == ABILITY) {
+      for(t=op->randomitems->items; t != NULL; t=t->next)
+        if(t->item && (t->item->clone.type == ABILITY)) {
           if(first) {
             first = 0;
             strcat(retbuf,"(Spell abilities:)");
@@ -641,62 +737,65 @@ char *describe_item(object *op) {
  */
 
 int is_magical(object *op) {
-	int i;
+    int i;
 
-	/* living creatures are considered non magical */
+    /* living creatures are considered non magical */
+    if (QUERY_FLAG(op, FLAG_ALIVE)) return 0;
 
-	if (QUERY_FLAG(op, FLAG_ALIVE)) return 0;
+    /* This is a test for it being an artifact, as artifacts have titles */
+    if (op->title!=NULL) return 1;
 
-	/* This is a test for it being an artifact, as artifacts have titles
-	 */
-	if (op->title!=NULL) return 1;
-
-	/* Handle rings and amulets specially.  If they change any of these
-	 * values, it means they are magical.
-	 */
-	if ((op->type==AMULET || op->type==RING) && 
-	    (op->stats.ac || op->stats.food || op->stats.exp || op->stats.dam ||
-	    op->stats.wc || op->stats.sp || op->stats.hp || op->stats.luck))
+    /* Handle rings and amulets specially.  If they change any of these
+     * values, it means they are magical.
+     */
+    if ((op->type==AMULET || op->type==RING) && 
+	(op->stats.ac || op->stats.food || op->stats.exp || op->stats.dam ||
+	op->stats.wc || op->stats.sp || op->stats.hp || op->stats.luck))
 		 return 1;
 
-	/* Take care of polished shield and helmet of xray vision */
-
-	if ((op->type==AMULET || op->type==SHIELD) && (QUERY_FLAG(op, FLAG_REFL_SPELL) ||
-	    QUERY_FLAG(op, FLAG_REFL_MISSILE))) return 1;
-
-	if (op->type==HELMET && QUERY_FLAG(op,FLAG_XRAYS)) return 1;
-
-
-	/* Potions, rods & boots are always magical.  Wands/staves are also magical,
-	 * assuming they still have any charges left.
-	 */
-	if (op->type==POTION || op->type==ROD || op->type==BOOTS ||
-	    (op->type==WAND && op->stats.food)) return 1;
-
-
-	/* if something protects, immunes or vulnerables, it must be magical.
-	 * power crystal, spellbooks, and scrolls are always magical.
-	 */
-	if (op->immune || op->protected || op->vulnerable || op->magic ||
-	    op->type==POWER_CRYSTAL || op->type==SPELLBOOK || op->type==SCROLL)
+    /* Check for stealty, speed, flying, or just plain magic in the boots */
+    if (op->type== BOOTS && 
+	(QUERY_FLAG(op, FLAG_STEALTH) || QUERY_FLAG(op, FLAG_FLYING) ||
+	 op->stats.exp))
 		return 1;
 
-	/* Check to see if it increases/decreases any stats */
-	for (i=0; i<6; i++) 
-		if (get_attr_value(&(op->stats),i)!=0) return 1;
+    /* Take care of amulet/shield that reflects spells/missiles */
+    if ((op->type==AMULET || op->type==SHIELD) && 
+	(QUERY_FLAG(op, FLAG_REFL_SPELL) || QUERY_FLAG(op, FLAG_REFL_MISSILE)))
+	    return 1;
 
-	/* If it doesn't fall into any of the above categories, must
-	 * be non magical.
-	 */
-	return 0;
+    /* Take care of helmet of xrays */
+    if (op->type==HELMET && QUERY_FLAG(op,FLAG_XRAYS)) return 1;
+
+    /* Potions & rods are always magical.  Wands/staves are also magical,
+     * assuming they still have any charges left.
+     */
+    if (op->type==POTION || op->type==ROD ||
+	(op->type==WAND && op->stats.food))
+	    return 1;
+
+    /* if something protects, immunes or vulnerables, it must be magical.
+     * power crystal, spellbooks, and scrolls are always magical.
+     */
+    if (op->immune || op->protected || op->vulnerable || op->magic ||
+	op->type==POWER_CRYSTAL || op->type==SPELLBOOK || op->type==SCROLL ||
+	op->type==GIRDLE)
+	    return 1;
+
+    /* Check to see if it increases/decreases any stats */
+    for (i=0; i<7; i++) 
+	    if (get_attr_value(&(op->stats),i)!=0) return 1;
+
+    /* If it doesn't fall into any of the above categories, must
+     * be non magical.
+     */
+    return 0;
 }
-
+#if 0
 int always_magical(object *op) {
   switch(op->type) {
   case RING:
   case BRACERS:
-  case BOOTS: /* For now at least */
-  case GLOVES: /* ditto */
   case AMULET:
   case GIRDLE:
   case ROD:
@@ -710,6 +809,7 @@ int always_magical(object *op) {
   }
   return 0;
 }
+#endif
 
 /* need_identify returns true if the item should be identified.  This 
  * function really should not exist - by default, any item not identified
@@ -723,6 +823,8 @@ int need_identify(object *op) {
   case ROD:
   case HORN:
   case SCROLL:
+  case SKILL:
+  case SKILLSCROLL:
   case SPELLBOOK:
   case FOOD:
   case POTION:
@@ -738,10 +840,15 @@ int need_identify(object *op) {
   case BRACERS:
   case GIRDLE:
   case CONTAINER:
+  case DRINK:
+  case FLESH:
+  case INORGANIC:
   case CLOSE_CON:
   case CLOAK:
   case GEM:
   case POWER_CRYSTAL:
+  case POISON:
+  case BOOK:
     return 1;
   }
 /* Try to track down some stuff that may show up here.  Thus, the
