@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_c =
- *   "$Id: loader.c,v 1.30 1995/07/24 22:03:20 master Exp $";
+ *   "$Id: loader.c,v 1.37 1997/03/09 04:09:36 master Exp master $";
  */
 
 /*
@@ -35,7 +35,8 @@
 
 /* All the variable names.  Each entry should correspond to the 
  * V_ vlaue in loader.h (that is, the 20'th value in this array should
- * correspond to the 20'th V_(value) in loader.h)
+ * correspond to the 20'th V_(value) in loader.h)  The V_??? get
+ * used as indexes into this array.
  */
 
 /* If you change this list (add/delete/rename), also update the list
@@ -45,21 +46,24 @@
 static char *variable_const[NR_OF_VARIABLES] = {
   "Object","name","race","slaying","msg","endmsg",
   "Inventory","arch","other_arch","More",
-  "anim","mina","end","last_heal","last_sp","last_eat",
+  "anim","mina","end","last_heal","last_sp","last_grace","last_eat",
   "speed","speed_left","slow_move",
-  "face","Str","Dex","Con","Wis","Cha","Int","hp","maxhp","sp","maxsp",
+  "face","Str","Dex","Con","Wis","Cha","Int","Pow","hp","maxhp","sp","maxsp",
+  "grace","maxgrace",
   "exp","food","dam","wc","ac","x","y","nrof","level","direction",
   "type","material","value", "weight","carrying",
   "immune","protected","attacktype","vulnerable",
   "path_attuned","path_repelled","path_denied",
   "invisible","magic","state","alive","applied","unpaid","need_an","need_ie",
   "no_pick","no_pass","walk_on","walk_off","fly_on","fly_off","is_animated",
-  "flying","monster", "friendly","generator","auto_apply","treasure",
+  "flying","monster",
+  "friendly","generator","is_thrown","auto_apply","treasure",
   "apply_once","see_invisible","can_roll","is_turning","is_turnable",
   "is_used_up","identified","reflecting","changing","splitting","hitback",
   "startequip","blocksview","editable","undead","scared","unaggressive",
-  "color_fg","color_bg","reflect_missile","reflect_spell","no_magic",
-  "wiz","was_wiz","no_fix_player","tear_down", "luck", "run_away","pass_thru",
+  "reflect_missile","reflect_spell","no_magic",
+  "wiz","was_wiz","no_fix_player","tear_down", "luck",
+  "run_away","pass_thru",
   "can_pass_thru","pick_up","anim_speed","container","no_drop",
   "no_pretext","will_apply","random_movement", "can_apply",
   "can_cast_spell","can_use_scroll","can_use_wand","can_use_bow",
@@ -68,10 +72,12 @@ static char *variable_const[NR_OF_VARIABLES] = {
   "sleep","stand_still","random_move","only_attack","armour",
   "attack_movement","move_state","confused","stealth","connected",
   "cursed","damned","see_anywhere","known_magical","known_cursed",
-  "can_steal","been_applied","title","has_ready_rod","can_use_rod",
+  "can_use_skill","been_applied","title","has_ready_rod","can_use_rod",
   "has_ready_horn","can_use_horn","expmul",
   "unique","make_invisible","inv_locked",
-  "is_wooded","is_hilly","has_ready_skill","has_ready_weapon", 
+  "is_wooded","is_hilly","has_ready_skill","has_ready_weapon",
+  "no_skill_ident","glow_radius","is_blind","can_see_in_dark",
+  "is_cauldron","randomitems","is_dust", "no_steal",
 #ifdef NPC_PROG
   "npc_status","npc_program",
 #endif
@@ -80,7 +86,7 @@ static char *variable_const[NR_OF_VARIABLES] = {
 char *variables[NR_OF_VARIABLES];
 
 /* This array equates the FLAG_ values with the V_ values.  Use -1 to
- * put gaps in the array that should no be processed.
+ * put gaps in the array that should not be processed.
  * The order matches the order of the define values in 'define.h'.
  */
 
@@ -93,7 +99,7 @@ int flag_links[NUM_FLAGS+1][2] ={
 {FLAG_NO_PASS, V_NO_PASS},{FLAG_ANIMATE, V_IS_ANIMATED},
 {FLAG_SLOW_MOVE, -1}, {FLAG_FLYING, V_FLYING},
 {FLAG_MONSTER, V_MONSTER}, {FLAG_FRIENDLY, V_FRIENDLY},
-{FLAG_GENERATOR, V_GENERATOR}, {-1, -1},	/* FLAG_THROWN */
+{FLAG_GENERATOR, V_GENERATOR}, {FLAG_IS_THROWN, V_IS_THROWN},
 {FLAG_AUTO_APPLY, V_AUTO_APPLY}, {FLAG_TREASURE, V_TREASURE},
 {FLAG_APPLY_ONCE, V_APPLY_ONCE},{FLAG_SEE_INVISIBLE, V_SEE_INVISIBLE},
 {FLAG_CAN_ROLL, V_CAN_ROLL}, {FLAG_IS_TURNING, V_IS_TURNING}, 
@@ -124,13 +130,15 @@ int flag_links[NUM_FLAGS+1][2] ={
 {-1, -1 /*WIZPASS*/}, {-1, -1 /*IS_LINKED */},
 {FLAG_CURSED, V_CURSED}, {FLAG_DAMNED, V_DAMNED},
 {FLAG_SEE_ANYWHERE, V_SEE_ANYWHERE}, {FLAG_KNOWN_MAGICAL, V_KNOWN_MAGICAL},
-{FLAG_KNOWN_CURSED, V_KNOWN_CURSED}, {FLAG_CAN_STEAL, V_CAN_STEAL},
+{FLAG_KNOWN_CURSED, V_KNOWN_CURSED}, {FLAG_CAN_USE_SKILL, V_CAN_USE_SKILL},
 {FLAG_BEEN_APPLIED, V_BEEN_APPLIED},  {FLAG_READY_ROD, V_HAS_READY_ROD},
 {FLAG_USE_ROD, V_CAN_USE_ROD}, {FLAG_READY_HORN, V_HAS_READY_HORN}, 
 {FLAG_USE_HORN, V_CAN_USE_HORN},{FLAG_MAKE_INVIS,V_MAKE_INVIS},
 {FLAG_INV_LOCKED,V_INV_LOCKED},{FLAG_IS_WOODED,V_IS_WOODED},
 {FLAG_IS_HILLY,V_IS_HILLY},{FLAG_READY_SKILL,V_HAS_READY_SKILL},
-{FLAG_READY_WEAPON,V_HAS_READY_WEAPON}
+{FLAG_READY_WEAPON,V_HAS_READY_WEAPON},{FLAG_NO_SKILL_IDENT,V_NO_SKILL_IDENT},
+{FLAG_BLIND,V_BLIND},{FLAG_SEE_IN_DARK,V_SEE_IN_DARK},
+{FLAG_IS_CAULDRON,V_IS_CAULDRON},{FLAG_DUST,V_DUST},{FLAG_NO_STEAL,V_NO_STEAL}
 };
 
 void save_double(char *buf,char *name,double v)
@@ -169,7 +177,10 @@ int get_variable(char *name) {
 
 /*
  * Returns a pointer to a static string which contains all variables
- * which are different in the two given objects.
+ * which are different in the two given objects.  op is the what object
+ * the different values will be taken from.  This function is
+ * typically used to dump objects (op2=empty object), or to save objects
+ * (op2 is the objects original archetype)
  */
 
 char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
@@ -217,6 +228,8 @@ char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
     save_long(buf,variable_const[V_CON],op->stats.Con);
   if(op->stats.Wis!=op2->stats.Wis)
     save_long(buf,variable_const[V_WIS],op->stats.Wis);
+  if(op->stats.Pow!=op2->stats.Pow)
+    save_long(buf,variable_const[V_POW],op->stats.Pow);
   if(op->stats.Cha!=op2->stats.Cha)
     save_long(buf,variable_const[V_CHA],op->stats.Cha);
   if(op->stats.Int!=op2->stats.Int)
@@ -229,6 +242,10 @@ char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
     save_long(buf,variable_const[V_SP],op->stats.sp);
   if(op->stats.maxsp!=op2->stats.maxsp)
     save_long(buf,variable_const[V_MAXSP],op->stats.maxsp);
+  if(op->stats.grace!=op2->stats.grace)
+    save_long(buf,variable_const[V_GRACE],op->stats.grace);
+  if(op->stats.maxgrace!=op2->stats.maxgrace)
+    save_long(buf,variable_const[V_MAXGRACE],op->stats.maxgrace);
   if(op->stats.exp!=op2->stats.exp)
     save_long(buf,variable_const[V_EXP],op->stats.exp);
   if(op->expmul!=op2->expmul) 
@@ -301,10 +318,18 @@ char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
     save_long(buf,variable_const[V_LAST_HEAL],op->last_heal);
   if(op->last_sp!=op2->last_sp)
     save_long(buf,variable_const[V_LAST_SP],op->last_sp);
+  if(op->last_grace!=op2->last_grace)
+    save_long(buf,variable_const[V_LAST_GRACE],op->last_grace);
   if(op->last_eat!=op2->last_eat)
     save_long(buf,variable_const[V_LAST_EAT],op->last_eat);
   if(QUERY_FLAG(op,FLAG_IS_LINKED) && (tmp = get_button_value(op)))
     save_long(buf,variable_const[V_CONNECTED],tmp);
+  if(op->glow_radius!=op2->glow_radius) 
+    save_long(buf,variable_const[V_GLOW_RADIUS],op->glow_radius);
+  if (op->randomitems!=op2->randomitems) {
+    sprintf(buf2,"randomitems %s\n",(op->randomitems?op->randomitems->name:"none"));
+    strcat(buf,buf2);
+  }
 #ifdef NPC_PROG
   if(op->npc_status!=op2->npc_status)
     save_long(buf,variable_const[V_NPC_STATUS],op->npc_status);
@@ -312,16 +337,15 @@ char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
     save_long(buf,variable_const[V_NPC_PROGRAM],op->npc_program);
 #endif
 
-#if 0
-  if(op->run_away!=op2->run_away)
-    save_long(buf,variable_const[V_EDITABLE],op->arch->editable);
-#endif
 
 /* Eneq(@csd.uu.se): Handles run_away and pick_up */
   if(op->run_away!=op2->run_away)
     save_long(buf,variable_const[V_RUN_AWAY],op->run_away);
   if(op->pick_up!=op2->pick_up)
     save_long(buf,variable_const[V_PICK_UP],op->pick_up);
+  if(op->weight_limit!=op2->weight_limit)
+    save_long(buf,variable_const[V_CONTAINER],op->weight_limit);
+
 
 /* Vick (@bern.docs.uu.se) @921107 -> Handle 'will_apply' &
    'random_movement.*/
@@ -351,51 +375,62 @@ char *get_ob_diff(object *op,object *op2) {/* I plan to optimize this heavily */
 
 /*
  * Dumps all variables in an object to a file.
- * If bit 0 of flag is set, unpaid objects will be saved.
- * If bit 1 of flag is set, don't remove the object after save.
+ * If bit 0 of flag is set, unpaid objects will be saved.  As of now,
+ * the only place this is not set is when saving the player.
+ * If bit 1 of flag is set, don't remove the object after save.  As of now,
+ * all of the callers are setting this.
  */
 
 void save_object(FILE *fp,object *op, int flag) {
-  archetype *at;
-  char *cp;
-  object *tmp,*old;
+    archetype *at;
+    char *cp;
+    object *tmp,*old;
 
-  if(op->owner!=NULL || fp == NULL)
-    return;
-  if(!(flag&1)&&(QUERY_FLAG(op, FLAG_UNPAID))) {
-#if 0
-    remove_ob(op);
-    free_object(op);
-#endif
-    return;
-  }
-  if((at=op->arch)!=NULL)
+    /* Even if the object does have an owner, it would seem that we should
+     * still save it.
+     */
+    if(op->owner!=NULL || fp == NULL)
+	return;
+
+    /* If it is unpaid and we don't want to save those, just return. */
+    if(!(flag&1)&&(QUERY_FLAG(op, FLAG_UNPAID))) {
+	return;
+    }
+
+    if((at=op->arch)==NULL) at=empty_archetype;
     fprintf(fp,"arch %s\n",at->name);
-  if((cp=get_ob_diff(op,&((at==NULL?empty_archetype:at)->clone)))!=NULL)
-    fputs(cp,fp);
 
-/* Eneq(@csd.uu.se): Added this to allow containers being saved with contents*/
+    if((cp=get_ob_diff(op,&at->clone))!=NULL)
+	fputs(cp,fp);	/* We really should do some status checking on this */
 
-  old=NULL;
+    /* Eneq(@csd.uu.se): Added this to allow containers being saved with contents*/
 
-  if (flag & 2 )
-    for(tmp=op->inv;tmp!=NULL;tmp=tmp->below)
-      save_object(fp,tmp,flag);
-  else while ((tmp=op->inv)!=NULL) {
+    old=NULL;
+
+    if (flag & 2 )
+	for(tmp=op->inv;tmp!=NULL;tmp=tmp->below)
+	    save_object(fp,tmp,flag);
+
+    /* Slightly different logic because tmp/op will be removed by
+     * the save_object we call.  So we just keep looking at op->inv
+     * until there is nothing left.  In theory, the variable old
+     * should not be needed, as recursive loops shouldn't happen.
+     */
+    else while ((tmp=op->inv)!=NULL) {
 	if(old==tmp) {
-	  LOG(llevError,"Recursive loop in inventory\n");
-	  break;
+	    LOG(llevError,"Recursive loop in inventory\n");
+	    break;
 	}
 	save_object(fp,tmp,flag); 
 	old=tmp;
-  }
+    }
    
-  if (!(flag&2)) {
-      remove_ob(op);
-      free_object (op);
-  }
+    if (!(flag&2)) {
+	remove_ob(op);
+	free_object (op);
+    }
 
-  fprintf(fp,"end\n");
+    fprintf(fp,"end\n");
 }
 
 
@@ -530,25 +565,7 @@ int set_variable(object *op,char *buf) {
     op->weight_limit=(signed long) value;
     break;
   case V_FACE:
-#if 0
-    op->face.number=(Fontindex)FindFace (vbp,0);
-#else
     op->face = &new_faces[FindFace(vbp,0)];
-#endif
-    break;
-  case V_COLOR_FG:
-#if 0
-    op->face.fg = find_color(vbp);
-#else
-/*    LOG(llevDebug,"Got object with color set: %s\n", vbp);*/
-#endif
-    break;
-  case V_COLOR_BG:
-#if 0
-    op->face.bg = find_color(vbp);
-#else
-/*    LOG(llevDebug,"Got object with color set: %s\n", vbp);*/
-#endif
     break;
   case V_STR:
     op->stats.Str=(signed char) value;
@@ -562,6 +579,9 @@ int set_variable(object *op,char *buf) {
   case V_WIS:
     op->stats.Wis=(signed char) value;
     break;
+  case V_POW:
+    op->stats.Pow=(signed char) value;
+    break;
   case V_CHA:
     op->stats.Cha=(signed char) value;
     break;
@@ -580,6 +600,12 @@ int set_variable(object *op,char *buf) {
   case V_MAXSP:
     op->stats.maxsp=(signed short) value;
     break;
+  case V_GRACE:
+    op->stats.grace=(signed short) value;
+    break;
+  case V_MAXGRACE:
+    op->stats.maxgrace=(signed short) value;
+    break;
   case V_EXP:
     op->stats.exp=(signed long) value;
     break;
@@ -587,7 +613,7 @@ int set_variable(object *op,char *buf) {
     sscanf(vbp,"%lf",&(op->expmul));
     break;
   case V_FOOD:
-    op->stats.food=(signed short) value;
+    op->stats.food=(sint16) value;
     break;
   case V_DAM:
     op->stats.dam=(sint16) value;
@@ -672,6 +698,9 @@ int set_variable(object *op,char *buf) {
   case V_LAST_SP:
     op->last_sp=(signed short) value;
     break;
+  case V_LAST_GRACE:
+    op->last_grace=(signed short) value;
+    break;
   case V_LAST_EAT:
     op->last_eat=(signed short) value;
     break;
@@ -710,6 +739,10 @@ int set_variable(object *op,char *buf) {
     op->will_apply = (unsigned char) value;
     break;
 
+  case V_GLOW_RADIUS:
+    op->glow_radius = (signed short) value;
+    break;
+
 /* Note: With the new FLAG value method, most all of these bits are
  * set in the default area.  The only ones not set there are
  * special cases (those needing to check the object, print debugging
@@ -762,11 +795,14 @@ int set_variable(object *op,char *buf) {
     else
       CLEAR_FLAG(op,FLAG_WAS_WIZ);
     break;
+  case V_RANDOMITEMS:
+    op->randomitems=find_treasurelist(vbp);
+    break;
+    
 
  /* In the default statement, it will handle all cases above.
   * First search through the flag_links, to match corresponding
-  * variables.  IF it doesn't find anything, then to the check for
-  * randomitems, and then print the error messages.
+  * variables.  If it doesn't find anything, then print the error messages.
   */
   default:
     for (tmp=0; tmp<=NUM_FLAGS; tmp++) {
@@ -776,8 +812,6 @@ int set_variable(object *op,char *buf) {
 	    return 0;
 	}
     }
-    if(!strcmp(buf,"randomitems"))
-      return 0;
     sprintf(errmsg,"Warning, unknown (or oldfashioned) variable: %s",buf);
     return -1;
   }
@@ -797,7 +831,6 @@ int load_object(FILE *fp, object *op) {
   Fontindex faces[MAX_ANIMATIONS];
   int position=1; /* 1 = standalone, 2 = inventory, 3 = head/tail link */
   int anim_start=0,msg_start=0;
-  int i;
   object *tmp;
 
   buf[MAX_BUF - 1] = '\0';
@@ -837,7 +870,7 @@ int load_object(FILE *fp, object *op) {
 	else op->inv=tmp;
         return load_object (fp, op);
     }
-    switch(i=set_variable(op,buf)) {
+    switch(set_variable(op,buf)) {
     case -1:
       LOG(llevError,"%s\n",errmsg);
       break;
@@ -846,7 +879,6 @@ int load_object(FILE *fp, object *op) {
       break;
     case 2:
       return position;
-      break;
     case 3:
       position=2;
       break;
@@ -860,8 +892,13 @@ int load_object(FILE *fp, object *op) {
     case 6:
       msg_start=0;
       op->msg=add_string(msgbuf);
-      if (strlen(op->msg) > 500)
-        LOG(llevDebug, "message: %d\n", strlen(op->msg));
+	/* Only reason to worry about this is if we might be in danger
+	 * of going beyond the limits of the buffer.
+	 */
+      if (strlen(op->msg) > (HUGE_BUF/2))
+	LOG(llevDebug, "\n\tWarning message length > %d: %d\n>%.80s<\n",
+	    HUGE_BUF/2,
+	    strlen(op->msg),op->msg);
       break;
     }
   }
@@ -915,7 +952,17 @@ object *LoadObject (FILE *fp, char *inbuf) {
 	    if(!strncmp (buf, "arch ", 5)) {
 		object *tmp;
 		tmp = LoadObject(fp,buf);
-		(void) insert_ob_in_ob (tmp, op);
+		/* If we don't have an arch or name, chances are the object
+		 * is one that has been removed or isn't available.  Just
+		 * drop it then (can't really dump it - no information
+		 * to show.)
+		 */
+		if (!tmp->arch && !tmp->name) {
+		    LOG(llevDebug,"Dropping null object.\n");
+		    free_object(tmp);
+		}
+		else
+		    (void) insert_ob_in_ob (tmp, op);
 		break;
 	    }
 
@@ -934,7 +981,6 @@ object *LoadObject (FILE *fp, char *inbuf) {
 
 	      case 2:
 		return op;
-		break;
 
 	      case 5:
 		state = message;
