@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_loger_c =
- *   "$Id: logger.c,v 1.2 1994/10/11 07:22:11 master Exp $ ";
+ *   "$Id: logger.c,v 1.3 1997/03/09 04:13:24 master Exp master $ ";
  */
 
 /*
@@ -41,7 +41,10 @@
 void LOG (LogLevel logLevel, char *format, ...)
 {
 #ifdef SERVER
-  char buf[VERY_BIG_BUF];
+  char buf[20480];  /* This needs to be really really big - larger
+		     * than any other buffer, since that buffer may
+		     * need to be put in this one.
+		     */
   sockets *s;
 #endif
 
@@ -50,7 +53,7 @@ void LOG (LogLevel logLevel, char *format, ...)
 
 #ifdef SERVER
   buf[0] = '\0';
-  if (logLevel <= debug)
+  if (logLevel <= settings.debug)
   {
     vsprintf(buf, format, ap);
     fputs(buf, logfile);
@@ -64,7 +67,7 @@ void LOG (LogLevel logLevel, char *format, ...)
         perror("log/write");
     }
 #else
-  if (logLevel <= debug)
+  if (logLevel <= settings.debug)
     vfprintf(logfile, format, ap);
 #endif
   if (!exiting && !trying_emergency_save &&
