@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.42 1995/04/15 03:14:28 master Exp $";
+ *   "$Id: map.c,v 1.48 1997/03/09 04:15:09 master Exp master $";
  */
 
 /*
@@ -101,13 +101,16 @@ mapstruct *has_been_loaded (char *name) {
  */
 
 char *create_pathname (char *name) {
-  static char buf[MAX_BUF];
+    static char buf[MAX_BUF];
 
-  if (*name == '/')
-      sprintf (buf, "%s/%s%s", LibDir, MapDir, name);
-  else
-      sprintf (buf, "%s/%s/%s", LibDir, MapDir, name);
-  return (buf);
+    /* Why?  having extra / doesn't confuse unix anyplace?  Dependancies
+     * someplace else in the code? msw 2-17-97
+     */
+    if (*name == '/')
+      sprintf (buf, "%s/%s%s", settings.libdir, settings.mapdir, name);
+    else
+      sprintf (buf, "%s/%s/%s", settings.libdir, settings.mapdir, name);
+    return (buf);
 }
 
 #ifdef UNIQUE_ITEMS
@@ -124,7 +127,7 @@ static char *create_items_path (char *s) {
     if (*s == '/')
       s++;
 
-    sprintf (buf, "%s/%s/", LibDir, ItemsDir);
+    sprintf (buf, "%s/%s/", settings.libdir, settings.uniquedir);
 
     for (t=buf+strlen(buf); *s; s++,t++)
       if (*s == '/')
@@ -221,6 +224,16 @@ int check_path (char *name)
     return (mode);
 }
 
+void dump_map_lights(mapstruct *m) {
+  objectlink *obl=m->light;
+  object *tmp=NULL;
+  while(obl) {
+     if((tmp=obl->ob)) 
+       LOG(llevDebug,"%s (%d) radius:%d\n",tmp->name,tmp->count,tmp->glow_radius); 
+     obl=obl->next;
+  } 
+}
+
 /*
  * Prints out debug-information about a map.
  */
@@ -234,6 +247,10 @@ void dump_map(mapstruct *m) {
     LOG(llevError,"Message:\n%s",m->map_object->msg);
   if(m->tmpname!=NULL)
     LOG(llevError,"Tmpname: %s\n",m->tmpname);
+  LOG(llevError,"Difficulty: %d\n",m->difficulty); 
+#ifdef USE_LIGHTING
+  LOG(llevError,"Darkness: %d\n",m->darkness); 
+#endif
 }
 
 /*
@@ -553,7 +570,7 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
 #endif
 
     object *op, *prev=NULL,*last_more=NULL;
-    MapLook	maplook;
+/*    MapLook	maplook;*/
 
     op=get_object();
     op->map = m; /* To handle buttons correctly */
@@ -568,6 +585,14 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
 		LOG(llevDebug,"Discarded object %s - invalid archetype.\n",op->name);
 	    continue;
 	}
+	/* this hack allows for 'unused' altars (see below) to have a 
+ 	 * random god (see the 'god table' in god.c) associated w/ it -b.t. 
+	 */
+#ifdef MULTIPLE_GODS
+	if(op->type==ALTAR && !op->value && !editor && !op->stats.sp)
+	   if(baptize_altar(op))
+		LOG(llevDebug,"Baptized altar to %s\n",op->title);
+#endif
 	switch(i) {
 	  case 1:
 	    insert_ob_in_map(op,m);
@@ -598,11 +623,13 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
 	 * the floor only needs to be determined once.
 	 * Mark Wedel (master@rahul.net)
 	 */
+#if 0
 	if (QUERY_FLAG(op,FLAG_IS_FLOOR)) {
 		maplook.face = op->face;
 		maplook.flags = 0;
 		m->floor[op->x+ m->mapx*op->y] = maplook;
 	}
+#endif
 	op=get_object();
         op->map = m;
 #if defined(PROCESS_WHILE_LOADING) && CHECK_ACTIVE_MAPS
@@ -617,17 +644,21 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
 }
 
 void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
-    int i, j = 0;
+    int i, j = 0,unique=0;
     object *op, *tmp, *otmp;
 #if 0 /*defined(PROCESS_WHILE_LOADING) && CHECK_ACTIVE_MAPS*/
     int  elapsed = 0;
 #endif
     /* first pass - save one-part objects */
     for(i = 0; i < m->mapx; i++)
-	for (j = 0; j < m->mapy; j++)
+	for (j = 0; j < m->mapy; j++) {
+	    unique=0;
 	    for(op = get_map_ob (m, i, j); op; op = otmp) {
 		otmp = op->above;
 
+		if (QUERY_FLAG(op,FLAG_IS_FLOOR) && QUERY_FLAG(op, FLAG_UNIQUE))
+		    unique=1;
+
 		if(op->type == PLAYER) {
 			LOG(llevDebug, "Player on map that is being saved\n");
 			continue;
@@ -636,8 +667,10 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
 		if (op->head || op->more || op->owner)
 		    continue;
 
-              save_object(QUERY_FLAG(op, FLAG_UNIQUE) ? fp2 : fp, op, 3);
-		/* don't check and don't remove */
+		if (unique || QUERY_FLAG(op, FLAG_UNIQUE))
+		    save_object( fp2 , op, 3);
+		else
+		    save_object(fp, op, 3);
 
 /* Would be nice to let players still do stuff while a map is saving.
  * The problem is, having a player enter a map that is in the process
@@ -652,12 +685,17 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
 	      }
 #endif
 	    }
+	}
     /* second pass - save multi-part objects */
     for(i = 0; i < m->mapx; i++)
-        for (j = 0; j < m->mapy; j++)
+        for (j = 0; j < m->mapy; j++) {
+	    unique=0;
             for(op = get_map_ob (m, i, j); op; op = otmp) {
                 otmp = op->above;
 
+		if (QUERY_FLAG(op,FLAG_IS_FLOOR) && QUERY_FLAG(op, FLAG_UNIQUE))
+		    unique=1;
+
 		if(op->type == PLAYER) {
 			LOG(llevDebug, "Player on map that is being saved\n");
 			continue;
@@ -666,13 +704,19 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
 		if (op->head || !op->more || op->owner)
 		    continue;
 
-              save_object(QUERY_FLAG(op, FLAG_UNIQUE) ? fp2 : fp, op, 3);
-                /* don't check and don't remove */
+		if (unique || QUERY_FLAG(op, FLAG_UNIQUE))
+		    save_object( fp2 , op, 3);
+		else
+		    save_object(fp, op, 3);
 
 		for (tmp = op->more; tmp; tmp = tmp->more) {
-                  fprintf (QUERY_FLAG(tmp, FLAG_UNIQUE) ? fp2:fp, "More\n");
-                  save_object(QUERY_FLAG(tmp, FLAG_UNIQUE) ? fp2:fp, tmp, 3);
-		    /* don't check and don't remove */
+		    if (unique || QUERY_FLAG(op, FLAG_UNIQUE)) {
+			fprintf ( fp2, "More\n");
+			save_object( fp2 , tmp, 3);
+		    } else {
+			fprintf ( fp, "More\n");
+			save_object(fp, tmp, 3);
+		    }
 		}
 #if 0 /*defined(PROCESS_WHILE_LOADING) && CHECK_ACTIVE_MAPS*/
 	      if(elapsed++>CHECK_ACTIVE_MAPS) {
@@ -682,6 +726,7 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
 	      }
 #endif
 	    }
+	}
 }
 
 /*
@@ -734,7 +779,8 @@ mapstruct *load_original_map(char *filename, int block) {
     close_and_delete(fp, comp);
     m->in_memory=MAP_IN_MEMORY;
     m->difficulty=calculate_difficulty(m);
-
+    m->darkness=m->map_object->invisible;
+    m->do_los=0;
     set_map_reset_time(m);
 
     return (m);
@@ -748,17 +794,24 @@ void load_temporary_map(mapstruct *m) {
     FILE *fp;
     object *op;
     int comp;
+    char buf[MAX_BUF];
     
     if (!m->tmpname) {
 	LOG(llevError, "No temporary filename for map %s\n", m->path);
-        m = load_original_map(m->path,0);
+	strcpy(buf, m->path);
+	delete_map(m);
+        m = load_original_map(buf,0);
+	(*fix_auto_apply_func)(m); /* Chests which open as default */
 	return;
     }
 
     if((fp=open_and_uncompress(m->tmpname,0, &comp))==NULL) {
 	LOG(llevError,"Can't open %s\n",m->tmpname);
 	perror("Can't read map file");
-        m = load_original_map(m->path,0);
+	strcpy(buf, m->path);
+	delete_map(m);
+        m = load_original_map(buf,0);
+	(*fix_auto_apply_func)(m); /* Chests which open as default */
 	return;
     }
     
@@ -829,21 +882,28 @@ int match_filename(char *s1, char *s2)
 
 void delete_unique_items(mapstruct *m)
 {
-    int i,j;
+    int i,j,unique=0;
     object *op, *next;
 
     for(i=0; i<m->map_object->x; i++)
-      for(j=0; j<m->map_object->y; j++)
-          for (op=get_map_ob(m, i, j); op; op=next) {
-              next = op->above;
-              if(op->head == NULL && QUERY_FLAG(op, FLAG_UNIQUE)) {
-                  clean_object(op);
-                  remove_ob(op);
-                  free_object(op);
-              }
-          }
+	for(j=0; j<m->map_object->y; j++) {
+	    unique=0;
+	    for (op=get_map_ob(m, i, j); op; op=next) {
+		next = op->above;
+		if (QUERY_FLAG(op, FLAG_IS_FLOOR) && QUERY_FLAG(op, FLAG_UNIQUE))
+		    unique=1;
+		if(op->head == NULL && (QUERY_FLAG(op, FLAG_UNIQUE) || unique)) {
+		    clean_object(op);
+		    if (QUERY_FLAG(op, FLAG_IS_LINKED))
+			remove_button_link(op);
+		    remove_ob(op);
+		    free_object(op);
+		}
+	    }
+	}
 }
 
+
 /*
  * Loads unique objects from file(s) into the map which is in memory
  */
@@ -954,7 +1014,7 @@ int new_save_map(mapstruct *m, int flag) {
 	make_path_to_file (filename);
     } else {
 	if (!m->tmpname)
-	    m->tmpname = tempnam(TMPDIR,NULL);
+	    m->tmpname = tempnam(settings.tmpdir,NULL);
 	strcpy(filename, m->tmpname);
     }
     LOG(llevDebug,"Saving map %s\n",m->path);
@@ -1028,12 +1088,7 @@ int new_save_map(mapstruct *m, int flag) {
     else
       fclose(fp);
 
-#if 0
-    if (flag && m->read_only) /* Huh ! */
-	chmod (filename, 0644);
-    else
-#endif
-	chmod (filename, 0644);
+    chmod (filename, 0644);
     return 0;
 }
 /*
@@ -1141,15 +1196,17 @@ void relink_objs_offset(mapstruct *m1, mapstruct *m2, int dx, int dy) {
 
 void clean_object(object *op)
 {
-  object *tmp, *next;
+    object *tmp, *next;
 
-  for(tmp = op->inv; tmp; tmp = next)
-  {
-    next = tmp->below;
-    clean_object(tmp);
-    remove_ob(tmp);
-    free_object(tmp);
-  }
+    for(tmp = op->inv; tmp; tmp = next)
+    {
+	next = tmp->below;
+	clean_object(tmp);
+	if (QUERY_FLAG(tmp, FLAG_IS_LINKED))
+	    remove_button_link(tmp);
+	remove_ob(tmp);
+	free_object(tmp);
+    }
 }
 
 #if 0 /* Why make it so difficult?  I believe the below is buggy */
@@ -1289,6 +1346,7 @@ void free_map(mapstruct *m,int flag) {
 
 /*
  * Allocates, initialises, and returns a pointer to a mapstruct.
+ * takes a path argument it doesn't look like it uses.
  */
 
 mapstruct *get_linked_map(char *path) {
@@ -1310,6 +1368,9 @@ mapstruct *get_linked_map(char *path) {
   map->in_memory=MAP_SWAPPED;
 /*  map->read_only=0;*/
   map->timeout=0;
+  map->light = (objectlink *) NULL;
+  map->darkness = 0;
+  map->do_los=0;
   map->pending = (objectlink *) NULL;
   map->map_object = NULL;
   map->buttons = NULL;
@@ -1368,6 +1429,8 @@ mapstruct *get_empty_map (int sizex, int sizey) {
 
 /*
  * Makes sure the given map is loaded and swapped in.
+ * name is path name of the map.  flush is true if called from
+ * the editor, false otherwise.
  * Returns a pointer to the given map.
  */
 
@@ -1413,8 +1476,11 @@ mapstruct *ready_map_name(char *name, int flush) {
 	m->in_memory = MAP_IN_MEMORY;
 	/* tempnam() on sun systems (probably others) uses malloc
 	 * to allocated space for the string.  Free it here.
+	 * In some cases, load_temporary_map above won't find the
+	 * temporary map, and so has reloaded a new map.  If that
+	 * is the case, tmpname is now null
 	 */
-	free(m->tmpname);
+	if (m->tmpname) free(m->tmpname);
 	m->tmpname = NULL;
 	/* It's going to be saved anew anyway */
     }
