@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_c =
- *   "$Id: object.c,v 1.38 1995/09/11 02:44:37 master Exp master $";
+ *   "$Id: object.c,v 1.49 1997/03/09 04:19:06 master Exp master $";
  */
 
 /*
@@ -38,8 +38,6 @@
 #include <funcpoint.h>
 #include <skills.h> 
 
-extern int penalty;
-
 object objarray[STARTMAX]; /* All objects, allocated this way at first */
 object *objects;           /* Pointer to the list of used objects */
 object *free_objects;      /* Pointer to the list of unused objects */
@@ -84,13 +82,14 @@ signed long sum_weight(object *op) {
 
 /*
  * Eneq(@csd.uu.se): Since we can have items buried in a character we need
- * a better check
+ * a better check.  We basically keeping traversing up until we can't
+ * or find a player.
  */
 
 object *is_player_inv (object *op) { 
     for (;op!=NULL&&op->type!=PLAYER; op=op->env)
       if (op->env==op)
-      op->env = NULL;
+	op->env = NULL;
     return op;
 }
 
@@ -106,7 +105,7 @@ void dump_object2(object *op) {
 
   if(op->arch!=NULL) {
       strcat(errmsg,"arch ");
-      strcat(errmsg,op->arch->name);
+      strcat(errmsg,op->arch->name?op->arch->name:"(null)");
       strcat(errmsg,"\n");
       if((cp=get_ob_diff(op,&empty_archetype->clone))!=NULL)
 	strcat(errmsg,cp);
@@ -173,6 +172,7 @@ void animate_object(object *op) {
   if(op->arch==NULL) {
     dump_object(op);
     LOG(llevError,"Tried to animate object without archetype:\n%s\n", errmsg);
+    return;
   }
   if(!op->arch->animations) {
     LOG(llevError,"Archtype lacks animation.\n");
@@ -247,6 +247,34 @@ object *find_object_name(char *str) {
   return op;
 }
 
+void free_all_object_data() {
+    object *op, *next;
+
+    for (op=objects; op!=NULL; op=next) {
+	next=op->next;
+	if (!op->head && !QUERY_FLAG(op,FLAG_REMOVED))
+		remove_ob(op);
+	if (!op->head && !QUERY_FLAG(op,FLAG_FREED))
+		free_object(op);
+    }
+#if 0
+    /* In theory, we should do this.  In practice, it is fairly difficult
+     * because objects are created in groups of 100, and there is no record
+     * of those starting pointers.
+     */
+    for (op=free_objects; op!=NULL; ) {
+	next=op->next;
+	free(op);
+	nrofallocobjects--;
+	nroffreeobjects--;
+	last=op;
+	op=next;
+    }
+#endif
+    LOG(llevDebug,"%d allocated objects, %d free objects, STARMAX=%d\n", 
+	nrofallocobjects, nroffreeobjects,STARTMAX);
+}
+
 /*
  * Returns the object which this object marks as being the owner.
  * A id-scheme is used to avoid pointing to objects which have been
@@ -274,13 +302,25 @@ object *get_owner(object *op) {
 void set_owner(object *op, object *owner) {
   if(owner==NULL||op==NULL)
     return;
-
   /* next line added to allow objects which own objects */ 
   while(owner->owner&&owner!=owner->owner) owner=owner->owner;
     op->owner=owner;
 
   op->ownercount=owner->count;
   owner->refcount++;
+
+#ifdef ALLOW_SKILLS /* set the pointers in op to inherit owners skill, exp_obj */ 
+  if(owner->type==PLAYER&&owner->chosen_skill) {
+        op->chosen_skill = owner->chosen_skill;
+        op->exp_obj = owner->chosen_skill->exp_obj;
+
+	/* unfortunately, we can't allow summoned monsters skill use
+         * because we will need the chosen_skill field to pick the
+         * right skill/stat modifiers for calc_skill_exp(). See
+         * hit_player() in server/attack.c -b.t. */
+  } else if(op->type!=PLAYER && QUERY_FLAG(op,FLAG_READY_SKILL))
+               CLEAR_FLAG(op,FLAG_READY_SKILL);
+#endif
 }
 
 /*
@@ -338,12 +378,14 @@ void clear_object(object *op) {
   op->arch=NULL;
   op->other_arch=NULL;
   op->enemy=NULL;
+  op->contr = NULL;
+  op->sk_list = NULL;
   op->weight=0,op->carrying=0,op->weight_limit=0;
   op->anim_speed=0;
   op->level=0;
   op->value=op->run_away=0;
   op->invisible=0;
-  op->last_heal=0,op->last_sp=0,op->last_eat=0;
+  op->last_heal=0,op->last_sp=0,op->last_grace=0,op->last_eat=0;
   op->nrof=0;
   op->ownercount=0;
   op->immune=0,op->protected=0,op->attacktype=0,op->vulnerable=0;
@@ -354,14 +396,16 @@ void clear_object(object *op) {
   op->stats.luck=0;
   op->stats.food=0;
   op->stats.sp=op->stats.maxsp=op->stats.hp=op->stats.maxhp=0;
+  op->stats.grace=op->stats.maxgrace=0;
   op->expmul=1.0;
   op->direction=0;
+  op->glow_radius=0;
 #ifdef NPC_PROG
   op->npc_status=0;
   op->npc_program=0;
 #endif
   op->stats.Str=op->stats.Dex=op->stats.Con=0;
-  op->stats.Wis=op->stats.Cha=op->stats.Int=0;
+  op->stats.Wis=op->stats.Cha=op->stats.Int=op->stats.Pow=0;
   op->material=op->magic=op->state=op->type=0;
   op->face = blank_face;
 
@@ -379,9 +423,11 @@ void clear_object(object *op) {
   op->path_denied = 0;
 #ifdef CASTING_TIME
   op->casting = -1;
-  op->spell = NULL;
   op->spelltype = 0;
+  op->spell = NULL;
 #endif
+  op->randomitems=NULL;
+  op->spellitem = NULL;
 
 }
 
@@ -577,7 +623,12 @@ void update_object(object *op) {
   object *tmp;
 
   if(op->env!=NULL) {
-    if((tmp=is_player_inv(op->env)))
+    tmp=is_player_inv(op->env);
+    /* If we are in ericserver mode, I think we should send an update
+     * or something.  Ideally, client should handle animations on its
+     * own.
+     */
+    if (tmp && tmp->contr->eric_server<=0)
       (*draw_inventory_faces_func)(tmp);
     return;
   }
@@ -602,7 +653,7 @@ void update_object(object *op) {
 void update_position (mapstruct *m, int x, int y) {
     object *tmp, *last = NULL;
     unsigned int flags = 0;
-    MapLook f=blank_look;
+    MapLook f=blank_look,floor=blank_look;
 #ifdef DOUBLE_FLOOR
     MapLook f1 = blank_look;
 #endif
@@ -618,9 +669,19 @@ void update_position (mapstruct *m, int x, int y) {
 	 * Always put the player down for drawing.
 	 */
 	if (tmp->type==PLAYER) {
-	    (*draw_look_faces_func)(tmp);
+	    if (tmp->contr->eric_server<=0)
+		(*draw_look_faces_func)(tmp);
 	    f.face = tmp->face;
 	}
+	else if (QUERY_FLAG(tmp,FLAG_IS_FLOOR)) {
+#ifdef DOUBLE_FLOOR
+	    /* can't see anything that is below the floor, so reset it. */
+	    f1=blank_look;
+#endif
+	    if (!tmp->invisible)
+		floor.face = tmp->face;
+	}
+
 #ifdef DOUBLE_FLOOR
 	else if (f1.face->visibility < tmp->face->visibility && !tmp->invisible)
 		f1.face = tmp->face;
@@ -645,14 +706,20 @@ void update_position (mapstruct *m, int x, int y) {
     }
     if (last && f.face==blank_face) f.face = last->face;
 
-    /* If using the editor, show invisible faces */
+    /* If not using the editor, lets find the top most non
+       inivisible face. Stop once we find a floor.*/
     if (!editor) {
-	while (last && (last->invisible || !last->face->number))
+	while (last && (last->invisible || !last->face->number) &&
+	      !QUERY_FLAG(last, FLAG_IS_FLOOR))
 	    last = last->below;
 
 	if (last!=NULL) f.face = last->face;
 	else f.face = blank_face;
     }
+
+/* Basically, we want to try and set the floors first, since that some
+ * object that comes along later can be put in the top spot.
+ */
 #ifdef DOUBLE_FLOOR
     if (f.face == f1.face)
 	f1.face = blank_face;
@@ -660,6 +727,7 @@ void update_position (mapstruct *m, int x, int y) {
 #endif
 
     f.flags = flags;
+    set_map_floor(m,x,y,&floor);
     set_map (m, x, y, &f);
 
     return;
@@ -809,7 +877,6 @@ void sub_weight (object *op, signed long weight) {
       weight=(signed long)(weight*(100-op->stats.Str)/100);
     }
     op->carrying-=weight;
-    if (op->carrying<0) op->carrying=0;
     op = op->env;
   }
 }
@@ -863,7 +930,8 @@ void remove_ob(object *op) {
       sub_weight(op->env, op->weight*op->nrof);
     else
       sub_weight(op->env, op->weight+op->carrying);
-    if ((otmp=is_player_inv(op->env))!=NULL)
+    if ((otmp=is_player_inv(op->env))!=NULL && otmp->contr && 
+	!QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
       fix_player(otmp);
     if(op->above!=NULL)
       op->above->below=op->below;
@@ -875,8 +943,10 @@ void remove_ob(object *op) {
     op->ox=op->x,op->oy=op->y;
     op->map=op->env->map;
     op->above=NULL,op->below=NULL;
-    if(otmp!=NULL)
-      (*draw_inventory_func)(otmp);
+    if(otmp!=NULL) {
+      if (otmp->type==PLAYER && otmp->contr && !otmp->contr->eric_server)
+	(*draw_inventory_func)(otmp);
+    }
     op->env=NULL;
     return;
   }
@@ -933,7 +1003,12 @@ void remove_ob(object *op) {
     set_map(op->map,op->x,op->y,&blank_look);
   else			/* Redraw the object at the top */
     update_object(last);
-  if(QUERY_FLAG(op,FLAG_BLOCKSVIEW))	/* Should be harmless from editor, */
+
+  if(QUERY_FLAG(op,FLAG_BLOCKSVIEW)	/* Should be harmless from editor, */
+#ifdef USE_LIGHTING
+     ||(op->glow_radius>0) 
+#endif
+  ) 
     update_all_los(op->map);	/* first_player is no longer set there */
 
 }
@@ -1002,6 +1077,7 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
   op->map=m;
   CLEAR_FLAG(op,FLAG_APPLIED); /* hack for fixing F_APPLIED in items of dead people */
   CLEAR_FLAG(op, FLAG_INV_LOCKED);
+  CLEAR_FLAG(op, FLAG_NO_STEAL);
   /* If anything below then */
   if((top=get_map_ob(op->map,op->x,op->y))!=NULL) {
     /*
@@ -1014,32 +1090,20 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
 
     tmp = top;
 
-    /* Re-did this with nested if statements.  In this way, each check does
-     * not need to check for the previous state - to me, it makes the code
-     * cleaner, and perhaps faster.
+    /*
+     * Rule 0: SEE_ANYWHERE() objects stay on top of the top.
      */
-    if (!QUERY_FLAG(op, FLAG_SEE_ANYWHERE)) {
-      /* See anywhere objects stay on top. */
-      while(tmp != NULL && QUERY_FLAG(tmp,FLAG_SEE_ANYWHERE))
+    while(tmp != NULL && QUERY_FLAG(tmp,FLAG_SEE_ANYWHERE) &&
+       !QUERY_FLAG(op,FLAG_SEE_ANYWHERE))
           top = tmp, tmp = tmp->below;
 
-      if (!QUERY_FLAG(op, FLAG_ALIVE) && !QUERY_FLAG(op,FLAG_FLYING)) {
-	/* Alive objects also stay on to. */
-	while(tmp != NULL && QUERY_FLAG(tmp,FLAG_ALIVE))
-	  top = tmp, tmp = tmp->below;
-
-	/* Have object 'fall below' other objects that block view, except in
-	 * cases where the object has a high visibility (high meaning >0)
-	 */
-	if (blocks_view(op->map, op->x, op->y) && (op->face &&
-		!op->face->visibility)) {
-	  while (tmp != NULL && QUERY_FLAG(tmp, FLAG_BLOCKSVIEW) &&
-	    !QUERY_FLAG(op, FLAG_BLOCKSVIEW))
-	      top = tmp, tmp = tmp->below;
-	}
-      }
-    }
-
+    /*
+     * Rule 1: Alive objects stay on top.
+     */
+    while(tmp != NULL && QUERY_FLAG(tmp,FLAG_ALIVE) &&
+	!QUERY_FLAG(op,FLAG_ALIVE) && !QUERY_FLAG(op,FLAG_FLYING) &&
+          !QUERY_FLAG(op,FLAG_SEE_ANYWHERE))
+      top = tmp, tmp = tmp->below;
 
     /*
      * Link the new object in the two-way list:
@@ -1076,9 +1140,22 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
   else
     set_map_ob(op->map,op->x,op->y,op);   /* Tell the map that we're here */
   update_object(op);
-  check_walk_on(op);
-  if(QUERY_FLAG(op,FLAG_FREED))	/* check_walk_on() killed it */
-    return;
+  /* Only check this if we are the head of the object */
+  if (!op->head) {
+      check_walk_on(op);
+      if(QUERY_FLAG(op,FLAG_FREED))	/* check_walk_on() killed it */
+	return;
+
+      /* If we are a multi part object, lets work our way through the check
+       * walk on's.  See if the object has been freed after each check.
+       */
+      for (tmp=op->more; tmp!=NULL; tmp=tmp->more) {
+          check_walk_on(op);
+          if(QUERY_FLAG(op,FLAG_FREED))	/* check_walk_on() killed it */
+		return;
+      }
+
+  }
   if(op->type==PLAYER)
     op->contr->do_los=1;
   for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1104,6 +1181,7 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
 
 void insert_ob_in_map(object *op,mapstruct *m) {
   object *tmp, *top;
+
   if(m==NULL) {
     dump_object(op);
     LOG(llevError,"Trying to insert in null-map!\n%s\n",errmsg);
@@ -1119,8 +1197,9 @@ void insert_ob_in_map(object *op,mapstruct *m) {
     LOG(llevError,"Trying to insert (map) inserted object.\n%s\n", errmsg);
     return;
   }
-  if(op->more!=NULL)
+  if(op->more!=NULL) {
     insert_ob_in_map(op->more,m);
+  }
   CLEAR_FLAG(op,FLAG_REMOVED);
   if(op->nrof)
     for(tmp=get_map_ob(m,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1135,6 +1214,7 @@ void insert_ob_in_map(object *op,mapstruct *m) {
   op->map=m;
   CLEAR_FLAG(op,FLAG_APPLIED); /* hack for fixing F_APPLIED in items of dead people */
   CLEAR_FLAG(op, FLAG_INV_LOCKED);
+  CLEAR_FLAG(op, FLAG_NO_STEAL);
   /* If there are other objects, then */
   if((top=get_map_ob(op->map,op->x,op->y))!=NULL) {
     /*
@@ -1161,11 +1241,11 @@ void insert_ob_in_map(object *op,mapstruct *m) {
 	while(tmp != NULL && QUERY_FLAG(tmp,FLAG_ALIVE))
 	  top = tmp, tmp = tmp->below;
 
-	/* Have object 'fall below' other objects that block view, except in
-	 * cases where the object has a high visibility (high meaning >0)
-	 */
-	if (blocks_view(op->map, op->x, op->y) && (op->face &&
-		!op->face->visibility)) {
+        /* Have object 'fall below' other objects that block view, except in
+         * cases where the object has a high visibility (high meaning >0)
+         */
+        if (blocks_view(op->map, op->x, op->y) && (op->face &&
+                !op->face->visibility)) {
 	  while (tmp != NULL && QUERY_FLAG(tmp, FLAG_BLOCKSVIEW) &&
 	    !QUERY_FLAG(op, FLAG_BLOCKSVIEW))
 	      top = tmp, tmp = tmp->below;
@@ -1208,9 +1288,29 @@ void insert_ob_in_map(object *op,mapstruct *m) {
   else
     set_map_ob(op->map,op->x,op->y,op);   /* Tell the map that we're here */
   update_object(op);
-  check_walk_on(op);
-  if(QUERY_FLAG(op,FLAG_FREED))	/* check_walk_on() killed it */
-    return;
+
+  /* Only check this if we are the head of the object */
+  if (!op->head) {
+      mapstruct *map=op->map;
+      check_walk_on(op);
+      /* check_walk_on() might killed it.  We need to compare maps because
+       * if it was freed and a new map was loaded because it was freed
+       * (sacrifice for altar), the object will get recycled in the new map
+       * and no longer be free.
+       */
+      if(QUERY_FLAG(op,FLAG_FREED) || map!=op->map)
+	return;
+
+      /* If we are a multi part object, lets work our way through the check
+       * walk on's.  See if the object has been freed after each check.
+       */
+      for (tmp=op->more; tmp!=NULL; tmp=tmp->more) {
+          check_walk_on(op);
+          if(QUERY_FLAG(op,FLAG_FREED))	/* check_walk_on() killed it */
+		return;
+      }
+
+  }
   if(op->type==PLAYER)
     op->contr->do_los=1;
   for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1224,6 +1324,19 @@ void insert_ob_in_map(object *op,mapstruct *m) {
         update_button(tmp);
       break;
     }
+  /* build up linked list of light sources in each map. We do
+   * this even for non-dark maps, as down the line we might make
+   * a spell/effect which effects the global light of any map.
+   * -b.t.
+   */
+#ifdef USE_LIGHTING
+        if(op->glow_radius>0&&light_not_listed(op)) {
+		add_light_to_map(op,m);
+		update_all_los(m);
+	} else if(m->darkness&&(op->glow_radius>0||op->lights)) /* a light moved, check los */
+		update_all_los(m);
+#endif 
+
 }
 
 /*
@@ -1237,16 +1350,13 @@ void insert_ob_in_map(object *op,mapstruct *m) {
 object *get_split_ob(object *orig_ob,int nr) {
   object *tmp, *otmp;
   if(orig_ob->nrof<nr) {
-    sprintf(errmsg,"There are only %ld %ss.",
+    sprintf(errmsg,"There are only %d %ss.",
             orig_ob->nrof?orig_ob->nrof:1, orig_ob->name);
     return NULL;
   }
   tmp=get_object();
   copy_object(orig_ob,tmp);
   if((orig_ob->nrof-=nr)<1) {
-#if 0 /* Isn't this wrong? */
-    orig_ob->weight=orig_ob->nrof=0;
-#endif
     remove_ob(orig_ob);
     free_object(orig_ob);
   }
@@ -1284,6 +1394,16 @@ void decrease_ob_nr(object *op,int i) {
 
   if((long)(op->nrof-=i)<1) {
     op->nrof=0;	/* just to be sure that remove_ob handles weight properly */
+    /* We don't have remove object send delete to the new client, because
+     * in many areas, more intelligent strategies are employed.  However, if
+     * we are decreasing the number, we can be sure that the object is gone
+     * for good, so send a delete down the line.  Need to check for
+     * environment before we call remove_ob, since that deletes the
+     * environment.
+     */
+    tmp=is_player_inv (op->env);
+    if(tmp!=NULL && tmp->contr->eric_server)
+	(*esrv_del_item_func)(tmp->contr->eric_server, op->count);
     remove_ob(op);
     free_object(op);
   }
@@ -1291,12 +1411,19 @@ void decrease_ob_nr(object *op,int i) {
     if(op->env!=NULL) {
       sub_weight(op->env,op->weight*i);
       tmp=is_player_inv (op->env);
-      if(tmp!=NULL)
-        (*draw_inventory_func)(tmp);
+      if(tmp!=NULL) {
+	if (tmp->contr->eric_server)
+	    (*esrv_send_item_func)(tmp, op);
+	else
+	    (*draw_inventory_func)(tmp);
+      }
     } else {
       for(tmp=op->above;tmp!=NULL;tmp=tmp->above)
-        if(tmp->type==PLAYER)
-          (*draw_look_func)(tmp);
+        if(tmp->type==PLAYER) 
+	  if (tmp->contr->eric_server)
+	    (*esrv_send_item_func)(tmp, op);
+	  else
+	    (*draw_look_func)(tmp);
     }
 }
 
@@ -1331,6 +1458,8 @@ void add_weight (object *op, signed long weight) {
 
 object *insert_ob_in_ob(object *op,object *where) {
   object *tmp, *otmp;
+  int sort_inventory=0;
+
   if(!QUERY_FLAG(op,FLAG_REMOVED)) {
     dump_object(op);
     LOG(llevError,"Trying to insert (ob) inserted object.\n%s\n", errmsg);
@@ -1358,6 +1487,10 @@ object *insert_ob_in_ob(object *op,object *where) {
 	/* return the original object and remove inserted object
            (client needs the original object) */
         tmp->nrof += op->nrof;
+	/* Weight handling gets pretty funky.  Since we are adding to
+	 * tmp->nrof, we need to increase the weight.
+	 */
+	add_weight (where, op->weight*op->nrof);
         SET_FLAG(op, FLAG_REMOVED);
         free_object(op); /* free the inserted object */
         op = tmp;
@@ -1366,14 +1499,22 @@ object *insert_ob_in_ob(object *op,object *where) {
 	break;
       }
 
-    /* I assume combined objects have no inventory */
+    /* I assume combined objects have no inventory
+     * We add the weight - this object could have just been removed
+     * (if it was possible to merge).  calling remove_ob will subtract
+     * the weight, so we need to add it in again, since we actually do
+     * the linking below
+     */
     add_weight (where, op->weight*op->nrof);
   } else
     add_weight (where, (op->weight+op->carrying));
 
   otmp=is_player_inv(where);
-  if (otmp&&otmp->contr!=NULL&&!QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
+  if (otmp&&otmp->contr!=NULL) {
+    if (!QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
       fix_player(otmp);
+    sort_inventory = otmp->contr->sort_inventory;
+  }
 
   op->map=NULL;
   op->env=where;
@@ -1381,11 +1522,25 @@ object *insert_ob_in_ob(object *op,object *where) {
   op->below=NULL;
   op->x=0,op->y=0;
   op->ox=0,op->oy=0;
+
+#ifdef USE_LIGHTING /* reset the light list and los of the players on the map */
+  if(op->glow_radius>0&&where->map)
+  {
+#ifdef DEBUG_LIGHTS
+      LOG(llevDebug, " insert_ob_in_ob(): got %s to insert in map/op\n",
+	op->name);
+#endif /* DEBUG_LIGHTS */ 
+      add_light_to_list(op,where);
+      if(light_not_listed(op)) add_light_to_map(op,where->map);
+      if(where->map->darkness&&where->map->players) update_all_los(where->map);
+  }
+#endif /* USE_LIGHTING */ 
+
   if (where->inv==NULL)
       where->inv=op;
   else {
     tmp = where->inv;
-    if (! tmp->invisible) {
+    if (! tmp->invisible || where->type==PLAYER) {
       /*
        * Rule 1: Move to the similar type
        */
@@ -1393,6 +1548,8 @@ object *insert_ob_in_ob(object *op,object *where) {
              tmp->type != op->type)
         tmp = tmp->below;
 
+/*	LOG(llevDebug,"insert_ob_in_ob: Found same type at %s\n",query_base_name(tmp));*/
+
       /*
        * Rule 2: Within similar type, sort by archetype
        */
@@ -1401,35 +1558,68 @@ object *insert_ob_in_ob(object *op,object *where) {
              tmp->arch != op->arch)
         tmp = tmp->below;
 
-      /*
-       * Rule 3: Within similar type & archetype, sort by magic
-       * First check a special case, then scroll down until
-       * we are below items with more magic.
-       */
-      if (tmp->type == op->type &&
-          tmp->arch == op->arch &&
-          abs(tmp->magic) < abs(op->magic))
-        tmp = tmp->above; /* Beware: tmp might be NULL now. */
-      else {
-        while (tmp != NULL &&
-               tmp->below != NULL &&
-               tmp->below->type == op->type &&
-               tmp->below->arch == op->arch &&
-               abs(tmp->magic) > abs(op->magic))
-          tmp = tmp->below;
+/*	LOG(llevDebug,"insert_ob_in_ob: Found same arch at %s\n",query_base_name(tmp));*/
+	if (!sort_inventory) {
+	    /*
+	     * Rule 3: Within similar type & archetype, sort by magic
+	     * First check a special case, then scroll down until
+	     * we are below items with more magic.
+	     */
+	    if (tmp->type == op->type &&
+		tmp->arch == op->arch &&
+		abs(tmp->magic) < abs(op->magic))
+			tmp = tmp->above; /* Beware: tmp might be NULL now. */
+	    else {
+		while (tmp != NULL &&
+		   tmp->below != NULL &&
+		   tmp->below->type == op->type &&
+		   tmp->below->arch == op->arch &&
+		   abs(tmp->magic) > abs(op->magic))
+			tmp = tmp->below;
 
-        /*
-         * Rule 4: Insert what seems like a similar item below.
-         */
-        while (tmp != NULL &&
-               tmp->below != NULL &&
-               tmp->below->type == op->type &&
-               tmp->below->arch == op->arch &&
-               abs(tmp->below->magic) == abs(op->magic))
-          tmp = tmp->below;
-      }
-    }
+		/*
+		 * Rule 4: Insert what seems like a similar item below.
+		 */
+		while (tmp != NULL &&
+		   tmp->below != NULL &&
+		   tmp->below->type == op->type &&
+		   tmp->below->arch == op->arch &&
+		   abs(tmp->below->magic) == abs(op->magic))
+			tmp = tmp->below;
+	    }
+	} else { /* If we do sort inventory */
+	    /*
+	     * Rule 5: Try to sort alphabetically.  This is probably a costly
+	     * operation.  It might be worthwhile to store the complete strings
+	     * in the object structure
+	     */
+	    char obuf[MAX_BUF],*nbuf;
 
+	    strcpy(obuf, query_base_name(op));
+
+	    /* We insert after tmp.  As such, we may want to insert before
+	     * the first object that we have matched so far, so lets back up
+	     */
+	    if (tmp->type == op->type &&
+		tmp->arch == op->arch &&
+		abs(tmp->magic) <= abs(op->magic) &&
+		(strcmp(obuf, query_base_name(tmp))<0))
+		    tmp = tmp->above; /* Beware: tmp might be NULL now. */
+	    else {
+		while (tmp != NULL &&
+		   tmp->below != NULL &&
+		   tmp->below->type == op->type &&
+		   tmp->below->arch == op->arch &&
+		   abs(tmp->below->magic) == abs(op->magic)) {
+			nbuf=query_base_name(tmp->below);
+			if (strcmp(obuf,nbuf)>0) {
+				tmp = tmp->below;
+			}
+			else break;
+		}
+	    }
+	} /*end of inventory sort */
+    } /* if this is a playr */
     /*
      * Now link the new object into the two-way list, below tmp.
      */
@@ -1437,13 +1627,6 @@ object *insert_ob_in_ob(object *op,object *where) {
       op->below = where->inv;
       op->below->above = op;
       where->inv = op;
-#if 0 /* commented out, it's not needed and close_container works better */
-    } else if (tmp->above == NULL) { 
-      where->inv = op;
-      op->above = NULL;
-      op->below = tmp;
-      tmp->above = op;
-#endif
     } else if (tmp->below != NULL) {
       tmp->below->above = op;
       op->below = tmp->below;
@@ -1455,7 +1638,7 @@ object *insert_ob_in_ob(object *op,object *where) {
     }
   }
   tmp=is_player_inv(where);
-  if (tmp && tmp->contr->eric_server <= 0) {
+  if (tmp && tmp->contr && tmp->contr->eric_server <= 0) {
       (*draw_inventory_func)(tmp);
   }
   return op;
@@ -1514,7 +1697,7 @@ void check_walk_on(object *op) {
 
 object *present_arch(archetype *at, mapstruct *m, int x, int y) {
   object *tmp;
-  if(out_of_map(m,x,y)) {
+  if(m==NULL || out_of_map(m,x,y)) {
     LOG(llevError,"Present_arch called outside map.\n");
     return NULL;
   }
@@ -1610,6 +1793,7 @@ int find_free_spot(archetype *at, mapstruct *m,int x,int y,int start,int stop) {
  * find_first_free_spot(archetype, mapstruct, x, y) works like
  * find_free_spot(), but it will search max number of squares.
  * But it will return the first available spot, not a random choice.
+ * Changed 0.93.2: Have it return -1 if there is no free spot available.
  */
 
 int find_first_free_spot(archetype *at, mapstruct *m,int x,int y) {
@@ -1617,7 +1801,7 @@ int find_first_free_spot(archetype *at, mapstruct *m,int x,int y) {
   for(i=0;i<SIZEOFFREE;i++)
     if(!arch_blocked(at,m,x+freearr_x[i],y+freearr_y[i]))
       return i;
-  return 0;
+  return -1;
 }
 
 /*
