@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_c =
- *   "$Id: player.c,v 1.24 1995/09/11 02:47:03 master Exp master $";
+ *   "$Id: player.c,v 1.30 1997/03/09 04:21:48 master Exp master $";
  */
 
 /*
@@ -27,6 +27,7 @@
 */
 
 #include <global.h>
+#include <funcpoint.h>
 
 /*
  * Returns a newly allocated and initialised and correctly
@@ -51,16 +52,20 @@ player *get_player_ob() {
   }
   new->next=NULL;
   new->menu=NULL;
-  new->sync = synchronize;
+  new->sync = settings.synchronize;
   new->cur_sync = 1;
   new->known_spell = 0;
   new->removed=0;
-  new->party_number=0;
+#ifdef SIMPLE_PARTY_SYSTEM
+  new->party_number=-1;
+#endif
   new->last_known_spell = 0;
   new->infoline=0;
   new->infopos=0;
   new->infofull=0;
   new->keyboard_flush=0;
+  new->show_inv_icon=0;
+  new->sort_inventory=0;
   new->infolines=INFOLINES;
   new->infochars=INFOCHARS;
   new->gc_root=NULL;
@@ -132,6 +137,11 @@ player *get_player_ob() {
   }
   new->outputs_sync=16;		/* Every 2 seconds */
   new->outputs_count=1;		/* Keeps present behaviour */
+  new->mark=NULL;
+  new->mark_count=0;
+  new->draw_fire_on=0;
+  new->draw_run_on=0;
+  new->show_buttons=0;
   return new;
 }
 
@@ -156,9 +166,11 @@ void free_player(player *pl) {
   if(pl->username != NULL)
     free(pl->username);
   if(pl->info) {	
-    for(i=0;i<pl->infolines;i++) 
+    for(i=0;i<pl->infolines;i++)  {
 	  free(pl->info[i]);
-	free(pl->info);
+	  pl->info[i]=NULL;
+    }
+    free(pl->info);
   }
   /*
    * Binded keys
