@@ -1,7 +1,6 @@
-
 /*
  * static char *rcsid_porting_c =
- *   "$Id: porting.c,v 1.6 1995/07/24 22:06:38 master Exp $";
+ *   "$Id: porting.c,v 1.10 1997/03/09 04:21:59 master Exp master $";
  */
 
 /*
@@ -52,6 +51,14 @@
 #    include <unistd.h>
 #  endif
 
+#if defined(MACH) || defined(vax) || defined(ibm032) || defined(NeXT) || defined(__MACH__)
+#ifndef S_ISDIR
+#define S_ISDIR(x) (((x) & S_IFMT) == S_IFDIR)
+#endif
+#endif
+
+
+
 static unsigned int curtmp = 0;
 
 /*
@@ -96,7 +103,6 @@ char *tempnam(char *dir, char *pfx)
  * A replacement of strdup(), since it's not defined at some
  * unix variants.
  */
-
 char *strdup_local(char *str) {
   char *c=(char *)malloc(sizeof(char)*(strlen(str)+1));
   strcpy(c,str);
@@ -204,7 +210,7 @@ struct xdirect *xreaddir(dir_ptr, mask)
 }
 
 /* This seems to be lacking on some system */
-#if defined(SVR4) && !defined(__sgi__) && !defined(sun)
+#if defined(SVR4) && !defined(__sgi__) && !defined(__sun__)
 int strncasecmp(char *s1, char *s2, int n)
 {
   register int c1, c2;
@@ -243,3 +249,14 @@ int strcasecmp(char *s1, char*s2)
 }
 #endif
 #endif
+
+/* This takes an err number and returns a string with a description of
+ * the error.
+ */
+char *strerror_local(int errno)
+{
+#if defined(SVR4) || defined(linux)
+    return(strerror(errno));
+#endif
+    return("strerror_local not implemented");
+}
