@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shstr_c =
- *   "$Id: shstr.c,v 1.16 1994/04/16 00:24:28 master Exp $";
+ *   "$Id: shstr.c,v 1.18 1997/02/04 10:02:23 master Exp $";
  *
  * shstr.c
  *
@@ -30,6 +30,10 @@
 
 #include "shstr.h"
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 static shared_string *hash_table[TABLESIZE];
 
 /*
@@ -361,7 +365,7 @@ ss_dump_table(int what) {
 	if ((ss = hash_table[i])!=NULL) {
 	    ++entries;
 	    refs += (ss->refcount & ~TOPBIT);
-#if 0 /* Can't use stderr any longer, need to include global.h and
+#if 1 /* Can't use stderr any longer, need to include global.h and
 	    if (what & SS_DUMP_TABLE)
        * use logfile. */
 		fprintf(stderr, "%4d -- %4d refs '%s' %c\n",
@@ -372,7 +376,7 @@ ss_dump_table(int what) {
 		ss = ss->next;
 		++links;
 		refs += (ss->refcount & ~TOPBIT);
-#if 0
+#if 1
 		if (what & SS_DUMP_TABLE)
 		    fprintf(stderr, "     -- %4d refs '%s' %c\n",
 			    (ss->refcount & ~TOPBIT), ss->string,
