@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c,v 1.38 1995/09/11 02:47:35 master Exp master $";
+ *   "$Id: treasure.c,v 1.48 1997/03/09 04:26:52 master Exp master $";
  */
 
 /*
@@ -35,15 +35,14 @@
  */
 #define TREASURE_DEBUG
 
-
+/* TREASURE_VERBOSE enables copious output concerning artifact generation */
+/* #define TREASURE_VERBOSE */
+ 
 #include <global.h>
 #include <treasure.h>
 #include <spellist.h>
 #include <funcpoint.h>
 
-treasurelist *first_treasurelist;
-artifactlist *first_artifactlist;
-
 /*
  * Initialize global archtype pointers:
  */
@@ -174,7 +173,7 @@ void load_treasures() {
   treasure *t;
   int comp;
 
-  sprintf(filename,"%s/%s",LibDir,Treasures);
+  sprintf(filename,"%s/%s",settings.libdir,settings.treasures);
   if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
     LOG(llevError,"Can't open treasure file.\n");
     return;
@@ -234,6 +233,13 @@ void load_treasures() {
 treasurelist *find_treasurelist(char *name) {
   char *tmp=find_string(name);
   treasurelist *tl;
+
+  /* Special cases - randomitems of none is to override default.  If
+   * first_treasurelist is null, it means we are on the first pass of
+   * of loading archetyps, so for now, just return - second pass will
+   * init these values.
+   */
+  if (!strcmp(name,"none") || (!first_treasurelist)) return NULL;
   if(tmp!=NULL)
     for(tl=first_treasurelist;tl!=NULL;tl=tl->next)
       if(tmp==tl->name)
@@ -271,8 +277,9 @@ void create_all_treasures(treasure *t, object *op, int flag, int difficulty, int
         tmp=arch_to_object(t->item);
         if(t->nrof&&tmp->nrof<=1)
           tmp->nrof = RANDOM()%((int) t->nrof) + 1;
-        fix_generated_item(tmp,difficulty,t->magic);
+        fix_generated_item(tmp,op,difficulty,t->magic);
         if(flag != GT_ENVIRONMENT) {
+	  fix_flesh_item(tmp, op);
           tmp = insert_ob_in_ob(tmp,op);
           (void) (*monster_check_apply_func)(op,tmp);
         } else {
@@ -319,8 +326,9 @@ void create_one_treasure(treasurelist *tl, object *op, int flag, int difficulty,
 	object *tmp=arch_to_object(t->item);
         if(t->nrof&&tmp->nrof<=1)
           tmp->nrof = RANDOM()%((int) t->nrof) + 1;
-        fix_generated_item(tmp,difficulty,t->magic);
+        fix_generated_item(tmp,op,difficulty,t->magic);
         if(flag != GT_ENVIRONMENT) {
+	  fix_flesh_item(tmp, op);
           tmp = insert_ob_in_ob(tmp,op);
           (void) (*monster_check_apply_func)(op,tmp);
         } else {
@@ -649,10 +657,17 @@ int get_magic(int diff) {
  * order to set it up right.  This produced magical bonuses, puts spells
  * into scrolls/books/wands, makes it unidentified, hides the value, etc.
  */
+/* 4/28/96 added creator object from which op may now inherit properties based on
+ * op->type. Right now, which stuff the creator passes on is object type 
+ * dependant. I know this is a spagetti manuever, but is there a cleaner 
+ * way to do this? b.t. */
 
-void fix_generated_item(object *op,int difficulty, int max_magic) {
+void fix_generated_item(object *op,object *creator,int difficulty, int max_magic) {
   int was_magic = op->magic;
 
+  if(!creator||creator->type==op->type) creator=op; /*safety & to prevent polymorphed 
+						     * objects giving attributes */ 
+
   if (difficulty<1) difficulty=1;
   if (op->arch == crown_arch) {
     set_magic(difficulty>25?30:difficulty+5, op, max_magic);
@@ -660,7 +675,8 @@ void fix_generated_item(object *op,int difficulty, int max_magic) {
   } else {
     if(!op->magic && max_magic)
       set_magic(difficulty,op,max_magic);
-    if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT)) || op->type == HORN)
+    if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT)) || op->type == HORN ||
+	difficulty >= 100000 )
       generate_artifact(op, difficulty);
   }
   if (!op->title) /* Only modify object if not special */
@@ -680,6 +696,27 @@ void fix_generated_item(object *op,int difficulty, int max_magic) {
           op->value*=3;
       }
       break;
+    case POTION: {
+	int too_many_tries=0,is_special=0;
+
+	while(!(is_special=special_potion(op)) && !op->stats.sp) {
+	   generate_artifact(op,difficulty);
+	   if(too_many_tries++ > 10) break;
+	}
+	if(!is_special) { 
+	   int nrof_potion=spells[op->stats.sp].scrolls-RANDOM()%3;
+           /* these 'potions' have been generated as artifacts 
+	    * and therefore need a few adjustments 
+            */ 
+           /* give fewer potions than scrolls */
+           if(nrof_potion>0)
+              op->nrof=RANDOM()%(nrof_potion)+1;
+           /* value multiplier is same as for scrolls */
+           op->value=(op->value*spells[op->stats.sp].level)/
+                 (spells[op->stats.sp].level+4);
+	}
+	break; 
+    }
     case AMULET:
       if(op->arch==amulet_arch)
         op->value*=5; /* Since it's not just decoration */
@@ -713,12 +750,55 @@ void fix_generated_item(object *op,int difficulty, int max_magic) {
         op->face=&new_faces[op->arch->faces
 		[RANDOM()%((int) op->arch->animations)]];
       break;
+    case BOOK:
+      /* Is it an empty book?, if yes lets make a special 
+       * msg for it, and tailor its properties based on the 
+       * creator and/or map level we found it on.
+       */
+      if(!op->msg&&RANDOM()%10) { 
+	/* set the book level properly */
+	if(creator->level==0 || QUERY_FLAG(creator,FLAG_ALIVE))
+            if(op->map&&op->map->difficulty) 
+	      op->level=RANDOM()%(op->map->difficulty)+RANDOM()%10+1;
+            else
+	      op->level=RANDOM()%20+1;
+	else 
+	    op->level=RANDOM()%creator->level;
+
+	tailor_readable_ob(op,(creator&&creator->stats.sp)?creator->stats.sp:-1);
+        /* books w/ info are worth more! */
+      	op->value*=((op->level>10?op->level:(op->level+1)/2)*((strlen(op->msg)/250)+1));
+	/* creator related stuff */
+	if(QUERY_FLAG(creator,FLAG_NO_PICK)) /* for library, chained books! */
+	    SET_FLAG(op,FLAG_NO_PICK);
+	if(creator->slaying&&!op->slaying) /* for check_inv floors */
+	    op->slaying = add_string(creator->slaying);
+#ifdef ALLOW_SKILLS /* add exp so reading it gives xp (once)*/
+        op->stats.exp = op->value>10000?op->value/5:op->value/10;
+#endif
+      }
+      break;
     case SPELLBOOK:
-      do
-        op->stats.sp=RANDOM()%NROFREALSPELLS;
-      while (RANDOM()%10>=spells[op->stats.sp].books);
+      if(!strcmp(op->arch->name,"cleric_book")) 
+	 do { 
+	     do
+               op->stats.sp=RANDOM()%NROFREALSPELLS;
+	     while(RANDOM()%10>=spells[op->stats.sp].books); 
+	 } while (!spells[op->stats.sp].cleric);
+      else
+	 do {
+             do   
+               op->stats.sp=RANDOM()%NROFREALSPELLS;
+             while(RANDOM()%10>=spells[op->stats.sp].books); 
+         } while (spells[op->stats.sp].cleric);
+
       op->value=(op->value*spells[op->stats.sp].level)/
                  (spells[op->stats.sp].level+4);
+      change_book(op,-1);
+#ifdef ALLOW_SKILLS /* add exp so learning gives xp */
+      op->level = spells[op->stats.sp].level;
+      op->stats.exp = op->value;
+#endif
       break;
     case WAND:
       do 
@@ -763,12 +843,15 @@ void fix_generated_item(object *op,int difficulty, int max_magic) {
       if (op->level<1) op->level=1;
       op->value=(op->value*spells[op->stats.sp].level)/
                  (spells[op->stats.sp].level+4);
+#ifdef ALLOW_SKILLS /* add exp so reading them properly gives xp */ 
+      op->stats.exp = op->value/5;
+#endif
       break;
     case RUNE:
       (*trap_adjust_func)(op,difficulty);
       break;
     }
-  if (op->type == POTION) {
+  if (op->type == POTION && special_potion(op)) {
     op->face = potion_face;
     free_string(op->name);
     op->name = add_string("potion");
@@ -782,6 +865,16 @@ void fix_generated_item(object *op,int difficulty, int max_magic) {
   else
     op->value = op->arch->clone.value / 2;
 #endif
+  /* Often times cursed rings show up with the only 'bad' effect being that
+   * they are cursed - they are still providing normal protections and
+   * immunities.  This makes cursed items really bad - make it vulnerable
+   * to whatever it protected/immune, and clear the protections/immunities.
+   */
+  if (QUERY_FLAG(op, FLAG_CURSED)) {
+    op->vulnerable |= op->protected | op->immune;
+    op->immune=0;
+    op->protected=0;
+  }
 }
 
 /*
@@ -878,8 +971,8 @@ void init_artifacts() {
   if (has_been_inited) return;
   else has_been_inited = 1;
 
-  LOG(llevDebug, "Reading artifacts...");
-  sprintf(filename, "%s/artifacts", LibDir);
+  sprintf(filename, "%s/artifacts", settings.libdir);
+  LOG(llevDebug, "Reading artifacts from %s...",filename);
   if ((fp = open_and_uncompress(filename, 0, &comp)) == NULL) {
     LOG(llevError, "Can't open %s.\n", filename);
     return;
@@ -948,12 +1041,7 @@ void init_artifacts() {
 #endif
   }
 
-#ifdef DUMP_SWITCHES
-  if (dump_monsters == 3) {
-    dump_artifacts();
-    exit(0);
-  }
-#endif
+  LOG(llevDebug,"done.\n");
 }
 
 
@@ -962,13 +1050,15 @@ void init_artifacts() {
  * is modified by the bonuses of the second object.
  */
 
-static void add_abilities(object *op, object *change) {
+void add_abilities(object *op, object *change) {
   int i, tmp;
   if (change->face != blank_face) {
+#ifdef TREASURE_VERBOSE
     LOG(llevDebug, "FACE: %d\n", change->face->number);
+#endif
     op->face = change->face;
   }
-  for (i = 0; i < 6; i++)
+  for (i = 0; i < 7; i++)
     change_attr_value(&(op->stats), i, get_attr_value(&(change->stats), i));
   if (QUERY_FLAG(change,FLAG_CURSED))
     SET_FLAG(op, FLAG_CURSED);
@@ -995,6 +1085,12 @@ static void add_abilities(object *op, object *change) {
     SET_FLAG(op,FLAG_FLYING);
   if (QUERY_FLAG(change,FLAG_XRAYS))
     SET_FLAG(op,FLAG_XRAYS);
+  if (QUERY_FLAG(change,FLAG_BLIND))
+    SET_FLAG(op,FLAG_BLIND);
+#ifdef USE_LIGHTING
+  if (QUERY_FLAG(change,FLAG_SEE_IN_DARK))
+    SET_FLAG(op,FLAG_SEE_IN_DARK);
+#endif
   if (QUERY_FLAG(change,FLAG_REFL_MISSILE))
     SET_FLAG(op,FLAG_REFL_MISSILE);
   if (QUERY_FLAG(change,FLAG_MAKE_INVIS))
@@ -1095,7 +1191,9 @@ static int legal_artifact_combination(object *op, artifact *art) {
   if (art->allowed == (linked_char *) NULL)
     return 1; /* Ie, "all" */
   for (tmp = art->allowed; tmp; tmp = tmp->next) {
+#ifdef TREASURE_VERBOSE
     LOG(llevDebug, "legal_art: %s\n", tmp->name);
+#endif
     if (*tmp->name == '!')
       name = tmp->name + 1, neg = 1;
     else
@@ -1189,8 +1287,10 @@ void generate_artifact(object *op, int difficulty) {
 	continue;
 
     if (!legal_artifact_combination(op, art)) {
+#ifdef TREASURE_VERBOSE
       LOG(llevDebug, "%s of %s was not a legal combination.\n",
           op->name, art->item->name);
+#endif
       continue;
     }
     give_artifact_abilities(op, art->item);
@@ -1198,4 +1298,98 @@ void generate_artifact(object *op, int difficulty) {
   }
 }
 
+/* fix_flesh_item() - objects of type FLESH are similar to type
+ * FOOD, except they inherit properties (name, food value, etc).
+ * based on the original owner (or 'donor' if you like). -b.t.
+ */
 
+void fix_flesh_item(object *item, object *donor) {
+  char tmpbuf[MAX_BUF];
+
+  if(item->type==FLESH && donor) {
+      /* change the name */
+      sprintf(tmpbuf,"%s's %s",donor->name,item->name);
+      free_string(item->name);
+      item->name=add_string(tmpbuf);
+     /* weight is FLESH weight/100 * donor */
+      if((item->weight = (float) (item->weight/100.0) * donor->weight)==0)
+		item->weight=1;
+     /* value is multiplied by level of donor */
+      item->value *= isqrt(donor->level*2);
+     /* food value */
+      item->stats.food += (donor->stats.hp/100) + donor->stats.Con;
+     /* flesh items inherit vuln, prot of donor */
+      item->vulnerable = donor->vulnerable;
+      item->protected = donor->protected;
+     /* if donor has some attacktypes, the flesh is poisonous */
+      if(donor->attacktype&AT_POISON)
+	item->type=POISON;
+      if(donor->attacktype&AT_ACID) item->stats.hp = -1*item->stats.food;
+      SET_FLAG(item,FLAG_NO_STEAL);
+  }
+}
+
+/* special_potion() - so that old potion code is still done right. */
+
+int special_potion (object *op) {
+
+  if(op->attacktype) return 1;
+  if(op->immune || op->vulnerable || op->protected) return 1;
+  if(op->stats.Str || op->stats.Dex || op->stats.Con || op->stats.Pow
+ 	|| op->stats.Wis || op->stats.Int || op->stats.Cha ) return 1;
+
+  return 0;
+}
+
+void free_treasurestruct(treasure *t)
+{
+	if (t->next) free_treasurestruct(t->next);
+	if (t->next_yes) free_treasurestruct(t->next_yes);
+	if (t->next_no) free_treasurestruct(t->next_no);
+	free(t);
+}
+
+void free_charlinks(linked_char *lc)
+{
+	if (lc->next) free_charlinks(lc->next);
+	free(lc);
+}
+
+void free_artifact(artifact *at)
+{
+
+    if (at->next) free_artifact(at->next);
+    if (at->allowed) free_charlinks(at->allowed);
+    if (at->item) {
+	if (at->item->name) free_string(at->item->name);
+	if (at->item->msg) free_string(at->item->msg);
+	if (at->item->title) free_string(at->item->title);
+	free(at->item);
+    }
+    free(at);
+}
+
+void free_artifactlist(artifactlist *al)
+{
+    artifactlist *nextal;
+    for (al=first_artifactlist; al!=NULL; al=nextal) {  
+	nextal=al->next;
+	if (al->items) {
+		free_artifact(al->items);
+	}
+	free(al);
+    }
+}
+
+void free_all_treasures() {
+treasurelist *tl, *next;
+
+
+    for (tl=first_treasurelist; tl!=NULL; tl=next) {
+	next=tl->next;
+	if (tl->name) free_string(tl->name);
+	if (tl->items) free_treasurestruct(tl->items);
+	free(tl);
+    }
+    free_artifactlist(first_artifactlist);
+}
