@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_xutil_c =
- *   "$Id: xutil.c,v 1.25 1995/09/11 02:48:04 master Exp master $";
+ *   "$Id: xutil.c,v 1.29 1997/03/09 04:28:06 master Exp master $";
  */
 
 /*
@@ -141,12 +141,16 @@ int ReadPixmaps(Display *gdisp, Pixmap **pixmaps, Pixmap **masks,
     LOG(llevDebug,"Building color pixmaps...");
     for (file_num=1; file_num<=num_files; file_num++) {
 	sprintf (buf,"%s/%s.pix.%d",LIBDIR, FONTNAME,file_num);
+	LOG (llevDebug,"\n\tprocessing file %s",buf);
 again:	error=XpmReadFileToPixmap(gdisp, root,
 	      buf,&pixmap,&mask, &xpmatribs);
 	if (error!=XpmSuccess) {
 		if (error==XpmColorFailed && !use_private_cmap) {
-			LOG(llevError,"XPM could not allocate colors - trying to switch to private colormap\n");
-			*cmap=XCopyColormapAndFree(gdisp, *cmap);
+			LOG(llevError,"XPM could not allocate colors - trying to switch to private colormap..");
+			if (!(*cmap=XCopyColormapAndFree(gdisp, *cmap)))
+			    LOG(llevError,"Failed...\n");
+			else
+			    LOG(llevError,"Successful...\n");
 			xpmatribs.colormap=*cmap;
 			use_private_cmap=1;
 			goto again;
@@ -201,7 +205,10 @@ again:	error=XpmReadFileToPixmap(gdisp, root,
 	    LOG(llevDebug, "Warning, pixmap %d is not defined, setting it to blank\n", i);
 	    pixmaps[i] = pixmaps[blank_face->number];
 	}
+    LOG(llevDebug,"done.\n");
     return use_private_cmap;
+#else
+    return 0;	/* Prevents some warning messages */
 #endif
 }
 
@@ -383,10 +390,10 @@ int fixfontpath(Display *disp, char *name) {
   if (check_font_path(disp))
     return 0;
 
-  if(fix_fontpath) {
+  if(settings.fixfontpath) {
       LOG(llevError,"Trying to fix fontpath for display %s.\n",name);
       fflush(logfile);
-      set_font_path(disp,FontDir);
+      set_font_path(disp,settings.fontdir);
       if(check_font_path(disp))
         return 0;
   }
@@ -418,7 +425,8 @@ int allocate_colors(Display *disp, Window w, long screen_num,
   Visual *vis;
   XColor exactcolor;
 
-  if(no_color) {
+  /* Should perhaps be setable by player? */
+  if(settings.blackwhite) {
     return 0;
   }
   iscolor = 1;
@@ -428,7 +436,7 @@ int allocate_colors(Display *disp, Window w, long screen_num,
     depth = DefaultDepth(disp,screen_num);
   }
   else {
-    *colormap=NULL;
+    *colormap=(Colormap )NULL;
     LOG(llevError,"Switching to black and white.\n");
     LOG(llevError,"You have a black and white terminal.\n");
     return 0;
