@@ -96,7 +96,7 @@ static void Picks(XtPointer client,String entryPath)
   char mapdir[PATH_MAX+1];
   int i=0;
 
-  sprintf(mapdir,"%s/%s",LibDir,MapDir);
+  sprintf(mapdir,"%s/%s",settings.libdir,settings.mapdir);
   for(path = entryPath; mapdir[i++] == *path++;);
 
   debug1("Picks() %s\n",path);
@@ -110,7 +110,7 @@ static void Walls(XtPointer client,String entryPath)
   char mapdir[PATH_MAX+1];
   int i = 0;
 
-  sprintf(mapdir,"%s/%s",LibDir,MapDir);
+  sprintf(mapdir,"%s/%s",settings.libdir,settings.mapdir);
   for(path = entryPath; mapdir[i++] == *path++;);
 
   debug1("Walls() %s\n",path);
@@ -137,6 +137,7 @@ static void Info(XtPointer client,String entryPath)
 static Boolean AppArchFilter (App self,archetype * at)
 {
     if(self->arch.all) return True;
+    if (!at->editable) return False;
     return (self->arch.flags & at->editable) ? True : False;
 }
 
@@ -633,7 +634,7 @@ static void Layout(App self)
 	("infoButton",menuButtonWidgetClass, box,
 	 XtNmenuName,"info",
 	 NULL);
-    sprintf(path,"%s/%s",LibDir,"doc");
+    sprintf(path,"%s/%s",settings.libdir,"doc");
     self->infof = CnvFilesCreate("info",use,Info,(XtPointer)self,path);
     
     /*** look ***/
@@ -670,12 +671,12 @@ static void Layout(App self)
 	("pickButton",menuButtonWidgetClass, box,
 	 XtNmenuName,"picks",
 	 NULL);
-    sprintf(path,"%s/%s/%s",LibDir,MapDir,"editor/picks");
+    sprintf(path,"%s/%s/%s",settings.libdir,settings.mapdir,"editor/picks");
     self->picks = CnvFilesCreate("picks",use,Picks,(XtPointer)self,path);
     use = XtVaCreateManagedWidget 
       ("wallButton",menuButtonWidgetClass, box,
        NULL);
-    sprintf(path,"%s/%s/%s",LibDir,MapDir,"editor/walls");
+    sprintf(path,"%s/%s/%s",settings.libdir,settings.mapdir,"editor/walls");
     self->walls = CnvFilesCreate("menu",use,Walls,(XtPointer)self,path);
 
     use = XtVaCreateManagedWidget 
@@ -782,24 +783,21 @@ App AppCreate(XtAppContext appCon,
     /*** images & colors ***/
     InitializeColors(XtDisplay(self->shell));
 
-    use_pixmaps = self->res.usePixmaps; /* kludge */
+    if (self->res.usePixmaps) settings.displaymode=Dm_Bitmap;
 #ifdef Xpm_Pix
-    color_pix = self->res.useColorPixmaps; /* kludge, do not use */
-    if (color_pix) use_pixmaps=0;
-#else
-    color_pix = 0;
+    else if (self->res.useColorPixmaps) settings.displaymode=Dm_Pixmap;
 #endif
 
     CnvInitialize(self->shell);
 
-    if(color_pix) {
+    if(settings.displaymode==Dm_Pixmap) {
 	if (ReadPixmaps(self->display, &pixmaps, &masks, &colormap)) {
 	    /* We really should do something better than this */
-	    fprintf(stderr,"Not enough space in colormap - exiting.\n");
-	    exit(1);
+	    fprintf(stderr,"Not enough space in colormap - switch colormap.\n");
+/*	    exit(1);*/
 	}
     } else {
-	if (use_pixmaps)
+	if (settings.displaymode==Dm_Bitmap)
 	    pixmaps = ReadBitmaps(self->display);
 	else if (!check_font_path(XtDisplay (self->shell))) {
 	    fprintf(stderr, "Trying to fix fontpath...\n");
@@ -813,7 +811,7 @@ App AppCreate(XtAppContext appCon,
     XtVaSetValues(self->shell, XtNcolormap, colormap, NULL);
 
     /*** creating ***/
-    sprintf(path,"%s/%s",LibDir,MapDir);
+    sprintf(path,"%s/%s",settings.libdir,settings.mapdir);
     self->path = CnvPathCreate("fileSelect",path, "");
     self->clip = EditCreate(self,ClipBoard,"/Clipboard"); /* separate from */
     Layout(self);
@@ -829,6 +827,7 @@ App AppCreate(XtAppContext appCon,
 void AppDestroy(App self)
 {
     Edit edit;
+    Edit temp;
 
     debug0("AppDestroy()");
     /*
@@ -841,8 +840,12 @@ void AppDestroy(App self)
 	AttrDestroy(self->attr);
     }
     self->attr = NULL;
-    for(edit = self->edit; edit ;edit = edit->next)
-	EditDestroy(edit);
+    edit = self->edit;
+    while(edit) {
+      temp = edit->next;
+      EditDestroy(edit);
+      edit = temp;
+    }
 }
 
 /*
