@@ -73,6 +73,8 @@ static void AttrReset(Attr self);
 
 /*
  * transfer type from game to editor, return negative on error
+ *
+ * Basically, for certain things, we want to have default values set up.
  */
 int GetType (object *tmp)
 {
@@ -99,6 +101,8 @@ int GetType (object *tmp)
     case MAGIC_EAR:
     case TRIGGER:
 	return T_Connected;
+    case CHECK_INV:
+	return T_Connected | T_Lockcode;
     case ALTAR:
     case TRIGGER_ALTAR:
 	return T_Altar;
@@ -124,7 +128,6 @@ int GetType (object *tmp)
 	    return T_Monster;
 	else
 	    return T_Default;
-	break;
     }
     /* return -1; */
 }
@@ -356,34 +359,34 @@ AttrDef AttrDescription[] = {
 
 char *allowed_variables[] = {
 
-    "name","race","slaying","other_arch",
-    "last_heal","last_sp","last_eat",
-    "speed","speed_left","slow_move",
-    "face","Str","Dex","Con","Wis","Cha","Int","hp","maxhp","sp","maxsp",
-    "exp","food","dam","wc","ac","nrof","level","direction",
-    "type","material","value","weight","carrying",
-    "immune","protected","attacktype","vulnerable",
-    "path_attuned","path_repelled","path_denied",
-    "invisible","magic","state","alive","applied","unpaid","need_an","need_ie",
-    "no_pick","no_pass","walk_on","walk_off","fly_on","fly_off",
-    "flying","monster","friendly","generator","auto_apply","treasure",
-    "apply_once","see_invisible","can_roll","is_turning","is_turnable",
-    "is_used_up","identified","reflecting","changing","splitting","hitback",
-    "startequip","blocksview","undead","scared","unaggressive",
-    "reflect_missile","reflect_spell","no_magic",
-    "wiz","was_wiz","tear_down","luck","run_away","pass_thru",
-    "can_pass_thru","pick_up","anim_speed","container","no_drop",
-    "no_pretext","will_apply","random_movement","can_apply",
-    "can_cast_spell","can_use_scroll","can_use_wand","can_use_bow",
-    "can_use_armour","can_use_weapon","can_use_ring","has_ready_wand",
-    "has_ready_bow","xrays","is_floor","lifesave","no_strength",
-    "sleep","stand_still","random_move","only_attack","armour",
-    "attack_movement", "move_state","confused","stealth","connected",
-    "cursed","damned","see_anywhere","known_magical","known_cursed",
-    "can_steal","been_applied","has_ready_rod","can_use_rod",
-    "has_ready_horn","can_use_horn","expmul",
-    "unique","make_invisible",
-    "is_wooded","is_hilly","has_ready_skill","has_ready_weapon",
+    "name", "race", "slaying", "other_arch", "last_heal", "last_sp",
+    "last_eat", "speed", "speed_left", "slow_move", "face", "Str", "Dex",
+    "Con", "Wis", "Cha", "Int", "Pow", "hp", "maxhp", "sp", "maxsp", "exp",
+    "food", "dam", "wc", "ac", "nrof", "level", "direction", "type",
+    "material", "value", "weight", "carrying", "immune", "protected",
+    "attacktype", "vulnerable", "path_attuned", "path_repelled",
+    "path_denied", "invisible", "magic", "state", "alive", "applied",
+    "unpaid", "need_an", "need_ie", "no_pick", "no_pass", "walk_on",
+    "walk_off", "fly_on", "fly_off", "flying", "monster", "friendly",
+    "generator", "is_thrown", "auto_apply", "treasure", "apply_once",
+    "see_invisible", "can_roll", "is_turning", "is_turnable", "is_used_up",
+    "identified", "reflecting", "changing", "splitting", "hitback",
+    "startequip", "blocksview", "undead", "scared", "unaggressive",
+    "reflect_missile", "reflect_spell", "no_magic", "wiz", "was_wiz",
+    "tear_down", "luck", "run_away", "pass_thru", "can_pass_thru",
+    "pick_up", "anim_speed", "container", "no_drop", "no_pretext",
+    "will_apply", "random_movement", "can_apply", "can_cast_spell",
+    "can_use_scroll", "can_use_wand", "can_use_bow", "can_use_armour",
+    "can_use_weapon", "can_use_ring", "has_ready_wand", "has_ready_bow",
+    "xrays", "is_floor", "lifesave", "no_strength", "sleep", "stand_still",
+    "random_move", "only_attack", "armour", "attack_movement", "move_state",
+    "confused", "stealth", "connected", "cursed", "damned", "see_anywhere",
+    "known_magical", "known_cursed", "can_use_skill", "been_applied",
+    "title", "has_ready_rod", "can_use_rod", "has_ready_horn",
+    "can_use_horn", "expmul", "unique", "make_invisible", "is_wooded",
+    "is_hilly", "has_ready_skill", "has_ready_weapon", "no_skill_ident",
+    "glow_radius", "is_blind", "can_see_in_dark", "is_cauldron",
+    "randomitems", "is_dust", "no_steal",
 #ifdef NPC_PROG
     "npc_status", "npc_program", 
 #endif
