@@ -85,6 +85,14 @@ static XtResource resources[] = {
 	Offset(stacking),
 	XtRImmediate,
 	(XtPointer)0
+    },{
+	XtNshow_weak_walls,
+	XtCShow_weak_walls,
+	XtRInt,
+	sizeof(int),
+	Offset(show_weak_walls),
+	XtRImmediate,
+	(XtPointer)0
     },{
 	XtNselectArea,
 	XtCSelectArea,
@@ -111,7 +119,6 @@ static Boolean SetValues(Widget cur,Widget req,Widget new,
 			 Arg args[],Cardinal *num);
 
 #define EmptyObject "editor_archetype"
-static object *emptyObject;
 
 /**********************************************************************
  * actions & translations
@@ -290,7 +297,7 @@ static void AnimateCursor (Widget w, int x, int y) {
     int yd = y - old_y;
     int i; 
     
-    if (use_pixmaps && XtIsRealized (w)) {	
+    if (settings.displaymode==Dm_Bitmap && XtIsRealized (w)) {	
 	if (!cursors_initialized) { 	    
 	    archetype *wizu = find_archetype ("editcursor");
 
@@ -384,7 +391,10 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	    yb = diff + y * self->crEdit.fontSize - 1;
 
 	    while (op) {
-		New_Face f = *op->face; 
+		New_Face f;
+		if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
+		    f=new_faces[op->arch->faces[op->arch->animations/2]];
+		else f = *op->face;
 		if (inv  == True) {
 		    int tmp = f.fg;
 		    f.fg = f.bg;
@@ -404,28 +414,36 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 		op = op->above;
 	    }
 	}
-    } else {
+    } else { /* Normal map drawing routine */
         New_Face f;
-        if (color_pix) {
+        if (settings.displaymode==Dm_Pixmap) {
           f = *(get_map_floor (self->crEdit.map, x, y)->face);
           FaceDraw (w, self->crEdit.gc, &f,
                     x * self->crEdit.fontSize,
                     y * self->crEdit.fontSize);
-	}
-#if 0
-	for (op = get_map_ob (self->crEdit.map, x, y); 
-	     op && op->above; 
-	     op = op->above);
-	f = op ? op->face : blank_face;
-#else
-	f = *(get_map (self->crEdit.map, x, y)->face); 
+#ifdef DOUBLE_FLOOR
+          f = *(get_map_floor2 (self->crEdit.map, x, y)->face);
+	  if (f.number != blank_face->number)
+	    FaceDraw (w, self->crEdit.gc, &f,
+                    x * self->crEdit.fontSize,
+                    y * self->crEdit.fontSize);
 #endif
+	}
+	op=get_map_ob(self->crEdit.map, x, y);
+	while (op && op->above) op=op->above;
+	if (!op) f=*blank_face;
+	else if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
+		f=new_faces[op->arch->faces[op->arch->animations/2]];
+
+	else f = *op->face;
+
 	if (inv  == True) {
 	    int tmp = f.fg;
 	    f.fg = f.bg;
 	    f.bg = tmp;
 	}
-	FaceDraw (w, self->crEdit.gc, &f, 
+	if (settings.displaymode!=Dm_Pixmap || f.number != blank_face->number)
+	    FaceDraw (w, self->crEdit.gc, &f, 
 		  x * self->crEdit.fontSize, 
 		  y * self->crEdit.fontSize);
     }
@@ -442,6 +460,7 @@ static void DrawRectangle(Widget w,XRectangle area,Boolean inv)
     CrEditWidget self = (CrEditWidget)w;
     int i, j;
 
+    if (!self->crEdit.map) return;
     for (j = area.x; 
 	 j < area.x + area.width &&  j < self->crEdit.map->mapx; 
 	 j++) {
@@ -463,6 +482,7 @@ static void DrawBorder(Widget w,XSegment seg,Boolean inv)
     CrEditWidget self = (CrEditWidget)w;
     int i,x1,x2,y1,y2;
     
+    if (!self->crEdit.map) return;
     x1 = seg.x1 < seg.x2 ? seg.x1 : seg.x2;
     y1 = seg.y1 < seg.y2 ? seg.y1 : seg.y2;
     x2 = abs(seg.x1 - seg.x2) + x1;
@@ -585,7 +605,7 @@ static void InsertAc (Widget w, XEvent * event,
 {
     CrEditWidget self = (CrEditWidget)w;
     int x, y;
-    static oldx, oldy;
+    static int oldx, oldy;
     struct CrEditCall call;
 
     AnimateCursor (w, event->xbutton.x, event->xbutton.y);
@@ -642,7 +662,7 @@ static void SelectExpandAc(Widget w, XEvent * event,
 {
     CrEditWidget self = (CrEditWidget)w;
     int x,y;
-    static oldx, oldy;
+    static int oldx, oldy;
 
     AnimateCursor (w, event->xbutton.x, event->xbutton.y);
     if(!self->crEdit.selectArea) return;
@@ -744,7 +764,7 @@ static void DeleteAc(Widget w, XEvent * event,
 {
     CrEditWidget self = (CrEditWidget)w;
     int x, y;
-    static oldx, oldy;
+    static int oldx, oldy;
     struct CrEditCall call;
 
     AnimateCursor (w, event->xbutton.x, event->xbutton.y);
@@ -775,7 +795,7 @@ static void FeedAc (Widget w, XEvent * event, String * argv, Cardinal * argc)
 {
     CrEditWidget self = (CrEditWidget)w;
     int x, y;
-    static oldx, oldy;
+    static int oldx, oldy;
     struct CrEditCall call;
     
     x = event->xbutton.x / self->crEdit.fontSize;
@@ -927,7 +947,6 @@ static void ClassInitialize()
 {
     debug0("CrEdit-ClassInitialize()\n");
 
-    emptyObject = &((find_archetype(EmptyObject))->clone);
 }
 
 static void Initialize(Widget req,Widget new,Arg args[],Cardinal *num)
@@ -997,7 +1016,11 @@ static Boolean SetValues(Widget cur,Widget req,Widget new,
 	SetSize(new);
 	redpy = True;
     }
-
+    /*** weak wall change ***/
+    if(self->crEdit.show_weak_walls != old->crEdit.show_weak_walls) {
+	SetSize(new);
+	redpy = True;
+    }
     return redpy; /* redisplay */
 }
 
