@@ -99,13 +99,17 @@ static void getStacking (object *ob, char *str, XtPointer c) {
 }
 
 static void getTimeout (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%ld", ob->value);
+    sprintf(str,"%d", ob->value);
 }
 
 static void getReset (object *ob, char *str, XtPointer c) {
     sprintf(str,"%ld", ob->weight);
 }
 
+static void getDarkness (object *ob, char *str, XtPointer c) {
+    sprintf(str,"%d", ob->invisible);
+}
+
 static void getDifficulty (object *ob, char *str, XtPointer c) {
     sprintf(str,"%d", ob->level);
 }
@@ -129,6 +133,14 @@ static void getNoReset (object *ob, char *str, XtPointer c) {
     *str = (QUERY_FLAG(ob, FLAG_STAND_STILL))? ~0 : 0;
 }
 
+static void getShowWeakWalls(object *ob, char *str, XtPointer c) {
+    Cardinal weak_walls;
+    Edit edit = (Edit)c;
+
+    XtVaGetValues (edit->w, XtNshow_weak_walls, &weak_walls, NULL);
+    *str= weak_walls ? ~0 : 0;
+}
+
 /*
  * putValue functions
  *
@@ -204,6 +216,10 @@ static void putReset (object *ob, char *str, XtPointer c) {
     ob->weight = atoi(str);
 }
 
+static void putDarkness (object *ob, char *str, XtPointer c) {
+    ob->invisible = atoi(str);
+}
+
 static void putDifficulty (object *ob, char *str, XtPointer c) {
     ob->level = atoi(str);
 }
@@ -213,6 +229,16 @@ static void putNoReset(object *ob, char *str, XtPointer c) {
 	else CLEAR_FLAG(ob, FLAG_STAND_STILL);
 }
 
+static void putShowWeakWalls (object *ob, char *str, XtPointer c) {
+    Edit edit = (Edit)c;
+    Cardinal weak_walls;
+
+    if (*str) weak_walls=1;
+    else weak_walls=0;
+
+    XtVaSetValues(edit->w,XtNshow_weak_walls,weak_walls,NULL);
+}
+
 
 /**********************************************************************
  * tags
@@ -228,7 +254,12 @@ static AttrDef MapDescription[] = {
     {"Timeout",		TypeString, getTimeout,		putTimeout},
     {"Reset",		TypeString, getReset,		putReset},
     {"Difficulty",	TypeString, getDifficulty,	putDifficulty},
+    {"Darkness",	TypeString, getDarkness,	putDarkness},
     {"FixedReset",	TypeToggle, getNoReset,		putNoReset},
+    {"Show Weak Walls",	TypeToggle, getShowWeakWalls,	putShowWeakWalls},
+    /* Make sure the ReadOnly is the last thing on the list, otherwise
+     * things won't work properly.
+     */
     {"ReadOnly",	TypeToggle, getReadOnly,	putReadOnly},
     {NULL,		TypeString, 0,			0},
     {"Overwrite",	TypeToggle, getOverwrite,	putOverwrite},
@@ -302,6 +333,12 @@ static Boolean Load(Edit self, char *name)
 	strcpy(save,self->emap->path); /* klydge */
     EdFreeMap (self);
     if(!(tmp = ready_map_name (path, 1))) {
+	/* Make the map null, since right now we don't have a valid map
+	 * in memory.  The CnfNotify call will force a redraw - we
+	 * can check for Null map data pretty easy, but can't really check
+	 * for unknown map data.
+	 */
+	XtVaSetValues(self->w, XtNmap, NULL, NULL);
 	sprintf(buf,"Cannot load map %s",path);
 	CnvNotify(buf,"Continue",NULL);
 	if(!(tmp = ready_map_name(save,1))) return False;
@@ -951,12 +988,6 @@ static Widget OptionMenu(String name,Edit self,Widget parent)
 	(refresh,XtNcallback,RefreshCb,(XtPointer)self);
     /*** toggles ***/
     XtVaCreateManagedWidget ("line", smeLineObjectClass, shell, NULL);
-#if 0
-    self->iw.read_only
-	= XtVaCreateManagedWidget ("readOnly", smeBSBObjectClass, shell, NULL);
-    XtAddCallback (self->iw.read_only,
-		   XtNcallback,ToggleReadCb, (XtPointer) self);
-#endif
     self->iw.overwrite = XtVaCreateManagedWidget 
 	("overWrite", smeBSBObjectClass, shell, NULL);
     XtAddCallback (self->iw.overwrite,
@@ -1683,8 +1714,8 @@ Boolean EditInsert (Edit self,int x, int y, int z)
 	draw_add (self, x, y);
 
     EditObjectCalc(self,op,&rect);
-    CrEditRefresh(self->w,rect);
     update_position (self->emap, x, y);
+    CrEditRefresh(self->w,rect);
 
     EditModified(self);
     return True;
