@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_config_h =
- *   "$Id: config.h,v 1.37 1994/12/30 06:32:58 master Exp master $";
+ *   "$Id: config.h,v 1.49 1997/03/09 04:58:11 master Exp master $";
  */
 
 /*
@@ -23,9 +23,41 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to master@rahul.net
+    The author can be reached via e-mail to mark@pyramid.com
 */
 
+/*
+ * Set this if you want the temporary maps to be saved and reused across
+ * crossfire runs.  This can be especially useful for single player
+ * servers, but even holds use for multiplayer servers.  The file used
+ * is updated each time a temp map is updated.
+ * Note that the file used to store this information is stored in
+ * the LIB directory.  Running multiple crossfires with the same LIB
+ * directory will cause serious problems, simply because in order for
+ * this to really work, the filename must be constant so the next run
+ * knows where to find the information.
+ */
+
+#define RECYCLE_TMP_MAPS
+
+/* Set this if you want to activate the new client/server code.
+ * Being that eutl dependencies have been removed, we now select it in
+ * the config file here.
+ * Unless you want to work with it, I would recommend you leave this
+ * off, as the code isn't totally debugged, and if someone came through
+ * with a new client, it could cause problems.
+ */
+
+/*
+#define ERIC_SERVER
+*/
+
+/* CSPORT is the port used for the new client/server code.  Change
+ * if desired.  Only of relevance if ERIC_SERVER is set above
+ */
+
+#define CSPORT 13327 /* old port + 1 */
+
 /* This file contains various #defines that select various options.
  * Some may not be desirable, and some just may not work.
  *
@@ -40,6 +72,20 @@
  * ench_armour: Allow characters to enchant their armor.
  */
 
+/* Enables Brian Thomas's new alchemy code.  Might unbalance the game - lets
+ * characters make potions and other special items.
+ */
+
+#define ALCHEMY
+
+/*
+ * The following enables the new light/darkness code.  Basically, darkness
+ * affects line of sight, depending how dark the actual map is.  There are
+ * also objects that create light (torches, spells, etc), which counteract
+ * this effect.
+ */
+
+#define USE_LIGHTING
 
 /* Uncommenting the following line allows the new skill code to be used
  * (look at skills.h to see some of the skill names.)  Allowing skills
@@ -52,12 +98,30 @@
 
 #define ALLOW_SKILLS
 
+/* Following allows characters to worshsip different gods.  Has no effect
+ * if ALLOW_SKILLS (above) is not defined.
+ */
+
+#define MULTIPLE_GODS
+
+
+/* IF this is set, then the range type will not switch to skill when
+ * you use a melee weapon - this better emulates pre-skill code.
+ */
+
+#define NO_AUTO_SKILL_SWITCH
+
 /* Following uses a new weapon improvement system.  This makes it more
  * difficult to get a super weapon, and makes it more likely that
  * players will create weapons that help certain areas (damage, sp, etc)
  * See the doc/SurvivalGuide for exact information on how the new method
  * works.
  */
+/* READ THIS:  NEW_IMPROVE_WEAPON will likely become a standard (non
+ * configurable) option in the next version.  If you are still using the old
+ * code and plan to in the future, send me mail (mark@pyramid.com) so that
+ * I know not to remove it.
+ */
 
 #define NEW_IMPROVE_WEAPON
 
@@ -73,17 +137,6 @@
 #define NO_CONE_PROPOGATE
 */
 
-/* SHOW_INV_ICON: this draws another icon next to the object image in
- * the players inventory.  This icon shows the status of the object, such
- * as cursed, magic, equipped, etc.  Right now, these icons are just words.
- * Someday, hopefully better images will be done for thse.
- * This does cut down the number of characters that can be printed for
- * the item name (because this image takes up some space), so it might not
- * always be diserable.  If you don't want this feature, comment the
- * define out.
- */
-
-/*#define SHOW_INV_ICON*/
 
 /*
  * BANFILE
@@ -153,20 +206,6 @@
 
 #define FULL_RING_DESCRIPTION
 
-/* SHOP_LISTINGS allows you to get a quick listing of what the shop has
- * in stock (unfortunately, it is still up to you to find the item.)
- * To get the listing, a 'menu' sign must be in the store, and then
- * must be A)pplied.  There probably isn't much reason not to define
- * this, as searching through the shop to find the item you want should
- * not be part of the game (IMHO). As time passes, more and more shops
- * will actually have signs put in them.  At some time, this should
- * probably become a standard part of the game.
- * Mark Wedel (master@rahul.net)
- * 
- * CF 0.91.2 - now standard part of game
- * #define SHOP_LISTINGS
- */
-
 /* if EXPLORE_MODE is defined, it allows players to enter explore mode,
  * where they can not die.  Unlike other games (nethack for instance) explore
  * modes, the only thing Crossfire explore mode does is prevent death.
@@ -175,7 +214,7 @@
  * only be entered if the person doing so is the only player in the
  * game, and once done, that person can not add new players to the
  * game.  To get into explore mode, type 'explore
- * Explore mode code added by Mark Wedel (master@rahul.net)
+ * Explore mode code added by Mark Wedel (mark@pyramid.com)
  * It's a good idea to turn off EXPLORE_MODE if you will only be running
  * on server, since it would block new players.
  */
@@ -234,28 +273,33 @@
 
 #define SIMPLE_PARTY_SYSTEM
 
-/*
- * If defined, some buttons for walking pops up.  They're mostly
- * useless though.
- */
-
-/* #define USE_BUTTONS */
-
 /*
  * If SECURE is defined, crossfire will not accept enviromental variables
  * as changes to the LIBDIR and other defines pointing to specific
- * directories.
+ * directories.  The only time this should really be an issue if you are
+ * running crossfire setuid/setgid and someone else could change the
+ * values and get special priveledges.  If you are running with normal uid
+ * privelidges, then this is not any more/less secure (user could compile
+ * there own version pointing wherever they want.)  However, having nonsecure
+ * can make debugging or other developement much easier, because you can
+ * change the paths without forcing a recompile.
+ *
+ * NOTE: Prior to 0.93.2, the default was for this to be defined (ie,
+ * could not change values on the command line/through environmental
+ * variables.)
  */
 
-#define SECURE 
+/* #define SECURE */
 
 /*
  * SERVER:
  * Leave this in if you want the magic server stuff.  eanders@cmu.edu
  * It is now also able to accept new players via socket without
  * the -server switch.
+ *
  * LOGFILE specifies which file to log to when playing with the
  * -server option as well.
+ *
  * NO_ADD:
  * If you want players to only be able to join the game by using the
  * client, and not by having other players type "'add <display>", then
@@ -263,16 +307,24 @@
  * depend on the username when making sure that one player can only
  * have one entry pr. class, and it's only possible to determine
  * username if they use the client.
+ *
  * ONE_PLAYER_PR_UID:
  * A user can only play one player in a single crossfire at any given time.
  * Rumor has it the ONE_PLAYER_PR_UID causes some problems even with multiple
  * user ids.
+ *
+ * NOTE: AS of now (0.92.8) all the client really is is just a telnet front
+ * end (server can not tell the different between a crossclient addition
+ * and a person who telneted in.)  As such, both NO_ADD ans ONE_PLAYER_PR_UID
+ * only make things slightly more difficult for some people.
  */
 
 #define SERVER
+
 #ifdef SERVER
-#define PORT 13326 /* Crossfire => 1cfe => 13326 (huh?) CrossFirE :)*/
+#define PORT 13326 /* Crossfire => 10000+cfe => 13326 (huh?) CrossFirE :)*/
 #endif
+
 #ifndef LOGFILE
 #define LOGFILE "/tmp/cross.log"
 #endif
@@ -281,16 +333,8 @@
 
 /*
  * DMFILE
- * A file containing valid names that can be dm, one on each line.
- * ie, something like:
- *
- * master
- * root
- * crossfire
- *
- * Thus, a global password is no longer required.  This file is located
- * in the lib directory.
- * 
+ * A file containing valid names that can be dm, one on each line.  See
+ * example dm_file for syntax help.
  */
 
 #ifndef DMFILE
@@ -303,13 +347,14 @@
  */
 
 #ifndef DM_MAIL
-/* #define DM_MAIL "master@rahul.net" */
+/* #define DM_MAIL "mark@pyramid.com" */
 #endif
 
 /*
  * If undefined, malloc is always used.
- * There is still a calloc() call in player.c which is not affected by this
- * definition due to a bug which hasn't been found yet.
+ * It looks like this can be oboleted.  However, it can be useful to
+ * track down some bugs, as it will make sure that the entire data structure
+ * is set to 0, at the expense of speed.
  */
 /* #define USE_CALLOC */
 
@@ -371,7 +416,8 @@ Fri 8-12
 msg
 Playing is forbidden during working hours.
  * -----
- * Unfortunately IBMR2 isn't supported by PERM_FILE yet.
+ * Unfortunately IBMR2 isn't supported by PERM_FILE yet.  I am not sure
+ * why this is so - the code doesn't look all that strange.
  */
 
 #ifndef _IBMR2
@@ -380,9 +426,10 @@ Playing is forbidden during working hours.
 
 /*
  * random() is much better than rand().  If you have random(), use it instead.
+ * You shouldn't need to change any of this
  */
 
-#if defined (sgi) || defined (hpux) || defined(linux)
+#if defined (hpux) || defined(linux)
 #define RANDOM() rand()
 #define SRANDOM(xyz) srand(xyz)
 #else
@@ -449,7 +496,7 @@ Playing is forbidden during working hours.
  * on how many to define.  If it is too low, maps just get swapped out
  * immediately, causing a performance hit.  If it is too high, the program
  * consumes more memory.  If you have gobs of free memory, a high number
- * might not be a bad idea.
+ * might not be a bad idea.  Each object is around 300 bytes right now.
  */
 
 #define MAX_OBJECTS	6000
@@ -650,6 +697,7 @@ Playing is forbidden during working hours.
  * slower than local disk)
  */
 
+/*#define TMPDIR "/home/sleipner/a/crossfire/crossfire/tmp"*/
 #define TMPDIR "/tmp"
 
 /*
@@ -665,7 +713,8 @@ Playing is forbidden during working hours.
 /*
  * If you want to regenerate the spoiler.ps file, you have to recompile
  * the game with DUMP_SWITCHES defined.  If defined, it turns on the
- * -m -m2 -m3 -m4 switches.
+ * -m -m2 -m3 -m4 switches.  There is probably no reason not to define
+ * this
  */
 
 #define DUMP_SWITCHES
@@ -720,6 +769,11 @@ Playing is forbidden during working hours.
 #  endif
 #endif
 
+/*
+ * These define the players starting map and location on that map, and where
+ * emergency saves are defined.  This should be left as is unless you make
+ * major changes to the map.
+ */
 
 #define EMERGENCY_MAPPATH "/city/city"
 #define EMERGENCY_X 15
@@ -756,8 +810,13 @@ Playing is forbidden during working hours.
 
 #define COMMAND_HASH_SIZE 107	/* If you change this, delete all characters :) */
 
+/* DEBUG generates copious amounts of output.  I tend to change the CC options
+ * in the crosssite.def file if I want this.  By default, you probably
+ * dont want this defined.
+ */
+
 #ifndef DEBUG
-/* #define DEBUG */		/* You don't really want to define DEBUG... */
+/* #define DEBUG */
 #endif
 
 
@@ -779,7 +838,8 @@ Playing is forbidden during working hours.
  * should be.  Very large numbers have the disadvantage of
  * not being especially efficient on space, and also require more
  * server memory.  The server memory could be a problem for
- * X Terminals.
+ * X Terminals.  If you change these values, you will need to rebuild
+ * and reinstall the lib/crossfire.pix files.
  */
 
 #define XPM_MONTAGE_X 25
@@ -850,6 +910,6 @@ Playing is forbidden during working hours.
  * Apparantly, there are still some bugs associated with this option, so
  * it is better to leave commented out.
  *
- * Mark Wedel (master@rahul.net)
+ * Mark Wedel (mark@pyramid.com)
  */
 /*#define PROCESS_WHILE_LOADING*/
