@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h,v 1.43 1995/09/11 03:01:15 master Exp master $";
+ *   "$Id: define.h,v 1.54 1997/03/09 04:52:49 master Exp master $";
  */
 
 /*
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to master@rahul.net
+    The author can be reached via e-mail to mark@pyramid.com
 */
 
 /* This file is really too large.  With all the .h files
@@ -130,9 +130,15 @@
 #define SKILL		43	/* Skills are similar to abilites, but
 				 * not related to spells.  by njw@cs.city.ac.uk
 				 */
+#define EXPERIENCE	44	/* An experience 'object'. Needed for multi-exp/skills
+				 * hack. -b.t. thomas@astro.psu.edu
+			         */
 #define EARTHWALL	45
 #define GOLEM		46
 #define BOMB		47
+#define THROWN_OBJ	48
+#define BLINDNESS	49
+#define GOD		50
 
 #define SPEEDBALL	52
 #define DEAD_OBJECT	53
@@ -157,6 +163,12 @@
 #define SHOP_MAT	69
 #define RING		70
 
+#define FLESH		72	/* animal 'body parts' -b.t. */
+#define INORGANIC	73	/* metals and minerals */ 
+
+#define LIGHTER		75
+#define TRAP_PART	76	/* Needed by set traps skill -b.t. */ 
+
 #define SPELLBOOK	85
 
 #define CLOAK		87
@@ -202,7 +214,7 @@
 
 #define CANCELLATION	141
 
-/* Mark Wedel (master@cats.ucsc.edu) Shop inventories */
+/* Mark Wedel (mark@pyramid.com) Shop inventories */
 #define MENU		150
 
 /* peterm:  ball lightning and color spray */
@@ -247,7 +259,7 @@
  * (Already done, Vick 930307 :)
  */
 
-#define NROFATTACKS 21
+#define NROFATTACKS 23
 
 #define AT_PHYSICAL	0x00000001 /*     1 */
 #define AT_MAGIC	0x00000002 /*     2 */
@@ -270,6 +282,8 @@
 #define AT_CHAOS        0x00040000 /*262144 peterm@soda.berkeley.edu*/
 #define AT_COUNTERSPELL 0x00080000 /*524288 peterm@soda.berkeley.edu*/
 #define AT_GODPOWER	0x00100000 /*1048576  peterm@soda.berkeley.edu */
+#define AT_HOLYWORD	0x00200000 /*2097152 race selective attack thomas@astro.psu.edu */ 
+#define AT_BLIND	0x00400000 /*4194304 thomas@astro.psu.edu */ 
 
 
 #define PATH_NULL	0x00000000
@@ -289,10 +303,16 @@
 #define PATH_INFO	0x00002000
 #define PATH_TRANSMUTE	0x00004000
 #define PATH_TRANSFER	0x00008000
+#define PATH_TURNING	0x00010000
+#define PATH_WOUNDING	0x00020000
+#define PATH_DEATH	0x00040000
+#define PATH_LIGHT	0x00080000
 
-#define NRSPELLPATHS	18
+#define NRSPELLPATHS	20
 
-#define NROFREALSPELLS	140	/* Number of different spells */
+#define NROFREALSPELLS	171	/* Number of different spells */
+
+#define NROFSKILLS  32  /* match to the number of elements in skills[] array */
 
 #define NROF_SOUNDS (23 + NROFREALSPELLS) /* Number of sounds */
 
@@ -320,6 +340,7 @@
 #define P_IS_ALIVE      0x00000010 /* */
 #define P_NO_CLERIC     0x00000020 /* no clerical spells cast here */
 
+
 /* Flag structure now changed.
  * Each flag is now a bit offset, starting at zero.  The macros
  * will update/read the appropriate flag element in the object
@@ -342,6 +363,11 @@
  * 
  * If any FLAG's are added, be sure to add them to the flag_links structure
  * in common/loader.c, if necessary.
+ *
+ * flags[0] is 0 to 31
+ * flags[1] is 32 to 63
+ * flags[2] is 64 to 95
+ * flags[3] is 96 to 127
  */
 /* Basic routines to do above */
 #define SET_FLAG(xyz, p) \
@@ -369,7 +395,7 @@
 #define FLAG_MONSTER		14 /* Will attack players */
 #define FLAG_FRIENDLY		15 /* Will help players */
 #define FLAG_GENERATOR		16 /* Will generate type ob->stats.food */
-#define FLAG_THROWN		17 /* Will continue to fly through the air. */
+#define FLAG_IS_THROWN		17 /* Object is designed to be thrown. */
 #define FLAG_AUTO_APPLY		18 /* Will be applied when created */
 #define FLAG_TREASURE		19 /* Will generate treasure when applied */
 #define FLAG_APPLY_ONCE		20 /* Will dissapear when applied */
@@ -433,7 +459,7 @@
 #define FLAG_SEE_ANYWHERE	76 /* The object will be visible behind walls */
 #define FLAG_KNOWN_MAGICAL	77 /* The object is known to be magical */
 #define FLAG_KNOWN_CURSED	78 /* The object is known to be cursed */
-#define FLAG_CAN_STEAL		79 /* The monster can steal objects */
+#define FLAG_CAN_USE_SKILL	79 /* The monster can use skills */
 #define FLAG_BEEN_APPLIED	80 /* The object has been applied */
 #define FLAG_READY_ROD		81 /* (Monster) has a rod readied... 8) */
 #define FLAG_USE_ROD		82 /* (Monster) can apply and use rods */
@@ -445,11 +471,16 @@
 #define FLAG_IS_HILLY		88 /* Item is hilly/mountain terrain */
 #define FLAG_READY_SKILL	89 /* (Monster or Player) has a skill readied */
 #define FLAG_READY_WEAPON	90 /* (Monster or Player) has a weapon readied */
-
-#define NUM_FLAGS		90 /* Should always be equal to the last
+#define FLAG_NO_SKILL_IDENT	91 /* If set, item cannot be identified w/ a skill */
+#define FLAG_BLIND		92 /* If set, object cannot see (visually) */
+#define FLAG_SEE_IN_DARK	93 /* if set ob not effected by darkness */ 
+#define FLAG_IS_CAULDRON	94 /* container can make alchemical stuff */
+#define FLAG_DUST		95 /* item is a 'powder', effects throwing */
+#define FLAG_NO_STEAL		96 /* Item can't be stolen */
+#define NUM_FLAGS		96 /* Should always be equal to the last
 				    * defined flag */
 
-/* Values can go up to 95 before the size of the flags array in the
+/* Values can go up to 127 before the size of the flags array in the
  * object structure needs to be enlarged.
  */
 
@@ -497,6 +528,17 @@
  */
 #define MAP_DIFFICULTY(m)	((m)->map_object->level)
 
+/* for use by the lighting code */
+#define MAX_LIGHT_RADII 	4	/* max radii for 'light' object, really
+					 * large values allow objects that can
+					 * slow down the game */
+#define MAX_DARKNESS		5	/* maximum map darkness, there is no
+					 * practical reason to exceed this */ 
+#define MAP_DARKNESS(m)	   	(m)->map_object->invisible>MAX_DARKNESS? \
+				  MAX_DARKNESS:(m)->map_object->invisible;
+#define BRIGHTNESS(xyz)		(xyz)->glow_radius>MAX_LIGHT_RADII? \
+				  MAX_LIGHT_RADII:(xyz)->glow_radius;
+
 #define F_BUY 0
 #define F_SELL 1
 #define F_TRUE 2	/* True value of item, unadjusted */
@@ -512,7 +554,7 @@
 #define WEAPON_SPEED(xyz)	(xyz)->last_sp
 
 
-#ifdef USE_BUTTONS
+/*#ifdef USE_BUTTONS*/
 #ifndef NUMDIRBUTTS
 #define NUMDIRBUTTS 9
 #define NW_button 0
@@ -530,7 +572,7 @@
 #define PEACE_button 2
 #define TALK_button 3
 #endif
-#endif
+/*#endif*/
 
 #define MAP_IN_MEMORY 1
 #define MAP_SWAPPED 2
@@ -540,6 +582,8 @@
 /*
  * Check nrof variable *before* calling CAN_MERGE()
  * stats.sp is used in wands, spellbooks and scrolls.
+ * the 0x400000 on flags2 is FLAG_INV_LOCK.  I don't think something
+ * being locked in inventory should prevent merging.
  */
 
 
@@ -552,7 +596,8 @@
    (ob1)->stats.sp == (ob2)->stats.sp && \
    (ob1)->flags[0] == (ob2)->flags[0] && \
    (ob1)->flags[1] == (ob2)->flags[1] && \
-   (ob1)->flags[2] == (ob2)->flags[2] && \
+   ((ob1)->flags[2] & ~0x400000) == ((ob2)->flags[2] & ~ 0x400000) && \
+   (ob1)->flags[3] == (ob2)->flags[3] && \
    (ob1)->magic == (ob2)->magic && \
    (ob1)->name == (ob2)->name && \
    (ob1)->title == (ob2)->title && \
@@ -664,11 +709,42 @@
 #define STIPPLE2_FACE_NAME "stipple.112"
 #define POTION_FACE_NAME	"potiongen.111" 
 
-#ifdef SHOW_INV_ICON
 #define INV_CURSE_FACE_NAME	"inv-curse.111"
 #define INV_DAMN_FACE_NAME	"inv-damn.111"
 #define INV_EQUIP_FACE_NAME	"inv-equip.111"
 #define INV_LOCK_FACE_NAME	"inv-lock.111"
 #define INV_MAGIC_FACE_NAME	"inv-magic.111"
 #define INV_UNPAID_FACE_NAME	"inv-unpaid.111"
-#endif
+
+/* separated this from the common/item.c file. b.t. Dec 1995 */
+
+#define DESCRIBE_ABILITY(retbuf, variable, name) \
+    if(variable) { \
+      int i,j=0; \
+      strcat(retbuf,"(" name ": "); \
+      for(i=0; i<NROFATTACKS; i++) \
+        if(variable & (1<<i)) { \
+          if (j) \
+            strcat(retbuf,", "); \
+          else \
+            j = 1; \
+          strcat(retbuf, attacks[i]); \
+        } \
+      strcat(retbuf,")"); \
+    }
+ 
+ 
+#define DESCRIBE_PATH(retbuf, variable, name) \
+    if(variable) { \
+      int i,j=0; \
+      strcat(retbuf,"(" name ": "); \
+      for(i=0; i<NRSPELLPATHS; i++) \
+        if(variable & (1<<i)) { \
+          if (j) \
+            strcat(retbuf,", "); \
+          else \
+            j = 1; \
+          strcat(retbuf, spellpathnames[i]); \
+        } \
+      strcat(retbuf,")"); \
+    }
