@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_funcpoint_h =
- *   "$Id: funcpoint.h,v 1.9 1994/12/30 06:35:55 master Exp $";
+ *   "$Id: funcpoint.h,v 1.11 1997/01/05 09:57:58 master Exp $";
  */
 
 /*
@@ -22,7 +22,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to frankj@ifi.uio.no.
+    The author can be reached via e-mail to mark@pyramid.com.
 
     This file was made after an idea by vidarl@ifi.uio.no
 */
@@ -51,4 +51,5 @@ extern void	(*info_map_func)		(int, mapstruct *, char *);
 extern void	(*move_teleporter_func)		(object *);
 extern void	(*move_creator_func)		(object *);
 extern void     (*trap_adjust_func)		(object *, int);
-extern void	(*esrv_send_item_func)		(int, object *, object *);
+extern void	(*esrv_send_item_func)		(object *, object *);
+extern void	(*esrv_del_item_func)		(int, int);
