@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.26 1994/12/30 06:36:51 master Exp $";
+ *   "$Id: global.h,v 1.34 1997/03/09 04:53:44 master Exp master $";
  */
 
 /*
@@ -22,7 +22,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to frankj@ifi.uio.no.
+    The author can be reached via e-mail to mark@pyramid.com.
 */
 
 #ifndef GLOBAL_H
@@ -34,7 +34,10 @@
 
 #include "includes.h"
 
-#define NUM_COLORS	13
+#define NUM_COLORS		13
+#define DEFAULT_PLAYER_FG	0
+#define DEFAULT_PLAYER_BG	8
+#define DEFAULT_PLAYER_FIELD	12
 
 #define XDRAWIMAGESTRING(disp,win,gc,x,y,str,len) \
 XDrawImageString16(disp,win,gc,x,y,str,len)
@@ -54,6 +57,8 @@ typedef XChar2b XChar;
  *****************************************************************************/
 
 
+extern Settings settings;
+
 extern char *colorname[NUM_COLORS][2];
 extern New_Face *new_faces;
 
@@ -67,25 +72,14 @@ EXTERN artifactlist *first_artifactlist;
 EXTERN archetype *first_archetype;
 EXTERN objectlink *first_friendly_object;	/* Objects monsters will go after */
 EXTERN sockets *first_socket;			/* Start of linked list of sockets */
+EXTERN godlink *first_god;
+EXTERN racelink *first_race;
 
 /*
  * The editor uses these (will get them out of here later):
  */
 
-EXTERN long fix_fontpath;
-EXTERN int synchronize;
-EXTERN long no_color;
 EXTERN long editor;     /* if true, edit maps instead of playing (almost obsolete) */
-EXTERN int write_state; /* 1= write to level, 2=write to name */
-EXTERN int write_pos;   /* Position in write_buf */
-EXTERN int write_d;     /* Which position in the button we start writing from */
-EXTERN int button_flag;
-EXTERN int last_bard;
-EXTERN int fix_walls;
-EXTERN int monsters;
-EXTERN int generators;
-EXTERN int hidden;
-EXTERN int others;
 
 /*
  * Variables set by different flags (see init.c):
@@ -97,37 +91,33 @@ EXTERN long warn_archetypes;	/* If true, write warnings when failing */
 				/* to find archetypes when loading from file */
 EXTERN long init_done;			/* Ignores signals until init_done is true */
 EXTERN long trying_emergency_save;	/* True when emergency_save() is reached */
-EXTERN LogLevel debug;			/* Set with -d flag */
-EXTERN long default_split_window;	/* Set with -w flag */
-EXTERN long dump_monsters;		/* Set with -m flag */
-EXTERN int use_pixmaps;		/* Set with -pix flag */
-EXTERN int color_pix;		/* Set with -xpm flag */
 EXTERN long nroferrors;		/* If it exceeds MAX_ERRORS, call fatal() */
 
-EXTERN long pticks;		/* used by various function to determine */
+extern long pticks;		/* used by various function to determine */
 				/* how often to save the character */
-/*
- * Global pointers to the original argv, argc and genv given to main()
- */
-
-EXTERN int gargc;
-EXTERN char **gargv,**genv;
-
 /*
  * Misc global variables:
  */
 EXTERN FILE *logfile;			/* Used by server/daemon.c */
 EXTERN int exiting;			/* True if the game is about to exit */
-EXTERN int server_mode;		/* True if the game is in server mode */
 EXTERN char *name;			/* The name of the game, usually "crossfire" */
 EXTERN long last_micro;		/* When process_active_maps() was last exec */
 EXTERN long nroftreasures;		/* Only used in malloc_info() */
 EXTERN long nrofartifacts;		/* Only used in malloc_info() */
 EXTERN long nrofallowedstr;		/* Only used in malloc_info() */
 
+/* global stuff used by new skill/experience system -b.t. */ 
+#define MAX_EXP_CAT 7 		/* This should be => # of exp obj in the game 
+				 * remember to include the "NULL" exp object  
+			         * EXP_NONE as part of the overall tally. 
+				 */
+#define EXP_NONE (MAX_EXP_CAT - 1)  /* "NULL" exp. object. This is the last 
+				     * experience obj always.*/ 
+EXTERN short nrofexpcat;	/* Current number of experience categories in the game */
+EXTERN object *exp_cat[MAX_EXP_CAT];	/* Array of experience objects in the game */ 
+
 EXTERN archetype *empty_archetype;	/* Nice to have fast access to it */
 EXTERN archetype *map_archeytpe;
-EXTERN char *chrfont;
 EXTERN char first_map_path[MAX_BUF];	/* The start-level */
 
 #ifdef LONGJUMP
@@ -144,36 +134,23 @@ EXTERN archetype *ring_arch,*amulet_arch,*staff_arch,*crown_arch;
 EXTERN char *undead_name;	/* Used in hit_player() in main.c */
 EXTERN sockets *active_socket;
 
-EXTERN int freearr_x[SIZEOFFREE], freearr_y[SIZEOFFREE];
-EXTERN int maxfree[SIZEOFFREE], freedir[SIZEOFFREE];
+extern int freearr_x[SIZEOFFREE], freearr_y[SIZEOFFREE];
+extern int maxfree[SIZEOFFREE], freedir[SIZEOFFREE];
 
-EXTERN New_Face *blank_face;
-EXTERN New_Face *blocked_face;
-EXTERN New_Face *stipple1_face;
-EXTERN New_Face *stipple2_face;
-EXTERN New_Face *potion_face;
-EXTERN MapLook blank_look;
-#ifdef SHOW_INV_ICON
-EXTERN New_Face *inv_curse_face, *inv_damn_face, *inv_equip_face, *inv_lock_face,
+extern New_Face *blank_face;
+extern New_Face *blocked_face;
+extern New_Face *stipple1_face;
+extern New_Face *stipple2_face;
+extern New_Face *potion_face;
+extern MapLook blank_look;
+extern New_Face *inv_curse_face, *inv_damn_face, *inv_equip_face, *inv_lock_face,
          *inv_magic_face, *inv_unpaid_face;
-#endif
 
 
 #ifdef SOUND_EFFECTS
 EXTERN RPLAY	*sound_table[NROF_SOUNDS];
 #endif
 
-EXTERN char *LibDir;
-EXTERN char *FontDir;
-EXTERN char *PlayerDir;
-EXTERN char *MapDir;
-EXTERN char *ArcheTypes;
-EXTERN char *Treasures;
-
-#ifdef UNIQUE_ITEMS
-EXTERN char *ItemsDir;
-#endif
-
 extern long max_time;	/* loop time */
 
 EXTERN char *font_graphic;
@@ -193,6 +170,7 @@ EXTERN char *font_graphic;
 #define get_map_ob(M,X,Y)	((M)->map_ob[(X)+((M)->mapx*(Y))])
 #define out_of_map(M,X,Y)	((X)<0||(X)>=(M)->mapx||(Y)<0||(Y)>=(M)->mapy)
 #define get_map_floor(M,X,Y)	(&(M)->floor[(X)+((M)->mapx*(Y))])
+#define set_map_floor(M,X,Y,C)	((M)->floor[(X)+((M)->mapx*(Y))]=*(C))
 
 #ifdef DOUBLE_FLOOR
 #define get_map_floor2(M,X,Y)	(&(M)->floor2[(X)+((M)->mapx*(Y))])
@@ -225,7 +203,7 @@ EXTERN char *font_graphic;
  * if Xpm_Pix is not defined.  IT would be better to actually have the
  * #ifdef's inside the macro itself, instead of having two versions, but
  * I was not able to get that to work.
- * Mark Wedel (master@rahul.net)
+ * Mark Wedel (mark@pyramid.com)
  */
 
 #ifdef Xpm_Pix
