@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h,v 1.7 1994/12/30 06:39:41 master Exp $";
+ *   "$Id: player.h,v 1.9 1997/01/05 10:00:05 master Exp $";
  */
 
 /*
@@ -22,13 +22,13 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to frankj@ifi.uio.no.
+    The author can be reached via e-mail to mark@pyramid.com.
 */
 
-#ifdef USE_BUTTONS
+/*#ifdef USE_BUTTONS*/
 extern int dirW;
 extern int dirX [NUMDIRBUTTS];
 extern int dirY [NUMDIRBUTTS];
 extern int opX [NUMOPBUTTS];
 extern int opY [NUMOPBUTTS];
-#endif
+/*#endif*/
