@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_h =
- *   "$Id: skills.h,v 1.3 1995/09/11 03:03:23 master Exp master $";
+ *   "$Id: skills.h,v 1.6 1997/01/05 10:00:17 master Exp $";
  */
 
 /*
@@ -23,8 +23,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to master@rahul.net
+    The author can be reached via e-mail to mark@pyramid.com
 */
+
 /*
  * First written 6 Sep 1994, Nick Williams (njw@cs.city.ac.uk)
  *
@@ -52,13 +53,31 @@
  * curse to woodsman - b.t. (thomas@nomad.astro.psu.edu)
  */ 
 
-/* Modification May/June 1995 - Alchemy skill changed--now player's can make potions, 
+/* Modification May/June 1995 - 
  *  HTH skills to allow hitting while moving. 
  *  Modified the bargaining skill to allow incremental CHA increase (based on level)
  *  Added the inscription skill, finished play-testing meditation skill.
  *  - b.t. (thomas@astro.psu.edu)
  */ 
 
+/* Modification June/July 1995 - 
+ *  1- Expansion of the skills code to fit within a scheme of multiple categories 
+ *  of experience. Henceforth, 2 categories of skills will exist: "associated"
+ *  skills which are associated with one of the categories of experience, and
+ *  "miscellaneous" skills, which are not related to any experience category.
+ *  2- Moved the attacking and spellcasting player activities into skills. 
+ *  Now have "hand weapons" "missile weapons" "throwing" and "spellcasting". 
+ *  see doc/??? for details on this system.
+ *  - b.t.  
+ */
+
+/* define this if you want to have player skills stored for 
+ * faster access from a linked list. If skill tools are heavily used
+ * calls to malloc from this code can actually make performance worse. 
+ * -b.t. */
+
+/* #define LINKED_SKILL_LIST */
+
 enum skillnrs { 
     /* 0 */
     SK_STEALING,		/* steal from other players/NPCs */
@@ -82,13 +101,24 @@ enum skillnrs {
     SK_FIND_TRAPS,		/* player can find traps better */ 
     SK_MEDITATION,		/* player can regain sp/hp at a faster rate */ 
     SK_BOXING,			/* can attack hand-to-hand, see attack_hth() */ 
-    SK_WRESTLING,		/* can attack hand-to-hand, see attack_hth() */ 
+    SK_FLAME_TOUCH,		/* player attack for fireborn characters */ 
     SK_KARATE,			/* can attack hand-to-hand, see attack_hth() */ 
     /* 20 */
     SK_CLIMBING,		/* player moves quickly over hills/mountains  */ 
     SK_WOODSMAN,		/* player moves quickly through jungle/forest */ 
-    SK_INSCRIPTION		/* player may write spell scrolls */ 
-
+    SK_INSCRIPTION, 		/* player may write spell scrolls */ 
+    SK_MELEE_WEAPON,		/* player can attack with melee weapons */ 
+    SK_MISSILE_WEAPON,		/* player can attack with missile weapons */ 
+    /* 25 */
+    SK_THROWING,		/* player can throw items */ 
+    SK_SPELL_CASTING,		/* player can cast magic spells */ 
+    SK_REMOVE_TRAP, 		/* player can disarm traps */ 
+    SK_SET_TRAP, 		/* player can set traps - not implemented */ 
+    SK_USE_MAGIC_ITEM, 		/* player use wands/horns/rods */ 
+    /* 30 */
+    SK_PRAYING,			/* player can cast cleric spells, regen grace points */ 
+    SK_CLAWING			/* player attack for troll, dragon characters */ 
+ 
 /* Here follows a list of others which could be implemented, but are
  * not */
 #if 0
@@ -99,3 +129,4 @@ enum skillnrs {
 };
 
 extern skill skills[];
+
