@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_h =
- *   "$Id: spells.h,v 1.22 1995/09/11 03:04:23 master Exp master $";
+ *   "$Id: spells.h,v 1.27 1997/02/04 10:39:38 master Exp $";
  */
 
 /*
@@ -22,7 +22,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to frankj@ifi.uio.no.
+    The author can be reached via e-mail to mark@pyramid.com.
 */
 
 extern Fontindex fbface[];
@@ -127,20 +127,43 @@ enum spellnrs {
   SP_BANISHMENT,	SP_CREATE_MISSILE,
 
   SP_SHOW_INVIS,	SP_XRAY,		SP_PACIFY,		/*135*/
-  SP_SUMMON_FOG,	SP_STEAMBOLT
+  SP_SUMMON_FOG,	SP_STEAMBOLT,
+
+  /* lots of new cleric spells, many need MULTIPLE_GODS defined to be
+   * very usefull - b.t. */
+  SP_COMMAND_UNDEAD,	SP_HOLY_ORB,		SP_SUMMON_AVATAR,	/*140*/
+  SP_HOLY_POSSESSION,	SP_BLESS,		
+
+  SP_CURSE, 		SP_REGENERATION,	SP_CONSECRATE,		/*145*/
+  SP_SUMMON_CULT,	SP_CAUSE_CRITICAL, 
+
+  SP_HOLY_WRATH,	SP_RETRIBUTION,		SP_FINGER_DEATH,	/*150*/
+  SP_INSECT_PLAGUE,	SP_HOLY_SERVANT,
+  
+  SP_WALL_OF_THORNS,	SP_STAFF_TO_SNAKE,	SP_LIGHT,	  	/*155*/
+  SP_DARKNESS,		SP_NIGHTFALL,
+
+  SP_DAYLIGHT,		SP_SUNSPEAR,		SP_FAERY_FIRE, 		/*160*/
+  SP_CURE_BLINDNESS,	SP_DARK_VISION,
+
+  SP_BULLET_SWARM,      SP_BULLET_STORM,        SP_CAUSE_MANY,  /* 165 */
+  SP_S_SNOWSTORM, SP_M_SNOWSTORM,
+
+  SP_L_SNOWSTORM
 
 };
 
 
 #define IS_SUMMON_SPELL(spell) (((spell) > SP_BOMB && (spell) < SP_D_DOOR) \
-	|| ((spell) == SP_MYSTIC_FIST))
+	|| ((spell) == SP_MYSTIC_FIST) || ((spell) == SP_SUMMON_AVATAR) \
+	|| ((spell) == SP_HOLY_SERVANT))
 
 #define PATH_SP_MULT(op,spell) (((op->path_attuned & s->path) ? 0.8 : 1) * \
 				((op->path_repelled & s->path) ? 1.25 : 1))
 #define PATH_TIME_MULT(op,spell) (((op->path_attuned & s->path) ? 0.8 : 1) * \
 				((op->path_repelled & s->path) ? 1.25 : 1))
 	
-archetype *spellarch[NROFREALSPELLS];
 extern char *range_name[range_size];
 extern spell_parameters SP_PARAMETERS[];
 extern char *spellpathnames[NRSPELLPATHS];
+extern archetype *spellarch[NROFREALSPELLS];
