@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_structs_h =
- *   "$Id: structs.h,v 1.48 1995/09/11 03:05:07 master Exp master $";
+ *   "$Id: structs.h,v 1.57 1997/03/09 04:55:29 master Exp master $";
  */
 
 /*
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to master@rahul.net
+    The author can be reached via e-mail to mark@pyramid.com
 */
 
 #ifndef STRUCTS_H
@@ -51,6 +51,41 @@ typedef signed short	sint16;
 typedef unsigned char	uint8;
 typedef signed char	sint8;
 
+/* Even if a particular compile time option may not use some of
+ * these values, we just leave them in in all cases.  We let
+ * the argument parsing routine ignore options that would
+ * change these and not apply.
+ */
+
+enum DisplayMode {Dm_Font, Dm_Bitmap, Dm_Pixmap};
+
+typedef struct Settings {
+    char    *chrfont;	    /* special font to use for images */
+    char    *logfilename;   /* logfile to use */
+    uint16  listen_port;    /* port for old server connections */
+    uint16  csport;	    /* port for new client/server */
+    uint8   servermode;	    /* did we get -server option? */
+    uint8   fixfontpath;    /* If true, we try to set the fontpath for the player */
+    enum DisplayMode displaymode;	/* Default display method */
+    uint8   synchronize;    /* Default synchronize value for player */
+    uint8   blackwhite;     /* If true, don't use color. */
+    uint8   noadd;	    /* If true, don't add the default display */
+    LogLevel debug;	    /* Default debugging level */
+    uint8   splitwindows;   /* If true, players default to split windows mode */
+    uint8   dumpvalues;	    /* Set to dump various values/tables */
+    uint8   daemonmode;     /* If true, detach and become daemon */
+    int	    argc;	    /* parameters that were passed to the program */
+    char    **argv;	    /* Only used by xio.c, so will go away at some time */
+    char    *libdir;	    /* Library location (ARchetypes, scores, etc. */
+    char    *fontdir;	    /* where the font is located */
+    char    *playerdir;	    /* Where the player files are */
+    char    *mapdir;	    /* Where the map files are */
+    char    *archetypes;    /* name of the archetypes file - libdir is prepended */
+    char    *treasures;	    /* location of the treasures file. */
+    char    *uniquedir;	    /* directory for the unique items */
+    char    *tmpdir;	    /* Directory to use for temporary files */
+} Settings;
+
 typedef struct sockets {
   struct sockets *next;
   int fd;
@@ -68,16 +103,18 @@ typedef struct sockets {
 } sockets;
 
 typedef struct liv { /* Mostly used by "alive" objects */
-  signed char Str,Dex,Con,Wis,Cha,Int;
-  signed char wc,ac;    /* Weapon Class and Armour Class */
-  signed short hp;      /* Hit Points. */
-  signed short maxhp;
-  signed short sp;      /* Spell points.  Used to cast spells. */
-  signed short maxsp;	/* Max spell points. */
-  signed long exp;      /* Experience.  Killers gain 1/10. */
-  signed short food;    /* How much food in stomach.  0 = starved. */
-  sint16 dam;		/* How much damage this object does when hitting */
-  signed char luck;	/* Affects thaco and ac from time to time */
+  sint8		Str,Dex,Con,Wis,Cha,Int,Pow;
+  sint8		wc,ac;		/* Weapon Class and Armour Class */
+  sint16	hp;		/* Hit Points. */
+  sint16	maxhp;
+  sint16	sp;		/* Spell points.  Used to cast mage spells. */
+  sint16	maxsp;		/* Max spell points. */
+  sint16	grace;		/* Grace.  Used to invoke clerical prayers. */
+  sint16	maxgrace;	/* Grace.  Used to invoke clerical prayers. */
+  sint32	exp;		/* Experience.  Killers gain 1/10. */
+  sint16	food;		/* How much food in stomach.  0 = starved. */
+  sint16	dam;		/* How much damage this object does when hitting */
+  sint8		luck;		/* Affects thaco and ac from time to time */
 } living;
 
 
@@ -91,11 +128,12 @@ typedef struct new_face_struct {
     uint8	fg, bg;
     char	*name;
     uint8	visibility;
+    uint8	magicmap;
 } New_Face;
 
 typedef struct map_look_struct {
     New_Face *face;
-    unsigned short flags;
+    uint8	flags;
 } MapLook;
 
 typedef struct obj {
@@ -120,8 +158,10 @@ typedef struct obj {
   struct obj *more;    /* Pointer to the rest of a large body of objects */
   struct obj *head;    /* Points to the main object of a large body */
   struct mapdef *map;  /* Pointer to the map in which this object is present */
-  unsigned short count;         /* Which nr. of object created this is. */
-  unsigned short refcount;	/* How many objects points to this object */
+
+  uint32	count;         /* Which nr. of object created this is. */
+  uint16	refcount;	/* How many objects points to this object */
+  struct oblnk *sk_list;	/* Pointer to linked list of skill objects for this ob*/
 
 /* These get an extra add_refcount(), after having been copied by memcpy() */
   char *name;			/* The name of the object, obviously... */
@@ -135,41 +175,44 @@ typedef struct obj {
   sint16 ox,oy;			/* For debugging: Where it was last inserted */
   float speed;                  /* The overall speed of this object */
   float speed_left;             /* How much speed is left to spend this round */
-  unsigned long nrof;
-  New_Face *face;			/* Face with colors */
-  signed char direction;        /* Means the object is moving that way. */
+  uint32	nrof;		/* How many of the objects */
+  New_Face *face;		/* Face with colors */
+  sint8		direction;	/* Means the object is moving that way. */
   sint8 facing;			/* Object is oriented/facing that way. */
   uint8 type; 		        /* PLAYER, BULLET, etc.  See define.h */
-  unsigned int immune;          /* Attacks which the object is immune against */
-  unsigned int protected;	/* Attacks which does half damage */
-  unsigned int attacktype;      /* Same bitmask as immune/protected */
-  unsigned int vulnerable;	/* Attacks which does double damage */
-  uint32 path_attuned;		/* Paths the object is attuned to */
-  uint32 path_repelled;		/* Paths the object is repelled from */
-  uint32 path_denied; 		/* Paths the object is denied access to */
-  uint16 material;       	/* What materials this object consist of */
-  signed char magic;            /* Any magical bonuses to this item */
-  signed char thrownthaco;      /* How precise the throw is */
-  unsigned char state;          /* How the object was last drawn (animation) */
-  signed long value;            /* How much money it is worth (or contains) */
+  /* The next 6 variables actually only really use 24 bits each */
+  uint32	immune;		/* Attacks which the object is immune against */
+  uint32	protected;	/* Attacks which does half damage */
+  uint32	attacktype;	/* Same bitmask as immune/protected */
+  uint32	vulnerable;	/* Attacks which does double damage */
+  uint32	path_attuned;	/* Paths the object is attuned to */
+  uint32	path_repelled;	/* Paths the object is repelled from */
+  uint32	path_denied; 	/* Paths the object is denied access to */
+  uint16	material;      	/* What materials this object consist of */
+  sint8		magic;		/* Any magical bonuses to this item */
+  sint8		thrownthaco;	/* How precise the throw is */
+  uint8		state;          /* How the object was last drawn (animation) */
+  sint32	value;		/* How much money it is worth (or contains) */
   signed short level;
-  signed short last_heal;       /* Last healed. Depends on constitution */
-  signed short last_sp;         /* As last_heal, but for spell points */
+  sint32 last_heal;       /* Last healed. Depends on constitution */
+  sint32 last_sp;         /* As last_heal, but for spell points */
+  signed short last_grace;      /* as last_sp, except for grace */
   signed short last_eat;	/* How long since we last ate */
   signed short invisible;	/* How much longer the object will be invis */
-  signed char armour;		/* How many % is subtracted from phys. damage */
+  sint8		armour;		/* How many % is subtracted from phys. damage */
   unsigned char pick_up;        /* See crossfire.doc */
-  struct obj *owner;	/* Pointer to the object which controls this one */
-			/* Owner should not be referred to directly - */
-			/* get_owner should be used instead. */
+  struct obj *owner;  /* Pointer to the object which controls this one */
+                      /* Owner should not be referred to directly - */
+                      /* get_owner should be used instead. */
   struct obj *enemy;   /* Monster/player to follow even if not closest */
   struct archt *arch;           /* Pointer to archetype */
   struct archt *other_arch;	/* Pointer used for various things */
   signed long weight;           /* Attributes of the object */
   signed long carrying;         /* How much weight this object contains */
-  uint32 flags[3];		/* various flags */
-  unsigned short ownercount;	/* What count the owner had (in case owner */
+  uint32 flags[4];		/* various flags */
+  uint32 ownercount;		/* What count the owner had (in case owner */
 				/* has been freed) */
+  struct treasureliststruct *randomitems; /* Items to be generated */
 #if 0
   unsigned short thrown;        /* How much further the object will fly */
 #endif
@@ -185,9 +228,13 @@ typedef struct obj {
  * information
  */
   struct obj *chosen_skill;	/* the skill chosen to use */
+  struct obj *exp_obj;		/* the exp. obj (category) assoc. w/ this object */
   uint32 hide;			/* The object is hidden, not invisible */
 
- 
+/* lighting code uses these - b.t. */
+  struct oblnk *lights;   	/* list of lights the object carries */
+  signed short glow_radius;	/* indicates the glow radius of the object */
+
 /* changes made by kholland@sunlab.cit.cornell.edu */
 /* allows different movement patterns for attackers */
   signed long move_status;      /* What stage in attack mode */
@@ -201,13 +248,15 @@ typedef struct obj {
 
 #ifdef CASTING_TIME
   signed short casting;         /* time left before spell goes off */
-  struct spell_struct *spell;
-  unsigned int spelltype;
   unsigned short spell_state;
   unsigned short start_holding;
+  struct spell_struct *spell;
+  unsigned int spelltype;
   char *spellarg;
 #endif
 
+  struct obj *spellitem;
+
 /* This modification will allow variable experience requirements 
    for advancement (eanders@cmu.edu) */
   double expmul;        /* needed experience = (calc_exp*expmul) */
@@ -227,12 +276,51 @@ typedef struct oblinkpt { /* Used to link together several object links */
 
 /*
  * So far only used when dealing with artifacts.
+ * (now used by alchemy and other code too. Nov 95 b.t).
  */
 typedef struct linked_char {
   char *name;
   struct linked_char *next;
 } linked_char;
 
+/* 'title' and 'titlelist' are used by the readable code */
+typedef struct titlestruct {
+        char *name;     /* the name of the book */
+        char *authour;  /* the name of the book authour */
+        char *archname;  /* the archetype name of the book */
+        int level;       /* level of difficulty of this message */ 
+        int size;       /* size of the book message */ 
+        int msg_index;  /* an index value derived from book message */
+        struct titlestruct *next;
+} title;
+ 
+typedef struct titleliststruct {
+        int number;       /* number in the list */ 
+        struct titlestruct *first_book;     /* pointer to first book in this list */
+        struct titleliststruct *next;  /* pointer to next book list */
+} titlelist;
+
+/* 'recipe' and 'recipelist' are used by the alchemy code */
+typedef struct recipestruct {
+        char *title;      /* distinguishing name of product */
+        char *arch_name;   /* the archetype of the final product made */
+        int chance;       /* chance that recipe for this item will appear
+                           * in an alchemical grimore */
+	int index;	  /* an index value derived from formula ingredients */
+	int transmute;	  /* if defined, one of the formula ingredients is
+			   * used as the basis for the product object */
+	int yield;        /*  The maximum number of items produced by the recipe */
+        linked_char *ingred;    /* comma delimited list of ingredients */
+        struct recipestruct *next;
+} recipe;
+
+typedef struct recipeliststruct {
+        int total_chance;
+        int number;			/* number of recipes in this list */ 
+        struct recipestruct *items;	/* pointer to first recipe in this list */ 
+	struct recipeliststruct *next; 	/* pointer to next recipe list */ 
+} recipelist;
+
 /*
  * An obwin structure contains a window with scrollbar, where the user
  * can click on items and move the scrollbar.
@@ -337,8 +425,8 @@ typedef struct pl {
   sint16 chosen_spell;		/* Type of readied spell */
   sint16 last_spell;		/* What spell draw_stats() last displayed */
   sint16 chosen_item_spell;	/* Type of spell that the item fires */
-  uint16 count;       /* Any numbers typed before a command */
-  uint16 count_left;  /* How many instances of prev_cmd are left */
+  uint32 count;       /* Any numbers typed before a command */
+  uint32 count_left;  /* How many instances of prev_cmd are left */
 			      /* to execute */
   unsigned char prev_cmd;     /* Previous command executed */
   unsigned char prev_fire_on;
@@ -349,6 +437,7 @@ typedef struct pl {
   signed char digestion;      /* Any bonuses/penalties to digestion */
   signed char gen_hp;         /* Bonuses to regeneration speed of hp */
   signed char gen_sp;         /* Bonuses to regeneration speed of sp */
+  signed char gen_grace;      /* Bonuses to regeneration speed of grace */
   uint8 berzerk;      /* Affects usage of CTRL-direction */
   uint32 split_window:1;    /* info-window will be separate */
   uint32 braced:1;          /* Will not move if braced, only attack */
@@ -359,6 +448,9 @@ typedef struct pl {
 #endif
   uint32 infofull:1;		/* True if info structure has been fileed up */
   uint32 keyboard_flush:1;	/* True if keyboard events should be flushed each time */
+  uint32 show_inv_icon:1;	/* True if we should show the inv. icons */
+  uint32 sort_inventory:1;	/* If true, sort inventory alphabetically */
+  uint32 show_buttons:1;	/* Ture if we should show buttons in message win */
   unsigned char state,infoline,infopos;
   unsigned char writing;   /* When true, the player is writing text */
 		     /* to be displayed */
@@ -384,7 +476,7 @@ typedef struct pl {
   signed short nrofdrawn_inv;	  /* How many items drawn. Delete extras */
   signed short nrofdrawn_look;    /* ditto for the look-window */
   signed short inv_chars;         /* How wide the text-field is */
-  sint16 look_chars;		  /* How wide the text-field is */
+  sint16 look_chars;            /* How wide the text-field is */
   char format_inv[16];            /* printf-format for text in inv. window */
   char format_look[16];           /* printf-format for text in look window */
   unsigned short barlength_inv;   /* height of scrollbar in pixels */
@@ -400,8 +492,12 @@ typedef struct pl {
   unsigned short scrollhp_alert;  /* Was the last scrollbar red */
   unsigned short scrollsize_sp;   /* How large the last scrollbar was */
   unsigned short scrollsp_alert;  /* Was the last scrollbar red */
+  unsigned short scrollsize_grace;  /* How large was the last scrollbar */
+  unsigned short scrollgrace_alert;  /* Was the last scrollbar red */
   unsigned short scrollsize_food; /* How large the last scrollbar was */
   unsigned short scrollfood_alert;  /* Was the last scrollbar red */
+  uint32	draw_fire_on:1;	/* Did we draw a fire on message */
+  uint32	draw_run_on:1;  /* Did we draw a run on message */
   short shootstrength;     /* Strength of spell */
   object *golem;           /* Which golem is controlled */
   living orig_stats;       /* Can be less in case of poisoning */
@@ -452,9 +548,7 @@ typedef struct pl {
   Window win_root,win_game,win_stats,win_info,win_inv,win_look,win_message;
   GC gc_root,gc_game,gc_stats,gc_info,gc_inv_text,gc_inv_icon,
   gc_look_text,gc_look_icon,gc_message, gc_xpm_floor, gc_xpm_object;
-#ifdef SHOW_INV_ICON
   GC gc_inv_status_icon;
-#endif
   XSizeHints gamehint,stathint,roothint,infohint,invhint,lookhint,messagehint;
   long gscreen;
   unsigned long gforeground,gbackground;
@@ -470,7 +564,7 @@ typedef struct pl {
   New_Face *floor2[WINRIGHT-WINLEFT+1][WINLOWER-WINUPPER+1];
 #endif
 #endif
-  char blocked_los[WINRIGHT-WINLEFT+1][WINLOWER-WINUPPER+1];
+  sint8 blocked_los[WINRIGHT-WINLEFT+1][WINLOWER-WINUPPER+1];
   char *name;   	 /* What display the player has */
   char *username;	 /* Username supplied by client */
   char title[MAX_NAME];
@@ -483,8 +577,10 @@ typedef struct pl {
   
   signed char levhp[11]; /* What the player gained on that level */
   signed char levsp[11];
+  signed char levgrace[11];
   signed char last_level;
 
+
   KeySym commandkeysym, firekeysym[2], runkeysym[2];
   KeyCode commandkey, firekey[2], runkey[2];
   Key_s *keys[COMMAND_HASH_SIZE];
@@ -493,11 +589,8 @@ typedef struct pl {
   uint32 name_changed:1;      /* If true, the player has set a name. */
   char killer[BIG_NAME];  /* Who killed this player. */
   char last_cmd;
-/*  Visual *vis;	Value does not need to be stored */
   Colormap colormap;
   int peaceful;
-/*  long depth;		Value only needed temporarily - don't store */
-/*  XColor exactcolor;  " */
   XColor discolor[NUM_COLORS];
   int iscolor:1;
 
@@ -540,6 +633,8 @@ typedef struct pl {
   Output_Buf	outputs[NUM_OUTPUT_BUFS];
   uint16	outputs_sync;	/* How often to print, no matter what */
   uint16	outputs_count;	/* Print if this count is exceeded */
+  object	*mark;		/* marked object */
+  uint32	mark_count;	/* count or mark object */
 } player;
   
 /*
@@ -581,7 +676,7 @@ typedef struct mapdef {
   char path[MAX_BUF];	/* Filename of the map */
   char *tmpname;	/* Name of temporary file */
   long reset_time;	/* When < sec since 1970, load original instead */
-  sint32 timeout;		/* When it reaches 0, the map will be swapped out */
+  sint32 timeout;	/* When it reaches 0, the map will be swapped out */
   uint32 need_refresh:1;	/* Something went wrong, need sanity check */
 #if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
   sint32 lock_items;       /* The number of lockfile or 0 if not locked */
@@ -590,7 +685,10 @@ typedef struct mapdef {
   uint32 in_memory;	/* If not true, the map has been freed and must
 		         * be loaded before used.  The map,omap and map_ob
 		         * arrays will be allocated when the map is loaded */
-  uint32 encounter:1;	/* True if this is a randomly generated map */
+  uint32 encounter:1;	/* True if this is a randomly generated map -
+			 * prevents random encounter maps within random
+			 * encounter maps
+			 */
   uint8 compressed;  /* Compression method used */
   MapLook *map;
   MapLook *floor;
@@ -600,6 +698,13 @@ typedef struct mapdef {
   object **map_ob;	/* What object lies on the floor */
   uint16 difficulty;	/* What level the player should be to play here */
   objectlink *pending;  /* Objects waiting for the map to be loaded */
+
+  /* lighting code uses these */
+  uint32 do_los:1;	/* True if line of sight needs to be recalculated for 
+		         * players on this map*/
+  uint8 darkness;    	/* indicates level of darkness of map */
+  objectlink *light;    /* linked list of lights (type object) in map */
+
   object *map_object;   /* Map object in new map format */
   oblinkpt *buttons;	/* Linked list of linked lists of buttons */
 } mapstruct;
@@ -614,7 +719,6 @@ typedef struct archt {
   struct archt *next;	/* Next archetype in a linked list */
   struct archt *head;   /* The main part of a linked object */
   struct archt *more; /* Next part of a linked object */
-  struct treasureliststruct *randomitems; /* Items to be generated */
   object clone;		/* An object from which to do copy_object() */
   unsigned long editable;         /* editable flags (mainly for editor) */
   long used_by;		/* A count of how many objects use this archetype */
@@ -664,12 +768,32 @@ typedef enum SpellTypeFrom {
   spellNormal, spellWand, spellRod, spellHorn, spellScroll, spellPotion
 } SpellTypeFrom;
 
+
+typedef struct glnk { /* Used to link together the gods */
+  char *name;       		/* name of this god */
+  struct archt *arch;      	/* pointer to the archetype of this god */
+  int id;               	/* id of the god */ 
+  char *pantheon;      		/* the name of the group this god belongs to */
+  struct glnk *next;    
+} godlink; 
+
+typedef struct ralnk {  /* Used to link the race lists together */
+  char *name;		/* name of this race entry */ 
+  int nrof;		/* nrof things belonging to this race */
+  struct oblnk *member; /* linked object list of things belonging to this race */
+  struct ralnk *next;
+} racelink;
+
+
 typedef struct skill_struct {
-    char *name;		/* how to describe it to player */
-    float time;		/* How many ticks it takes to use the skill */
-    short level;	/* base level required to learn this skill */
-      /* Cost? */
-      /* constraints ? */
+      char *name;       /* how to describe it to the player */
+      short category;   /* the experience category to which this skill belongs */
+      short time;       /* base number of ticks it takes to use the skill */
+      long bexp;        /* base exp gain for this skill */
+      float lexp;       /* level multiplier of exp gain for using this skill */
+      short stat1;      /* primary stat effecting use of this skill */
+      short stat2;      /* secondary stat for this skill */
+      short stat3;      /* tertiary stat for this skill */
 } skill;
 
 
@@ -744,6 +868,7 @@ typedef int shared_string;
  * Some function types
  */
 typedef void (*type_func_int)(int);
+typedef void (*type_func_int_int)(int,int);
 typedef void (*type_func_void)(void);
 typedef void (*type_func_map)(mapstruct *);
 typedef void (*type_func_map_char)(mapstruct *, char *);
