@@ -5,7 +5,7 @@ $archetypes = "archetypes";
 $bmaps = "bmaps";
 $faces = "faces";
 $paths = $bmaps."."."paths";
-$faceExt = "\\.[A-Z0-9][A-Z0-9][A-Z0-9]";
+$faceExt = "\\.[a-zA-Z0-9][A-Z0-9][A-Z0-9]";
 
 ### main
 &info("looking ...");
@@ -33,6 +33,8 @@ sub traverse {
 	$name = &basename($file,""); # DIR
         if( -d $file && $name ne "dev" && $name ne "trashbin") {
 	    &traverse($file);
+	} elsif ( -d $file && ( $name eq "dev" || $name eq "trashbin" ) ) {
+# Empty directive to prevent warnings below
 	} elsif( $file =~ /.*\.arc$/) {	# ARCHETYPE
 	    $archsNum++;
 	    push(@archs,$file);
@@ -43,12 +45,14 @@ sub traverse {
 	    $faces{$name} = $file;
 	    # &warn($file." has no archetype") 
 	    #	if ! -r $dir."/".&basename($file,$faceExt).".arc";
+	} elsif( $file =~ /.*$faceExt.xpm$/) { # FACE
+# Empty directive to prevent warnings below
 	} elsif ( $file =~ /.*\.face$/) {	# Face information file
 	    $facesFileNum++;
 	    push(@face_files, $file);
 	}else {
 	    $trashNum++;
-	    &msg($file." no match");
+	    print "Warning: $file might be a junk file\n";
 	}
 # Supress warning messages.  Almost all of the .xpm files are over
 # 14 characters
@@ -66,7 +70,9 @@ sub storeFaceInfo {
 #	not relevant.  Several monsters use blank.111 as part of their
 #	animation to make them appear invisible, but have some other
 #	foreground color set.
-	if ($fg{$lface} && $fg{$lface} ne $values[0] && $lface ne "blank.111") {
+#	Same applies to empty also.
+	if ($fg{$lface} && $fg{$lface} ne $values[0] && $lface ne "blank.111"
+		&& $lface ne "empty.111") {
 	    &warn($arch." duplicate fg color ".$fg{$lface}."/".$values[0]." face ".$lface);
 	} else {
 		$fg{$lface} = $values[0];
@@ -79,13 +85,29 @@ sub storeFaceInfo {
 		$bg{$lface} = $values[1];
 	}
     }
-    if ($values[2] ne "") {
+    if ($values[2] ne "" && $lface ne "blank.111" && $lface ne "empty.111") {
+#      blank.111 is a special case - see above explanation
+#      Its visibility is always 0.
 	if ($visibility{$lface} && $visibility{$lface} ne $values[2]) {
 	    &warn($arch." duplicate visibilty ".$visibility{$lface}."/".$values[2]." face ".$lface);
 	} else {
 		$visibility{$lface} = $values[2];
 	}
     }
+    if ($values[3] ne "" && lface ne "blank.111" && $lface ne "empty.111") {
+	if ($magicmap{$lface} && $magicmap{$lface} ne $values[3]) {
+	    &warn($arch." duplicate magicmap color ".$magicmap{$lface}."/".$values[3]." face ".$lface);
+	} else {
+		$magicmap{$lface} = $values[3];
+	}
+    }
+    if ($values[4] ne "") {
+	if ($floor{$lface} && $floor{$lface} ne $values[4]) {
+	    &warn($arch." duplicate floor information ".$floor{$lface}."/".$values[4]." face ".$lface);
+	} else {
+		$floor{$lface} = $values[4];
+	}
+    }
 }
 	
 
@@ -101,11 +123,13 @@ line:	while(<ARC>) {
 		$lfg = "";
 		$lbg = "";
 		$lvis = "";
+		$mm = "";
+		$floor = "";
 	    }
 	    if ($var eq "end" &&  $#lface !=0) {
 		$#lface--;
 		foreach $face (@lface) {
-		    &storeFaceInfo($face, $lfg, $lbg, $lvis);
+		    &storeFaceInfo($face, $lfg, $lbg, $lvis,$mm,$floor);
 		}
 	    }
 	    if ($var eq "color_fg") {
@@ -120,6 +144,15 @@ line:	while(<ARC>) {
 		$lvis = $values[0];
 		next line;
 	    }
+	    if ($var eq "magicmap") {
+		$mm = $values[0];
+		next line;
+	    }
+	    if ($var eq "is_floor") {
+		$floor = $values[0];
+		# is_floor is also needed for archs, so let it pass
+		# through
+	    }
 	    $lface[$#lface++] = $values[0]
 		if ($var eq "face");
 	    $anim = 0 if $var eq "mina";
@@ -183,6 +216,8 @@ sub facesOut {
 		$lfg = "";
 		$lbg = "";
 		$lvis = "";
+		$mm = "";
+		$floor = "";
 	    }
 	    elsif ($var eq "color_fg") {
 		$lfg = $values[0];
@@ -193,15 +228,22 @@ sub facesOut {
 	    elsif ($var eq "visibility") {
 		$lvis = $values[0];
 	    }
+	    if ($var eq "magicmap") {
+		$mm = $values[0];
+	    }
+	    if ($var eq "is_floor") {
+		$floor = $values[0];
+	    }
 	    elsif ($var eq "end") {
-		&storeFaceInfo($lface, $lfg, $lbg, $lvis);
+		&storeFaceInfo($lface, $lfg, $lbg, $lvis, $mm, $floor);
 	    }
 	}
 	close(FACE);
     }
     print FACES "# This file is generated by $0, do not edit\n";
     foreach $face (sort(keys %faces)) {
-	if ($fg{$face} ne "" || $bg{$face} ne "" || $visibility{$face} ne "")
+	if ($fg{$face} ne "" || $bg{$face} ne "" || $visibility{$face} ne "" ||
+	    $magicmap{$face} ne "" || $floor{$face} ne "")
 	{
 	print FACES "face ".$face."\n";
 	print FACES "color_fg ".$fg{$face}."\n"
@@ -210,6 +252,10 @@ sub facesOut {
 	    if $bg{$face} ne "";
 	print FACES "visibility ".$visibility{$face}."\n"
 	    if $visibility{$face} ne "";
+	print FACES "magicmap ".$magicmap{$face}."\n"
+	    if $magicmap{$face} ne "";
+	print FACES "is_floor ".$floor{$face}."\n"
+	    if $floor{$face} ne "";
 	print FACES "end\n";
 	}
     }
