@@ -26,16 +26,17 @@ void load_xpm(char *filename)
 {
     FILE *fin;
     int  colors=0, dummy, num_color_trans=0;
-    char buf[256], color_translate[MAX_COLORS][2];
+    char buf[256], color_translate[MAX_COLORS][2], *cp;
 
     if ((fin=fopen(filename, "r"))==NULL) {
 	fprintf(stderr, "Could not open %s\n", filename);
 	exit(8);
     }
+    /* Discard top of file comments */
     while ((fgets(buf, 256, fin)!=NULL) && (strncmp(buf, "static", 6)));
 
     fgets(buf, 256,fin);
-    if (buf[0]=='/') {
+    while (buf[0]=='/') {
 	    fgets(buf, 256,fin);
     }
     sscanf(buf,"\"%d %d %d", &dummy, &dummy, &colors);
@@ -54,11 +55,23 @@ void load_xpm(char *filename)
 	fgets(buf, 256, fin);
 	buf[strlen(buf)-3] ='\0'; /* removing newline, comma, and quote */
 
+	/* Turn all the tabs into spaces. */
+	for (cp=buf+1; *cp!='\0'; cp++) 
+	    if (*cp=='\t') *cp=' ';
+
+	/* Try to find a match string.  Start at position 1
+	 * to we skip the color id for the string.
+	 * If the id for the color matches, set that so we know
+	 * it isn't free.
+	 */
 	for (count=0; count<num_color_match; count++) {
 	    if (!strcasecmp(color_match[count]+1, buf+1)) break;
 	    if (color_match[count][0] == buf[0])
 		color_char_match = count;
 	}
+	if (buf[0]=='\t') 
+	    fprintf(stderr,"File %s using a tab for the color index?\n",filename);
+	/* Didn't find the color - lets add it in. */
 	if (count==num_color_match) {
 	    /* If the character for that color is already used,
 	     * then increase the last one in the array by one and
@@ -66,6 +79,9 @@ void load_xpm(char *filename)
 	     * added.
 	     */
 	    if (color_char_match!=-1 || (buf[0]>(START_COLOR + num_color_match))) {
+		/* color_translate[0] contains the color in the xpm file.
+		 * color_translate[1] contains what it should become.
+		 */
 		color_translate[num_color_trans][0] = buf[0];
 		buf[0] = START_COLOR + num_color_match;
 		color_translate[num_color_trans][1] = buf[0];
@@ -81,6 +97,9 @@ void load_xpm(char *filename)
 		exit(16);
 	    }
 	}
+	/* We already have that color in our database.  Just set up
+	 * the translation table.
+	 */
 	else if (color_match[count][0]!=buf[0]) {
 	    color_translate[num_color_trans][0] = buf[0];
 	    color_translate[num_color_trans][1] = color_match[count][0];
@@ -197,7 +216,7 @@ int main(int argc, char *argv[])
 }
 
 
-#if defined(SVR4) && !defined(sun) && !defined(sgi)
+#if defined(SVR4) && !defined(__sun__) && !defined(sgi)
 int strcasecmp(char *s1, char *s2)
 {
   register int c1, c2;
