@@ -5,6 +5,7 @@
 
 #include <global.h>
 #include <sproto.h>
+#include <skills.h>
 
 static int move_internal (object *op, char *params, int dir)
 {
@@ -18,6 +19,7 @@ static int move_internal (object *op, char *params, int dir)
       }
     } else if (params[0] == 'r' && !op->contr->run_on)
       op->contr->run_on =1;
+      draw_message_window(op);
   }
   move_player(op, dir);
   return 0;
@@ -82,12 +84,11 @@ int command_stay (object *op, char *params)
   It searches both around the player--9 squares counting the one he's on
   and the inventories of all the objects around him for traps.  (Including his own!!  Hmmm.)
 
-  4-21-95 I modified trap_see to allow greater trapdetection with the find trap
-  skill. This needs to be carried over into this routine here. -b.t.
 */
 
-int command_search (object *op, char *params)
-{  object *tmp,*tmp2;
+int command_search (object *op, char *params) {  
+#ifndef ALLOW_SKILLS
+   object *tmp,*tmp2;
    int i;
   /*First we search all around us for runes and traps, which are
     all type RUNE */
@@ -100,14 +101,23 @@ int command_search (object *op, char *params)
 	    /*  And now we'd better do an inventory traversal of each
 	        of these objects' inventory */
 	    for(tmp2=tmp->inv;tmp2!=NULL;tmp2=tmp2->below) 
-		if(tmp2->type==RUNE) { if(trap_see(op,tmp2,0)) trap_show(tmp2,tmp); }
-	    if(tmp->type==RUNE) { if(trap_see(op,tmp,0)) trap_show(tmp,tmp);}
+		if(tmp2->type==RUNE) { if(trap_see(op,tmp2)) trap_show(tmp2,tmp); }
+	    if(tmp->type==RUNE) { if(trap_see(op,tmp)) trap_show(tmp,tmp);}
 	    }
    }
   return 0;
+#else	/* ALLOW_SKILLS */ 
+   if(!change_skill(op,SK_FIND_TRAPS))
+        return 0;
+   else {
+        int success = do_skill(op,0,NULL);
+        return success;
+   }
+#endif /* ALLOW_SKILLS */ 
 }
 
 int command_disarm (object *op, char *params) {
+#ifndef ALLOW_SKILLS
   object *tmp,*tmp2;
   int i;
 
@@ -134,6 +144,14 @@ int command_disarm (object *op, char *params) {
    }
 
 return 0;
+#else
+   if(!change_skill(op,SK_REMOVE_TRAP)) 
+        return 0;
+   else {
+        int success = do_skill(op,0,NULL); 
+        return success;
+   }
+#endif
 }
 
   
