@@ -55,10 +55,22 @@ int execute_newserver_command(object *pl, char *command)
 	    new_draw_info(NDI_UNIQUE, 0,pl, buf);
 	    return 0;
 	}
+	pl->speed_left -= csp->time;
+#if 0
 	if (csp->time && csp->time>pl->speed_left) {
+#endif
+	/* A character time can never exceed his speed (which in many cases,
+	 * if wearing armor, is less than one.)  Thus, in most cases, if
+	 * the command takes 1.0, the player's speed will be less than zero.
+	 * it is only really an issue if time goes below -1
+	 * Due to various reasons that are too long to go into here, we will
+	 * actually still execute player even if his time is less than 0,
+	 * but greater than -1.  This is to improve the performance of the
+	 * new client/server.  In theory, it shouldn't make much difference.
+	 */
+	if (csp->time && pl->speed_left<-2.0) {
 		LOG(llevDebug,"execute_newclient_command: Player issued command that takes more time than he has left.\n");
 	}
-	pl->speed_left -= csp->time;
 	return csp->func(pl, NULL);
   }
   /*
@@ -77,34 +89,38 @@ int execute_newserver_command(object *pl, char *command)
 	new_draw_info(NDI_UNIQUE, 0,pl, buf);
 	return 0;
     }
-    if (csp->time && csp->time>pl->speed_left) {
+    pl->speed_left -= csp->time;
+    if (csp->time && pl->speed_left<-1.0) {
 	LOG(llevDebug,"execute_newclient_command: Player issued command that takes more time than he has left.\n");
     }
-    pl->speed_left -= csp->time;
     return csp->func(pl, cp);
 }
 
 int command_run(object *op, char *params)
 {
     op->contr->run_on=1;
+    draw_message_window(op);
     return (move_player(op, params?atoi(params):0));
 }
 
 int command_run_stop(object *op, char *params)
 {
     op->contr->run_on=0;
+    draw_message_window(op);
     return 1;
 }
 
 int command_fire(object *op, char *params)
 {
     op->contr->fire_on=1;
+    draw_message_window(op);
     return move_player(op, params?atoi(params):0);
 }
 
 int command_fire_stop(object *op, char *params)
 {
     op->contr->fire_on=0;
+    draw_message_window(op);
     return 1;
 }
 
