@@ -26,6 +26,126 @@
 	  last->ob=(NEW);\
           last->id=(COUNT);
 
+/* This is a subset of the parse_id command.  Basically, name can be
+ * a string seperated lists of things to match, with certain keywords.
+ * pl is the player (only needed to set count properly)
+ * op is the item we are trying to match.  Calling function takes care
+ * of what action might need to be done and if it is valid
+ * (pickup, drop, etc.)  Return NONZERO if we have a match.  A higher
+ * value means a better match.  0 means no match.
+ *
+ * Brief outline of the procedure:
+ * We take apart the name variable into the individual components.
+ * cases for 'all' and unpaid are pretty obvious.
+ * Next, we check for a count (either specified in name, or in the
+ * player object.)
+ * If count is 1, make a quick check on the name.
+ * IF count is >1, we need to make plural name.  Return if match.
+ * Last, make a check on the full name.
+ */
+int item_matched_string(object *pl, object *op, char *name)
+{
+
+    char *cp, local_name[MAX_BUF];
+    int count,retval=0;
+
+    strcpy(local_name, name);	/* strtok is destructive to name */
+    for (cp=strtok(local_name,","); cp; cp=strtok(NULL,",")) {
+	while (cp[0]==' ') ++cp;	/* get rid of spaces */
+/*	LOG(llevDebug,"Trying to match %s\n", cp);*/
+	/* All is a very generic match - low match value */
+	if (!strcmp(cp,"all")) return 1;
+	/* unpaid is a little more specific */
+	if (!strcmp(cp,"unpaid") && QUERY_FLAG(op,FLAG_UNPAID)) return 2;
+	if (!strcmp(cp,"cursed") && QUERY_FLAG(op,FLAG_KNOWN_CURSED) &&
+	    (QUERY_FLAG(op,FLAG_CURSED) ||QUERY_FLAG(op,FLAG_DAMNED)))
+		return 2;
+	/* Allow for things like '100 arrows' */
+	if ((count=atoi(cp))!=0) {
+	    cp=strchr(cp, ' ');
+	    while (cp[0]==' ') ++cp;	/* get rid of spaces */
+	}
+	else count=pl->contr->count;
+	if (!cp || cp[0]=='\0' || count<0) return 0;
+	/* base name matched - not bad */
+	if (strcasecmp(cp,op->name)==0 && !count) return 4;
+	else if (count>1) {	/* Need to plurify name for proper match */
+	    char newname[MAX_BUF];
+	    strcpy(newname, op->name);
+ 	    if (QUERY_FLAG(op,FLAG_NEED_IE)) {
+		char *cp1=strrchr(newname,'y');
+                if(cp1!=NULL)
+                    *cp1='\0'; /* Strip the 'y' */
+                strcat(newname,"ies");
+	    }
+	    else strcat(newname,"s");
+	    if (!strcasecmp(newname,cp)) {
+		pl->contr->count=count;	/* May not do anything */
+		return 6;
+	    }
+	}
+	else if (count==1) {
+	    if (!strcasecmp(op->name,cp)) {
+		pl->contr->count=count;	/* May not do anything */
+		return 6;
+	    }
+	}
+	if (!strcasecmp(cp,query_name(op))) retval=20;
+	else if (!strcasecmp(cp,query_short_name(op))) retval=18;
+	else if (!strcasecmp(cp,query_base_name(op))) retval=16;
+	else if (!strncasecmp(cp,query_base_name(op),
+		MIN(strlen(cp),strlen(query_base_name(op))))) retval=14;
+	if (retval) {
+	    pl->contr->count=count;
+	    return retval;
+	}
+    }
+    return 0;
+}
+
+/* Search the inventory of 'pl' for what matches best with params.
+ * we use item_matched_string above - this gives us consistent behaviour
+ * between many commands.  Return the best match, or NULL if no match.
+ */
+object *find_best_object_match(object *pl, char *params)
+{
+    object *tmp, *best=NULL;
+    int match_val=0,tmpmatch;
+
+    for (tmp=pl->inv; tmp; tmp=tmp->below) {
+	if (tmp->invisible) continue;
+	if ((tmpmatch=item_matched_string(pl, tmp, params))>match_val) {
+	    match_val=tmpmatch;
+	    best=tmp;
+	}
+    }
+    return best;
+}
+
+/* This looks for the first item in the players inventory.  In theory,
+ * you can also use non player objects.  In all cases, it skips over
+ * invisible objects.  In the case of players, it will also match up
+ * with what the inventory window shows.
+ */
+
+object *find_first_inventory_item(object *pl)
+{
+
+    object *inv;
+
+    inv= pl->inv;
+    if (pl->type!=PLAYER) {
+	while(inv&&inv->invisible)
+	    inv=inv->below;
+    }
+    else { /*show_what_object also looks for invisibility */
+	while (inv && !show_what_object(inv, pl->contr->show_what))
+	    inv=inv->below;
+    }
+    return inv;
+}
+
+#if 0
 /* returns nonzero if the appropriate action can be
  * done.  Really a stupid function - this should be handled in
  * the calling function.
@@ -38,10 +158,10 @@ static int may_do(object *op,object *ob,int state)
     {
     case 0:			/* drop */
       return (!QUERY_FLAG(ob,FLAG_NO_DROP) && !ob->invisible);
-      break;
+
     case 1:			/* take */
       return (can_pick(op,ob) && !ob->invisible);
-      break;
+
     default:
       return 0;
     }
@@ -128,6 +248,13 @@ static objectlink *parse_id(object *op,object *from, char *name, int state)
   return first;
 }
 
+/* Tries to find a matching object.
+ * op is the player/monster.
+ * from is the potential source object.
+ * ids is a string to match for
+ * state is what we are tryign to do (0=drop, 1=pickup)
+ */
+
 objectlink *id(object *op,object *from,char *ids,int state)
 {
   objectlink *first=NULL,*last=NULL,*tmp;
@@ -136,8 +263,12 @@ objectlink *id(object *op,object *from,char *ids,int state)
     return NULL;
   if(ids==NULL || !ids[0])
     {
-      while(from && from->invisible && may_do(op, from, state))
+      int show_what=show_all;
+      if (op->type==PLAYER && state==0)  show_what=op->contr->show_what;
+
+      while(from && !show_what_object(from,show_what) && may_do(op, from, state)) {
 	from=from->below;
+      }
       if(!from)
 	return NULL;
       OBLINKMALLOC(first);
@@ -193,14 +324,52 @@ objectlink *id(object *op,object *from,char *ids,int state)
   return first;
 }
 
-
+#endif
 int command_uskill ( object *pl, char *params) {
-   if (!params) { 
+   if (!params) {
         new_draw_info(NDI_UNIQUE, 0,pl, "Usage: use_skill <skill name>");
         return 0;
    }
-   change_skill(pl,params);
-   return 0;
+   return use_skill(pl,params);
+}
+
+int command_rskill ( object *pl, char *params) {
+   int skillno;
+
+   if (!params) { 
+        new_draw_info(NDI_UNIQUE, 0,pl, "Usage: ready_skill <skill name>");
+        return 0;
+   }
+   skillno=lookup_skill_by_name(params);
+   if (skillno==-1) {
+	new_draw_info_format(NDI_UNIQUE,0,pl,"Couldn't find skill %s", params);
+	return 0;
+   }
+   return change_skill(pl,skillno);
+}
+
+void apply_inventory(object *op) {
+  object *inv;
+  if(op->contr->last_used!=NULL) {
+    inv=op->contr->last_used;
+    if(QUERY_FLAG(inv, FLAG_FREED)||inv->count!=op->contr->last_used_id||
+       inv->env==NULL||(inv->env!=op&&inv->env->env!=op)) {
+      op->contr->last_used=NULL;
+      op->contr->last_used_id=0;
+      apply_inventory(op);
+      return;
+    }
+  } else {
+    inv= find_first_inventory_item(op);
+  }
+  if(inv) {
+    if(!apply(op,inv)) {
+      new_draw_info_format(NDI_UNIQUE, 0, op,
+	"I don't know how to apply the %s.",query_name(inv));
+    }
+  } else {
+    new_draw_info(NDI_UNIQUE, 0,op, "You have nothing to apply!");
+  }
 }
 
 int command_apply (object *op, char *params)
@@ -209,7 +378,17 @@ int command_apply (object *op, char *params)
     apply_below(op);
     return 0;
   }
-  apply_inventory(op);
+  else {
+    object *inv=find_best_object_match(op, params);
+    if (inv) {
+	if(!apply(op,inv)) {
+	  new_draw_info_format(NDI_UNIQUE, 0, op,
+	    "I don't know how to apply the %s.",query_name(inv));
+	}
+    } else
+	  new_draw_info_format(NDI_UNIQUE, 0, op,
+	    "Could not find any match to %s.",params);
+  }
   /* By applying something what, the applied/unapplied state will change,
    * so the window needs to be updated.
    */
@@ -219,12 +398,21 @@ int command_apply (object *op, char *params)
 }
 
 /* Pick up commands follow */
-
+/* pl = player, op is the object to put tmp into, 
+ * tmp is the object to pick up, nrof is the number to
+ * pick up (0 means all of them)
+ */
 void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 {
     char buf[MAX_BUF];
 
-    if(QUERY_FLAG(pl, FLAG_FLYING) && !QUERY_FLAG(pl, FLAG_WIZ)) {
+    /* IF the player is flying & trying to take the item out of a container 
+     * that is in his inventory, let him.  tmp->env points to the container 
+     * (sack, luggage, etc), tmp->env->env then points to the player (nested 
+     * containers not allowed as of now)
+     */
+    if(QUERY_FLAG(pl, FLAG_FLYING) && !QUERY_FLAG(pl, FLAG_WIZ) && 
+       is_player_inv(tmp)!=pl) {
 	new_draw_info(NDI_UNIQUE, 0,pl, "You are levitating, you can't reach the ground!");
 	return;
     }
@@ -272,7 +460,7 @@ void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 		query_cost_string(tmp,op,F_BUY));
     else
 	(void) sprintf(buf,"You pick up %s.", query_name(tmp));
-    new_draw_info(NDI_UNIQUE, 0,op,buf);
+    new_draw_info(NDI_UNIQUE, 0,pl,buf);
 
     tmp = insert_ob_in_ob(tmp, op);
     if(pl->type!=PLAYER) return;
@@ -293,7 +481,8 @@ void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 void pick_up(object *op,object *alt) 
 /* modified slightly to allow monsters use this -b.t. 5-31-95 */
 {
-    object *tmp=NULL, *split=NULL;
+    object *tmp=NULL;
+    int count;
 
     if(alt)
 	tmp=alt;
@@ -303,27 +492,27 @@ void pick_up(object *op,object *alt)
     } else
 	tmp=op->below;
 
+    if (op->type==PLAYER && op->contr->count && op->contr->count <tmp->nrof)
+	count=op->contr->count;
+    else
+	count=tmp->nrof;
+
     /* container is open, so use it */
     if (op->container) {
 	alt = op->container;
-	if (alt != tmp->env && !sack_can_hold (op, alt, tmp))
+	if (alt != tmp->env && !sack_can_hold (op, alt, tmp,count))
 	    return;
-    } else {
-	if (op->type==PLAYER && op->contr->count && op->contr->count <tmp->nrof)
-	    split = get_split_ob(tmp, op->contr->count);
-	else
-	    split = tmp;
-
+    } else { /* non container pickup */
 	for (alt=op->inv; alt; alt=alt->below)
 	    if (alt->type==CONTAINER && QUERY_FLAG(alt, FLAG_APPLIED) &&
-		alt->race && alt->race==split->race &&
-		sack_can_hold (NULL, alt, split))
+		alt->race && alt->race==tmp->race &&
+		sack_can_hold (NULL, alt, tmp,count))
 		break;  /* perfect match */
 	
 	if (!alt)
 	    for (alt=op->inv; alt; alt=alt->below)
 		if (alt->type==CONTAINER && QUERY_FLAG(alt, FLAG_APPLIED) &&
-		    sack_can_hold (NULL, alt, split))
+		    sack_can_hold (NULL, alt, tmp,count))
 		    break;  /* General container comes next */
 	if (!alt)
 	    alt = op; /* No free containers */
@@ -334,14 +523,6 @@ void pick_up(object *op,object *alt)
 	 */
 	alt = op;
     }
-    /* Need to put the objects back together.  Bit of a waist, since
-     * pick_up_object will split them again, but no easy solution
-     * to that problem.
-     */
-    if (split && tmp!=split) {
-	tmp->nrof += split->nrof;
-	free_object(split);
-    }
 #ifdef PICKUP_DEBUG
     printf ("Pick_up(): %s picks %s (%d) and inserts it %s.\n",op->name, tmp->name,  op->contr->count, alt->name);
 #endif
@@ -350,99 +531,8 @@ void pick_up(object *op,object *alt)
        op->contr->count=0;
     } else 
        pick_up_object(op, alt, tmp, tmp->nrof);
-    
 }
-#if 0
 
-void pick_up(object *op,object *alt) {
-  object *tmp;
-  char buf[MAX_BUF];
-
-  if(QUERY_FLAG(op,FLAG_FLYING)&&!QUERY_FLAG(op, FLAG_WIZ)) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You are levitating, you can't reach the ground!");
-    return;
-  }
-  if(alt!=NULL)
-    if(!can_pick(op,alt)) {
-	if (alt->name!=NULL) {
-	sprintf(buf,"You can't pick up a %s", alt->name);
-	new_draw_info(NDI_UNIQUE, 0,op, buf);
-	}
-      else
-	new_draw_info(NDI_UNIQUE, 0,op,"You can't take that!");
-      return;
-    } else
-      tmp=alt;
-  else if(op->below==NULL||!can_pick(op,op->below)) {
-    new_draw_info(NDI_UNIQUE, 0,op,"There is nothing to pick up here.");
-    return;
-  } else
-    tmp=op->below;
-  if (QUERY_FLAG (tmp, FLAG_NO_DROP))
-      return;
-  if(QUERY_FLAG(tmp,FLAG_WAS_WIZ)&&!QUERY_FLAG(op, FLAG_WAS_WIZ)) {
-    new_draw_info(NDI_UNIQUE, 0,op,"The object disappears in a puff of smoke!");
-    new_draw_info(NDI_UNIQUE, 0,op,"It must have been an illusion.");
-    remove_ob(tmp);
-    free_object(tmp);
-    return;
-  }
-
-  /* container is open, so use it */
-  if (op->container) {
-    alt = op->container;
-    if (alt != tmp->env && !sack_can_hold (op, alt, tmp))
-      return;
-  } else {
-    for (alt=op->inv; alt; alt=alt->below)
-      if (alt->type==CONTAINER && QUERY_FLAG(alt, FLAG_APPLIED) && 
-	  alt->race && alt->race==tmp->race && 
-	  sack_can_hold (NULL, alt, tmp))
-	break;  /* perfect match */
-
-      if (!alt)
-	for (alt=op->inv; alt; alt=alt->below)
-	  if (alt->type==CONTAINER && QUERY_FLAG(alt, FLAG_APPLIED) && 
-	      sack_can_hold (NULL, alt, tmp))
-	    break;  /* General container comes next */
-      if (!alt)
-	alt = op; /* No free containers */
-  }
-  if(tmp->env == alt) {
-    /* here it could be possible to check rent,
-     * if someone wants to implement it
-     */
-    alt = op;
-  }
-  if(QUERY_FLAG(op, FLAG_WAS_WIZ))
-    SET_FLAG(tmp, FLAG_WAS_WIZ);
-  if(op->contr->count) {
-    tmp=get_split_ob(tmp,op->contr->count);
-    op->contr->count=0;
-    if(tmp==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-      return;
-    }
-  }
-  else
-    remove_ob(tmp); /* Unlink it */
-  if(QUERY_FLAG(tmp, FLAG_UNPAID))
-    (void) sprintf(buf,"%s will cost you %s.",query_name(tmp),
-		   query_cost_string(tmp,op,F_BUY));
-  else
-    (void) sprintf(buf,"You pick up %s.",query_name(tmp));
-  new_draw_info(NDI_UNIQUE, 0,op,buf);
-  op->contr->last_used=tmp;
-  op->contr->last_used_id=tmp->count;
-  tmp = insert_ob_in_ob(tmp, alt);
-  draw_inventory(op);
-
-  /* Eneq(@csd.uu.se): Don't forget to draw the containers inv. */
-
-  if (op->container)
-      draw_look (op);
-}
-#endif
 
 int command_take (object *op, char *params)
 {
@@ -469,7 +559,7 @@ int command_take (object *op, char *params)
 	 * but that probably will make it more difficult to read, and
 	 * not make it any more efficient
 	 */
-	if (params && !strncasecmp(params, tmp->name, strlen(params))) {
+	if (params && item_matched_string(op, tmp, params)) {
 	    pick_up(op, tmp);
 	}
 	else if (can_pick(op, tmp) && !params) {
@@ -508,6 +598,7 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
     object *tmp2;
     char buf[MAX_BUF];
 
+    if (sack==tmp) return;	/* Can't put an object in itself */
     if (sack->type != CONTAINER) {
       new_draw_info_format(NDI_UNIQUE, 0,op,
 	"%s is not a container.", query_name(sack));
@@ -531,7 +622,7 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
       new_draw_info(NDI_UNIQUE, 0,op, buf);
       for (tmp2 = tmp->inv; tmp2; tmp2 = tmp) {
 	  tmp = tmp2->below;
-	if (sack_can_hold(op, op->container, tmp2))
+	if (sack_can_hold(op, op->container, tmp2,tmp2->nrof))
 	  put_object_in_sack (op, sack, tmp2, 0);
 	else {
 	  sprintf(buf,"Your %s fills up.", query_name(op->container));
@@ -542,7 +633,7 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
       return;
     }
 
-    if (! sack_can_hold (op, sack, tmp))
+    if (! sack_can_hold (op, sack, tmp,(nrof?nrof:tmp->nrof)))
       return;
 
     /* Small hack to avoid autojoining in apply_special(): */
@@ -679,6 +770,9 @@ void drop_object (object *op, object *tmp, long nrof)
     }
     fix_player(op); /* This is overkill, fix_player() is called somewhere */ 
 		    /* in object.c */
+#ifdef USE_LIGHTING
+    if(tmp->glow_radius>0) remove_light_from_list(tmp,op);
+#endif
 }
 
 void drop(object *op, object *tmp) 
@@ -835,40 +929,165 @@ int command_dropall (object *op, char *params) {
   draw_look(op);
   return 0;
 }
-    
+
+/* Object op wants to drop object(s) params.  params can be a
+ * comma seperated list.
+ */    
 
 int command_drop (object *op, char *params)
 {
-  objectlink *list,*tmp;
-  list=id(op,op->inv,params,0);
-  if(!list)
-    {
-      new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop!");
-      return 0;
+    object  *tmp, *next;
+    int did_one=0, show_what=show_all;
+
+    op->contr->freeze_inv=1;
+    op->contr->freeze_look=1;
+    if (!params) drop(op,find_first_inventory_item(op));
+    else {
+	if (op->type==PLAYER) show_what=op->contr->show_what;
+	for (tmp=op->inv; tmp; tmp=next) {
+	    next=tmp->below;
+	    if (QUERY_FLAG(tmp,FLAG_NO_DROP) || 
+		!show_what_object(tmp,show_what)) continue;
+	    if (item_matched_string(op,tmp,params)) {
+		drop(op, tmp);
+		did_one=1;
+	    }
+	}
+	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop.");
     }
-  op->contr->freeze_inv=1;
-  op->contr->freeze_look=1;
-  while(list)
-    {
-      op->contr->count=list->id;
-      drop(op,list->ob);
-      tmp=list->next;
-      free(list);
-      list=tmp;
-    }
-  op->contr->freeze_inv=0;
-  op->contr->freeze_look=0;
-  op->contr->count=0;
-  draw_all_inventory(op);
-  draw_look(op);
-  return 0;
+    op->contr->freeze_inv=0;
+    op->contr->freeze_look=0;
+    op->contr->count=0;
+    draw_all_inventory(op);
+    draw_look(op);
+    return 0;
 }
 
 int command_examine (object *op, char *params)
 {
   if (!params)
     examine(op,op->below);
-  else
-    examine(op,op->inv);
+  else {
+    object *tmp=find_best_object_match(op,params);
+    if (tmp) 
+        examine(op,tmp);
+    else
+	    new_draw_info_format(NDI_UNIQUE,0,op,"Could not find an object that matches %s",params);
+  }
   return 0;
 }
+
+/* op should be a player.
+ * we return the object the player has marked with the 'mark' command
+ * below.  If no match is found (or object has changed), we return 
+ * NULL.  We leave it up to the calling function to print messages if
+ * nothing is found.
+ */
+object *find_marked_object(object *op)
+{
+    object *tmp;
+
+    if (!op || !op->contr) return NULL;
+    if (!op->contr->mark) {
+/*	new_draw_info(NDI_UNIQUE,0,op,"You have no marked object");*/
+	return NULL;
+    }
+    /* This may seem like overkill, but we need to make sure that they
+     * player hasn't dropped the item.  We use count on the off chance that
+     * an item got reincarnated at some point.
+     */
+    for (tmp=op->inv; tmp; tmp=tmp->below) {
+	if (tmp->invisible) continue;
+	if (tmp == op->contr->mark) {
+	    if (tmp->count == op->contr->mark_count)
+		return tmp;
+	    else {
+		op->contr->mark=NULL;
+		op->contr->mark_count=0;
+/*		new_draw_info(NDI_UNIQUE,0,op,"You have no marked object");*/
+		return NULL;
+	    }
+	}
+    }
+    return NULL;
+}
+    
+
+/* op should be a player, params is any params.
+ * If no params given, we print out the currently marked object.
+ * otherwise, try to find a matching object - try best match first.
+ */
+int command_mark(object *op, char *params)
+{
+    if (!op->contr) return 1;
+    if (!params) {
+	object *mark=find_marked_object(op);
+	if (!mark) new_draw_info(NDI_UNIQUE,0,op,"You have no marked object.");
+	else new_draw_info_format(NDI_UNIQUE,0,op,"%s is marked.", query_name(mark));
+    }
+    else {
+	object *mark1=find_best_object_match(op, params);
+	if (!mark1) {
+	    new_draw_info_format(NDI_UNIQUE,0,op,"Could not find an object that matches %s",params);
+	    return 1;
+	}
+	else {
+	    op->contr->mark=mark1;
+	    op->contr->mark_count=mark1->count;
+	    new_draw_info_format(NDI_UNIQUE,0,op,"Marked item %s", query_name(mark1));
+	    return 0;
+	}
+    }
+    return 0;	/*shouldnt get here */
+}
+
+int command_lock (object *op, char *params)
+{
+    object  *tmp;
+    int show_what=show_all,did_one=0;
+
+    op->contr->freeze_inv=1;
+    op->contr->freeze_look=1;
+    if (!params) lock_inv(op,find_first_inventory_item(op));
+    else {
+	if (op->type==PLAYER) show_what=op->contr->show_what;
+	for (tmp=op->inv; tmp; tmp=tmp->below) {
+	    if (!show_what_object(tmp,show_what)) continue;
+	    if (item_matched_string(op,tmp,params)) {
+		lock_inv(op, tmp);
+		did_one=1;
+	    }
+	}
+	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to lock.");
+    }
+    op->contr->freeze_inv=0;
+    op->contr->count=0;
+    draw_inventory(op);
+    return 0;
+}
+
+int command_unlock (object *op, char *params)
+{
+    object  *tmp;
+    int show_what=show_all,did_one=0;
+
+    op->contr->freeze_inv=1;
+    op->contr->freeze_look=1;
+    if (!params) unlock_inv(op,find_first_inventory_item(op));
+    else {
+	if (op->type==PLAYER) show_what=op->contr->show_what;
+	for (tmp=op->inv; tmp; tmp=tmp->below) {
+	    if (!show_what_object(tmp,show_what)) continue;
+	    if (item_matched_string(op,tmp,params)) {
+		unlock_inv(op, tmp);
+		did_one=1;
+	    }
+	}
+	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to unlock.");
+    }
+    op->contr->freeze_inv=0;
+    op->contr->count=0;
+    draw_inventory(op);
+    return 0;
+}
+
