@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_input_c =
- *   "$Id: c_party.c,v 1.6 1994/12/30 07:46:25 master Exp $";
+ *   "$Id: c_party.c,v 1.8 1997/02/04 10:44:29 master Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -55,8 +55,7 @@ partylist * form_party(object *op, char *params, partylist * firstparty, partyli
 
     newparty->passwd[0] = '\0';
     newparty->next = NULL;
-    newparty->partyleader = (char *)malloc(strlen(op->name));
-    newparty->partyleader = strcpy(newparty->partyleader,op->name);
+    newparty->partyleader = strdup_local(op->name);
     if(firstparty != NULL)
         lastparty->next = newparty;
     new_draw_info_format(NDI_UNIQUE, 0, op,
@@ -102,9 +101,6 @@ int confirm_party_password(object *op) {
 }
 
 void receive_party_password(object *op, char k) {
-  partylist * tmplist;
- 
-  tmplist = firstparty;
  
   if(k!=13) {
     if((k==8 || k==127)&&(op->contr->writing==1))
@@ -156,7 +152,6 @@ int command_gsay(object *op, char *params)
 
 int command_party (object *op, char *params)
 {
-  int no;
   char buf[MAX_BUF];
   partylist * tmpparty; /* For iterating over linked list */
   char * currentparty; /* For iterating over linked list */
@@ -223,10 +218,11 @@ int command_party (object *op, char *params)
         return 1;
       }
     }
-
+    tmpparty=firstparty;
     while(tmpparty != NULL) {
       if(strcmp(tmpparty->partyname,params) == 0) {
-        sprintf("The party %s already exists, pick a new name",params);
+        new_draw_info_format(NDI_UNIQUE, 0, op,
+		"The party %s already exists, pick a new name",params);
         return 1;
       }
       tmpparty = tmpparty->next;
@@ -237,7 +233,7 @@ int command_party (object *op, char *params)
   } /* form */
 
   if(strcmp(params, "leave")==0) {
-    if((no=op->contr->party_number)<=0)
+    if(op->contr->party_number<=0)
       {
         new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
         return 1;
