@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c,v 1.9 1995/04/15 04:52:01 master Exp $";
+ *   "$Id: c_wiz.c,v 1.13 1996/09/30 03:48:38 master Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -43,7 +43,6 @@
 
 int command_goto (object *op, char *params)
 {
-      mapstruct *m, *prev;
       char *name;
       object *dummy;
 
@@ -55,8 +54,6 @@ int command_goto (object *op, char *params)
         return 1;
       }
   name = params;
-      m = has_been_loaded(name);
-      prev = op->map;
       dummy=get_object();
       dummy->map = op->map;
       EXIT_PATH(dummy) = add_string (name);
@@ -253,8 +250,8 @@ int command_create (object *op, char *params)
           insert_ob_in_map(head,op->map);
         else
           head = insert_ob_in_ob(head,op);
-        if (at->randomitems!=NULL)
-          create_treasure(at->randomitems,head,GT_INVENTORY,
+        if (at->clone.randomitems!=NULL)
+          create_treasure(at->clone.randomitems,head,GT_INVENTORY,
                           op->map->difficulty,0);
 	if (op->contr->eric_server > 0) {
 	    esrv_send_item(op, head);
@@ -386,6 +383,9 @@ int command_addexp (object *op, char *params)
 {
     char buf[MAX_BUF];
     int i;
+#ifdef ALLOW_SKILLS
+  object *exp_ob,*skill;
+#endif
   player *pl;
 
   if(params==NULL || sscanf(params, "%s %d", buf, &i)!=2) {
@@ -399,6 +399,24 @@ int command_addexp (object *op, char *params)
       new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
       return 1;
     }
+
+/* In new system: for dm adding experience to a player, only can 
+ * add exp if we satisfy the following: 
+ * 1) there is an associated skill readied by the player 
+ * 2) added exp doesnt result in exp_ob->stats.exp>MAX_EXP_IN_OBJ 
+ */
+
+#ifdef ALLOW_SKILLS
+    if((skill = pl->ob->chosen_skill) && ((exp_ob = pl->ob->chosen_skill->exp_obj)
+       || link_player_skill(pl->ob, skill))) { 
+      i = check_dm_add_exp_to_obj(exp_ob,i);
+      exp_ob->stats.exp += i;
+    } else {
+      new_draw_info(NDI_UNIQUE, 0,op,"Can't find needed experience object.");
+      new_draw_info(NDI_UNIQUE, 0,op,"Player has no associated skill readied.");
+      return 1;
+    }
+#endif
     pl->ob->stats.exp += i;
     add_exp(pl->ob,0);
     SET_FLAG(pl->ob, FLAG_WAS_WIZ);
@@ -448,7 +466,7 @@ int command_stats (object *op, char *params)
          sprintf(buf,"Con : %-2d        AC : %-4d  WC  : %d",
                  pl->ob->stats.Con,pl->ob->stats.ac,pl->ob->stats.wc) ;
          new_draw_info(NDI_UNIQUE, 0,op,buf);
-         sprintf(buf,"Wis : %-2d       EXP : %ld",
+         sprintf(buf,"Wis : %-2d       EXP : %d",
                  pl->ob->stats.Wis,pl->ob->stats.exp);
          new_draw_info(NDI_UNIQUE, 0,op,buf);
          sprintf(buf,"Cha : %-2d      Food : %d",
@@ -456,6 +474,8 @@ int command_stats (object *op, char *params)
          new_draw_info(NDI_UNIQUE, 0,op,buf);
          sprintf(buf,"Int : %-2d    Damage : %d",
                  pl->ob->stats.Int,pl->ob->stats.dam) ;
+         sprintf(buf,"Pow : %-2d    Grace : %d",
+                 pl->ob->stats.Pow,pl->ob->stats.grace) ;
          new_draw_info(NDI_UNIQUE, 0,op,buf);
          break;
        }
@@ -502,6 +522,8 @@ int command_abil (object *op, char *params)
             pl->ob->stats.Cha = iii,pl->orig_stats.Cha = iii;
           if(!strcmp("int",thing2))
             pl->ob->stats.Int = iii,pl->orig_stats.Int = iii;
+          if(!strcmp("pow",thing2))
+            pl->ob->stats.Pow = iii,pl->orig_stats.Pow = iii;
           sprintf(buf,"%s has been altered.",pl->ob->name);
           new_draw_info(NDI_UNIQUE, 0,op,buf);
           fix_player(pl->ob);
@@ -586,3 +608,90 @@ int command_nowiz (object *op, char *params) /* 'noadm' is alias */
      CLEAR_FLAG(op, FLAG_FLYING);
      return 1;
   }
+
+
+static int checkdm(object *op, char *pl_name, char *pl_passwd, char *pl_host)
+{
+  FILE  *dmfile;
+  char  buf[MAX_BUF];
+  char  line_buf[160], name[160], passwd[160], host[160];
+
+  sprintf (buf, "%s/%s", LIBDIR, DMFILE);
+  if ((dmfile = fopen(buf, "r")) == NULL) {
+    LOG (llevDebug, "Could not find DM file.\n");
+    return(0);
+  }
+  while(fgets(line_buf, 160, dmfile) != NULL) {
+    if (line_buf[0]=='#') continue;
+    if (sscanf(line_buf,"%[^:]:%[^:]:%s\n",name, passwd, host)!=3) {
+	LOG(llevError,"Warning - malformed dm file entry: %s", line_buf);
+    }
+    else if ((!strcmp(name,"*") || (pl_name && !strcmp(pl_name, name)))
+      && (!strcmp(passwd,"*") || !strcmp(passwd,pl_passwd)) &&
+      (!strcmp(host,"*") || !strcmp(host, pl_host))) {
+	fclose(dmfile);
+	return (1);
+    }
+  }
+  fclose(dmfile);
+  return (0);
+}
+
+/* Actual command to perhaps become dm.  Changed aroun a bit in version 0.92.2
+ * - allow people on sockets to become dm, and allow better dm file
+ */
+
+int command_dm (object *op, char *params)
+{
+  if (op==NULL) {
+    if (checkdm(op, active_socket->name, params?params:"*", active_socket->host)) {
+      active_socket->wiz=1;
+      new_draw_info(NDI_UNIQUE, 0,op, "Ok, you are the Dungeon Master!");
+      new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
+	"The Dungeon Master has arrived!");
+    }
+    else {
+      new_draw_info(NDI_UNIQUE, 0,op, "Sorry Pal, I don't think so.");
+      LOG(llevDebug,"socket[%s]: DM mode attempted (%s %s)\n", 
+	  active_socket->name,params?params:"*", active_socket->host);
+    }
+    return 1;
+  }
+  /* If this isn't a player, quit out now */
+  else if (!op->contr) return 0;
+  else {
+    char *s1, player_host[160];
+    /* op->contr->name is actually the displayname */
+    if (op->contr->name) {
+	strcpy(player_host, op->contr->name);
+	/* Next line removes the :0 or :0.0 from the player hostname, if it
+	 * is there
+	 */
+	if ((s1=strchr(player_host, ':'))!=NULL) *s1='\0';
+    }
+    else strcpy(player_host,"*");
+    /* op->contr->username is the name supplied with the name command via
+     * the socket
+     */
+    if (checkdm(op, (op->contr->username?op->contr->username:"*"), 
+		(params?params:"*"), player_host)) {
+      SET_FLAG(op, FLAG_WIZ);
+      SET_FLAG(op, FLAG_WAS_WIZ);
+      SET_FLAG(op, FLAG_WIZPASS);
+      new_draw_info(NDI_UNIQUE, 0,op, "Ok, you are the Dungeon Master!");
+      new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
+  	"The Dungeon Master has arrived!");
+      SET_FLAG(op, FLAG_FLYING);
+      clear_los(op);
+      op->contr->writing =0;
+      op->contr->write_buf[0] ='\0';
+      return 1;
+    } else {
+      new_draw_info(NDI_UNIQUE, 0,op, "Sorry Pal, I don't think so.");
+      op->contr->writing =0;
+      op->contr->write_buf[0] ='\0';
+      return 1;
+    }
+  }
+}
+
