@@ -51,6 +51,7 @@ CommArray_s Commands[] = {
   {"drop", command_drop,	1.0},
   {"dropall", command_dropall,	1.0},
   {"examine", command_examine,	0.5},
+  {"fix_me", command_fix_me,	0.0},
   {"get", command_take,		1.0},
   {"help", command_help,	0.0},
   {"hiscore", command_hiscore,	0.0},
@@ -64,6 +65,7 @@ CommArray_s Commands[] = {
   {"malloc", command_malloc,	0.0},
   {"maps", command_maps,	0.0},
   {"mapinfo", command_mapinfo,	0.0},
+  {"mark", command_mark,	0.0},
   {"motd", command_motd,	0.0},
   {"output-sync", command_output_sync,	0.0},
   {"output-count", command_output_count,0.0},
@@ -79,14 +81,18 @@ CommArray_s Commands[] = {
   {"scroll", command_scroll,	0.0},
   {"shout", command_shout,	0.0},
   {"show", command_show,	0.0},
+  {"showinvicon", command_show_inv_icon, 0.0},
 #ifdef ALLOW_SKILLS
   {"skills", command_skills,	0.0},	/* shows player list of skills */
   {"use_skill", command_uskill, 0.0},
+  {"ready_skill", command_rskill, 0.0},
 #endif
   {"search",command_search,	1.0},
 #ifdef SEARCH_ITEMS
   {"search-items", command_search_items,	0.0},
 #endif
+  {"sort_inventory", command_sort_inventory, 0.0},
+  {"statistics", command_statistics,	0.0},
   {"strength", command_strength,	0.0},
   {"strings", command_strings,	0.0},
   {"sync", command_sync,	0.0},
@@ -98,7 +104,9 @@ CommArray_s Commands[] = {
   {"title", command_title,	0.0},
 #endif
   {"unbind", command_unbind,	0.0},
+  {"use-buttons", command_use_buttons,	0.0},
   {"version", command_version,	0.0},
+  {"wimpy", command_wimpy,	0.0},
   {"who", command_who,		0.0},
 
   {"stay", command_stay,	0.0},
@@ -163,6 +171,7 @@ CommArray_s WizCommands [] = {
   {"dump", command_dump,0.0},
   {"dumpbelow", command_dumpbelow,0.0},
   {"dumpfriendlyobjects", command_dumpfriendlyobjects,0.0},
+  {"dumplights", command_dumplights,0.0},
   {"dumpallarchetypes", command_dumpallarchetypes,0.0},
   {"dumpallmaps", command_dumpallmaps,0.0},
   {"dumpallobjects", command_dumpallobjects,0.0},
@@ -269,7 +278,8 @@ long xfire_kc2ks(player *p,KeyCode kc,long foozit)
 long xfire_ks2kc(player *p,KeySym ks)
 {
   if (p->eric_server>0) {
-    return esrv_ks2kc(p->eric_server,ks);
+    LOG(llevDebug,"xfire_ks2kc called in ericserver mode.\n");
+    return 0;
   } else {
     return XKeysymToKeycode(p->gdisp,ks);
   }
@@ -431,7 +441,7 @@ void load_default_keys(player *p)
       free(kp);
     }
   
-  sprintf(filename, "%s/def_keys", LibDir);
+  sprintf(filename, "%s/def_keys", settings.libdir);
   
   if ((fp=fopen(filename, "r")) == NULL) {
     LOG(llevError, "Can't open %s\n", filename);
@@ -441,7 +451,7 @@ void load_default_keys(player *p)
   
   while (fgets(line, MAX_BUF, fp))
     if (line[0] != '#' && line[0] != '\n') {
-      line[MAX_BUF] ='\0';
+      line[MAX_BUF-1] ='\0';
       insert_key(p, KEYF_DEFAULT, line);
     }
   
@@ -523,12 +533,14 @@ void configure_keys(object *op, KeyCode k, KeySym keysym)
   if (op->contr->write_buf[0] == 'B' || 
       op->contr->write_buf[0] == 'U') {
     if(k == op->contr->firekey[0] || k == op->contr->firekey[1]) {
-      op->contr->fire_on =1;
-      return;
+	op->contr->fire_on =1;
+	draw_message_window(op);
+	return;
     }
     if(k == op->contr->runkey[0] || k == op->contr->runkey[1]) {
-      op->contr->run_on =1;
-      return;
+	op->contr->run_on =1;
+	draw_message_window(op);
+	return;
     }
   }
   
@@ -581,6 +593,7 @@ void configure_keys(object *op, KeyCode k, KeySym keysym)
     "Binded to key '%s' (%i)", XKeysymToString(keysym), (int)k);
   op->contr->fire_on=0;
   op->contr->run_on=0;
+  draw_message_window(op);
   return;
 }
 
@@ -596,78 +609,77 @@ void configure_keys(object *op, KeyCode k, KeySym keysym)
 
 int parse_string(object *op, char *str)
 {
-  CommFunc f;
-  char *cp;
+    CommFunc f;
+    char *cp;
 
 #ifdef INPUT_DEBUG
-  LOG(llevDebug, "Command: '%s'\n", str);
+    LOG(llevDebug, "Command: '%s'\n", str);
 #endif
 
-  /*
-   * No arguments?
-   */
-  if (!(cp=strchr(str, ' '))) {
-    if (!op) {
-      if(active_socket == (sockets *) NULL) {
-	LOG(llevError,"parse_string without active socket: %s\n",str);
+    /*
+     * No arguments?
+     */
+    if (!(cp=strchr(str, ' '))) {
+	if (!op) {
+	    if(active_socket == (sockets *) NULL) {
+		LOG(llevError,"parse_string without active socket: %s\n",str);
+		return 0;
+	    }
+	    if ((f=find_socketcommand(str)))
+		return f(op, NULL);
+	    if (active_socket->wiz && (f=find_wizcommand(str)))
+		return f(op, NULL);
+	    new_draw_info_format(NDI_UNIQUE, 0, op, "Unknown command: %s", str);
+	    return 0;
+	}
+	op->contr->writing =0;
+	if(!op->contr->no_echo) {
+	    new_draw_info_format(NDI_UNIQUE, 0, op, ">%s", str);
+	}
+	if ((f=find_command(str)))
+	    return f(op, NULL);
+	if (QUERY_FLAG(op,FLAG_WIZ) && (f=find_wizcommand(str)))
+	    return f(op, NULL);
+
+	if(op) {
+	    op->contr->no_echo = 0;
+	    new_draw_info(NDI_UNIQUE, 0,op, "Unknown command.  Try help.");
+	}
 	return 0;
-      }
-      if ((f=find_socketcommand(str)))
-	return f(op, NULL);
-      if (active_socket->wiz && (f=find_wizcommand(str)))
-	return f(op, NULL);
-      return 0;
-    } 
+    }
+
+    /*
+     * Command with some arguments
+     */
+
+    *(cp++) ='\0';
+
+    if (!op) {
+	if(active_socket == (sockets *) NULL) {
+	    LOG(llevError,"parse_string without active socket: %s %s\n",str, cp);
+	    return 0;
+	}
+	if ((f=find_socketcommand(str)))
+	    return f(op, cp);
+	if (active_socket->wiz && (f=find_wizcommand(str)))
+	    return f(op, cp);
+	return 0;
+    }
+
     op->contr->writing =0;
     if(!op->contr->no_echo) {
-      new_draw_info_format(NDI_UNIQUE, 0, op,
-	">%s", str);
+	new_draw_info_format(NDI_UNIQUE, 0, op, ">%s %s", str, cp);
     }
     if ((f=find_command(str)))
-      return f(op, NULL);
-    if (QUERY_FLAG(op,FLAG_WIZ) && (f=find_wizcommand(str)))
-      return f(op, NULL);
+	return f(op, cp);
+    if (QUERY_FLAG(op, FLAG_WIZ) && (f=find_wizcommand(str)))
+	return f(op, cp);
 
     if(op) {
-      op->contr->no_echo = 0;
-      new_draw_info(NDI_UNIQUE, 0,op, "Unknown command.  Try help.");
+	op->contr->no_echo = 0;
+	new_draw_info(NDI_UNIQUE, 0,op, "Unknown command.  Try help.");
     }
     return 0;
-  }				/* ' ' */
-
-  /*
-   * Command with some arguments
-   */
-
-  *(cp++) ='\0';
-
-  if (!op) {
-    if(active_socket == (sockets *) NULL) {
-      LOG(llevError,"parse_string without active socket: %s %s\n",str, cp);
-      return 0;
-    }
-    if ((f=find_socketcommand(str)))
-      return f(op, cp);
-    if (active_socket->wiz && (f=find_wizcommand(str)))
-      return f(op, cp);
-    return 0;
-  } 
-
-  op->contr->writing =0;
-  if(!op->contr->no_echo) {
-    new_draw_info_format(NDI_UNIQUE, 0, op,
-	">%s %s", str, cp);
-  }
-  if ((f=find_command(str)))
-    return f(op, cp);
-  if (QUERY_FLAG(op, FLAG_WIZ) && (f=find_wizcommand(str)))
-    return f(op, cp);
-
-  if(op) {
-    op->contr->no_echo = 0;
-    new_draw_info(NDI_UNIQUE, 0,op, "Unknown command.  Try help.");
-  }
-  return 0;
 }
 
 
@@ -694,6 +706,7 @@ int parse_key(object *op, char k, KeyCode kc, KeySym keysym)
   Key_s *key;
   player *pl=op->contr;
   static char p_buff[MAX_BUF];
+  char *charkey;
 
 #ifdef INPUT_DEBUG
 
@@ -717,11 +730,17 @@ int parse_key(object *op, char k, KeyCode kc, KeySym keysym)
     return 1;
   }
   if ((kc == pl->firekey[0] && keysym == pl->firekeysym[0]) ||
-      (kc == pl->firekey[1] && keysym == pl->firekeysym[1]))
-    return pl->fire_on=1;
+      (kc == pl->firekey[1] && keysym == pl->firekeysym[1])) {
+	pl->fire_on=1;
+	draw_message_window(op);
+	return 1;
+  }
   if ((kc == pl->runkey[0] && keysym == pl->runkeysym[0]) ||
-      (kc == pl->runkey[1] && keysym == pl->runkeysym[1]))
-    return pl->run_on=1;
+      (kc == pl->runkey[1] && keysym == pl->runkeysym[1])) {
+	pl->run_on=1;
+	draw_message_window(op);
+	return 1;
+  }
   if(pl->writing) {
     if (keysym != NoSymbol &&
 	!(IsKeypadKey(keysym) || IsCursorKey(keysym) || IsPFKey(keysym) ||
@@ -766,17 +785,18 @@ int parse_key(object *op, char k, KeyCode kc, KeySym keysym)
   }
 
   if(k >= '0' && k <= '9') {
-    pl->count =((k-'0')+(int)pl->count*10) % 10000;
+    pl->count =((k-'0')+(int)pl->count*10) % 100000;
     new_draw_info_format(NDI_UNIQUE, 0, op,
 	"Count: %d.", pl->count);
     return 1;
   }
 
+  charkey=XKeysymToString(keysym);
   new_draw_info_format(NDI_UNIQUE, 0, op,
 	"Key unused (%s%s%s)",
 	  (pl->fire_on? "Fire&": ""),
 	  (pl->run_on ? "Run&" : ""),
-	  XKeysymToString(keysym));
+	  (charkey ? charkey:"(null)"));
   pl->count_left=0;
   pl->count=0;
   return 1;
@@ -795,13 +815,15 @@ void handle_keyrelease(object *op,KeyCode kc,KeySym ks)
 
   if ((kc == pl->firekey[0] && ks == pl->firekeysym[0]) ||
       (kc == pl->firekey[1] && ks == pl->firekeysym[1])) {
-    pl->fire_on=0;
-    return;
+	pl->fire_on=0;
+	draw_message_window(op);
+	return;
   }
   if ((kc == pl->runkey[0] && ks == pl->runkeysym[0]) ||
       (kc == pl->runkey[1] && ks == pl->runkeysym[1])) {
-    pl->run_on=0;
-    return;
+	pl->run_on=0;
+	draw_message_window(op);
+	return;
   }
 }
 
