@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_deamon_c =
- *   "$Id: daemon.c,v 1.8 1994/11/09 03:39:05 master Exp $";
+ *   "$Id: daemon.c,v 1.10 1997/03/09 05:07:54 master Exp master $";
  */
 
 /*
@@ -47,7 +47,7 @@ FILE *BecomeDaemon (char *filename)
   int forkresult;
  
   if((logfile=fopen(filename,"a"))==NULL){
-    printf("Couldn't create logfile.\n");
+    printf("Couldn't create logfile %s.\n",filename);
     exit(0);
   } 
   fputs("\n========================\n",logfile);    
@@ -88,7 +88,7 @@ FILE *BecomeDaemon (char *filename)
 	(void) ioctl (i, TIOCTTY, &zero);
 #else
 #ifndef TIOCNOTTY /********* !!!!!!!!! **********/
-#if /*defined(__sun__) &&*/ defined(SVR4)
+#if defined(SVR4)
 #include <sys/termios.h>
 #else
 #include <sys/ttycom.h>
@@ -100,7 +100,7 @@ FILE *BecomeDaemon (char *filename)
     }
 
 
-#ifdef SYSV
+#if defined(SYSV) || defined(SVR4)
 #if defined(__sun__) || defined(hpux) || defined(sgi) || defined(__osf__)
     setsid();
 #else
