@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_hiscore_c =
- *   "$Id: hiscore.c,v 1.13 1995/04/15 04:54:15 master Exp $";
+ *   "$Id: hiscore.c,v 1.14 1997/03/09 05:09:24 master Exp master $";
  */
 
 /*
@@ -122,6 +122,21 @@ score *get_score(char *bp) {
   return &sc;
 }
 
+static char * draw_one_high_score(score *sc) {
+    static char retbuf[MAX_BUF];
+
+    if(!strncmp(sc->killer,"quit",MAX_NAME))
+	sprintf(retbuf,"%3d %10ld %s quit the game on map %s [%d][%d].",
+            sc->position,sc->exp,sc->name,sc->maplevel,sc->maxhp,sc->maxsp);
+    else if(!strncmp(sc->killer,"left",MAX_NAME))
+	sprintf(retbuf,"%3d %10ld %s left the game on map %s [%d][%d].",
+            sc->position,sc->exp,sc->name,sc->maplevel,sc->maxhp,sc->maxsp);
+    else
+	sprintf(retbuf,"%3d %10ld %s was killed by %s on map %s [%d][%d].",
+            sc->position,sc->exp,sc->name,sc->killer,sc->maplevel,
+            sc->maxhp,sc->maxsp);
+    return retbuf;
+}
 /*
  * add_score() adds the given score-structure to the high-score list, but
  * only if it was good enough to deserve a place.
@@ -136,7 +151,7 @@ score *add_score(score *new_score) {
  
   new_score->position=HIGHSCORE_LENGTH+1;
   old_score.position= -1;
-  sprintf(filename,"%s/%s",LibDir,HIGHSCORE);
+  sprintf(filename,"%s/%s",settings.libdir,HIGHSCORE);
   if((fp=open_and_uncompress(filename,1,&comp))!=NULL) {
     while(fgets(buf,MAX_BUF,fp)!=NULL&&nrofscores<HIGHSCORE_LENGTH) {
       if((tmp_score=get_score(buf))==NULL) break;
@@ -189,134 +204,153 @@ score *add_score(score *new_score) {
 }
 
 void check_score(object *op) {
-  char buf[MAX_BUF];
-  score new_score;
-  score *old_score;
-  if(op->stats.exp==0)
-    return;
-  if(!op->contr->name_changed) {
-    if(op->stats.exp>0) {
-      new_draw_info(NDI_UNIQUE, 0,op,"As you haven't changed your name, you won't");
-      new_draw_info(NDI_UNIQUE, 0,op,"get into the high-score list.");
+    char buf[MAX_BUF];
+    score new_score;
+    score *old_score;
+
+    if(op->stats.exp==0)
+	return;
+
+    if(!op->contr->name_changed) {
+	if(op->stats.exp>0) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"As you haven't changed your name, you won't");
+	    new_draw_info(NDI_UNIQUE, 0,op,"get into the high-score list.");
+	}
+	return;
+    }
+    if(QUERY_FLAG(op,FLAG_WAS_WIZ)) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Since you have been in wizard mode,");
+	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	return;
     }
-    return;
-  }
-  if(QUERY_FLAG(op,FLAG_WAS_WIZ)) {
-    new_draw_info(NDI_UNIQUE, 0,op,"Since you have been in wizard mode,");
-    new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
-    return;
-  }
 #ifdef EXPLORE_MODE
-  if (op->contr->explore) {
-    new_draw_info(NDI_UNIQUE, 0,op,"Since you were in explore mode,");
-    new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
-    return;
-  }
-#endif
-  sprintf(buf,"%s the %s",op->name,op->contr->title);
-  strncpy(new_score.name,buf,BIG_NAME);
-  strncpy(new_score.killer,op->contr->killer,BIG_NAME);
-  if(new_score.killer[0]=='\0')
-    strcpy(new_score.killer,"a dungeon collapse");
-  new_score.exp=op->stats.exp;
-  if(op->map == NULL)
-    *new_score.maplevel = '\0';
-  else { 
-    strncpy(new_score.maplevel,op->map->map_object->name,BIG_NAME-1);
-    new_score.maplevel[BIG_NAME-1] = '\0';
-  }
-  new_score.maxhp=(int) op->stats.maxhp;
-  new_score.maxsp=(int) op->stats.maxsp;
-  if((old_score=add_score(&new_score))==NULL) {
-    new_draw_info(NDI_UNIQUE, 0,op,"Error in the highscore list.");
-    return;
-  }
-  if(new_score.position == -1) {
-    new_score.position = HIGHSCORE_LENGTH+1; /* Not strictly correct... */
-    if(!strcmp(old_score->name,new_score.name))
-      new_draw_info(NDI_UNIQUE, 0,op,"You didn't beat your last highscore:");
-    else
-      new_draw_info(NDI_UNIQUE, 0,op,"You didn't enter the highscore list:");
-    draw_one_high_score(old_score);
-    new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-    draw_one_high_score(&new_score);
-    new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-    return;
-  }
-  if(old_score->exp>=new_score.exp)
-    new_draw_info(NDI_UNIQUE, 0,op,"You didn't beat your last score:");
-  else
-    new_draw_info(NDI_UNIQUE, 0,op,"You beat your last score:");
-  draw_one_high_score(old_score);
-  new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-  draw_one_high_score(&new_score);
-  new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-}
-
-void draw_one_high_score(score *sc) {
-  if(!strncmp(sc->killer,"quit",MAX_NAME))
-    sprintf(errmsg,"%3d %10ld %s quit the game on map %s [%d][%d].",
-            sc->position,sc->exp,sc->name,sc->maplevel,sc->maxhp,sc->maxsp);
-  else if(!strncmp(sc->killer,"left",MAX_NAME))
-    sprintf(errmsg,"%3d %10ld %s left the game on map %s [%d][%d].",
-            sc->position,sc->exp,sc->name,sc->maplevel,sc->maxhp,sc->maxsp);
-  else
-    sprintf(errmsg,"%3d %10ld %s was killed by %s on map %s [%d][%d].",
-            sc->position,sc->exp,sc->name,sc->killer,sc->maplevel,
-            sc->maxhp,sc->maxsp);
-}
-
-
-void display_high_score(object *op,int max) {
-  FILE *fp;
-  char buf[MAX_BUF],buf2[MAX_BUF], filename[MAX_BUF], *bp;
-  int i=0,j=0,maxchar=(op==NULL?80:INFOCHARS),comp;
-  score *sc;
-
-  sprintf(filename,"%s/highscore",LibDir);
-  if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
-    perror("Couldn't open highscore file");
-    if(op!=NULL)
-      new_draw_info(NDI_UNIQUE, 0,op,"There is no highscore file.");
-    return;
-  }
-  sprintf(buf,"Nr    Score    Who [max hitpoints][max spellpoints]");
-  if(op==NULL && active_socket == (sockets *) NULL)
-    printf("%s\n",buf);
-  else {
-    if(op != NULL)
-      clear_win_info(op);
-    new_draw_info(NDI_UNIQUE, 0,op,buf);
-  }
-  while(fgets(buf,MAX_BUF,fp)!=NULL) {
-    if(j>=HIGHSCORE_LENGTH||i>=(max-1))
-      break;
-    if((sc=get_score(buf))==NULL)
-      break;
-    i++,j++;
-    sc->position=j;
-    draw_one_high_score(sc);
-    strncpy(buf,errmsg,MAX_BUF);
-    bp=buf+maxchar;
-    if((int)strlen(buf) > maxchar)
-      while(*bp!=' '&&bp>buf)
-        bp--;
-    strncpy(buf2,buf,bp-buf);
-    buf2[bp-buf]='\0';
-    if (op == NULL && active_socket == (sockets *) NULL)
-      printf("%s\n",buf2);
-    else
-      new_draw_info(NDI_UNIQUE, 0,op,buf2);
-
-    if((int)strlen(buf) > maxchar) {
-      i++;
-      strcpy(buf2,"            ");
-      strcat(buf2,bp);
-      if(op == NULL && active_socket == (sockets *) NULL)
-        printf("%s\n",buf2);
-      else
-        new_draw_info(NDI_UNIQUE, 0,op,buf2);
+    if (op->contr->explore) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Since you were in explore mode,");
+	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	return;
     }
-  }
-  close_and_delete(fp, comp);
+#endif
+    sprintf(buf,"%s the %s",op->name,op->contr->title);
+    strncpy(new_score.name,buf,BIG_NAME);
+    strncpy(new_score.killer,op->contr->killer,BIG_NAME);
+    if(new_score.killer[0]=='\0')
+	strcpy(new_score.killer,"a dungeon collapse");
+    new_score.exp=op->stats.exp;
+    if(op->map == NULL)
+	*new_score.maplevel = '\0';
+    else { 
+	strncpy(new_score.maplevel,op->map->map_object->name,BIG_NAME-1);
+	new_score.maplevel[BIG_NAME-1] = '\0';
+    }
+    new_score.maxhp=(int) op->stats.maxhp;
+    new_score.maxsp=(int) op->stats.maxsp;
+    if((old_score=add_score(&new_score))==NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Error in the highscore list.");
+	return;
+    }
+    if(new_score.position == -1) {
+	new_score.position = HIGHSCORE_LENGTH+1; /* Not strictly correct... */
+	if(!strcmp(old_score->name,new_score.name))
+	    new_draw_info(NDI_UNIQUE, 0,op,"You didn't beat your last highscore:");
+	else
+	    new_draw_info(NDI_UNIQUE, 0,op,"You didn't enter the highscore list:");
+	new_draw_info(NDI_UNIQUE, 0,op, draw_one_high_score(old_score));
+	new_draw_info(NDI_UNIQUE, 0,op, draw_one_high_score(&new_score));
+	return;
+    }
+    if(old_score->exp>=new_score.exp)
+	new_draw_info(NDI_UNIQUE, 0,op,"You didn't beat your last score:");
+    else
+	new_draw_info(NDI_UNIQUE, 0,op,"You beat your last score:");
+
+    new_draw_info(NDI_UNIQUE, 0,op, draw_one_high_score(old_score));
+    new_draw_info(NDI_UNIQUE, 0,op, draw_one_high_score(&new_score));
+}
+
+
+
+/* displays the high score file.  object is the calling object
+ * (null if being called via command line.)  max is the maximum
+ * number of scores to display.  match, if set, is the name or class
+ * to match to.
+ */
+
+void display_high_score(object *op,int max, char *match) {
+    FILE *fp;
+    char buf[MAX_BUF],*scorebuf, *bp, *cp;
+    int i=0,j=0,maxchar=(op==NULL?80:INFOCHARS),comp;
+    score *sc;
+
+    sprintf(buf,"%s/%s",settings.libdir,HIGHSCORE);
+    if((fp=open_and_uncompress(buf,0,&comp))==NULL) {
+	perror("Couldn't open highscore file");
+	if(op!=NULL)
+	    new_draw_info(NDI_UNIQUE, 0,op,"There is no highscore file.");
+	return;
+    }
+    if(op==NULL && active_socket == (sockets *) NULL)
+	printf("Nr    Score    Who [max hitpoints][max spellpoints]\n");
+    else {
+	if(op != NULL)
+	    clear_win_info(op);
+	new_draw_info(NDI_UNIQUE, 0,op,"Nr    Score    Who [max hitpoints][max spellpoints]");
+    }
+    while(fgets(buf,MAX_BUF,fp)!=NULL) {
+	if(j>=HIGHSCORE_LENGTH||i>=(max-1))
+	    break;
+	if((sc=get_score(buf))==NULL)
+	    break;
+	sc->position=++j;
+	if (match==NULL) {
+	    scorebuf=draw_one_high_score(sc);
+	    i++;
+	} else {
+	    char tmpbuff[MAX_BUF],*cp1,*cp2,*cp3 ;
+
+	    strcpy(tmpbuff, sc->name);
+	    cp1=strtok(tmpbuff," ");
+	    cp2=strtok(NULL," ");
+	    cp3=strtok(NULL," ");
+	    if (!strcasecmp(cp1, match) || !strcasecmp(cp3, match)) {
+		scorebuf=draw_one_high_score(sc);
+		i++;
+	    }
+	    else continue;
+	}
+	/* Replaced what seemed to an overly complicated word wrap method 
+	 * still word wraps, but assumes at most 2 lines of data.
+	 * mw - 2-12-97
+	 */
+	strncpy(buf,scorebuf,MAX_BUF);
+	cp=buf;
+	if (strlen(cp)> maxchar) {
+	    bp = strrchr(cp+maxchar, ' ');
+	    /* If there is no space, dump it all and get out */
+	    if (!bp) {
+		if (op == NULL && active_socket == (sockets *) NULL)
+		    printf("%s\n",cp);
+		else
+		    new_draw_info(NDI_UNIQUE, 0,op,cp);
+		i++;
+		break;
+	    }
+	    *bp='\0';
+	    bp++;
+	    if (op == NULL && active_socket == (sockets *) NULL) {
+		printf("%s\n",cp);
+		printf("          %s\n",bp);
+	    }
+	    else {
+		new_draw_info(NDI_UNIQUE, 0,op,cp);
+		new_draw_info_format(NDI_UNIQUE, 0, op,"          %s\n",bp);
+	    }
+	    i+=2;
+	} else { /* will fit on one line */
+	    if(op == NULL && active_socket == (sockets *) NULL)
+		printf("%s\n",cp);
+	    else
+		new_draw_info(NDI_UNIQUE, 0,op,cp);
+	}
+    }
+    close_and_delete(fp, comp);
 }
