@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_input_c =
- *   "$Id: input.c,v 1.51 1995/09/11 03:14:50 master Exp master $";
+ *   "$Id: input.c,v 1.60 1997/03/09 05:11:10 master Exp master $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -78,10 +78,15 @@ object **find_objects(object *op, char *line, int flags)
 }
 
 /*
- * Check if an item op can be put into a sack. If pl exits then tell
+ * Check if an item op can be put into a sack. If pl exists then tell
  * a player the reason of failure.
+ * returns 1 if it will fit, 0 if it will not.  nrof is the number of
+ * objects (op) we want to put in.  We specify it separately instead of
+ * using op->nrof because often times, a player may have specified a
+ * certain number of objects to drop, so we can pass that number, and
+ * not need to use split_ob and stuff.
  */
-int sack_can_hold (object *pl, object *sack, object *op) {
+int sack_can_hold (object *pl, object *sack, object *op, int nrof) {
     char buf[MAX_BUF];
     buf[0] = 0;
 
@@ -95,7 +100,7 @@ int sack_can_hold (object *pl, object *sack, object *op) {
 		 query_name(sack));
     if (op->type == SPECIAL_KEY && sack->slaying && op->slaying)
 	sprintf (buf, "You don't want put the key into %s.", query_name(sack));
-    if (sack->weight_limit && sack->carrying + (op->nrof ? op->nrof : 1) * 
+    if (sack->weight_limit && sack->carrying + (nrof ? nrof : 1) * 
 	op->weight * (100 - sack->stats.Str) / 100  > sack->weight_limit)
 	sprintf (buf, "That won't fit in %s!", query_name(sack));
     if (buf[0]) {
@@ -209,71 +214,6 @@ void rotate_left(object *op) {
   draw_inventory(op);
 }
 
-void change_skill(object *pl, char *name) {
-
-  char buf[MAX_BUF];
-  int i=0;
-  object *tmp;
-
-  if(pl->chosen_skill!=NULL
-     && QUERY_FLAG(pl->chosen_skill,FLAG_APPLIED)) { 
-      CLEAR_FLAG(pl->chosen_skill,FLAG_APPLIED); 
-              CLEAR_FLAG(pl,FLAG_READY_SKILL);
-  }
-
-   for(tmp=pl->inv; tmp; tmp=tmp->below) {	/* does player possess the skill? */ 
-     if(tmp->type == SKILL) {		/* player does have skills */ 
-	i++;
-        if(!strcmp(name, skills[tmp->stats.sp].name)) { /* player has named skill? */ 
-            sprintf(buf,"Readying skill: %s. ",skills[tmp->stats.sp].name);
-            new_draw_info(NDI_UNIQUE, 0,pl,buf);
-	    pl->contr->shoottype = range_skill;
-	    pl->chosen_skill=tmp;
-	    SET_FLAG(tmp, FLAG_APPLIED);
-	    if(!QUERY_FLAG(pl,FLAG_READY_SKILL)) 
-		SET_FLAG(pl,FLAG_READY_SKILL);
-	    fix_player(pl);
-	    draw_stats(pl);
-	    return;
-        }
-     }
-  }
-  if(i<1)
-    sprintf(buf,"You have no skills that you may ready!");
-  else
-    sprintf(buf,"You have no skill named %s.",name);
-  new_draw_info(NDI_UNIQUE, 0,pl,buf);
-  if(QUERY_FLAG(pl,FLAG_READY_SKILL))         /* shouldnt happen -b.t. */ 
-    CLEAR_FLAG(pl, FLAG_READY_SKILL);
-
-}
-
-void apply_inventory(object *op) {
-  object *inv;
-  if(op->contr->last_used!=NULL) {
-    inv=op->contr->last_used;
-    if(QUERY_FLAG(inv, FLAG_FREED)||inv->count!=op->contr->last_used_id||
-       inv->env==NULL||(inv->env!=op&&inv->env->env!=op)) {
-      op->contr->last_used=NULL;
-      op->contr->last_used_id=0;
-      apply_inventory(op);
-      return;
-    }
-  } else {
-    inv= op->inv;
-    while(inv&&inv->invisible)
-      inv=inv->below;
-  }
-  if(inv) {
-    if(!apply(op,inv)) {
-      new_draw_info_format(NDI_UNIQUE, 0, op,
-	"I don't know how to apply the %s.",query_name(inv));
-    }
-  } else {
-    new_draw_info(NDI_UNIQUE, 0,op, "You have nothing to apply!");
-  }
-}
-
 void drop_inventory(object *op) {
   object *inv,*n=NULL;
   if(op->contr->last_used!=NULL) {
@@ -347,6 +287,7 @@ char *long_desc(object *tmp) {
   buf[0]='\0';
   switch(tmp->type) {
   case RING:
+  case SKILL:
   case WEAPON:
   case ARMOUR:
   case BRACERS:
@@ -359,6 +300,9 @@ char *long_desc(object *tmp) {
   case BOW:
   case ARROW:
   case CLOAK:
+  case FOOD:
+  case DRINK:
+  case FLESH:
     if(*(cp=describe_item(tmp))!='\0')
       sprintf(buf,"%s %s",query_name(tmp),cp);
     break;
@@ -370,7 +314,7 @@ char *long_desc(object *tmp) {
 }
 
 void examine(object *op, object *tmp) {
-  char buf[MAX_BUF];
+  char buf[VERY_BIG_BUF];
 
   if (tmp == NULL || tmp->type == CLOSE_CON)
     return;
@@ -384,7 +328,11 @@ void examine(object *op, object *tmp) {
   switch(tmp->type) {
   case SPELLBOOK:
     if(QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->stats.sp > 0 && tmp->stats.sp <= NROFREALSPELLS )
-      sprintf(buf,"%s is a %d level spell.",
+      if(!strcmp(tmp->arch->name,"cleric_book"))
+        sprintf(buf,"%s is a %d level prayer.",
+              spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
+      else
+        sprintf(buf,"%s is a %d level spell.",
               spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
     break;
   case BOOK:
@@ -517,53 +465,11 @@ void inventory(object *op,object *inv) {
   }
   if(!inv && op) {
     sprintf(buf,"%-*s %-8s",
-            INFOCHARS-9,"Total weight carried:",query_weight(op));
+            INFOCHARS-9,"Total weight :",query_weight(op));
     new_draw_info(NDI_UNIQUE, 0,op,buf);
   }
 }
 
-/* Test function. Meant to allow players to examine
- * their current skill list. b.t. (thomas@nomad.astro.psu.edu) 
- * This is a complete hack of inventory command above!!!
- */
-
-void show_skills(object *op) {
-
-  object *tmp = op->inv;
-  char buf[MAX_BUF], *in;
-  int items = 0, length;
-
-  while (tmp) {
-    if (tmp->type == SKILL)
-      items++;
-    tmp=tmp->below;
-  }
-  if (items==0) {
-      new_draw_info(NDI_UNIQUE, 0,op,"You know no skills.");
-      op->chosen_skill=NULL;	/* double safety - for those who lose skills */ 
-      return;
-  } else {
-      length = INFOCHARS-19;
-      in = "";
-      if (op)
-        clear_win_info(op);
-      new_draw_info(NDI_UNIQUE, 0,op,"Skills known:");
-  }
-  for (tmp=op->inv; tmp; tmp=tmp->below) {
-    if((!op) || (tmp->type != SKILL)) 
-      continue;
-    if(tmp->invisible||QUERY_FLAG(tmp,FLAG_APPLIED)) { 
-      if((!op || QUERY_FLAG(op, FLAG_WIZ)))
-        (void) sprintf(buf,"%s- %-*s (%5d)", in, length, skills[tmp->stats.sp].name,
-                     tmp->count);
-      else
-        (void) sprintf(buf,"%s- %-*s", in, length+8, skills[tmp->stats.sp].name); 
-      new_draw_info(NDI_UNIQUE, 0,op,buf);
-    }
-  }
-
-}
-
 /*
  * look_at prints items on the specifc square.
  * [ removed EARTHWALL check and added check for containers inventory.
@@ -599,7 +505,9 @@ void look_at(object *op,int dx,int dy) {
       (void) sprintf(buf,"- %s.",query_name(tmp));
     new_draw_info(NDI_UNIQUE, 0,op,buf);
     if((tmp->inv!=NULL || (tmp->head && tmp->head->inv)) && 
-       ( (!dx&&!dy) || tmp->type != CONTAINER || QUERY_FLAG(op, FLAG_WIZ)))
+       ( (!dx&&!dy) || tmp->type != CONTAINER || QUERY_FLAG(op, FLAG_WIZ)
+/* added some cases here -b.t. */
+	|| !(tmp->type) || tmp->type!=FLESH ))
       inventory(op,tmp->head==NULL?tmp:tmp->head);
     if(QUERY_FLAG(tmp, FLAG_IS_FLOOR)&&!QUERY_FLAG(op, FLAG_WIZ))	/* don't continue under the floor */
       break;
@@ -734,19 +642,19 @@ static void help_topics(object *op, int what)
   
   switch (what) {
   case 1:
-    sprintf(filename, "%s/wizhelp", LibDir);
+    sprintf(filename, "%s/wizhelp", settings.libdir);
     new_draw_info(NDI_UNIQUE, 0,op, "      Wiz commands:");
     break;
   case 2:
-    sprintf(filename, "%s/sockethelp", LibDir);
+    sprintf(filename, "%s/sockethelp", settings.libdir);
     new_draw_info(NDI_UNIQUE, 0,op, "      Socket commands:");
     break;
   case 3:
-    sprintf(filename, "%s/mischelp", LibDir);
+    sprintf(filename, "%s/mischelp", settings.libdir);
     new_draw_info(NDI_UNIQUE, 0,op, "      Misc help:");
     break;
   default:
-    sprintf(filename, "%s/help", LibDir);
+    sprintf(filename, "%s/help", settings.libdir);
     new_draw_info(NDI_UNIQUE, 0,op, "      Commands:");
     break;
 }
@@ -814,50 +722,6 @@ static void show_commands(object *op, int what)
   new_draw_info(NDI_UNIQUE, 0,op, line);
 }
 
-int command_lock (object *op, char *params)
-{
-
-  objectlink *list;
-
-  list = id(op,op->inv,params,0);
-  if(!list) {
-    new_draw_info(NDI_UNIQUE, 0,op,"Nothing to lock!");
-    return 0;
-  }
-  op->contr->freeze_inv=1;
-  op->contr->freeze_look=1;
-  while(list) {
-    op->contr->count=list->id;
-    lock_inv(op,list->ob);
-    list=list->next;
-  }
-  op->contr->freeze_inv=0;
-  op->contr->count=0;
-  draw_inventory(op);
-  return 0;
-}
-
-int command_unlock (object *op, char *params)
-{
-  objectlink *list;
-
-  list = id(op,op->inv,params,0);
-  if(!list) {
-    new_draw_info(NDI_UNIQUE, 0,op,"Nothing to unlock!");
-    return 1;
-  }
-  op->contr->freeze_inv=1;
-  while(list) {
-    op->contr->count=list->id;
-    unlock_inv(op,list->ob);
-    list=list->next;
-  }
-  op->contr->freeze_inv=0;
-  op->contr->count=0;
-  draw_inventory(op);
-  return 0;
-}
-
 int command_help (object *op, char *params)
 {
   struct stat st;
@@ -872,14 +736,14 @@ int command_help (object *op, char *params)
    * Main help page?
  */
   if (!params) {
-    sprintf(filename, "%s/def_help", LibDir);
+    sprintf(filename, "%s/def_help", settings.libdir);
     if ((fp=fopen(filename, "r")) == NULL) {
       LOG(llevError, "Can't open %s\n", filename);
       perror("Can't read default help");
       return 0;
     }
     while (fgets(line, MAX_BUF, fp)) {
-      line[MAX_BUF] ='\0';
+      line[MAX_BUF-1] ='\0';
       len =strlen(line)-1;
       if (line[len] == '\n')
 	line[len] ='\0';
@@ -931,23 +795,23 @@ int command_help (object *op, char *params)
     return 0;
   }
 
-  sprintf(filename, "%s/mischelp/%s", LibDir, params);
+  sprintf(filename, "%s/mischelp/%s", settings.libdir, params);
   if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
     if (op) {
-      sprintf(filename, "%s/help/%s", LibDir, params);
+      sprintf(filename, "%s/help/%s", settings.libdir, params);
       if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
 	if (QUERY_FLAG(op, FLAG_WIZ)) {
-	  sprintf(filename, "%s/wizhelp/%s", LibDir, params);
+	  sprintf(filename, "%s/wizhelp/%s", settings.libdir, params);
 	  if (stat(filename, &st) || !S_ISREG(st.st_mode))
 	    goto nohelp;
 	} else
 	  goto nohelp;
       }
           } else {
-      sprintf(filename, "%s/sockethelp/%s", LibDir, params);
+      sprintf(filename, "%s/sockethelp/%s", settings.libdir, params);
       if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
 	if (active_socket->wiz) {
-	  sprintf(filename, "%s/wizhelp/%s", LibDir, params);
+	  sprintf(filename, "%s/wizhelp/%s", settings.libdir, params);
 	  if (stat(filename, &st) || !S_ISREG(st.st_mode))
 	    goto nohelp;
 	} else
@@ -967,7 +831,7 @@ int command_help (object *op, char *params)
   sprintf(line, "Help about '%s'", params);
   new_draw_info(NDI_UNIQUE, 0,op, line);
   while (fgets(line, MAX_BUF, fp)) {
-    line[MAX_BUF] ='\0';
+    line[MAX_BUF-1] ='\0';
     len =strlen(line)-1;
     if (line[len] == '\n')
       line[len] ='\0';
@@ -994,13 +858,13 @@ int command_chrfont (object *op, char *params)
     if ((tmp_font=XLoadQueryFont(op->contr->gdisp, params)) != NULL) {
       strcpy (op->contr->font_str, params);
       XFreeFont(op->contr->gdisp, tmp_font);
-      } else {
+    } else {
       LOG(llevError, "Crossfire: Couldn't load font %s.\n", params);
       new_draw_info(NDI_UNIQUE, 0,op, "Couldn't load font.");
-      }
+    }
   } else new_draw_info(NDI_UNIQUE, 0,op, "What font?");
       return 1;
-    }
+}
 
 
 int command_invoke(object *op, char *params)
@@ -1018,39 +882,16 @@ int command_prepare(object *op, char *params)
 	return command_cast_spell(op, params, 'p');
 }
 
-/* Some commands are not accepted from sockets */
-
-int command_cast_spell (object *op, char *params, char command)
+/* object *op is the caster, params is the spell name.  We return the index
+ * value of the spell in the spells array for a match, -1 if there is no
+ * match, -2 if there are multiple matches.  Note that 0 is a valid entry, so
+ * we can't use that as failure.
+ */
+static int find_spell_byname(object *op, char *params)
 {
-  int i, castnow=0;
-  char *cp=NULL;
-  char buf[MAX_BUF];
-
-      if(!op->contr->nrofknownspells&&!QUERY_FLAG(op, FLAG_WIZ)) {
-        new_draw_info(NDI_UNIQUE, 0,op,"You don't know any spells.");
-        return 1;
-      }
-
-      if(op->contr->golem!=NULL) {
-        remove_friendly_object(op->contr->golem);
-        remove_ob(op->contr->golem);
-        free_object(op->contr->golem);
-        op->contr->golem=NULL;
-      }
-
-  if (command=='i') castnow = 1;
-  if(params!=NULL) {
     int numknown; /* number of spells known by op */
     int spnum;  /* number of spell that is being cast */
-
-    /* rune of fire, rune of ... are special cases */
-    if (strncmp(params,"rune",4)) {
-	cp =strstr(params, " of ");
-        if (cp) {
-	      *cp='\0';
-	      cp +=4;
-	}
-    }
+    int match=-1,i;
 
     if(QUERY_FLAG(op, FLAG_WIZ))
 	numknown = NROFREALSPELLS;
@@ -1058,57 +899,151 @@ int command_cast_spell (object *op, char *params, char command)
 	numknown = op->contr->nrofknownspells;
 
     for(i=0;i<numknown;i++){
-	    if (QUERY_FLAG(op, FLAG_WIZ)) spnum = i;
-	    else spnum = op->contr->known_spells[i];
+	if (QUERY_FLAG(op, FLAG_WIZ)) spnum = i;
+	else spnum = op->contr->known_spells[i];
 
-	    if (!strncmp(params, spells[spnum].name, strlen(params))) {
-		/* matched the spell name. */
-/*		if (spells[spnum].range != 0) {*/
-		    if (castnow)
-			op->stats.sp-=cast_spell(op,op,op->facing,
-                      spnum,0,spellNormal,cp);
-              	    else {
-                        op->contr->shoottype=range_magic;
-                        op->contr->chosen_spell=spnum;
-		    }
-		    draw_stats(op);
-		    return 1;
-/*	    	}*/
-/*
-		op->stats.sp -= invoke_spell(op, op->facing, spnum, 0, spellNormal, cp);
-		draw_stats(op);
-		return 1;
-*/
-	    }
+	if (!strncmp(params, spells[spnum].name, strlen(params))) {
+	    /* We already found a match previously - thus params is not
+	     * not unique, so return -2 stating this.
+	     */
+	    if (match>=0) return -2;
+	    else match=spnum;
+	}
     }
-  }
-      new_draw_info(NDI_UNIQUE, 0,op,"Cast what spell?  Choose one of:");
+    return match;
+}
 
-      new_draw_info(NDI_UNIQUE, 0,op,"[sp] spell name");
-      if(QUERY_FLAG(op, FLAG_WIZ))
-        for(i=0;i<NROFREALSPELLS;i++)
-	  {
-#ifdef SPELLPOINT_LEVEL_DEPEND
-	    sprintf(buf,"[%02d] %s",SP_level_spellpoint_cost(op,i),
-#else
-	    sprintf(buf,"[%02d] %s",spells[i].sp,
-#endif
-		    spells[i].name);
-	    new_draw_info(NDI_UNIQUE, 0,op,buf);
-	  }
-      else
-        for(i=0;i<(int)op->contr->nrofknownspells;i++)
-	  {
-#ifdef SPELLPOINT_LEVEL_DEPEND
-	    sprintf(buf,"[%02d] %s",
-	    SP_level_spellpoint_cost(op,op->contr->known_spells[i]),
-#else
-	    sprintf(buf,"[%02d] %s",spells[op->contr->known_spells[i]].sp,
-#endif
-		    spells[op->contr->known_spells[i]].name);
-	    new_draw_info(NDI_UNIQUE, 0,op,buf);
-	  }
+/* Shows all spells that op knows.  If params is supplied, the must match
+ * that.  If cleric is 1, show cleric spells, if not set, show mage
+ * spells.
+ */
+static void show_matching_spells(object *op, char *params, int cleric)
+{
+    int i,spnum,first_match=0;
+    char lev[80];
+
+    for (i=0; i<(QUERY_FLAG(op, FLAG_WIZ)?NROFREALSPELLS:op->contr->nrofknownspells); i++) {	
+	if (QUERY_FLAG(op,FLAG_WIZ)) spnum=i;
+	else spnum = op->contr->known_spells[i];
+
+	if (spells[spnum].cleric != cleric) continue;
+	if (params && strncmp(spells[spnum].name,params, strlen(params)))
+		continue;
+	if (!first_match) {
+	    first_match=1;
+	    if (!cleric)
+		new_draw_info(NDI_UNIQUE, 0, op, "Mage spells");
+	    else
+		new_draw_info(NDI_UNIQUE, 0, op, "Priest spells");
+	    new_draw_info(NDI_UNIQUE, 0,op,"[sp] [lev] spell name");
+	}
+	if (spells[spnum].path & op->path_denied)
+	    strcpy(lev,"den");
+	else
+	    sprintf(lev," %02d",spells[spnum].level);
+
+	new_draw_info_format(NDI_UNIQUE,0,op,"[%02d] [%s] %s",
+		SP_level_spellpoint_cost(op,op,spnum),
+		lev,
+		spells[spnum].name);
+    }
+}
+
+
+
+/* sets up to cast a spell.  op is the caster, params is the spell name,
+ * and command is the first letter of the spell type (c=cast, i=invoke, 
+ * p=prepare).  Invoke casts a spell immediately, where as cast (and I believe
+ * prepare) just set up the range type.
+ */
+
+int command_cast_spell (object *op, char *params, char command)
+{
+    int castnow=0;
+    char *cp=NULL;
+    int spnum=-1;  /* number of spell that is being cast */
+
+    if(!op->contr->nrofknownspells&&!QUERY_FLAG(op, FLAG_WIZ)) {
+	new_draw_info(NDI_UNIQUE, 0,op,"You don't know any spells.");
         return 1;
+    }
+    /* Remove control of the golem */
+    if(op->contr->golem!=NULL) {
+        remove_friendly_object(op->contr->golem);
+        remove_ob(op->contr->golem);
+        free_object(op->contr->golem);
+        op->contr->golem=NULL;
+    }
+
+    if (command=='i') castnow = 1;
+    if(params!=NULL) {
+
+	/* rune of fire, rune of ... are special cases, break it into 'rune' and
+	 * then put 'fire, marking, whatever' in cp. */
+	if (strncmp(params,"rune",4)) {
+	    cp =strstr(params, " of ");
+	    if (cp) {
+		*cp='\0';
+		cp +=4;
+	    }
+	}
+	spnum = find_spell_byname(op, params);
+	if (spnum>=0) {
+	    rangetype orig_rangetype=op->contr->shoottype;
+	    op->contr->shoottype=range_magic;
+#ifdef ALLOW_SKILLS 
+	    if(op->type==PLAYER&&!QUERY_FLAG(op,FLAG_WIZ)) { 
+		 /* if we don't change to the correct spell numb,
+		  * check_skill_to_fire will be confused as to which
+		  * spell casting skill to ready for the player!
+		  * I set the code to change back to the old spellnum
+		  * after we check, but is this really needed?? -b.t. */
+
+		int orig_spn = op->contr->chosen_spell;
+		op->contr->chosen_spell=spnum;
+		if(!check_skill_to_fire(op)) {  
+		    op->contr->shoottype=orig_rangetype;
+		    return 0; 
+		    }
+		op->contr->chosen_spell=orig_spn;
+	    }
+#endif
+	    if (castnow) { 
+		int value;
+
+		    /* Need to switch shoottype to range_magic - otherwise 
+                     * cast_spell doesn't check to see if the character 
+                     * has enough spellpoints. 
+		     * Note: now done above this -b.t. */
+		    /* op->contr->shoottype=range_magic; */ 
+
+		value = cast_spell(op,op,op->facing,spnum,0,spellNormal,cp);
+		op->contr->shoottype=orig_rangetype;
+
+		if(spells[spnum].cleric) 
+			op->stats.grace -= value;
+		else 
+			op->stats.sp -= value;
+	    } 
+	    /* We are not casting now */
+	    else op->contr->chosen_spell=spnum; 
+	    
+	    draw_stats(op);
+	    return 1;
+	} /* found a matching spell */
+    } /* params supplied */
+
+    /* We get here if cast was given without options or we could not find
+     * the requested spell.  List all the spells the player knows (if
+     * spnum = -1) or spells matching params if spnum=-2
+     */
+
+    new_draw_info(NDI_UNIQUE, 0,op,"Cast what spell?  Choose one of:");
+    show_matching_spells(op, (spnum==-2)?params:NULL, 0);
+    new_draw_info(NDI_UNIQUE,0,op,"");
+    show_matching_spells(op, (spnum==-2)?params:NULL, 1);
+
+    return 1;
 }
 
 #ifdef SET_TITLE
@@ -1166,10 +1101,10 @@ int command_search_items (object *op, char *params)
 	}
 	op->contr->search_str[0]='\0';
 	new_draw_info(NDI_UNIQUE, 0,op,"Search mode turned off.");
-    fix_player(op);
-    draw_stats(op);
+	fix_player(op);
+	draw_stats(op);
 	return 1;
-      }
+  }
   if((int)strlen(params) >= MAX_BUF) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Search string too long.");
 	return 1;
@@ -1458,32 +1393,6 @@ int command_explore (object *op, char *params)
     }
 #endif
 
-int command_dm (object *op, char *params)
-{
-  if (op==NULL) {
-    new_draw_info(NDI_UNIQUE, 0,NULL, "You cannot enter dm mode from a socket.");
-    return 1;
-  }
-  if (checkdm(op->contr->username)) {
-    SET_FLAG(op, FLAG_WIZ);
-    SET_FLAG(op, FLAG_WAS_WIZ);
-    SET_FLAG(op, FLAG_WIZPASS);
-    new_draw_info(NDI_UNIQUE, 0,op, "Ok, you are the Dungeon Master!");
-    new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
-	"The Dungeon Master has arrived!");
-    SET_FLAG(op, FLAG_FLYING);
-    clear_los(op);
-    op->contr->writing =0;
-    op->contr->write_buf[0] ='\0';
-    return 1;
-  } else {
-    new_draw_info(NDI_UNIQUE, 0,op, "Sorry Pal, I don't think so.");
-    op->contr->writing =0;
-    op->contr->write_buf[0] ='\0';
-    return 1;
-  }
-}
-
 #ifdef SOUND_EFFECTS
 int command_sound (object *op, char *params)
 {
@@ -1497,10 +1406,10 @@ int command_sound (object *op, char *params)
 
 int command_add (object *op, char *params)
 {
-    int i, pixtmp = use_pixmaps, cpix = color_pix;
-    long splittmp = default_split_window;
-    int synctmp = synchronize;
     char username[9];
+    Settings oldsettings=settings;
+    int i;
+
 #ifdef EXPLORE_MODE
     if (explore_mode()) {
 	new_draw_info(NDI_UNIQUE, 0,op,"You are not able to add new players in explore mode.");
@@ -1513,6 +1422,7 @@ int command_add (object *op, char *params)
       return 1;
     }
 #endif
+
   if(params == NULL) {
       if(op != NULL)
         op->contr->writing=0;
@@ -1521,19 +1431,17 @@ int command_add (object *op, char *params)
     }
     if (active_socket != (sockets *) NULL)
     {
-      color_pix = active_socket->color_pix;
-      use_pixmaps = (int) active_socket->use_pix;
-      default_split_window = (long) active_socket->split;
-      synchronize = active_socket->sync;
-      strcpy(username, active_socket->name);
+	if (active_socket->color_pix) settings.displaymode=Dm_Pixmap;
+	else if (active_socket->use_pix) settings.displaymode=Dm_Bitmap;
+	settings.splitwindows=active_socket->split;
+	settings.synchronize=active_socket->sync;
+	strcpy(username, active_socket->name);
     }
     else
-      *username = '\0';
+	*username = '\0';
     i=add_player(params, *username ? username : NULL, NULL,0);
-    use_pixmaps = pixtmp;
-    default_split_window = splittmp;
-    synchronize = synctmp;
-    color_pix = cpix;
+    settings = oldsettings;
+
     /* This player can be freed during a add_player() */
     if(op == NULL || (op->type == PLAYER && !QUERY_FLAG(op,FLAG_FREED)))
       new_draw_info(NDI_UNIQUE, 0,op, (i ? errmsg : "OK."));
@@ -1710,7 +1618,6 @@ int legal_range(object *op,int r) {
     return 0;
   case range_scroll: /* Use scrolls */
     return 0;
-
   case range_skill:
       for (tmp = op->inv; tmp!=NULL; tmp=tmp->below) {
 	  if (tmp->type == SKILL) { 
@@ -1719,7 +1626,6 @@ int legal_range(object *op,int r) {
       }
       op->chosen_skill=NULL;	/* they have lost all skills :) */ 
       return 0;
-
   }
   return 0;
 }
@@ -1737,7 +1643,7 @@ void change_spell(object *op,char k) {
     if(op->contr->shoottype >= range_size)
       op->contr->shoottype = range_none;
     else if (op->contr->shoottype <= range_bottom)
-      op->contr->shoottype = range_size-1;
+      op->contr->shoottype = (rangetype) range_size-1;
   } while (!legal_range(op,op->contr->shoottype));
   switch(op->contr->shoottype) {
   case range_none:
@@ -1774,9 +1680,8 @@ void change_spell(object *op,char k) {
     break;
   case range_skill: 
     sprintf (buf, "Switched to skill: %s", op->chosen_skill ?  
-		 skills[op->chosen_skill->stats.sp].name : "none");
+		 op->chosen_skill->name : "none");
     break;
-
   default:
     break;
   }
@@ -1920,13 +1825,17 @@ int command_show (object *op, char *params)
 
 int command_throw (object *op, char *params)
 {
-      new_draw_info(NDI_UNIQUE, 0,op,"Throw is disabled (it was too buggy).");
-#if 0
-      op->contr->shoottype=10;
-      op->contr->count_left=0;
-      new_draw_info(NDI_UNIQUE, 0,op,"You will now throw things.");
-#endif
+
+#ifdef ALLOW_SKILLS
+   if(!change_skill(op,SK_THROWING))
+        return 0;
+   else {
+        int success = do_skill(op,op->facing,NULL);
+        return success;
+   }
+#else
   return 0;
+#endif
 }
 
 int command_brace (object *op, char *params)
@@ -1945,21 +1854,6 @@ int command_brace (object *op, char *params)
   return 0;
 }
 
-int command_keyboard_flush (object *op, char *params)
-{
-  if (!params)
-    op->contr->keyboard_flush =!op->contr->keyboard_flush;
-  else
-    op->contr->keyboard_flush =onoff_value(params);
-
-  if(op->contr->keyboard_flush)
-    new_draw_info(NDI_UNIQUE, 0,op, "Keystrokes will not be buffered.");
-  else
-    new_draw_info(NDI_UNIQUE, 0,op, "Keystrokes will be buffered.");
-
-  return 0;
-}
-
 int command_rotatespells (object *op, char *params)
 {
   player *pl=op->contr;
@@ -2057,24 +1951,3 @@ int command_savewinpos (object *op, char *params)
 }
 #endif /* SAVE_WINDOW_POSITIONS */
 
-int checkdm(char *name)
-{
-  FILE  *dmfile;
-  char  buf[MAX_BUF];
-  char  /*log_buf[64],*/ line_buf[160];
-/*  char	tmp[MAX_BUF];*/
-
-  sprintf (buf, "%s/%s", LIBDIR, DMFILE);
-  if ((dmfile = fopen(buf, "r")) == NULL) {
-    LOG (llevDebug, "Could not find DM file.\n");
-    return(0);
-  }
-  while(fgets(line_buf, 160, dmfile) != NULL) {
-    if(!strncmp (name, line_buf, (strlen (line_buf) - 2))) {
-      fclose(dmfile);
-      return (1);
-    }
-  }
-  fclose(dmfile);
-  return (0);
-}
