@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c,v 1.38 1995/09/11 03:16:04 master Exp master $";
+ *   "$Id: login.c,v 1.46 1997/03/09 05:11:32 master Exp master $";
  */
 
 /*
@@ -38,12 +38,12 @@ extern char *uncomp[NROF_COMPRESS_METHODS][3];
 extern char *range_name[range_size];
 extern long pticks;
 
-int penalty;
-
-void emergency_save(int d) {
+/* If flag is non zero, it means that we want to try and save everyone, but
+ * keep the game running.  Thus, we don't want to free any information.
+ */
+void emergency_save(int flag) {
   player *pl;
 #ifndef NO_EMERGENCY_SAVE
-  penalty=d;
   trying_emergency_save = 1;
   if(editor)
     return;
@@ -55,16 +55,19 @@ void emergency_save(int d) {
     }
     LOG(llevError,"%s ",pl->ob->name);
     new_draw_info(NDI_UNIQUE, 0,pl->ob,"Emergency save...");
-/* Mol(mol@meryl.csd.uu.se) 921121 Patch to get players back to a predefined
-   position  i.e. the village.  It's VERY frustrating to start where you
-   emergency saved if there's any monster hanging around.
+
+/* If we are not exiting the game (ie, this is sort of a backup save), then
+ * don't change the location back to the village.  Note that there are other
+ * options to have backup saves be done at the starting village
  */
-    strcpy(pl->maplevel, first_map_path);
-    if(pl->ob->map!=NULL)
-      pl->ob->map = NULL;
-    pl->ob->x = -1;
-    pl->ob->y = -1;
-    if(!save_player(pl->ob,0)) {
+    if (!flag) {
+	strcpy(pl->maplevel, first_map_path);
+	if(pl->ob->map!=NULL)
+	    pl->ob->map = NULL;
+	pl->ob->x = -1;
+	pl->ob->y = -1;
+    }
+    if(!save_player(pl->ob,flag)) {
       LOG(llevError, "(failed) ");
       new_draw_info(NDI_UNIQUE, 0,pl->ob,"Emergency save failed, checking score...");
     }
@@ -81,32 +84,33 @@ void emergency_save(int d) {
    * because they don't have a lot of memory.  The pointer is set
    * to NULL after we are finished to prevent another effort later.
    */
-  for(pl=first_player;pl!=NULL;pl=pl->next) {
-    if(!pl->ob) {
-      LOG(llevError, "No name, ignoring this.\n");
-      continue;
-    }
-    if (pl->pixmaps) {
+  if (!flag) {
+    for(pl=first_player;pl!=NULL;pl=pl->next) {
+      if(!pl->ob) {
+	LOG(llevError, "No name, ignoring this.\n");
+	continue;
+      }
+      if (pl->pixmaps) {
 	free_pixmaps(pl->gdisp, pl->pixmaps);
 	pl->pixmaps=NULL;
-    }
-    if (pl->masks) {
+      }
+      if (pl->masks) {
 	free_pixmaps(pl->gdisp, pl->masks);
 	pl->masks =NULL;
+      }
     }
   }
-
 }
 
 void delete_character(char *name) {
   char buf[MAX_BUF];
 
 #ifdef SAVE_HOMEDIR
-  sprintf(buf,"%s/%s/%s.pl",(char *) getenv("HOME"),PlayerDir,name);
+  sprintf(buf,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,name);
 #else
-  sprintf(buf,"%s/%s/%s.pl",LibDir,PlayerDir,name);
+  sprintf(buf,"%s/%s/%s.pl",settings.libdir,settings.playerdir,name);
 #endif
-  if(unlink(buf)== -1 && debug)
+  if(unlink(buf)== -1 && settings.debug)
     perror("crossfire (delete character)");
 }
 
@@ -117,12 +121,12 @@ void remove_lock(player *pl) {
   if(pl->ob == NULL)
     return;
 #ifdef SAVE_HOMEDIR
-  sprintf(buf, "%s/%s", (char *) getenv("HOME"), PlayerDir);
+  sprintf(buf, "%s/%s", (char *) getenv("HOME"), settings.playerdir);
   (void) create_savedir_if_needed(buf);
   
-  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),PlayerDir,pl->ob->name);
+  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),settings.playerdir,pl->ob->name);
 #else
-  sprintf(buf,"%s/%s/%s.lock",LibDir,PlayerDir,pl->ob->name);
+  sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,pl->ob->name);
 #endif
   if(!rmdir(buf)) {
 #ifdef DEBUG
@@ -137,12 +141,12 @@ int lock_player(char *name) {
   char buf[MAX_BUF];
 
 #ifdef SAVE_HOMEDIR
-  sprintf(buf, "%s/%s", (char *) getenv("HOME"), PlayerDir);
+  sprintf(buf, "%s/%s", (char *) getenv("HOME"), settings.playerdir);
   (void) create_savedir_if_needed(buf);
   
-  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),PlayerDir,name);
+  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),settings.playerdir,name);
 #else
-  sprintf(buf,"%s/%s/%s.lock",LibDir,PlayerDir,name);
+  sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,name);
 #endif
   if(!mkdir(buf,0770))
     return 0;
@@ -187,12 +191,6 @@ int check_name(player *me,char *name) {
   return 1;
 }
 
-#if defined(MACH)
-#ifndef S_ISDIR
-#define S_ISDIR(x) (((x) & S_IFMT) == S_IFDIR)
-#endif
-#endif
-
 int create_savedir_if_needed(char *savedir)
 {
   struct stat *buf;
@@ -223,6 +221,7 @@ void destroy_object (object *op)
     object *tmp;
     while ((tmp = op->inv))
 	destroy_object (tmp);
+
     if (!QUERY_FLAG(op, FLAG_REMOVED))
 	remove_ob(op);
     free_object(op);
@@ -246,13 +245,11 @@ int save_player(object *op, int flag) {
   sint16 backup_x, backup_y;
 #endif
 
-  penalty=flag&2;
-
   flag&=1;
 
   if(QUERY_FLAG(op,FLAG_WAS_WIZ)||!pl->name_changed||(!flag&&!op->stats.exp)) {
     if(!flag) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Your name is not valid,");
+      new_draw_info(NDI_UNIQUE, 0,op,"Your game is not valid,");
       new_draw_info(NDI_UNIQUE, 0,op,"Game not saved.");
     }
     return 0;
@@ -262,17 +259,18 @@ int save_player(object *op, int flag) {
     terminate_all_pets(op);
 
 #ifdef SAVE_HOMEDIR
-  sprintf(filename,"%s/%s", (char *) getenv("HOME"), PlayerDir);
+  sprintf(filename,"%s/%s", (char *) getenv("HOME"), settings.playerdir);
   (void) create_savedir_if_needed(filename);
   
-  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),PlayerDir,op->name);
+  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,op->name);
 #else
-  sprintf(filename,"%s/%s/%s.pl",LibDir,PlayerDir,op->name);
+  sprintf(filename,"%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name);
 #endif
-  tmpfilename = tempnam(TMPDIR,NULL);
+  tmpfilename = tempnam(settings.tmpdir,NULL);
   fp=fopen(tmpfilename, "w");
   if(!fp) {
     new_draw_info(NDI_UNIQUE, 0,op, "Can't open file for save.");
+    LOG(llevDebug,"Can't open file for save.");
     free(tmpfilename);
     return 0;
   }
@@ -289,7 +287,7 @@ int save_player(object *op, int flag) {
     fprintf(fp,"title %s\n",pl->own_title);
 #endif /* SET_TITLE */
 #ifdef SAVE_WINDOW_POSITIONS
-  if(pl->valid_save_positions && pl->split_window) {
+  if(pl->valid_save_positions) {
     fprintf(fp,"win_pos");
     for(i=0;i<6;i++)
       fprintf(fp," %d %d %u %u",
@@ -303,6 +301,7 @@ int save_player(object *op, int flag) {
 #endif
   fprintf(fp,"gen_hp %d\n",pl->gen_hp);
   fprintf(fp,"gen_sp %d\n",pl->gen_sp);
+  fprintf(fp,"gen_grace %d\n",pl->gen_grace);
   fprintf(fp,"listening %d\n",pl->listening);
   fprintf(fp,"spell %d\n",pl->chosen_spell);
   fprintf(fp,"shoottype %d\n",pl->shoottype);
@@ -312,8 +311,11 @@ int save_player(object *op, int flag) {
   fprintf(fp,"digestion %d\n",pl->digestion);
   fprintf(fp,"pickup %d\n", pl->mode);
   fprintf(fp,"keyboard_flush %d\n", pl->keyboard_flush);
+  fprintf(fp,"show_inv_icon %d\n", pl->show_inv_icon);
   fprintf(fp,"outputs_sync %d\n", pl->outputs_sync);
   fprintf(fp,"outputs_count %d\n", pl->outputs_count);
+  fprintf(fp,"sort_inventory %d\n", pl->sort_inventory);
+  fprintf(fp,"show_buttons %d\n", pl->show_buttons);
 
 #ifdef BACKUP_SAVE_AT_HOME
   if (op->map!=NULL && flag==0)
@@ -328,6 +330,7 @@ int save_player(object *op, int flag) {
   fprintf(fp,"Dex %d\n",pl->orig_stats.Dex);
   fprintf(fp,"Con %d\n",pl->orig_stats.Con);
   fprintf(fp,"Int %d\n",pl->orig_stats.Int);
+  fprintf(fp,"Pow %d\n",pl->orig_stats.Pow);
   fprintf(fp,"Wis %d\n",pl->orig_stats.Wis);
   fprintf(fp,"Cha %d\n",pl->orig_stats.Cha);
 
@@ -337,6 +340,7 @@ int save_player(object *op, int flag) {
   for(i=1;i<=pl->last_level&&i<=10;i++) {
     fprintf(fp,"%d\n",pl->levhp[i]);
     fprintf(fp,"%d\n",pl->levsp[i]);
+	 fprintf(fp,"%d\n",pl->levgrace[i]);
   }
   for(i=0;i<pl->nrofknownspells;i++)
     fprintf(fp,"known_spell %s\n",spells[pl->known_spells[i]].name);
@@ -470,16 +474,15 @@ void check_login(object *op) {
   FILE *fp;
   char filename[MAX_BUF];
   char buf[MAX_BUF],bufall[MAX_BUF];
-  object *tmp;
   int i,value,x,y,comp;
   long checksum = 0;
   player *pl = op->contr;
 
   strcpy (pl->maplevel, first_map_path);
 #ifdef SAVE_HOMEDIR
-  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),PlayerDir,op->name);
+  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,op->name);
 #else
-  sprintf(filename,"%s/%s/%s.pl",LibDir,PlayerDir,op->name);
+  sprintf(filename,"%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name);
 #endif
   if ((fp=open_and_uncompress(filename,1,&comp)) == NULL) {
     confirm_password(op);
@@ -527,6 +530,7 @@ void check_login(object *op) {
       pl->orig_stats.Dex=0;
       pl->orig_stats.Con=0;
       pl->orig_stats.Int=0;
+      pl->orig_stats.Pow=0;
       pl->orig_stats.Wis=0;
       pl->orig_stats.Cha=0;
       load_default_keys(op->contr);
@@ -535,11 +539,11 @@ void check_login(object *op) {
         if (!strcmp(buf,"endplst"))
           break;
 #ifdef SET_TITLE
-	else if (!strncmp(bufall,"title",6))
+	else if (!strcmp(bufall,"title"))
           sscanf(bufall,"title %s[^\n]",pl->own_title);
 #endif /* SET_TITLE */
 #ifdef SAVE_WINDOW_POSITIONS
-	else if (!strcmp(buf,"win_pos") && pl->split_window) {
+	else if (!strcmp(buf,"win_pos")) {
 	  pl->valid_save_positions=
 	    (sscanf(bufall,
 		    "win_pos %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u",
@@ -564,9 +568,11 @@ void check_login(object *op) {
         else if (!strcmp(buf,"gen_hp"))
           pl->gen_hp=value;
         else if (!strcmp(buf,"shoottype"))
-          pl->shoottype=value;
+          pl->shoottype=(rangetype)value;
         else if (!strcmp(buf,"gen_sp"))
           pl->gen_sp=value;
+        else if (!strcmp(buf,"gen_grace"))
+          pl->gen_grace=value;
         else if (!strcmp(buf,"spell"))
           pl->chosen_spell=value;
         else if (!strcmp(buf,"listening"))
@@ -583,6 +589,17 @@ void check_login(object *op) {
 	  pl->mode=value;
         else if (!strcmp(buf,"keyboard_flush"))
 	  pl->keyboard_flush = value;
+	else if (!strcmp(buf,"sort_inventory"))
+	  pl->sort_inventory = value;
+	else if (!strcmp(buf,"show_buttons"))
+	  pl->show_buttons = value;
+        else if (!strcmp(buf,"show_inv_icon")) {
+	  pl->show_inv_icon = value;
+	  if (pl->show_inv_icon)
+	    strcpy(pl->format_inv,"%-20.20s%-6s"); /* This can be changed by resize */
+	  else
+	    strcpy(pl->format_inv,"%-24.24s%-6s"); /* This can be changed by resize */
+	}
 	else if (!strcmp(buf,"outputs_sync"))
 	  pl->outputs_sync = value;
 	else if (!strcmp(buf,"outputs_count"))
@@ -599,6 +616,8 @@ void check_login(object *op) {
           pl->orig_stats.Con=value;
         else if (!strcmp(buf,"Int"))
           pl->orig_stats.Int=value;
+        else if (!strcmp(buf,"Pow"))
+          pl->orig_stats.Pow=value;
         else if (!strcmp(buf,"Wis"))
           pl->orig_stats.Wis=value;
         else if (!strcmp(buf,"Cha"))
@@ -610,6 +629,8 @@ void check_login(object *op) {
             pl->levhp[i]=j;
             fscanf(fp,"%d\n",&j);
             pl->levsp[i]=j;
+            fscanf(fp,"%d\n",&j);
+            pl->levgrace[i]=j;
           }
         } else if (!strcmp(buf,"spell_array")) {
           pl->nrofknownspells=value;
@@ -647,6 +668,9 @@ void check_login(object *op) {
       /* sigh */
       free_object (op);
       op = pl->ob = LoadObject (fp, NULL);
+      /* Somehow, the game worked before without this line - relied
+       * on getting the object back that we just freed? */
+      op->contr = pl;
       CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
 
       x=op->x; y=op->y;
@@ -672,6 +696,7 @@ void check_login(object *op) {
       }
 
       enter_exit(op,NULL); /* This won't insert the player any longer! */
+
       if (op->stats.hp<0||op->stats.food<0) {
 /* IF NOT_PERMADEATH is set, this should really do something different,
  * like bring the character back to life under the normal rules.
@@ -681,7 +706,7 @@ void check_login(object *op) {
         new_draw_info(NDI_UNIQUE, 0,op," ");
 	play_again(op);
         delete_character(op->name);
-      } else {
+      } else { /* character is alive */
         pl->name_changed=1;
         pl->state = ST_PLAYING;
 #ifdef AUTOSAVE
@@ -689,7 +714,6 @@ void check_login(object *op) {
 #endif
 	op->carrying=0;
 	pl->freeze_inv=1;
-	tmp=get_object();
 
 	pl->freeze_inv=0;
 	if (pl->eric_server > 0) {
@@ -710,6 +734,11 @@ void check_login(object *op) {
         fix_player(op);
 	fix_weight ();
 
+#ifdef ALLOW_SKILLS
+	(void) init_player_exp(op);
+        (void) link_player_skills(op);
+#endif
+
         if(!pl->split_window && pl->eric_server == 0)
           XClearWindow(pl->gdisp,pl->win_root);
         draw_all_info(op);
@@ -729,8 +758,8 @@ void check_login(object *op) {
         close_and_delete(fp, comp);
         LOG(llevDebug,"Checksums: %x %x\n",
                 checksum,calculate_checksum(filename,1));
-#ifdef CHECKSUM_ENABLED
-        if(calculate_checksum(filename) != checksum) {
+#ifdef ENABLE_CHECKSUM
+        if(calculate_checksum(filename,1) != checksum) {
           new_draw_info(NDI_UNIQUE, 0,op,"Since your savefile has been tampered with,");
           new_draw_info(NDI_UNIQUE, 0,op,"you will not be able to save again.");
           set_cheat(op);
@@ -778,6 +807,10 @@ void check_login(object *op) {
     xwc.x=pl->win_pos[5].x;xwc.y=pl->win_pos[5].y;
     xwc.width=pl->win_pos[5].w;xwc.height=pl->win_pos[5].h;
     XConfigureWindow(pl->gdisp,pl->win_message,xwc_mask,&xwc);
+    sprintf(pl->format_inv,"%%-%d.%ds%%-6s",pl->inv_chars-6,
+          pl->inv_chars-6);
+    draw_all_inventory(pl->ob);
+
   }
     
 #endif /* SAVE_WINDOW_POSITIONS */
