@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_main_c =
- *    "$Id: main.c,v 1.35 1995/07/24 22:15:00 master Exp $";
+ *    "$Id: main.c,v 1.41 1997/03/09 05:11:52 master Exp master $";
  */
 
 /*
@@ -258,6 +258,7 @@ void enter_exit(object *op, object *exit_ob) {
   op->contr->count=0;
   op->contr->count_left=0;
 
+  /* For exits that cause damages (like pits) */
   if(exit_ob && exit_ob->stats.dam && op->type==PLAYER)
        hit_player(op,exit_ob->stats.dam,exit_ob,exit_ob->attacktype);
 
@@ -379,6 +380,9 @@ void enter_exit(object *op, object *exit_ob) {
     LOG(llevDebug, "Players map is still loading!?\n");
     return; /* This will block the player until map has been loaded */
   }
+#ifdef USE_LIGHTING
+  if(op->lights||op->glow_radius>0) remove_carried_lights(op,m);
+#endif
   enter_map(op);
 }
 
@@ -398,6 +402,12 @@ void enter_map(object *op) {
 			   EXIT_Y(op->map->map_object),1,9);
     op->x = EXIT_X(op->map->map_object) + freearr_x[i];
     op->y = EXIT_Y(op->map->map_object) + freearr_y[i];
+    if (out_of_map (op->map, op->x, op->y)) {
+	LOG (llevDebug, "Broken map: Start position is out of map (%s)!\n",
+	     op->map->path);
+	op->x = 0;
+	op->y = 0;
+    }
   }
   if(!op->contr->removed) {
     SET_FLAG(op, FLAG_NO_APPLY);
@@ -411,6 +421,9 @@ void enter_map(object *op) {
    * draw_colorpix routine cannot find out, if the background of player has
    * changed, and would not redraw player...
    */
+#endif
+#ifdef USE_LIGHTING
+  if(op->lights) add_carried_lights(op);
 #endif
   enter_pending_objects(op->map);
 }
@@ -507,111 +520,104 @@ void process_map(mapstruct *map) {
 
 void process_players1(mapstruct *map)
 {
-  int flag;
-  player *pl,*plnext;
+    int flag;
+    player *pl,*plnext;
 
-  for(flag=1;flag!=0;) {
-    flag=0;
-    for(pl=first_player;pl!=NULL;pl=plnext) {
+    /* Basically, we keep looping until all the players have done their actions. */
+    for(flag=1;flag!=0;) {
+	flag=0;
+	for(pl=first_player;pl!=NULL;pl=plnext) {
 
 
-      plnext=pl->next; /* In case a player exits the game in handle_player() */
-      if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
+	    plnext=pl->next; /* In case a player exits the game in handle_player() */
+	    if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
 
 #ifdef AUTOSAVE
-      /* check for ST_PLAYING state so that we don't try to save off when
-       * the player is logging in.
-       */
-      if ((pl->last_save_tick+AUTOSAVE)<pticks && pl->state==ST_PLAYING) {
-	save_player(pl->ob,1);
-	pl->last_save_tick = pticks;
-      }
+	    /* check for ST_PLAYING state so that we don't try to save off when
+	     * the player is logging in.
+	     */
+	    if ((pl->last_save_tick+AUTOSAVE)<pticks && pl->state==ST_PLAYING) {
+		save_player(pl->ob,1);
+		pl->last_save_tick = pticks;
+	    }
 #endif
-      if(pl->ob->speed_left>0) {
-	  if (pl->eric_server>0) {
-		if (handle_newcs_player(pl->ob))
-		    flag=1;
-	  }
-	  else {
-	    if(--(pl->ob->speed_left)>0) /* Another loop needed */
-		flag=1;
-	    handle_player(pl->ob);
-	  }
-      }
-    }
-  }
-  for(pl=first_player;pl!=NULL;pl=pl->next) {
-    if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
+	    if(pl->ob->speed_left>0) {
+		if (pl->eric_server>0) {
+		    if (handle_newcs_player(pl->ob))
+			flag=1;
+		}
+		else {
+		    if(--(pl->ob->speed_left)>0) /* Another loop needed */
+			flag=1;
+		    handle_player(pl->ob);
+		}
+	    } /* end if player has speed left */
+	} /* end of for loop for all the players */
+    } /* for flag */
+    for(pl=first_player;pl!=NULL;pl=pl->next) {
+	if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
 #ifdef CASTING_TIME
-    if (pl->ob->casting > 0){
-      pl->ob->casting--;
-      pl->ob->start_holding = 1;
-    }
-    /* set spell_state so we can update the range in stats field */ 
-    if ((pl->ob->casting == 0) && (pl->ob->start_holding ==1)){
-      pl->ob->start_holding = 0;
-      pl->ob->spell_state = 1;
-    }
+	if (pl->ob->casting > 0){
+	    pl->ob->casting--;
+	    pl->ob->start_holding = 1;
+	}
+	/* set spell_state so we can update the range in stats field */ 
+	if ((pl->ob->casting == 0) && (pl->ob->start_holding ==1)){
+	    pl->ob->start_holding = 0;
+	    pl->ob->spell_state = 1;
+	}
 #endif
-    do_some_living(pl->ob);
-    draw(pl->ob);
-  }
+	do_some_living(pl->ob);
+	draw(pl->ob);
+    }
 }
 
 void process_players2(mapstruct *map)
 {
-  player *pl;
+    player *pl;
 
-/* Then check if any players should use weapon-speed instead of speed */
-  for(pl=first_player;pl!=NULL;pl=pl->next) {
-    if (map!=NULL) {
-      if(pl->ob == NULL || QUERY_FLAG(pl->ob,FLAG_REMOVED))
-        continue;
-      else if(pl->loading != NULL) /* Player is blocked */
-        pl->ob->speed_left -= pl->ob->speed;
-      if (pl->ob->map!=map) continue;
-    }
+    /* Then check if any players should use weapon-speed instead of speed */
+    for(pl=first_player;pl!=NULL;pl=pl->next) {
+	if (map!=NULL) {
+	    if(pl->ob == NULL || QUERY_FLAG(pl->ob,FLAG_REMOVED))
+		continue;
+	    else if(pl->loading != NULL) /* Player is blocked */
+		pl->ob->speed_left -= pl->ob->speed;
+	    if (pl->ob->map!=map) continue;
+	}
 
 #ifdef SOUND_EFFECTS
-    if (pl->play_count > 0)
-      pl->play_count--;
+	if (pl->play_count > 0)
+	    pl->play_count--;
 #endif
-    if(pl->has_hit) {
-      float hit_speed = pl->ob->speed/pl->weapon_sp - pl->ob->speed;
+	if(pl->has_hit) {
+	    float hit_speed = pl->ob->speed/pl->weapon_sp - pl->ob->speed;
 
-      if (hit_speed<pl->ob->speed) hit_speed = pl->ob->speed;
+	    if (hit_speed<pl->ob->speed) hit_speed = pl->ob->speed;
 
-      pl->ob->speed_left+=hit_speed;
-      /* This could happen if the player still has left over speed */
-      if (pl->ob->speed_left>hit_speed) pl->ob->speed_left=hit_speed;
+	    pl->ob->speed_left+=hit_speed;
+	    /* This could happen if the player still has left over speed */
+	    if (pl->ob->speed_left>hit_speed) pl->ob->speed_left=hit_speed;
+	}
+	else if (pl->ob->speed_left>pl->ob->speed)
+	    pl->ob->speed_left = pl->ob->speed;
     }
-    else if (pl->ob->speed_left>pl->ob->speed)
-	pl->ob->speed_left = pl->ob->speed;
-  }
 
 /* Then synchronize the different screens to avoid jerky movement */
-  for(pl=first_player;pl!=NULL;pl=pl->next) {
-    if (pl->eric_server > 0) {
-      esrv_bar(pl->eric_server,"sync");
-      continue;
+    for(pl=first_player;pl!=NULL;pl=pl->next) {
+	if (pl->eric_server > 0) {
+	    continue;
+	}
+	if (pl->sync && ++pl->cur_sync >= pl->sync) {
+	    XSync(pl->gdisp,False);
+	    pl->cur_sync = 0;
+	}
+	else
+	    XFlush(pl->gdisp);
     }
-    if (pl->sync && ++pl->cur_sync >= pl->sync) {
-      XSync(pl->gdisp,False);
-      pl->cur_sync = 0;
-    }
-    else
-      XFlush(pl->gdisp);
-  }
 }
 
 #define SPEED_DEBUG
-#define NEW_PROCESS_EVENTS
-/*
- * Withe NEW_PROCESS_EVENTS, this function no longer hogs as much
- * cpu time as it did.
- * Extended to take a map arguement.  This way, process_map is no
- * longer needed.
- */
 
 
 void process_events(mapstruct *map) {
@@ -619,22 +625,20 @@ void process_events(mapstruct *map) {
 
  process_players1(map);
 
-#ifdef NEW_PROCESS_EVENTS
  for (op=active_objects; op!=NULL; op=next) {
-#else
- for(op=objects;op!=NULL;op=next) {
-#endif
     if(QUERY_FLAG(op, FLAG_FREED)) { /* Was this object somehow freed as a result of */
                        /* the actions of the previous object? */ 
       LOG(llevDebug,"Free object on list, process_events\n");
+      next=op->active_next;
+	continue; /* If we have a free object, any values it might
+		   * have are likely garbage.  breaking is probabyl
+		   * best, but let as at least continue and hope
+		   * for the best
+		   */
 /*      break;*/
     }
 
-#ifdef NEW_PROCESS_EVENTS
     next=op->active_next;
-#else
-    next=op->next;
-#endif
 
     if (op->map == NULL && op->env == NULL && op->name &&
         op->type!=MAP && map==NULL)
@@ -648,20 +652,14 @@ void process_events(mapstruct *map) {
 	continue;
 
     if(!op->speed) {
-#ifdef NEW_PROCESS_EVENTS
       /* This is not a real problem, unless it repeats for the the same
        * object.  IF it just gets printed once, it likely means that that
        * object was killed/changed by the previous object
        */
       LOG(llevDebug, "Object %s has no speed, but is on active list\n",
 	op->arch->name);
-#endif
       continue;
     }
-#if defined(SPEED_DEBUG) && !defined(NEW_PROCESS_EVENTS)
-    if (op->active_prev==NULL && op->active_next==NULL)
-	LOG(llevDebug, "Object %s has speed, but not on active list\n", op->arch->name);
-#endif
 
 /* Eneq(@csd.uu.se): Handle archetype-field anim_speed differently when
    it comes to the animation. If we have a value on this we don't animate it
@@ -694,13 +692,36 @@ void clean_tmp_files() {
   LOG(llevError,"Cleaning up...\n");
   for(pl=first_player;pl!=NULL;pl=pl->next)
     remove_lock(pl);
-  for(m=first_map;m!=NULL;m=m->next)
+
+  /* We save the maps - it may not be intuitive why, but if there are unique
+   * items, we need to save the map so they get saved off.  Perhaps we should
+   * just make a special function that only saves the unique items.
+   */
+  for(m=first_map;m!=NULL;m=m->next) {
+    if (m->in_memory == MAP_IN_MEMORY)
+/* If we want to reuse the temp maps, swap it out (note that will also
+ * update the log file.  Otherwise, save the map (mostly for unique item
+ * stuff).  Note that the clean_tmp_map is called after the end of
+ * the for loop but is in the #else bracket.  IF we are recycling the maps,
+ * we certainly don't want the temp maps removed.
+ */
+#ifdef RECYCLE_TMP_MAPS
+	swap_map(m);
+#else
+	new_save_map(m, 0);
     clean_tmp_map(m);
+#endif
+  }
+/* I don't think we should care about lockfiles being cleaned up
+ * if recycling the temp maps.  Not sure, however.
+ */
 #if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
   clean_lockfiles();
 #endif
 }
 
+
+
 void leave(player *pl) {
   char buf[MAX_BUF];
 
@@ -748,10 +769,21 @@ void leave(player *pl) {
   pl->ob->type = DEAD_OBJECT; /* To avoid problems with inventory window */
   free_player(pl);
 #ifdef SERVER
-  if(server_mode != SERVER_ENABLED)
+  if(settings.servermode != SERVER_ENABLED)
 #endif
   if(first_player==NULL) { /* Last player left the game */
     clean_tmp_files();
+    write_book_archive();
+    free_all_object_data();
+    free_all_archs();
+    free_all_treasures();
+    free_all_images();
+    free_all_ericserver();
+    free_all_recipes();
+    free_all_readable();
+    free_all_god();
+    /* See what the string data that is out there that hasn't been freed. */
+/*    LOG(llevDebug, ss_dump_table(0xff));*/
     exit(0);
   }
   new_draw_info(NDI_UNIQUE | NDI_ALL, 5, NULL, buf);
@@ -769,7 +801,7 @@ int forbid_play()
     clock = time (NULL);
     tm = (struct tm *) localtime (&clock);
 
-    sprintf (buf, "%s/%s", LibDir, PERM_FILE);
+    sprintf (buf, "%s/%s", settings.libdir, PERM_FILE);
     if ((fp = open_and_uncompress(buf, 0, &comp)) == NULL)
 	return 0;
 
@@ -826,6 +858,8 @@ void do_specials() {
     if (!(special2%25))
       fix_luck();
   }
+  if (!(special2%10)) write_book_archive();
+
 }
 
 /*
@@ -833,21 +867,19 @@ void do_specials() {
  * We don't need to know the timezone since we're only interested in
  * the delta time since the last 'tick' .
  */
-struct timeval last_time;
+extern struct timeval last_time;
 struct timezone dummy_timezone;
 
-int main(int argc,char **argv,char **env) {
-
- /*
-  * Set up some global pointers, to make things easy:
-  */
-  gargc=argc, gargv=argv, genv=env;
+int main(int argc,char **argv)
+{
 
 #ifdef DEBUG_MALLOC_LEVEL
   malloc_debug(DEBUG_MALLOC_LEVEL);
 #endif
 
-  init();
+  settings.argc=argc;
+  settings.argv=argv;
+  init(argc, argv);
 
   for(;;) {
     nroferrors = 0;
