@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c,v 1.23 1995/07/24 22:15:14 master Exp $";
+ *    "$Id: monster.c,v 1.32 1997/03/09 05:12:29 master Exp master $";
  */
 
 /*
@@ -29,8 +29,12 @@
 #include <global.h>
 #ifndef __CEXTRACT__
 #include <sproto.h>
+#include <spells.h>
+#include <skills.h>
 #endif
 
+#define MIN_MON_RADIUS 3 /* minimum monster detection radius */ 
+
 extern spell spells[NROFREALSPELLS];
 
 object *get_enemy(object *npc) {
@@ -46,37 +50,52 @@ object *get_enemy(object *npc) {
        npc->enemy->map!=npc->map)
       npc->enemy=NULL;
   }
-  return npc->enemy;
+  return can_detect_enemy(npc,npc->enemy)?npc->enemy:NULL;
 }
 
 object *find_enemy(object *npc) {
-  object *tmp;
+  object *tmp=NULL;
+
   if ((npc->move_type & HI4) == PETMOVE)
     return get_pet_enemy(npc);
-  if((tmp=get_enemy(npc))!=NULL)
-    return tmp;
-  if(QUERY_FLAG(npc, FLAG_UNAGGRESSIVE))
-    return NULL;
-  tmp = get_nearest_player(npc);
-  if(QUERY_FLAG(npc, FLAG_FRIENDLY)) {
-    object *op;
-    if(tmp == NULL)
-      return NULL;
-    if((op = get_enemy(tmp))!=NULL)
-      return op;
-    return NULL;
-  }
+  if((tmp=get_enemy(npc))==NULL)
+    if(!QUERY_FLAG(npc, FLAG_UNAGGRESSIVE)) {
+      tmp = get_nearest_player(npc);
+      if(QUERY_FLAG(npc, FLAG_FRIENDLY)&&tmp)
+        tmp = get_enemy(tmp);
+    }
   return tmp;
 }
 
-int check_wakeup(object *op) {
+int check_wakeup(object *op, object *enemy) {
+  int radius = op->stats.Wis>MIN_MON_RADIUS?op->stats.Wis:MIN_MON_RADIUS;
   objectlink *ol;
+
+  /* blinded monsters can only find nearby objects to attack */
+  if(QUERY_FLAG(op, FLAG_BLIND) && !QUERY_FLAG(op, FLAG_SEE_INVISIBLE)) 
+	radius = MIN_MON_RADIUS;
+#ifdef USE_LIGHTING
+  /* This covers the situation where the monster is in the dark 
+   * and has an enemy. If the enemy has no carried light (or isnt 
+   * glowing!) then the monster has trouble finding the enemy. 
+   * Remember we already checked to see if the monster can see in 
+   * the dark. */
+  else if(op->map&&op->map->darkness>0&&enemy&&!enemy->invisible&&
+    !stand_in_light(enemy)&&(!QUERY_FLAG(op,FLAG_SEE_IN_DARK)||
+    !QUERY_FLAG(op,FLAG_SEE_INVISIBLE))) 
+  {
+	  int dark = radius/(op->map->darkness);
+	  radius = (dark>MIN_MON_RADIUS)?(dark+1):MIN_MON_RADIUS;
+  } 
+#endif
+  else if(!QUERY_FLAG(op,FLAG_SLEEP)) return 1;
+
   for(ol=first_friendly_object;ol!=NULL;ol=ol->next)
     if(ol->ob->map==op->map&&
-       (QUERY_FLAG(ol->ob,FLAG_STEALTH)?(abs(ol->ob->x-op->x)<op->stats.Wis/2+1 &&
-                         abs(ol->ob->y-op->y)<op->stats.Wis/2+1):
-                        (abs(ol->ob->x-op->x)<op->stats.Wis&&
-                         abs(ol->ob->y-op->y)<op->stats.Wis)))
+       (QUERY_FLAG(ol->ob,FLAG_STEALTH)?(abs(ol->ob->x-op->x)<radius/2+1 &&
+                         abs(ol->ob->y-op->y)<radius/2+1):
+                        (abs(ol->ob->x-op->x)<radius&&
+                         abs(ol->ob->y-op->y)<radius)))
     {
       CLEAR_FLAG(op,FLAG_SLEEP);
       return 1;
@@ -98,22 +117,42 @@ int move_randomly(object *op) {
  */
 
 int move_monster(object *op) {
-  int dir,diff;
-  object *enemy, *part, *owner;
+  int dir=0,diff;
+  object *part, *owner, *enemy = find_enemy(op);
 
-  if(QUERY_FLAG(op, FLAG_SLEEP)) {
-    if(!check_wakeup(op))
+
+  /* Monsters not on maps don't do anything.  These monsters are thins
+   * Like royal guards in city dwellers inventories.
+   */
+  if (!op->map) return 0;
+
+  if(QUERY_FLAG(op, FLAG_SLEEP)||QUERY_FLAG(op, FLAG_BLIND)
+#ifdef USE_LIGHTING  
+    ||((op->map->darkness>0)&&!QUERY_FLAG(op,FLAG_SEE_IN_DARK)
+	&&!QUERY_FLAG(op,FLAG_SEE_INVISIBLE))
+#endif
+  ) {
+    if(!check_wakeup(op,enemy))
       return 0;
   }
 
+  /* check if monster pops out of hidden spot */
+  if(op->hide) do_hidden_move(op);
+
   if(op->pick_up)
     monster_check_pickup(op);
 
   if(op->will_apply)
     monster_apply_below(op); /* Check for items to apply below */
-
+  /*  generate hp, if applicable */
   if(op->stats.Con&&op->stats.hp<op->stats.maxhp) {
-    op->stats.hp+=op->stats.Con;
+    /*  last heal is in funny units.  Dividing by speed puts
+        the regeneration rate on a basis of time instead of
+        #moves the monster makes.  The scaling by 8 is
+        to capture 8th's of a hp fraction regens */
+	 op->last_heal+= (int)((float)(8*op->stats.Con)/FABS(op->speed));
+    op->stats.hp+=op->last_heal/32; /* causes Con/4 hp/tick */
+    op->last_heal%=32;
     if (QUERY_FLAG(op,FLAG_RUN_AWAY) &&
         op->stats.hp >= (signed short)(((float)op->run_away/(float)100)*
                         (float)op->stats.maxhp))
@@ -121,17 +160,30 @@ int move_monster(object *op) {
     if(op->stats.hp>op->stats.maxhp)
       op->stats.hp=op->stats.maxhp;
   }
+  /* generate sp, if applicable */
+  if(op->stats.Pow&&op->stats.sp<op->stats.maxsp) {
+    /*  last_sp is in funny units.  Dividing by speed puts
+        the regeneration rate on a basis of time instead of
+        #moves the monster makes.  The scaling by 8 is
+        to capture 8th's of a sp fraction regens */
+	 op->last_sp+= (int)((float)(8*op->stats.Pow)/FABS(op->speed));
+    op->stats.sp+=op->last_sp/128;  /* causes Pow/16 sp/tick */
+    op->last_sp%=128;
+    if(op->stats.sp>op->stats.maxsp)
+      op->stats.sp=op->stats.maxsp;
+  }
+
   if(QUERY_FLAG(op, FLAG_SCARED)&&!(RANDOM()%20))
     CLEAR_FLAG(op,FLAG_SCARED); /* Time to regain some "guts"... */
 
-  if((enemy = find_enemy(op)) == NULL) {
+  if(!enemy) {
     if(QUERY_FLAG(op, FLAG_ONLY_ATTACK)) {
       remove_ob(op);
       free_object(op);
       return 1;
     }
     if (op->move_type & HI4) {
-      switch (op->move_type) {
+      switch (op->move_type & HI4) {
         case (PETMOVE):
           pet_move (op);
           if(QUERY_FLAG(op, FLAG_REMOVED)) {
@@ -194,12 +246,18 @@ int move_monster(object *op) {
     strcpy(op->name,enemy->name);
   }
 
+#define NEW_MON_ATTACK
+#ifndef NEW_MON_ATTACK
   part = get_nearest_part(op,enemy);
-
+#else
+  for (part=op; part!=NULL; part=part->more) {
+#endif
   dir=find_dir_2(part->x-enemy->x,part->y-enemy->y);
+
   if(QUERY_FLAG(op,FLAG_IS_TURNING))
     op->value=(enemy->x>op->x);
 
+
   if(QUERY_FLAG(op, FLAG_SCARED) || QUERY_FLAG(op,FLAG_RUN_AWAY))
     dir=absdir(dir+4);
 
@@ -219,15 +277,19 @@ int move_monster(object *op) {
     if(QUERY_FLAG(op,FLAG_READY_HORN)&&!(RANDOM()%5))
       if(monster_use_horn(op,part,enemy,dir))
         return 0;
+#ifdef ALLOW_SKILLS
+    if(QUERY_FLAG(op,FLAG_READY_SKILL)&&!(RANDOM()%3))
+      if(monster_use_skill(op,part,enemy,dir))
+	return 0;
+#endif
     if(QUERY_FLAG(op,FLAG_READY_BOW)&&!(RANDOM()%2))
       if(monster_use_bow(op,part,enemy,dir))
         return 0;
-    if(QUERY_FLAG(op,FLAG_READY_SKILL)&&!(RANDOM()%3))
-#ifdef ALLOW_SKILLS
-      if(monster_use_skill(op,part,enemy,dir))
-#endif
-	return 0;
   }
+#ifdef NEW_MON_ATTACK
+  }
+  part = get_nearest_part(op,enemy);
+#endif
   if ((op->move_type & LO4) && !QUERY_FLAG(op, FLAG_SCARED)) {
      switch (op->move_type & LO4) {
      case DISTATT:
@@ -302,10 +364,18 @@ int move_monster(object *op) {
     {
       signed char tmp = (signed char)((float)part->stats.wc*(float)2);
       part->stats.wc+=tmp;
+#ifdef ALLOW_SKILLS
+      (void)skill_attack(enemy,part,0,NULL);
+#else
       (void)attack_ob(enemy,part);
+#endif
       part->stats.wc-=tmp;
     } else
+#ifdef ALLOW_SKILLS
+      (void)skill_attack(enemy,part,0,NULL);
+#else
       (void) attack_ob(enemy,part);
+#endif
   }
   if(QUERY_FLAG(part,FLAG_FREED))    /* Might be freed by ghost-attack or hit-back */
     return 1;
@@ -330,21 +400,27 @@ int can_apply(object *who,object *item) {
 
 #define MAX_KNOWN_SPELLS 20
 
-object *choose_random_spell(object *monster) {
+object *monster_choose_random_spell(object *monster) {
   object *altern[MAX_KNOWN_SPELLS];
   object *tmp;
+  spell *spell;
   int i=0,j;
 
   for(tmp=monster->inv;tmp!=NULL;tmp=tmp->below)
     if(tmp->type==ABILITY||tmp->type==SPELLBOOK) {
-      if(tmp->stats.maxsp)
-        for(j=0;i<MAX_KNOWN_SPELLS&&j<tmp->stats.maxsp;j++)
-          altern[i++]=tmp;
-      else
-        altern[i++]=tmp;
-      if(i==MAX_KNOWN_SPELLS)
-        break;
-    }
+	 /*  Check and see if it's actually a useful spell */
+		if((spell=find_spell(tmp->stats.sp))!=NULL 
+			&&!(spell->path&(PATH_INFO|PATH_TRANSMUTE|PATH_TRANSFER|PATH_LIGHT))) {
+			 if(tmp->stats.maxsp)
+				for(j=0;i<MAX_KNOWN_SPELLS&&j<tmp->stats.maxsp;j++)
+				  altern[i++]=tmp;
+			 else
+				altern[i++]=tmp;
+			 if(i==MAX_KNOWN_SPELLS)
+				break;
+		}
+	
+	 }
   if(!i)
     return NULL;
   return altern[RANDOM()%i];
@@ -356,10 +432,9 @@ int monster_cast_spell(object *head, object *part,object *pl,int dir) {
   int sp_typ, ability;
   object *owner;
 
-  if(head->stats.sp<head->stats.maxsp) /* Generate spell-points */
-    head->stats.sp+=head->stats.Int;
   if(!(RANDOM()%3)) /* Don't want to cast spells so often */
     return 0;
+
   if(!(dir=path_to_player(part,pl,0)))
     return 0;
   if(QUERY_FLAG(head,FLAG_FRIENDLY) && (owner = get_owner(head)) != NULL) {
@@ -369,11 +444,19 @@ int monster_cast_spell(object *head, object *part,object *pl,int dir) {
   }
   if(QUERY_FLAG(head,FLAG_CONFUSED))
     dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
-  if((spell_item=choose_random_spell(head))==NULL) {
-    LOG(llevMonster,"Turned off spells in %s\n",head->name);
-    CLEAR_FLAG(head, FLAG_CAST_SPELL); /* Will be turned on when picking up books */
-    return 0;
+
+  /*  If the monster hasn't already chosen a spell, choose one */
+  if(head->spellitem==NULL) {
+    if((spell_item=monster_choose_random_spell(head))==NULL) {
+      LOG(llevMonster,"Turned off spells in %s\n",head->name);
+      CLEAR_FLAG(head, FLAG_CAST_SPELL); /* Will be turned on when picking up book
+s */
+      return 0;
+    }
   }
+  else
+    spell_item=head->spellitem; 
+
   if(spell_item->stats.hp) {
     /* Alternate long-range spell: check how far away enemy is */
     if(isqrt(distance(part,pl))>6)
@@ -388,15 +471,32 @@ int monster_cast_spell(object *head, object *part,object *pl,int dir) {
   }
   if (sp->onself) /* Spell should be cast on caster (ie, heal, strength) */
     dir = 0;
-  if(head->stats.sp<sp->sp) /* Monster doesn't have enough spell-points */
+  
+  if(head->stats.sp<SP_level_spellpoint_cost(head,head,sp_typ))
+   /* Monster doesn't have enough spell-points */
     return 0;
-  head->stats.sp-=sp->sp;
   ability = (spell_item->type==ABILITY && !(spell_item->attacktype&AT_MAGIC));
+
+
+/* If we cast a spell, only use up casting_time speed */
+  head->speed_left+=1.0 - spells[sp_typ].time/20.0*FABS(head->speed); 
+
+  head->stats.sp-=SP_level_spellpoint_cost(head,head,sp_typ);
+  /* choose the spell the monster will cast next */
+  /* choose the next spell to cast */
+  head->spellitem = monster_choose_random_spell(head);  
   return cast_spell(part,part,dir,sp_typ,ability, spellNormal,NULL);
 }
 
-/* implemented 95-04-28 to allow monster skill use - bt. */
 #ifdef ALLOW_SKILLS
+/* monster_use_skill()-implemented 95-04-28 to allow monster skill use.
+ * Note that monsters do not need the skills SK_MELEE_WEAPON and
+ * SK_MISSILE_WEAPON to make those respective attacks, if we 
+ * required that we would drastically increase the memory
+ * requirements of CF!! 
+ *
+ * The skills we are treating here are all but those. -b.t. 
+ */  
 
 int monster_use_skill(object *head, object *part, object *pl,int dir) {
 object *skill, *owner;
@@ -411,20 +511,26 @@ object *skill, *owner;
   if(QUERY_FLAG(head,FLAG_CONFUSED))
     dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
 
+  /* skill selection - monster will use the next unused skill.
+   * well...the following scenario will allow the monster to 
+   * toggle between 2 skills. One day it would be nice to make
+   * more skills available to monsters.  
+   */
+ 
   for(skill=head->inv;skill!=NULL;skill=skill->below)
-    if(skill->type==SKILL
-	/* &&QUERY_FLAG(skill,FLAG_APPLIED) */
-      )
-      break;
-  if(!skill) {
-    LOG(llevError,"Error: Monster %s (%d) HAS_READY_SKILL without skill.\n",
+    if(skill->type==SKILL && skill!=head->chosen_skill) { 
+        head->chosen_skill=skill; 
+        break;
+    }
+
+  if(!skill && !head->chosen_skill) {
+    LOG(llevDebug,"Error: Monster %s (%d) has FLAG_READY_SKILL without skill.\n",
         head->name,head->count);
     CLEAR_FLAG(head, FLAG_READY_SKILL);
     return 0;
   }
-/* use_skill */
-  head->chosen_skill=skill; 
-  return do_skill(head,dir);
+/* use skill */
+  return do_skill(head,dir,NULL);
 }
 
 #endif 
@@ -676,8 +782,14 @@ int monster_can_pick(object *monster, object *item) {
   case ARMOUR:
   case SHIELD:
   case HELMET:
+  case BOOTS:
+  case GLOVES:
+  case GIRDLE:
     flag=(monster->pick_up&16)||QUERY_FLAG(monster,FLAG_USE_ARMOUR);
     break;
+  case SKILL:
+    flag=QUERY_FLAG(monster,FLAG_CAN_USE_SKILL);
+    break;
   case RING:
     flag=QUERY_FLAG(monster,FLAG_USE_RING);
     break;
@@ -734,6 +846,8 @@ void monster_apply_below(object *monster) {
  * (so that other monsters can pick it up and use it)
  */
 
+/* Sept 96, fixed this so skills will be readied -b.t.*/
+
 void monster_check_apply(object *mon, object *item) {
 
   if(item->type==SPELLBOOK&&
@@ -784,6 +898,15 @@ void monster_check_apply(object *mon, object *item) {
       flag=((mon->can_apply&16)||QUERY_FLAG(mon,FLAG_USE_ARMOUR))&&
             check_good_armour(mon,item);
       break;
+    case SKILL:
+#ifdef ALLOW_SKILLS
+      if((flag=QUERY_FLAG(mon,FLAG_CAN_USE_SKILL))) {
+        if(!QUERY_FLAG(item,FLAG_APPLIED)) apply(mon,item);
+        if (item->type==SKILL&&present_in_ob(SKILL,mon)!=NULL)
+	  SET_FLAG(mon, FLAG_READY_SKILL);
+      }
+#endif
+      break;
     case RING:
       flag=QUERY_FLAG(mon,FLAG_USE_RING);
       break;
@@ -895,7 +1018,7 @@ int wait_att2 (int dir, object *ob,object *enemy,object *part) {
 }
 
 void circ1_move (object *ob) {
-  static circle [12] = {3,3,4,5,5,6,7,7,8,1,1,2};
+  static int circle [12] = {3,3,4,5,5,6,7,7,8,1,1,2};
   if(++ob->move_status > 11)
     ob->move_status = 0;
   if (!(move_object(ob,circle[ob->move_status])))
@@ -903,7 +1026,7 @@ void circ1_move (object *ob) {
 }
 
 void circ2_move (object *ob) {
-  static circle[20] = {3,3,3,4,4,5,5,5,6,6,7,7,7,8,8,1,1,1,2,2};
+  static int circle[20] = {3,3,3,4,4,5,5,5,6,6,7,7,7,8,8,1,1,1,2,2};
   if(++ob->move_status > 19)
     ob->move_status = 0;
   if(!(move_object(ob,circle[ob->move_status])))
@@ -1076,14 +1199,20 @@ msglang *parse_message(char *msg) {
 
 void free_messages(msglang *msgs) {
   int messages, keywords;
+
+  if (!msgs)
+    return;
   for(messages = 0; msgs->messages[messages]; messages++) {
-    if(msgs->keywords[messages])
+    if(msgs->keywords[messages]) {
       for(keywords = 0; msgs->keywords[messages][keywords]; keywords++)
         free(msgs->keywords[messages][keywords]);
+      free(msgs->keywords[messages]);
+    }
     free(msgs->messages[messages]);
   }
   free(msgs->messages);
   free(msgs->keywords);
+  free(msgs);
 }
 
 void dump_messages(msglang *msgs) {
@@ -1144,7 +1273,7 @@ int talk_to_wall(object *npc, char *txt) {
     return 0;
   if((msgs = parse_message(npc->msg)) == NULL)
     return 0;
-  if(debug)
+  if(settings.debug)
     dump_messages(msgs);
   for(i=0; msgs->messages[i]; i++)
     for(j=0; msgs->keywords[i][j]; j++)
@@ -1158,3 +1287,224 @@ int talk_to_wall(object *npc, char *txt) {
   free_messages(msgs);
   return 0;
 }
+
+/* find_mon_throw_ob() - modeled on find_throw_ob */
+
+object *find_mon_throw_ob( object *op ) {
+  object *tmp = NULL, *heaviest = NULL;
+  int stupid,dumb;
+
+  if(op->head) tmp=op->head; else tmp=op;  
+  stupid = tmp->stats.Int<6?1:0;
+  dumb=tmp->stats.Int<10?1:0;
+   
+  if(!op) { /* safety */
+    LOG(llevError,"find_mon_throw_ob(): confused! have a NULL thrower!\n");
+    return (object *) NULL;
+  }
+
+  /* look through the inventory. Smarter creatures will throw
+   * thrown weapons first, followed by heaviest objects. */
+
+  for(tmp=op->inv;tmp;tmp=tmp->below) {
+    if(!tmp) break;
+     /* no throwing your arms,hands, head, etc. or invisible 
+      * items like skills. No throwing special keys either! */
+    if(tmp->invisible||tmp->type==FLESH||tmp->type==SPECIAL_KEY) continue;
+     /* this should prevent us from throwing away
+      * cursed items, worn armour, etc. */
+    if(QUERY_FLAG(tmp,FLAG_APPLIED)&&
+	(QUERY_FLAG(tmp,FLAG_CURSED)||QUERY_FLAG(tmp,FLAG_DAMNED)))
+ 	  continue;
+
+    /* based on smarts, we decline to throw some stuff */
+    if(!dumb) { 
+       /* throwing weapons are *always* preferable */ 
+      if(QUERY_FLAG(tmp,FLAG_IS_THROWN)&&!QUERY_FLAG(tmp,FLAG_APPLIED)) 
+	  break;
+       /* toss dirt/dust for blindness/damage */
+      if(QUERY_FLAG(tmp,FLAG_DUST)) break;
+       /* toss those non-self, non-summoning, spellcasting potions for damage! */
+      if(tmp->type==POTION&&tmp->stats.sp&&!IS_SUMMON_SPELL(tmp->stats.sp)
+	&&!spells[tmp->stats.sp].onself) break;
+      if(QUERY_FLAG(tmp,FLAG_APPLIED)) continue;
+      else if(tmp->type==BOW||tmp->type==ARROW||tmp->type==SCROLL
+	||tmp->type==BOOK||tmp->type==SPELLBOOK||tmp->type==AMULET
+	||tmp->type==MONEY||tmp->type==GEM||tmp->type==SKILL
+	||tmp->type==KEY||(tmp->type==CONTAINER&&tmp->inv)
+	||(tmp->glow_radius>0&&!QUERY_FLAG(op,FLAG_SEE_IN_DARK)) 
+        ||tmp->magic>0||tmp->value>(10*SK_level(op))) 
+	  continue;
+       /* 'throw' away 'trash'--spent wands and cursed items*/ 
+      if((tmp->type==WAND&&tmp->stats.food==0)||QUERY_FLAG(tmp,FLAG_CURSED)
+	||QUERY_FLAG(tmp,FLAG_DAMNED))
+	  continue;
+    } else if(!stupid) { 
+      if(QUERY_FLAG(tmp,FLAG_IS_THROWN)) break;
+      if(monster_can_pick(op,tmp)) continue;
+    } else
+      if(tmp->type==FOOD) continue;
+
+    if(!heaviest) heaviest=tmp;
+    if(tmp->weight>heaviest->weight) heaviest=tmp;
+  }
+
+  if(!tmp) tmp=heaviest;
+  else if(QUERY_FLAG(tmp,FLAG_APPLIED)) CLEAR_FLAG(tmp,FLAG_APPLIED);
+
+#ifdef DEBUG_THROW
+  LOG(llevDebug,"%s chooses to throw: %s (%d)\n",op->name,
+	!(tmp)?"(nothing)":query_name(tmp),tmp?tmp->count:-1);
+#endif
+  return tmp;
+}
+
+/* determine if we can 'detect' the enemy. Check for walls blocking the
+ * los. Also, just because its hidden/invisible, we may be sensitive/smart 
+ * enough (based on Wis & Int) to figure out where the enemy is. -b.t. 
+ */
+
+int can_detect_enemy (object *op, object *enemy) {
+
+  /* null detection for any of these condtions always */
+  if(!op || !enemy || !op->map || !enemy->map || op->map!=enemy->map) 
+    return 0;
+
+#if 0
+  /* this causes problems, dunno why.. */
+  /* are we trying to look through a wall? */ 
+  if(path_to_player(op->head?op->head:op,enemy,0)==0) return 0;
+#endif
+
+  /* opponent is unseen? */
+  if(!can_see_enemy(op,enemy)) {
+    object *part = get_nearest_part(op,enemy);
+    int radius = MIN_MON_RADIUS;
+
+
+    /* detect non-hidden (eg dark/invis enemy) */
+    if(!enemy->hide)  
+      radius = (op->stats.Wis/5)+1>MIN_MON_RADIUS?(op->stats.Wis/5)+1:MIN_MON_RADIUS;
+    else { /* a level/INT/Dex adjustment for hiding */
+      object *sk_hide;
+      int bonus = op->level + op->stats.Int;
+      if(enemy->type==PLAYER)
+        if((sk_hide = find_skill(enemy,SK_HIDING)))
+          bonus -= sk_hide->level;
+        else { 
+          LOG(llevError,"can_detect_enemy() got hidden player w/o hiding skill!");
+          make_visible(enemy);
+          radius=radius<MIN_MON_RADIUS?MIN_MON_RADIUS:radius;
+        }
+      else
+        bonus -= enemy->level;
+      radius += (bonus/10);
+    }
+
+    /* the stealth adjustment. Only if you are stealthy 
+     * will you be able to sneak up closer to creatures */ 
+    if(QUERY_FLAG(enemy,FLAG_STEALTH)) 
+      radius = radius/2;
+    else /* never less than minimum radii */ 
+      radius=radius<MIN_MON_RADIUS?MIN_MON_RADIUS:radius;
+
+
+    /* sanity check if the enemy is visible */
+    if(!enemy->invisible) {
+       /* on dark maps body heat can help indicate location with infravision */
+       if(QUERY_FLAG(op,FLAG_SEE_IN_DARK)&&op->map->darkness>0
+         &&(!enemy->immune&AT_FIRE||QUERY_FLAG(enemy,FLAG_UNDEAD))) 
+	radius += op->map->darkness/2;
+
+       /* op next to a monster (and not in complete darkness) 
+	* the monster should see you. */
+	if(radius<MIN_MON_RADIUS&&op->map->darkness<5) 
+	  radius = MIN_MON_RADIUS;
+    }
+
+    /* now check if enemy detected/found */ 
+    if(abs(part->x-enemy->x)<radius&&abs(part->y-enemy->y)<radius) { 
+	if(enemy->hide) { /* make hidden enemy visible */
+	   make_visible(enemy);
+	   /* inform players of new status */
+	   if(enemy->type==PLAYER) 
+	     new_draw_info(NDI_UNIQUE,0, enemy,"You are discovered!");
+        }
+    } else
+       return 0;
+  }
+  return 1;
+} 
+
+/* determine if op stands in a lighted square. This is not a very
+ * intellegent algorithm. For one thing, we ignore los here, SO it 
+ * is possible for a bright light to illuminate a player on the 
+ * other side of a wall (!). 
+ */
+
+int stand_in_light( object *op) {
+  objectlink *obl;
+
+  if(!op) return 0;
+  if(has_carried_lights(op)) return 1;
+
+  if(op->map)
+    for(obl=op->map->light;obl;obl=obl->next)
+      if(obl->ob&&(!obl->ob->env&&(!obl->ob->above||obl->ob->invisible))) { 
+	int dist=isqrt(distance(op,obl->ob));
+        if(dist && dist<obl->ob->glow_radius) return 1;
+      }
+  return 0;
+}
+
+/* assuming no walls/barriers, lets check to see if its *possible* 
+ * to see an enemy. Note, "detection" is different from "seeing".
+ * See can_detect_enemy() for more details. -b.t.
+ */
+
+int can_see_enemy (object *op, object *enemy) {
+  object *looker = op->head?op->head:op;
+
+  /* safety */
+  if(!looker||!enemy||looker->map!=enemy->map||!QUERY_FLAG(looker,FLAG_ALIVE))
+    return 0; 
+
+  /* we dont give a full treatment of xrays here (shorter range than normal,
+   * see through walls). Should we change the code elsewhere to make you 
+   * blind even if you can xray? */
+  if(QUERY_FLAG(looker,FLAG_BLIND)&&
+    (!QUERY_FLAG(looker,FLAG_SEE_INVISIBLE)||QUERY_FLAG(looker,FLAG_XRAYS)))
+    return 0;
+
+  /* checking for invisible things */
+  if(enemy->invisible) {
+  
+    /* HIDDEN ENEMY. by definition, you can't see hidden stuff! 
+     * However,if you carry any source of light, then the hidden
+     * creature is seeable (and stupid) */
+    if(has_carried_lights(enemy)) { 
+      if(enemy->hide) { 
+	make_visible(enemy);
+	new_draw_info(NDI_UNIQUE,0, enemy,
+	  "Your light reveals your hidding spot!");
+      }
+      return 1;
+    } else if (enemy->hide) return 0; 
+
+    /* INVISIBLE ENEMY. */
+    if(!QUERY_FLAG(looker,FLAG_SEE_INVISIBLE)
+      &&(is_true_undead(looker)==QUERY_FLAG(enemy,FLAG_UNDEAD)))
+      return 0;
+
+  }
+
+#ifdef USE_LIGHTING
+  /* ENEMY IN DARK MAP. Without infravision, the enemy is not seen 
+   * unless they carry a light or stand in light. */
+  if(looker->map->darkness>0&&!stand_in_light(enemy) 
+     &&(!QUERY_FLAG(looker,FLAG_SEE_IN_DARK)||!QUERY_FLAG(looker,FLAG_XRAYS)))
+    return 0;
+#endif
+
+  return 1;
+}
