@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_pets_c =
- *    "$Id: pets.c,v 1.8 1994/12/30 07:58:14 master Exp $";
+ *    "$Id: pets.c,v 1.9 1996/10/29 10:52:08 master Exp $";
  */
 
 /*
@@ -146,13 +146,19 @@ void remove_all_pets(mapstruct *map) {
     {
       if(owner->map == NULL || owner->map->in_memory != MAP_IN_MEMORY) {
         object *ob = obl->ob;
-        if(owner->map != NULL) {
+	/* a bit of a hack, but necessary.  IF a player is cycling through many
+	 * maps very quickly, it is possible that a monster can be caught in
+	 * limbo (being transferred to some other map.)  This should fix
+	 * that problem.
+	 */
+	if (!QUERY_FLAG(ob, FLAG_REMOVED)) {
           remove_ob(ob);
+	}
+        if(owner->map != NULL) {
           add_pending_object(ob, owner->map);
           continue;
         }
         LOG(llevMonster,"(pet no map)..");
-        remove_ob(ob);
         remove_friendly_object(ob);
         free_object(ob);
         continue;
