@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_rune_c =
- *   "$Id: rune.c,v 1.11 1995/07/24 22:17:08 master Exp $";
+ *   "$Id: rune.c,v 1.17 1996/12/01 07:59:53 master Exp $";
  */
 
 /*
@@ -86,6 +86,14 @@ int write_rune(object *op,int dir,int inspell,int level,char *runename) {
 			  return 0;
 	}
 	at=find_archetype(runename);
+
+	/* What it compares to should probably be expanded.  But basically,
+	 * creating a rune of sword should not be allowed. */
+	if (at && at->clone.type!=RUNE) {
+	     new_draw_info_format(NDI_UNIQUE, 0, op,
+		"You can't make a rune of %s", runename);
+		return 0;
+	}
         /* next it attempts to look up a rune_archetype for this spell
 		by doing some string manipulations */
 	if(!at) {
@@ -131,7 +139,7 @@ int write_rune(object *op,int dir,int inspell,int level,char *runename) {
     tmp->y=ny;
     tmp->map = op->map;
     tmp->direction=dir;  /* where any spell will go upon detonation */
-    tmp->level=op->level;  /* what level to cast the spell at */
+    tmp->level=SK_level(op);  /* what level to cast the spell at */
     set_owner(tmp,op);
     insert_ob_in_map(tmp,op->map);
     return 1;
@@ -198,17 +206,20 @@ void spring_trap(object *trap,object *victim)
   if((spell_in_rune=look_up_spell_by_name(NULL,trap->slaying))!=-1) trap->stats.sp=spell_in_rune;
   if(victim) if(victim->type==PLAYER) new_draw_info(NDI_UNIQUE, 0,victim,trap->msg);
   if(!trap->stats.sp) rune_attack(trap,victim); 
-  else {
+  else {  /*  This is necessary if the trap is inside something else */
     remove_ob(trap);
     trap->x=victim->x;trap->y=victim->y;
     insert_ob_in_map(trap,victim->map);
-    cast_spell(trap,trap,trap->direction,trap->stats.sp,1,0,NULL);
+    cast_spell(trap,trap,trap->direction,trap->stats.sp,1,spellNormal,NULL);
   }
+  /*  Flash an image of the trap on the map so the poor sod
+   *   knows what hit him.  */
   for(env=trap;env!=NULL&&env->env!=NULL;env=env->env);
   trap_show(trap,env);  
+
   if(!trap->stats.hp) {
     trap->type=98;  /* make the trap impotent */
-    trap->stats.food=20;  /* make it stick around until it's spells are gone */
+    trap->stats.food=20;  /* make it stick around until its spells are gone */
     SET_FLAG(trap,FLAG_IS_USED_UP);
   }
 }
@@ -248,18 +259,14 @@ int dispel_rune(object *op,int dir,int risk)
 	
 }
 
-/* I modified this slightly to allow greater trap detection
- * with the detect traps skill (has_skill==1) - b.t.
- */
-
-int trap_see(object *op,object *trap,int has_skill) {
+int trap_see(object *op,object *trap) {
   char buf[MAX_BUF];
-  int chance,trapworth;
+  int chance;
+#ifndef ALLOW_SKILLS 
+  int trapworth=0;  
+#endif
 
-  if(has_skill>0) 
-	chance = RANDOM()%200;
-  else
-	chance = RANDOM()%100;
+  chance = RANDOM()%100;
   
   /*  decide if we see the rune or not */
   if((trap->stats.Cha==1) || (chance >
@@ -269,9 +276,11 @@ int trap_see(object *op,object *trap,int has_skill) {
       sprintf(buf,"You spot a %s!",trap->name);
       new_draw_info(NDI_UNIQUE, 0,op,buf);
 
+#ifndef ALLOW_SKILLS /* this stuff is handled by find_trap skill code -b.t. */ 
       trapworth = trap->stats.Cha * trap->level;
       if(trap->stats.Cha>1) add_exp(op,trapworth);  /* finding a trap awards exp */
       trap->stats.Cha=1;  /* unhide the rune/trap */
+#endif
       return 1;
   }
   return 0;
@@ -306,12 +315,21 @@ int trap_disarm(object *disarmer, object *trap, int risk) {
 			   +5 - disarmer->stats.Dex/2)))))
         {
             new_draw_info(NDI_UNIQUE, 0,disarmer,"You successfuly disarm it!");
-	    if(!trap->owner) add_exp(disarmer,trapworth);
-            else if(trap->owner && trap->owner->type!=PLAYER)
-	      add_exp(disarmer,trapworth);
             remove_ob(trap);
             free_object(trap);
-            return 1;
+	    /* If it is your own trap, (or any players trap), don't you don't
+	     * get exp for it.
+	     */
+#ifdef ALLOW_SKILLS 
+	    if (trap->owner && trap->owner->type!=PLAYER && risk)
+		return trapworth;
+	    else return 1; /* give minimal exp and say success */
+#else
+	    if(!trap->owner) add_exp(disarmer,trapworth);
+            else if(trap->owner && trap->owner->type!=PLAYER && risk)
+	      add_exp(disarmer,trapworth);
+	    return 1;
+#endif
         }
     else
         {
