@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c,v 1.31 1995/09/11 03:16:47 master Exp master $";
+ *   "$Id: shop.c,v 1.33 1996/09/30 03:56:52 master Exp $";
  */
 
 /*
@@ -56,7 +56,7 @@ static char *coins[] = {"platinacoin", "goldcoin", "silvercoin", NULL};
  * divisions took place, the value got rounded to 0 (Being an int), and
  * thus remained 0.
  *
- * Mark Wedel (master@rahul.net)
+ * Mark Wedel (mwedel@pyramid.com)
  */
 int query_cost(object *tmp, object *who, int flag) {
   int val;
@@ -89,8 +89,15 @@ int query_cost(object *tmp, object *who, int flag) {
       else	/* Trying to sell something, or get true value */
         if (tmp->type == POTION)
           val = number * 1000; /* Don't want to give anything away */
-        else
-          val = number * tmp->arch->clone.value / 3;
+        else {
+	  /* Get 2/3'rd value for applied objects, 1/3'rd for totally 
+	   * unknown objects
+	   */
+	  if (QUERY_FLAG(tmp, FLAG_BEEN_APPLIED))
+	    val = number * tmp->arch->clone.value *2 / 3;
+	  else
+	    val = number * tmp->arch->clone.value / 3;
+	}
     } else { /* No archetype with this object */
       LOG(llevDebug,"In sell item: Have object with no archetype: %s\n", tmp->name);
       if (flag == F_BUY) {
@@ -101,8 +108,6 @@ int query_cost(object *tmp, object *who, int flag) {
         val = number * tmp->value / 5;
     }
   }
-  if (!QUERY_FLAG(tmp, FLAG_IDENTIFIED) && QUERY_FLAG(tmp, FLAG_BEEN_APPLIED))
-    val = val * 2 / 3;
 
   /* If the item has been applied or identifed or does not need to be
    * identified, AND the object is magical and the archetype is non
@@ -124,18 +129,23 @@ int query_cost(object *tmp, object *who, int flag) {
 	   */
           val/=(1-tmp->magic);
      }
+#if 0 /* cha_bonus now takes this into account */
   if (flag==F_BUY)
     val*=6; /* For charisma 25, the price is now x1.5 */
-
+#endif
   if (tmp->type==WAND) {
     if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) || !need_identify(tmp))
+      /* Give money based on how many charges the item could have.  Not
+       * really a good method - some spells are certainly better than others.
+       */
       val=(val*tmp->stats.food)/spells[tmp->stats.sp].charges;
     else
       val/=3;
   }
 
+  /* Limit amount of money you can get for really great items. */
   if (flag==F_TRUE || flag==F_SELL) {
-    if (val/number>10000) {
+    if (val/number>25000) {
 	val=8000+isqrt((int)val/number)*20;
 	val *= number;
     }
@@ -148,16 +158,32 @@ int query_cost(object *tmp, object *who, int flag) {
  */
 
   if (who!=NULL && who->type==PLAYER) { 
+      float diff;
+
       charisma = who->stats.Cha;  /* used for SK_BARGAINING modification */ 
       if(who->contr->shoottype == range_skill && who->chosen_skill)
          if(who->chosen_skill->stats.sp == SK_BARGAINING) { 
 	    charisma = who->stats.Cha + (who->level+2)/3;
 	    if(charisma>30) charisma = 30;
          }
+      if (cha_bonus[charisma]<=1.0) {
+	LOG(llevError,"Illegal charisma bonus, %d <=1.0", cha_bonus[charisma]);
+	diff=0.9;	/*pretty bad case */
+      }
+      else 
+	/* Diff is now a float between 0 and 1 */
+	diff=(cha_bonus[charisma]-1)/(1+cha_bonus[charisma]);
+
+      /* we need to multiply these by 4.0 to keep buy costs roughly the same
+       * (otherwise, you could buy a potion of charisma for around 400 pp.
+       * Arguable, the costs in the archetypes should be updated to better
+       * reflect values (potion charisma list for 1250 gold)
+       */
       if(flag==F_BUY)
-          val=(val*(100-cha_bonus[charisma]))/100;
+          val=(4.0*(float)val*(1.0+diff));
       else if (flag==F_SELL)
-          val=(val*100)/(100-cha_bonus[charisma]);
+          val=(4.0*(float)val*(1.0-diff));
+      else val *=4;
   }
   if(val<0)
     val=1000000;
@@ -268,7 +294,7 @@ int query_money(object *op) {
  */
 
 int pay_for_item(object *op,object *pl) {
-    int item_cost=query_cost(op,pl,F_BUY),i, insert_obj[NUM_COINS], count;
+    int item_cost=query_cost(op,pl,F_BUY),i, count;
     int to_pay;
     object *tmp, *coin_objs[NUM_COINS];
     archetype *at;
@@ -279,7 +305,6 @@ int pay_for_item(object *op,object *pl) {
 
 
     for (i=0; i<NUM_COINS; i++) {
-	insert_obj[i] = 0;
 	coin_objs[i] = NULL;
     }
     pl->contr->freeze_inv=1;
@@ -323,7 +348,6 @@ int pay_for_item(object *op,object *pl) {
 	    coin_objs[i] =get_object();
 	    copy_object(&at->clone, coin_objs[i]);
 	    coin_objs[i]->nrof = 0;
-	    insert_obj[i] = 1;
 	}
 
     to_pay = item_cost;
@@ -456,10 +480,6 @@ void sell_item(object *op, object *pl) {
     return;
   }
   D_LOCK(pl);
-#if 0
-  otmp2=pl->container;
-  pl->container=NULL;
-#endif
   for (count=0; coins[count]!=NULL; count++) {
     at = find_archetype(coins[count]);
     if (at==NULL) LOG(llevError, "Could not find %s archetype", coins[count]);
@@ -478,10 +498,6 @@ void sell_item(object *op, object *pl) {
 	LOG(llevError,"Warning - payment not zero: %d\n", i);
 
   D_UNLOCK(pl);
-#if 0	/* done in drop */
-  draw_look(pl);
-  draw_inventory(pl);
-#endif
   new_draw_info_format(NDI_UNIQUE, 0, pl,
 	"You receive %s for %s.",query_cost_string(op,pl,1),
           query_name(op));
