@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_sockets_c =
- *    "$Id: socket.c,v 1.28 1995/04/15 05:03:12 master Exp $";
+ *    "$Id: socket.c,v 1.30 1997/03/09 08:16:45 master Exp master $";
  */
 
 /*
@@ -61,13 +61,12 @@ struct sockaddr_in insock;
 struct sockaddr_in addr;
 int addrlen=sizeof(struct sockaddr);
 int doneone=0;
-unsigned short listen_port = PORT;
 #endif /* SERVER */
 
 void init_socket() {
 #ifdef SERVER
-  if(server_mode == SERVER_ENABLED || debug)
-    LOG(llevError,"Opening add user socket on %d\n",listen_port);
+  if(settings.servermode == SERVER_ENABLED)
+    LOG(llevDebug,"Opening add user socket on %d\n",settings.listen_port);
 
 #if defined(hpux) || defined (SVR4)
   max_filedescriptors = sysconf(_SC_OPEN_MAX);
@@ -90,7 +89,7 @@ void init_socket() {
   }
 
   insock.sin_family = AF_INET;
-  insock.sin_port = htons(listen_port);
+  insock.sin_port = htons(settings.listen_port);
   insock.sin_addr.s_addr = htonl(INADDR_ANY);
 
   {
@@ -115,7 +114,7 @@ void init_socket() {
    perror("error on setsockopt REUSEADDR");
 #endif
   if (bind(acceptfd,(struct sockaddr *)&insock,sizeof(insock)) == (-1)) {
-    if(server_mode == SERVER_ENABLED || debug)
+    if(settings.servermode == SERVER_ENABLED || settings.debug)
       perror("error on bind command");
     abort_sockets();
     return;
@@ -127,7 +126,7 @@ void init_socket() {
     return;
   }
 
-  if(server_mode == SERVER_ENABLED)
+  if(settings.servermode == SERVER_ENABLED)
     block_until_new_connection();
 #endif
 }
@@ -138,7 +137,7 @@ void check_socket() {
   fd_set tmp_exceptions;
   sockets *next=NULL;
 
-  if(server_mode == SERVER_DISABLED)
+  if(settings.servermode == SERVER_DISABLED)
     return;
 
   FD_ZERO(&tmp_read);
@@ -186,15 +185,14 @@ void check_socket() {
       if (readct==0) {
         close(active_socket->fd);
         remove_socket(active_socket);
-        if (server_mode == SERVER_ABORTED) {
+        if (settings.servermode == SERVER_ABORTED) {
           LOG(llevError, "Server mode ended.\n");
-          server_mode = SERVER_OFF;
+          settings.servermode = SERVER_OFF;
         }
       } else {
 	/* The following should never ever happen. */
         if (readct>SOCKET_BUFLEN) {
-          if(debug)
-            LOG(llevError,"Error, Protocol Violation, input too long\n");
+          LOG(llevError,"Error, Protocol Violation, input too long\n");
           draw_socket(active_socket->fd,"Line too long, ignored.");
 
 	  /* Having this be fatal probably does not make much
@@ -209,9 +207,9 @@ void check_socket() {
           buf[--readct] = '\0';
         if (buf[readct-1] == '\r')
           buf[--readct] = '\0';
-        if (debug) /* Would get useless echos if I used LOG() here */
-          if (strncmp(buf, "dm ", 3)) /* Better not log the password */
-            fprintf(logfile,"socket[%s]: %s\n",active_socket->name,buf);
+        if (strncmp(buf, "dm ", 3)) /* Better not log the password */
+          LOG(llevDebug,"socket[%s]:(%d) %s\n",active_socket->name,
+	      readct,buf);
         if(buf[0]=='\0')
           continue; /* Empty line */
 
@@ -281,7 +279,7 @@ void remove_socket(sockets *socket) {
   LOG(llevDebug,"Removing socket %d\n",tmp->fd);
   free(tmp);
   if (first_player == (player *) NULL &&
-      first_socket == (sockets *) NULL && server_mode == SERVER_ENABLED)
+      first_socket == (sockets *) NULL && settings.servermode == SERVER_ENABLED)
     block_until_new_connection();
 #endif
 }
@@ -289,7 +287,7 @@ void remove_socket(sockets *socket) {
 void close_all_sockets() {
 #ifdef SERVER
   info_all_sockets("Crossfire died, disconnecting...");
-  server_mode = SERVER_ABORTED; /* Don't want to block */
+  settings.servermode = SERVER_ABORTED; /* Don't want to block */
   while(first_socket != (sockets *) NULL) {
     close(first_socket->fd);
     remove_socket(first_socket);
@@ -332,11 +330,11 @@ void info_all_sockets(char *txt) {
 
 void abort_sockets() {
 #ifdef SERVER
-  if(server_mode != SERVER_ENABLED) {
+  if(settings.servermode != SERVER_ENABLED) {
     LOG(llevDebug,"Turning off sockets.\n");
     if(close(acceptfd))
       perror("Disabling sockets/close");
-    server_mode = SERVER_DISABLED; /* No need to check them in the future */
+    settings.servermode = SERVER_DISABLED; /* No need to check them in the future */
     return;
   }
   exit(1);
@@ -350,7 +348,7 @@ void block_until_new_connection()
 
   FD_ZERO(&readfs);
   FD_SET(acceptfd, &readfs);
-#if ERIC_SERVER
+#ifdef ERIC_SERVER
   FD_SET(ericfd(),&readfs);
 #endif
 
