@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sound_c =
- *   "$Id: sounds.c,v 1.9 1995/07/24 22:18:01 master Exp $";
+ *   "$Id: sounds.c,v 1.12 1997/03/09 08:17:13 master Exp master $";
  */
 
 /* Send bug reports to Raphael Quinet (quinet@montefiore.ulg.ac.be) */
@@ -12,9 +12,6 @@
 #endif
 
 int volume_table[NROF_SOUNDS];
-#ifdef SOUND_EFFECTS
-RPLAY   *sound_table[NROF_SOUNDS];
-#endif
 
 void setup_sounds()
 {
@@ -23,8 +20,8 @@ void setup_sounds()
   FILE *fp;
   int	num_sounds = 0, volume;
 
-  LOG(llevDebug, "Setup_sounds()...");
-  sprintf(name_buf, "%s/%s", LibDir, SOUNDS_LIST);
+  sprintf(name_buf, "%s/%s", settings.libdir, SOUNDS_LIST);
+  LOG(llevDebug, "Reading sounds from %s...",name_buf);
   fp = fopen(name_buf, "r");
   if (fp == NULL)
     {
@@ -93,7 +90,7 @@ int init_disp_sound(char *player_disp)
   int   rplay_fd;
   char  hostname[MAX_BUF];
 
-  LOG(llevDebug, "Init_disp_sounds(%s)...", player_disp);
+  LOG(llevDebug, "Init display (%s) with sounds...", player_disp);
   strcpy(hostname, player_disp);
   if ((p = strrchr(hostname, ':')) != NULL)
     *p = '\0';
