@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_c =
- *   "$Id: spell_util.c,v 1.9 1995/09/11 03:18:57 master Exp master $";
+ *   "$Id: spell_util.c,v 1.20 1997/03/09 08:18:22 master Exp master $";
  */
 
 /*
@@ -49,12 +49,15 @@ char *range_name[range_size] = {
 #endif
 };
 
+archetype *spellarch[NROFREALSPELLS];
+
 void init_spells() {
   static int init_spells_done = 0;
   int i;
 
   if (init_spells_done)
     return;
+  LOG(llevDebug, "Initializing spells...");
   init_spell_param(); /*  peterm:  read the spell parameter file--newspells.c */
   init_spells_done = 1;
   for (i = 0; i < NROFREALSPELLS; i++)
@@ -65,8 +68,13 @@ void init_spells() {
             spells[i].name,spells[i].archname);
     } else
       spellarch[i] = (archetype *) NULL;
-#ifdef DUMP_SWITCHES
-  if (dump_monsters == 4) {
+  LOG(llevDebug, "done.\n");
+}
+
+void dump_spells()
+{
+    int i;
+
     for (i = 0; i < NROFREALSPELLS; i++) {
       char *name1 = NULL, *name2 = NULL;
       if (spellarch[i]) {
@@ -75,11 +83,9 @@ void init_spells() {
         if (spellarch[i]->clone.other_arch)
           name2 = spellarch[i]->clone.other_arch->name;
       }
-      fprintf(stderr, "%s:%s:%s\n", spells[i].name, name1, name2);
+      fprintf(stderr, "%s:%s:%s\n", spells[i].name, (name1?name1:"null"), 
+	     (name2?name2:"null"));
     }
-    exit(0);
-  }
-#endif
 }
 
 void spell_effect(int spell_type, int x, int y, mapstruct *map) {
@@ -128,68 +134,101 @@ int check_spell_known(object *op,int sp) {
  * Fires spell "type" in direction "dir".
  * If "ability" is true, the spell is the innate ability of a monster.
  * (ie, don't check for blocks_magic(), and don't add AT_MAGIC to attacktype.
+ *
+ * op is the creature that is owner of the object that is casting the spell
+ * caster is the actual object (wand, potion) casting the spell. can be
+ *    same as op.
+ * dir is the direction to cast in.
+ * ability is true if it is an ability casting the spell.  These can be
+ *    cast in no magic areas.
+ * item is the type of object that is casting the spell.
+ * stringarg is any options that are being used.
  */
 
+/* Oct 95 - added cosmetic touches for MULTIPLE_GODS hack -b.t. */
+
 int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeFrom item,char *stringarg) {
+#ifdef MULTIPLE_GODS
+  char *godname;
+#endif
   spell *s=find_spell(type);
   int success=0,bonus;
   int duration=SP_PARAMETERS[type].bdur;  /*  get the base duration */
-  object casting_object, *orig_op=op;
 
-  if(item!=spellNormal&&caster!=op) {
-	reset_object(&casting_object);
-	(void) memcpy((void *)((char *) &casting_object +
-		offsetof(object,name)),
-                (void *)((char *) op+offsetof(object,name)),
-                sizeof(object)-offsetof(object, name));
-
-	/* Things not cleared by reset_object */
-	casting_object.next = NULL;
-	casting_object.prev = NULL;
-	casting_object.active_next = NULL;
-	casting_object.active_prev = NULL;
-      /*  these should come from the source object for the spell */
-	casting_object.level=caster->level;
-	casting_object.path_attuned=caster->path_attuned;
-	casting_object.path_repelled=caster->path_repelled;
-	casting_object.path_denied=caster->path_denied;
-      /* the following must come from the monster/player initiating the spell*/
-	casting_object.type=op->type;
-	casting_object.env=op->env;
-	casting_object.inv=op->inv;
-	casting_object.contr=op->contr;
-	casting_object.facing=op->facing;
-	casting_object.map=op->map;
-	set_owner(&casting_object, op);
-	casting_object.x=op->x;
-	casting_object.y=op->y; 
-      /* these are needed for some spells */
-	casting_object.stats.luck=op->stats.luck;
-	casting_object.stats.Wis=op->stats.Wis;
-	casting_object.stats.Cha=op->stats.Cha;
-	casting_object.stats.Int=op->stats.Int; 
-      /*  set the object */
-	op= &casting_object;
-  }
+#ifdef MULTIPLE_GODS
+  if(!strcmp((godname=determine_god(op)),"none")) godname="A random spirit"; 
+#endif
 
+  /* It looks like the only properties we ever care about from the casting
+   * object (caster) is spell paths and level.
+   */
+  if (!caster && item==spellNormal)
+	caster=op;
 
   if(s==NULL) {
     LOG(llevError,"Error, unknown spell: %d\n",type);
     return 0;
   }
-  if(!QUERY_FLAG(op, FLAG_WIZ)&&op->type==PLAYER&&op->contr->shoottype==range_magic&&
-     item!=spellPotion&& op->stats.sp<SP_level_spellpoint_cost(op,type) &&
-     !(IS_SUMMON_SPELL(type)&&op->contr->golem!=NULL))
+
+  if(!(QUERY_FLAG(op, FLAG_WIZ))&&
+	  (op->type==PLAYER)&&
+	  (op->contr->shoottype==range_magic)&&
+     (item!=spellPotion)&& 
+     (!(IS_SUMMON_SPELL(type)&&op->contr->golem!=NULL)))
   {
-    new_draw_info(NDI_UNIQUE, 0,op,"You don't have enough spellpoints.");
-    op->contr->count_left=0;
-    return 0;
+	if(!spells[type].cleric&& op->stats.sp<SP_level_spellpoint_cost(op,caster,type))
+	{
+	  new_draw_info(NDI_UNIQUE, 0,op,"You don't have enough mana.");
+	  op->contr->count_left=0;
+	  return 0;
+	}
+	if(spells[type].cleric&&op->stats.grace<SP_level_spellpoint_cost(op,caster,type))
+	{
+	  /* it's possible for grace to go negative */
+	  /* Fine - let grace go negative, but how negative it is should really
+	   * put a limit on things - in the old method, chance was the same
+	   * no matter how negative it was.
+	   *Instead of subtracting 10 from the roll, add in grace (which is
+	   * negative).  This puts a real limit on things.
+	   */
+	  if( (RANDOM()%op->stats.Wis) +op->stats.grace -
+		  10*SP_level_spellpoint_cost(op,caster,type)/op->stats.maxgrace >0) {
+#ifdef MULTIPLE_GODS
+			  new_draw_info_format(NDI_UNIQUE, 0,op, 
+				"%s grants your prayer, though you are unworthy.",godname);
+#else
+			  new_draw_info(NDI_UNIQUE, 0,op, 
+				"God grants your prayer, though you are unworthy.");
+#endif
+	  }
+	  else
+	  {
+		  prayer_failure(op,op->stats.grace,SP_level_spellpoint_cost(op,caster,type));
+#ifdef MULTIPLE_GODS
+			  new_draw_info_format(NDI_UNIQUE, 0,op,"%s ignores your prayer.",godname);
+#else
+			  new_draw_info(NDI_UNIQUE, 0,op,"God ignores your prayer.");
+#endif
+		  op->contr->count_left=0;
+			  return 0;
+	  }
+	}
   }
-  if (op->path_denied & s->path) {
+
+  if (caster->path_denied & s->path) {
     new_draw_info(NDI_UNIQUE, 0,op, "You are unable to cast that spell.");
-    op->contr->count_left=0;
+    if(op->type==PLAYER) op->contr->count_left=0;
     return RANDOM()%(s->sp)+1;
   }
+  /* If it is an ability, assume that the designer of the archetype knows
+   * what they are doing.
+   */
+  if (item==spellNormal && !ability && SK_level(caster) < s->level && !QUERY_FLAG(op,FLAG_WIZ)) {
+    new_draw_info(NDI_UNIQUE, 0,op, "You lack enough skill to cast that spell.");
+    if(op->type==PLAYER) op->contr->count_left=0;
+    return 0;
+  }
+
 #ifdef CASTING_TIME
   if (op->casting==-1) /* begin the casting */
    {
@@ -222,7 +261,12 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
 	  (( s->cleric)&&blocks_cleric(op->map,op->x,op->y)))) {
     if (op->type!=PLAYER)
       return 0;
-    if(s->cleric) new_draw_info(NDI_UNIQUE, 0,op,"This ground is unholy!  God ignores you.");
+    if(s->cleric) 
+#ifdef MULTIPLE_GODS
+      new_draw_info_format(NDI_UNIQUE, 0,op,"This ground is unholy!  %s ignores you.",godname);
+#else
+      new_draw_info(NDI_UNIQUE, 0,op,"This ground is unholy!  God ignores you.");
+#endif
     else
     switch(op->contr->shoottype) {
     case range_magic:
@@ -258,7 +302,7 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
 #endif    
     if(s->sp==0) /* Shouldn't happen... */
       return 0;
-    return RANDOM()%(SP_level_spellpoint_cost(op,type)+1)+1;
+    return RANDOM()%(SP_level_spellpoint_cost(op,caster,type)+1)+1;
   }
 #ifdef SPELL_ENCUMBRANCE
   if(item == spellNormal && op->type==PLAYER && (!s->cleric) ) {
@@ -267,9 +311,9 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
     if( failure < 0) {
 	new_draw_info(NDI_UNIQUE, 0,op,"You bungle the spell because you have too much heavy equipment in use.");
 #ifdef SPELL_FAILURE_EFFECTS
-        spell_failure(op,failure,SP_level_spellpoint_cost(op,type));
+        spell_failure(op,failure,SP_level_spellpoint_cost(op,caster,type));
 #endif
-	return RANDOM()%(SP_level_spellpoint_cost(op,type)+ 1);
+	return RANDOM()%(SP_level_spellpoint_cost(op,caster,type)+ 1);
 	}
    }
 #endif /*SPELL_ENCUMBRANCE*/
@@ -298,15 +342,21 @@ if (item == spellNormal && !ability ){
   switch((enum spellnrs) type) {
   case SP_BULLET:
   case SP_LARGE_BULLET:
-    success = fire_arch(op,dir,spellarch[type],type,1);
+    success = fire_arch(op,caster,dir,spellarch[type],type,1);
+    break;
+  case SP_HOLY_ORB:
+    success = fire_arch(op,caster,dir,spellarch[type],type,0);
     break;
   case SP_S_FIREBALL:
   case SP_M_FIREBALL:
   case SP_L_FIREBALL:
+  case SP_S_SNOWSTORM:
+  case SP_M_SNOWSTORM:
+  case SP_L_SNOWSTORM:
   case SP_HELLFIRE:
   case SP_POISON_CLOUD:
   case SP_M_MISSILE:
-    success = fire_arch(op,dir,spellarch[type],type, !ability);
+    success = fire_arch(op,caster,dir,spellarch[type],type, !ability);
     break;
   case SP_MASS_CONFUSION:
   case SP_SHOCKWAVE:
@@ -320,29 +370,60 @@ if (item == spellNormal && !ability ){
   case SP_FIREBREATH:
   case SP_LARGE_ICESTORM:
   case SP_BANISHMENT:	
-    success = cast_cone(op,dir,duration,type,spellarch[type],!ability);
+    success = cast_cone(op,caster,dir,duration,type,spellarch[type],!ability);
     break;
   case SP_TURN_UNDEAD:
+    if(QUERY_FLAG(op,FLAG_UNDEAD)) { /* the undead *don't* cast this */
+       new_draw_info(NDI_UNIQUE, 0,op,"Your undead nature prevents you from turning undead!");
+       success=0; break;
+    }
   case SP_HOLY_WORD:
-    success = cast_cone(op,dir,duration+turn_bonus[op->stats.Wis],type,
+    success = cast_cone(op,caster,dir,duration+(turn_bonus[op->stats.Wis]/5),type,
 	spellarch[type],0);
     break;
+  case SP_HOLY_WRATH:
+  case SP_INSECT_PLAGUE:
+  case SP_RETRIBUTION:
+    success = cast_smite_spell(op,caster,dir,type);
+    break;
+  case SP_SUNSPEAR:
   case SP_FIREBOLT:
   case SP_FROSTBOLT:
   case SP_S_LIGHTNING:
   case SP_L_LIGHTNING:
   case SP_STEAMBOLT:
-    success = fire_bolt(op,dir,type,!ability);
+    success = fire_bolt(op,caster,dir,type,!ability);
     break;
   case SP_BOMB:
-    success = create_bomb(op,dir,"bomb");
+    success = create_bomb(op,caster,dir,"bomb");
     break;
   case SP_GOLEM:
   case SP_FIRE_ELEM:
   case SP_WATER_ELEM:
   case SP_EARTH_ELEM:
   case SP_AIR_ELEM:
-    success = summon_monster(op,dir,spellarch[type],type);
+    success = summon_monster(op,caster,dir,spellarch[type],type);
+    break;
+  case SP_STAFF_TO_SNAKE: 
+    success = staff_to_snake(op,caster,dir,spellarch[type],type);
+    break;
+  case SP_FINGER_DEATH: 
+    success = finger_of_death(op,caster,dir);
+    break;
+  case SP_SUMMON_AVATAR: 
+  case SP_HOLY_SERVANT: {
+    archetype *spat = find_archetype((type==SP_SUMMON_AVATAR)?"avatar":"holy_servant");
+    success = summon_avatar(op,caster,dir,spat,type);
+    break; } 
+  case SP_CONSECRATE: 
+    success = cast_consecrate(op); 
+    break;
+  case SP_SUMMON_CULT:
+#ifdef MULTIPLE_GODS
+    success = summon_cult_monsters(op,dir);
+#else
+    success = summon_pet(op,dir,item);
+#endif
     break;
   case SP_PET:
     success = summon_pet(op,dir, item);
@@ -351,14 +432,16 @@ if (item == spellNormal && !ability ){
     /* dimension door needs the actual caster, because that is what is
      * moved.
      */
-    success = dimension_door(orig_op,dir);
+    success = dimension_door(op,dir);
     break;
+  case SP_DARKNESS:
+  case SP_WALL_OF_THORNS:
   case SP_CHAOS_POOL:
   case SP_COUNTERWALL:
   case SP_FIRE_WALL:
   case SP_FROST_WALL:
   case SP_EARTH_WALL:
-    success = magic_wall(op,dir,type);
+    success = magic_wall(op,caster,dir,type);
     break;
   case SP_MAGIC_MAPPING:
     if(op->type==PLAYER) {
@@ -371,14 +454,14 @@ if (item == spellNormal && !ability ){
     if(op->type==PLAYER)
       bonus=fear_bonus[op->stats.Cha];
     else
-      bonus=op->head==NULL?op->level/3+1:op->head->level/3+1;
-    success = cast_cone(op,dir,duration+bonus,SP_FEAR,spellarch[type],!ability);
+      bonus=caster->head==NULL?caster->level/3+1:caster->head->level/3+1;
+    success = cast_cone(op,caster,dir,duration+bonus,SP_FEAR,spellarch[type],!ability);
     break;
   case SP_WOW:
     success = cast_wow(op,dir,ability, item);
     break;
   case SP_DESTRUCTION:
-    success = cast_destruction(op,5+op->stats.Int,AT_MAGIC);
+    success = cast_destruction(op,caster,5+op->stats.Int,AT_MAGIC);
     break;
   case SP_PERCEIVE:
     success = perceive_self(op);
@@ -389,17 +472,21 @@ if (item == spellNormal && !ability ){
   case SP_INVIS:
   case SP_INVIS_UNDEAD:
   case SP_IMPROVED_INVIS:
-    success = cast_invisible(op,type);
+    success = cast_invisible(op,caster,type);
     break;
   case SP_PROBE:
     success = probe(op,dir);
     break;
   case SP_CREATE_FOOD:
-    success = cast_create_food(op,dir,stringarg);
+    success = cast_create_food(op,caster,dir,stringarg);
     break;
   case SP_EARTH_DUST:
-    success = cast_earth2dust(op);
+    success = cast_earth2dust(op,caster);
     break;
+  case SP_REGENERATION: 
+  case SP_BLESS:
+  case SP_CURSE:
+  case SP_HOLY_POSSESSION:
   case SP_STRENGTH:
   case SP_DEXTERITY:
   case SP_CONSTITUTION:
@@ -421,7 +508,8 @@ if (item == spellNormal && !ability ){
   case SP_HEROISM:
   case SP_CONFUSION:
   case SP_XRAY:
-    success = cast_change_attr(op,dir,type);
+  case SP_DARK_VISION:
+    success = cast_change_attr(op,caster,dir,type);
     break;
   case SP_RESTORATION:
   case SP_HEAL:
@@ -430,6 +518,7 @@ if (item == spellNormal && !ability ){
   case SP_MAJOR_HEAL:
   case SP_CURE_POISON:
   case SP_CURE_CONFUSION:
+  case SP_CURE_BLINDNESS:
     success = cast_heal(op,dir,type);
     break;
   case SP_REGENERATE_SPELLPOINTS:
@@ -471,22 +560,45 @@ if (item == spellNormal && !ability ){
     break;
 /* peterm: following spells added */
   case SP_BALL_LIGHTNING:
-    success = fire_arch(op,dir,find_archetype("ball_lightning"),type,!ability);
+    success = fire_arch(op,caster,dir,find_archetype("ball_lightning"),type,!ability);
     break;
   case SP_METEOR_SWARM: {
     int n;
     n=RANDOM()%3 + RANDOM()%3 + RANDOM()%3 +3 +
-      SP_level_strength_adjust(op, type);
+      SP_level_strength_adjust(op,caster, type);
     success = 1;
-    fire_swarm(op,dir,find_archetype("meteor"),SP_METEOR,n);
+    fire_swarm(op,dir,spellarch[type],SP_METEOR,n);
     break;
-}
-
+  }
+  case SP_BULLET_SWARM: {
+    int n;
+    n=RANDOM()%3 + RANDOM()%3 + RANDOM()%3 +3 +
+      SP_level_strength_adjust(op,caster, type);
+    success = 1;
+    fire_swarm(op,dir,spellarch[type],SP_BULLET,n);
+    break;
+  }
+  case SP_BULLET_STORM: {
+    int n;
+    n=RANDOM()%3 + RANDOM()%3 + RANDOM()%3 +3 +
+      SP_level_strength_adjust(op,caster, type);
+    success = 1;
+    fire_swarm(op,dir,spellarch[type],SP_LARGE_BULLET,n);
+    break;
+  }
+  case SP_CAUSE_MANY: {
+    int n;
+    n=RANDOM()%3 + RANDOM()%3 + RANDOM()%3 +3 +
+      SP_level_strength_adjust(op,caster, type);
+    success = 1;
+    fire_swarm(op,dir,spellarch[type],SP_CAUSE_HEAVY,n);
+    break;
+  }
   case SP_METEOR:
-    success = fire_arch(op,dir,find_archetype("meteor"),type,0);
+    success = fire_arch(op,caster,dir,find_archetype("meteor"),type,0);
     break;
   case SP_MYSTIC_FIST:
-    success = summon_monster(op,dir,spellarch[type],type);
+    success = summon_monster(op,caster,dir,spellarch[type],type);
     break;
   case SP_RAISE_DEAD:
   case SP_RESURRECTION:
@@ -505,14 +617,14 @@ if (item == spellNormal && !ability ){
   case SP_INVULNERABILITY:
   case SP_PROTECTION:
   case SP_HASTE:
-    success=cast_change_attr(op,dir,type);
+    success=cast_change_attr(op,caster,dir,type);
     break;
   /* peterm, additional spells added */
   case SP_BUILD_DIRECTOR:
   case SP_BUILD_BWALL:
   case SP_BUILD_LWALL:
   case SP_BUILD_FWALL:
-    success=create_the_feature(op,dir,type);
+    success=create_the_feature(op,caster,dir,type);
     break;
   case SP_RUNE_FIRE: 
   case SP_RUNE_FROST:
@@ -520,13 +632,13 @@ if (item == spellNormal && !ability ){
   case SP_RUNE_BLAST: 
   case SP_RUNE_DEATH: 
   case SP_RUNE_ANTIMAGIC:
-    success = write_rune(op,dir,0,op->level,s->archname);
+    success = write_rune(op,dir,0,caster->level,s->archname);
     break;
   case SP_RUNE_DRAINSP:
-    success = write_rune(op,dir,SP_MAGIC_DRAIN,op->level,s->archname);
+    success = write_rune(op,dir,SP_MAGIC_DRAIN,caster->level,s->archname);
     break;
   case SP_RUNE_TRANSFER:
-    success= write_rune(op,dir,SP_TRANSFER,op->level,s->archname);
+    success= write_rune(op,dir,SP_TRANSFER,caster->level,s->archname);
     break;
   case SP_TRANSFER:
     success = cast_transfer(op,dir);
@@ -560,7 +672,7 @@ if (item == spellNormal && !ability ){
     { int total_sp_cost, spellinrune;
       spellinrune=look_up_spell_by_name(op,stringarg);
       if(spellinrune!=-1) {
-      total_sp_cost=SP_level_spellpoint_cost(op,spellinrune)
+      total_sp_cost=SP_level_spellpoint_cost(op,caster,spellinrune)
 			+spells[spellinrune].sp;
 	if(op->stats.sp<total_sp_cost) {
 	  new_draw_info(NDI_UNIQUE, 0,op,"Not enough spellpoints.");
@@ -570,7 +682,7 @@ if (item == spellNormal && !ability ){
 #endif
 	  return 0;
 	}
-	success=write_rune(op,dir,spellinrune,op->level,stringarg);
+	success=write_rune(op,dir,spellinrune,caster->level,stringarg);
 	return (success ? total_sp_cost : 0);
       }
 #ifdef CASTING_TIME
@@ -588,26 +700,43 @@ if (item == spellNormal && !ability ){
 #endif
 
 
+    break;
+  case SP_LIGHT:
+    success = cast_light(op,caster,dir);
+    break;
+  case SP_DAYLIGHT:
+    success = cast_daylight(op); 
+    break; 
+  case SP_NIGHTFALL:
+    success = cast_nightfall(op);
+    break;
+  case SP_FAERY_FIRE:
+    success = cast_faery_fire(op,caster);
     break;
   case SP_CAUSE_LIGHT:
   case SP_CAUSE_HEAVY:
   case SP_CAUSE_MEDIUM:
-	success = fire_arch(op,dir,spellarch[type],type,1); /* don't want to OR magic */
+  case SP_CAUSE_CRITICAL:
+	success = fire_arch(op,caster,dir,spellarch[type],type,1); /* don't want to OR magic */
   break;
   case SP_SUMMON_FOG:
-	success = summon_fog(op,dir,type);
+	success = summon_fog(op,caster,dir,type);
 	break;
   case SP_PACIFY:
 	cast_pacify(op,caster,spellarch[type],type);
 	success = 1;
   	break;
+  case SP_COMMAND_UNDEAD:
+	cast_charm_undead(op,caster,spellarch[type],type);
+	success = 1;
+  	break;
   case SP_CHARM:
-	cast_charm(op,spellarch[type],type);
+	cast_charm(op,caster,spellarch[type],type);
 	success = 1;
 	break;
   /* huma */
   case SP_CREATE_MISSILE:
-    success = cast_create_missile(op,dir,stringarg);
+    success = cast_create_missile(op,caster,dir,stringarg);
     break;
 
   }
@@ -620,14 +749,14 @@ if (item == spellNormal && !ability ){
 #endif
 
 #ifdef SPELLPOINT_LEVEL_DEPEND
-  return success?SP_level_spellpoint_cost(op,type):0;
+  return success?SP_level_spellpoint_cost(op,caster,type):0;
 #else
   return success?(s->sp*PATH_SP_MULT(op,s)):0;
 #endif
 }
 
 
-int cast_create_obj(object *op,object *new_op, int dir)
+int cast_create_obj(object *op,object *caster,object *new_op, int dir)
 {
   if(dir && blocked(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir])) {
     new_draw_info(NDI_UNIQUE, 0,op,"Something is in the way.");
@@ -640,7 +769,7 @@ int cast_create_obj(object *op,object *new_op, int dir)
   return dir;
 }
 
-int summon_monster(object *op,int dir,archetype *at,int spellnum) {
+int summon_monster(object *op,object *caster,int dir,archetype *at,int spellnum) {
   object *tmp;
   if(op->type==PLAYER)
     if(op->contr->golem!=NULL&&!QUERY_FLAG(op->contr->golem,FLAG_FREED)) {
@@ -679,12 +808,12 @@ int summon_monster(object *op,int dir,archetype *at,int spellnum) {
   }
   /*  This sets the level dependencies on dam and hp for monsters */
   tmp->stats.hp = SP_PARAMETERS[spellnum].bdur +
-			 10 * SP_level_strength_adjust(op,spellnum);
+			 10 * SP_level_strength_adjust(op,caster,spellnum);
   tmp->stats.dam= SP_PARAMETERS[spellnum].bdam +
-			 2* SP_level_dam_adjust(op,spellnum);
+			 2* SP_level_dam_adjust(op,caster,spellnum);
   if(tmp->stats.dam<0) tmp->stats.dam=127;  /*seen this go negative!*/
  /*  make experience increase in proportion to the strength of the summoned creature. */
-  tmp->stats.exp *= SP_level_spellpoint_cost(op,spellnum)/spells[spellnum].sp;
+  tmp->stats.exp *= SP_level_spellpoint_cost(op,caster,spellnum)/spells[spellnum].sp;
   tmp->speed_left= -1;
   tmp->x=op->x+freearr_x[dir],tmp->y=op->y+freearr_y[dir];
   tmp->direction=dir;
@@ -709,7 +838,7 @@ int ok_to_put_more(mapstruct *m,int x,int y,object *op,int immune_stop) {
 
 
 
-int fire_bolt(object *op,int dir,int type,int magic) {
+int fire_bolt(object *op,object *caster,int dir,int type,int magic) {
   object *tmp=NULL;
   if (!spellarch[type])
     return 0;
@@ -717,8 +846,8 @@ int fire_bolt(object *op,int dir,int type,int magic) {
   if(tmp==NULL)
     return 0;
   /*  peterm:  level dependency for bolts  */
-  tmp->stats.dam = SP_PARAMETERS[type].bdam + SP_level_dam_adjust(op,type);
-  tmp->stats.hp = SP_PARAMETERS[type].bdur + SP_level_strength_adjust(op,type);
+  tmp->stats.dam = SP_PARAMETERS[type].bdam + SP_level_dam_adjust(op,caster,type);
+  tmp->stats.hp = SP_PARAMETERS[type].bdur + SP_level_strength_adjust(op,caster,type);
   if(magic)
     tmp->attacktype|=AT_MAGIC;
   tmp->x=op->x,tmp->y=op->y;
@@ -751,8 +880,9 @@ int fire_bolt(object *op,int dir,int type,int magic) {
     Later added a ball-lightning firing routine.
  */
 
-int fire_arch(object *op,int dir,archetype *at, int type, int magic) {
+int fire_arch(object *op,object *caster,int dir,archetype *at, int type, int magic) {
   object *tmp, *env;
+ 
   if(at==NULL)
     return 0;
   for(env=op;env->env!=NULL;env=env->env);
@@ -762,10 +892,15 @@ int fire_arch(object *op,int dir,archetype *at, int type, int magic) {
   if(tmp==NULL)
     return 0;
   tmp->stats.sp=type;
-  tmp->stats.dam=SP_PARAMETERS[type].bdam+SP_level_dam_adjust(op,type);
-  tmp->stats.hp=SP_PARAMETERS[type].bdur+SP_level_strength_adjust(op,type);
+  tmp->stats.dam=SP_PARAMETERS[type].bdam+SP_level_dam_adjust(op,caster,type);
+  tmp->stats.hp=SP_PARAMETERS[type].bdur+SP_level_strength_adjust(op,caster,type);
   tmp->x=op->x,tmp->y=op->y;
   tmp->direction=dir;
+  set_owner(tmp,op);
+#ifdef MULTIPLE_GODS /* needed for AT_HOLYWORD,AT_GODPOWER stuff */
+  if(tmp->attacktype&AT_HOLYWORD||tmp->attacktype&AT_GODPOWER)
+	      if(!tailor_god_spell(tmp,op)) return 0; 
+#endif 
   if(magic)
     tmp->attacktype|=AT_MAGIC;
   if(QUERY_FLAG(tmp, FLAG_IS_TURNABLE))
@@ -774,7 +909,6 @@ int fire_arch(object *op,int dir,archetype *at, int type, int magic) {
   if(op->type==PLAYER)
     tmp->stats.hp=(op->contr->shootstrength-10)/10+10;
 #endif
-  set_owner(tmp,op);
   if(op->type==PLAYER)
     D_LOCK(op);
   insert_ob_in_map(tmp,op->map);
@@ -788,7 +922,7 @@ int fire_arch(object *op,int dir,archetype *at, int type, int magic) {
 	break;
     case SP_BALL_LIGHTNING:
 	tmp->stats.food=SP_PARAMETERS[type].bdur +
-                      SP_level_strength_adjust(op,type);
+                      SP_level_strength_adjust(op,caster,type);
 	move_ball_lightning(tmp);
 	break;
     default:
@@ -800,10 +934,11 @@ int fire_arch(object *op,int dir,archetype *at, int type, int magic) {
 }
 
 int
-cast_cone(object *op, int dir, int strength, int spell_type,archetype *spell_arch, int magic)
+cast_cone(object *op, object *caster,int dir, int strength, int spell_type,archetype *spell_arch, int magic)
 {
   object *tmp;
   int i,success=0,range_min= -1,range_max=1;
+
   if(!dir)
     range_min= -3,range_max=4,strength/=2;
 
@@ -815,10 +950,15 @@ cast_cone(object *op, int dir, int strength, int spell_type,archetype *spell_arc
     success=1;
     tmp=arch_to_object(spell_arch);
     set_owner(tmp,op);
-    tmp->level=op->level;  /*  Make face of death work? */
+/*  Make face of death work? */
+    /* tmp->level=op->level;  */
+    tmp->level=SK_level(op); /* need to use the cleric level, not overall value -b.t. */ 
     tmp->x=x,tmp->y=y;
-    /*if (tmp->type != CONE)
-      tmp->type = CONE; */
+#ifdef MULTIPLE_GODS /* holy word stuff */                
+    if((tmp->attacktype&AT_HOLYWORD)||(tmp->attacktype&AT_GODPOWER))
+            if(!tailor_god_spell(tmp,op)) return 0;  
+    else /* god/holy word isnt really 'magic' */
+#endif
     if(magic)
       tmp->attacktype|=AT_MAGIC;  /* JWI cone attacks should be considered
                                      magical in nature ;) */
@@ -826,9 +966,9 @@ cast_cone(object *op, int dir, int strength, int spell_type,archetype *spell_arc
       tmp->stats.sp=dir;
     else
       tmp->stats.sp=i;
-    tmp->stats.hp=strength+SP_level_strength_adjust(op,spell_type);
+    tmp->stats.hp=strength+SP_level_strength_adjust(op,caster,spell_type);
     tmp->stats.dam=SP_PARAMETERS[spell_type].bdam +
-                  SP_level_dam_adjust(op,spell_type); 
+                  SP_level_dam_adjust(op,caster,spell_type); 
     tmp->stats.maxhp=tmp->count;
     SET_FLAG(tmp, FLAG_FLYING);
     insert_ob_in_map(tmp,op->map);
@@ -839,6 +979,13 @@ cast_cone(object *op, int dir, int strength, int spell_type,archetype *spell_arc
 void move_cone(object *op) {
   int i;
 
+  /* if no map then hit_map will crash so just ignore object */
+  if (! op->map) {
+    LOG(llevError,"Tried to move_cone object %s without a map.\n",
+      op->name ? op->name : "unknown");
+    return;
+  }
+
   if (QUERY_FLAG(op, FLAG_LIFESAVE)) {/* lava saves it's life, but not yours  :) */
       hit_map(op,0,op->attacktype);
       return;
@@ -874,7 +1021,13 @@ void move_cone(object *op) {
       object *tmp=arch_to_object(op->arch);
       set_owner(tmp,op->owner);
       tmp->x=x, tmp->y=y;
-      tmp->level=op->level;  /* added to make face of death work,and counterspell */
+/* added to make face of death work,and counterspell */
+      /* tmp->level=op->level;*/  
+      tmp->level=SK_level(op);  
+#ifdef MULTIPLE_GODS /* holy word stuff */
+      if(tmp->attacktype&AT_HOLYWORD||tmp->attacktype&AT_GODPOWER) 
+            if(!tailor_god_spell(tmp,op)) return;
+#endif
       tmp->stats.sp=op->stats.sp,tmp->stats.hp=op->stats.hp+1;
       tmp->stats.maxhp=op->stats.maxhp;
       tmp->attacktype=op->attacktype;
@@ -1014,7 +1167,10 @@ void move_bolt(object *op) {
   }
 }
 
+/* updated this to allow more than the golem 'head' to attack */
 void move_golem(object *op) {
+  int made_attack=0;
+  object *tmp;
   if(QUERY_FLAG(op, FLAG_MONSTER))
     return; /* Has already been moved */
   if(get_owner(op)==NULL) {
@@ -1024,19 +1180,57 @@ void move_golem(object *op) {
     return;
   }
   if(--op->stats.hp<0) {
-    new_draw_info(NDI_UNIQUE, 0,op->owner,"Your golem dissolved.");
+    char buf[MAX_BUF];
+    sprintf(buf,"Your %s dissolved.",op->name);
+    if(op->exp_obj && op->exp_obj->stats.Wis)
+        sprintf(buf,"Your %s departed this plane.",op->name);
+    new_draw_info(NDI_UNIQUE, 0,op->owner,buf);
     remove_friendly_object(op);
     op->owner->contr->golem=NULL;
     remove_ob(op);
     free_object(op);
     return;
   }
-  if(!move_ob(op,op->direction)&&
-     !out_of_map(op->map,op->x+freearr_x[op->direction],
-                 op->y+freearr_y[op->direction])) {
-    update_object(op);
-    hit_map(op,op->direction,AT_PHYSICAL);
+
+  /* Do golem attacks/movement for single & multisq golems. 
+   * Assuming here that op is the 'head' object. Pass only op to 
+   * move_ob (makes recursive calls to other parts) 
+   */
+  if(!move_ob(op,op->direction))
+    for(tmp=op;tmp;tmp=tmp->more) { 
+      int x=tmp->x+freearr_x[op->direction],y=tmp->y+freearr_y[op->direction];
+      if(!out_of_map(op->map,x,y)&&!will_hit_self(op,x,y)) {
+        object *victim;
+        for(victim=get_map_ob(op->map,x,y);victim;victim=victim->above) 
+             if(QUERY_FLAG(victim,FLAG_ALIVE)) break;
+
+        if(victim) {
+	  /* for golems with race fields, we don't attack
+	   * aligned races */
+	  if(victim->race&&op->race&&strstr(op->race,victim->race)) {
+	    if(op->owner) new_draw_info_format(NDI_UNIQUE, 0,op->owner,
+		    "%s avoids damaging %s.",op->name,victim->name);
+	  } else { 
+            hit_map(tmp,op->direction,op->attacktype);
+            made_attack=1;
+	  }
+	}
+      }
+    }
+  if(made_attack) update_object(op);
+}
+
+/* check to see if the monster will hit itself at
+ * location x,y 
+ */
+int will_hit_self(object *head, int x, int y) {
+  object *tmp;
+
+  for(tmp=head;tmp;tmp=tmp->more) { 
+    int cx=tmp->arch->clone.x, cy=tmp->arch->clone.y;
+    if((cx+head->x==x)&&(cy+head->y==y)) return 1; 
   }
+  return 0;
 }
 
 void control_golem(object *op,int dir) {
@@ -1108,15 +1302,26 @@ int explode_object(object *op) {
 
   /*  op->stats.sp stores the spell which made this object here. */
   if(op->owner)
-      tmp->stats.dam += SP_level_dam_adjust(op->owner,op->stats.sp);
+      tmp->stats.dam += SP_level_dam_adjust(op->owner,op->owner,op->stats.sp);
   if(op->attacktype&AT_MAGIC)
     tmp->attacktype|=AT_MAGIC;
-  if((owner = get_owner(op)) != (object *) NULL)
+  if((owner = get_owner(op)) != (object *) NULL) {
     set_owner(tmp,owner);
+    if(op->chosen_skill && (op->chosen_skill != tmp->chosen_skill)){
+          tmp->exp_obj = op->exp_obj;
+          tmp->chosen_skill = op->chosen_skill;
+    }
+  }
   if(op->stats.hp)
     tmp->stats.hp=op->stats.hp;
   tmp->stats.maxhp=op->count; /* Unique ID */
   tmp->x=env->x,tmp->y=env->y;
+
+#ifdef MULTIPLE_GODS /* needed for AT_HOLYWORD stuff -b.t. */
+  if(tmp->attacktype&AT_HOLYWORD||tmp->attacktype&AT_GODPOWER) 
+          if(!tailor_god_spell(tmp,op)) return 0;   
+#endif
+
   if (wall(env->map,env->x,env->y))
     tmp->x-=DIRX(env),tmp->y-=DIRY(env);
   if (out_of_map(env->map, env->x, env->y))
@@ -1131,6 +1336,10 @@ void check_fired_arch(object *op) {
   if(blocked(op->map,op->x,op->y)) {
     object *tmp;
     remove_ob(op);
+    if(out_of_map(op->map,op->x,op->y)) {
+	free_object(op);
+	return;
+    }
     if(explode_object(op))
       return;
     for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1173,6 +1382,10 @@ void move_fired_arch(object *op) {
   }
   if(blocked(op->map,op->x,op->y)) {
     object *tmp;
+    if(out_of_map(op->map,op->x,op->y)) {
+	free_object(op);
+	return;
+    }
     if(explode_object(op))
       return;
     for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1228,12 +1441,17 @@ object *find_target_for_friendly_spell(object *op,int dir)
      */
     if(!tmp || !QUERY_FLAG(tmp,FLAG_MONSTER)) tmp=op;
   }
-  else
-    for(tmp=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
+  else {
+    if (out_of_map(op->map, op->x+freearr_x[dir],op->y+freearr_y[dir]))
+	tmp=NULL;
+    else  {
+      for(tmp=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
         tmp!=NULL;
         tmp=tmp->above)
       if(tmp->type==PLAYER)
         break;
+    }
+  }
   if(tmp==NULL)               /* didn't find a player there, look in current square for a player */
     for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
       if(tmp->type==PLAYER)
@@ -1335,9 +1553,18 @@ for turning undead and whatnot.
 
   The arrays are defined in spells.h*/
 
-int SP_level_dam_adjust(object *op, int spell_type)
+/* July 1995 - I changed the next 3 functions slightly by replacing
+ * the casters level (op->level) with the skill level (SK_level(op)) 
+ * instead for when we have compiled with ALLOW_SKILLS - b.t. 
+ */
+
+int SP_level_dam_adjust(object *op, object *caster, int spell_type)
 {  int adj;
-   int level=op->level+path_level_mod(op, spell_type);
+#ifdef ALLOW_SKILLS
+   int level=SK_level(op)+path_level_mod(caster, spell_type);
+#else
+   int level=op->level+path_level_mod(caster, spell_type);
+#endif
 
     adj=(level-spells[spell_type].level);
     if(adj < 0) adj=0;
@@ -1347,10 +1574,15 @@ int SP_level_dam_adjust(object *op, int spell_type)
  return adj;
 }
 
-int SP_level_strength_adjust(object *op, int spell_type)
-{  int adj;
-   int level=op->level+path_level_mod(op, spell_type);
+/* July 1995 - changed slightly (SK_level) for ALLOW_SKILLS - b.t. */
 
+int SP_level_strength_adjust(object *op, object *caster, int spell_type)
+{  int adj;
+#ifdef ALLOW_SKILLS
+   int level=SK_level(op)+path_level_mod(caster, spell_type);
+#else
+   int level=op->level+path_level_mod(caster, spell_type);
+#endif
     adj= (level-spells[spell_type].level);
     if(adj < 0) adj=0;
     if(SP_PARAMETERS[spell_type].ldur)
@@ -1365,11 +1597,17 @@ lower level spells become incredibly vicious at high
 levels.  Very cheap mass destruction.  This function is
 intended to keep the sp cost related to the effectiveness. */
 
-int SP_level_spellpoint_cost(object *op, int spell_type)
+/* July 1995 - changed slightly (SK_level) for ALLOW_SKILLS - b.t. */
+
+int SP_level_spellpoint_cost(object *op, object *caster, int spell_type)
 {
   spell *s=find_spell(spell_type);
+# ifdef ALLOW_SKILLS 
+  int level=SK_level(op)+path_level_mod(caster, spell_type);
+#else
+  int level=op->level+path_level_mod(caster, spell_type);
+#endif /* ALLOW_SKILLS */
 #ifdef SPELLPOINT_LEVEL_DEPEND
-  int level=op->level+path_level_mod(op, spell_type);
   int sp;
   if(SP_PARAMETERS[spell_type].spl)
    sp= (int) (spells[spell_type].sp * 
@@ -1377,10 +1615,11 @@ int SP_level_spellpoint_cost(object *op, int spell_type)
 	(MAX(0,	(float)(level-spells[spell_type].level)/
 	(float)SP_PARAMETERS[spell_type].spl ))));
   else sp= spells[spell_type].sp;
-  sp *= PATH_SP_MULT(op,s);
+  sp *= PATH_SP_MULT(caster,s);
   return MIN(sp,(spells[spell_type].sp + 50));
-#endif  
-  return s->sp*PATH_SP_MULT(op,s);
+#else
+  return s->sp*PATH_SP_MULT(caster,s);
+#endif /* SPELLPOINT_LEVEL_DEPEND */  
 }
 
 
@@ -1408,7 +1647,7 @@ void move_swarm_spell(object *op)
 /*  for level dependence, we need to know what spell is fired.  */
 /*  that's stored in op->stats.sp  by fire_swarm  */
    if(!wall(op->map,op->x,op->y))
-   fire_arch(op,op->direction,op->other_arch,op->stats.sp,0);
+   fire_arch(op,op,op->direction,op->other_arch,op->stats.sp,0);
    op->x=x; op->y=y;  /* reset original location */
 
 }
@@ -1489,7 +1728,7 @@ void put_a_monster(object *op,char *monstername) {
   /* first we check the closest square for free squares */
   if((at=find_archetype(monstername))==NULL) return;
   dir=find_first_free_spot(at,op->map,op->x,op->y);
-  if((tmp=arch_to_object(at))!=NULL) {
+  if(dir!=-1 && (tmp=arch_to_object(at))!=NULL) {
     nx=op->x+freearr_x[dir];
     ny=op->y+freearr_y[dir];
     tmp->x=nx;
@@ -1678,7 +1917,7 @@ void init_spell_param()
   ATTACKS[21].fg=color_array[black];
   ATTACKS[21].bg=color_array[khaki];  
 
-  sprintf(fname,"%s/%s",LibDir,"spell_params");
+  sprintf(fname,"%s/spell_params",settings.libdir);
   if(! (spell_params=fopen(fname,"r")))
     {
 	perror(fname);
@@ -1703,4 +1942,88 @@ void init_spell_param()
         SP_PARAMETERS[spellindex].ldur=ldur;
 	SP_PARAMETERS[spellindex].spl=spl;
     }
+    fclose(spell_params);
 }
+
+/* get_pointed_target() - this is used by finger of death
+ * and the 'smite' spells. Returns the pointer to the first
+ * monster in the direction which is pointed to by op. b.t.
+ */
+ 
+object *get_pointed_target(object *op, int dir) {
+  object *target;
+  int x,y;
+
+  if (dir==0) return NULL;
+  for(x=op->x+freearr_x[dir],y=op->y+freearr_y[dir]
+     ;!out_of_map(op->map,x,y)&&!blocks_view(op->map,x,y)
+     &&!wall(op->map,x,y);x+=freearr_x[dir],y+=freearr_y[dir])
+        for(target=get_map_ob(op->map,x,y);target;target=target->above) {
+              if(QUERY_FLAG(target->head?target->head:target,FLAG_MONSTER))
+                   if(!blocks_magic(op->map,x,y))
+                          return target;
+                   else break;
+        }
+ 
+  return ((object *) NULL);
+}
+
+/* cast_smite_arch() - the priest points to a creature and causes
+ * a 'godly curse' to decend. I generalized this a bit so that several
+ * spells will be possible to use w/ this code (eg fire_arch, cast_cone).
+ * -b.t.
+ */
+
+int cast_smite_spell (object *op, object *caster,int dir, int type) {
+   object *effect, *target = get_pointed_target(op,dir);
+#ifdef MULTIPLE_GODS
+   object *god = find_god(determine_god(op));
+#endif
+
+   if(!target || QUERY_FLAG(target,FLAG_REFL_SPELL)
+#ifdef MULTIPLE_GODS /* if we don't worship a god, or target a creature
+                      * of our god, the spell will fail.  */
+      ||!god
+      ||(target->title&&!strcmp(target->title,god->name))
+      ||(target->race&&strstr(target->race,god->race))
+#endif
+   ) {
+        new_draw_info(NDI_UNIQUE,0,op,"Your request is unheeded.");
+        return 0;
+   }
+
+   if (spellarch[type] != (archetype *) NULL)
+      effect = arch_to_object(spellarch[type]);
+   else
+      return 0;
+
+  /* tailor the effect by priest level and worshipped God */
+   effect->level=SK_level(op);
+#ifdef MULTIPLE_GODS
+   if(effect->attacktype&AT_HOLYWORD||effect->attacktype&AT_GODPOWER)
+        if(tailor_god_spell(effect,op))
+           new_draw_info_format(NDI_UNIQUE,0,op,
+                "%s answers your call!",determine_god(op));
+        else {
+           new_draw_info(NDI_UNIQUE,0,op,"Your request is ignored.");
+           return 0;
+        }
+#endif 
+ 
+   /* size of the area of destruction */
+   effect->stats.hp=SP_PARAMETERS[type].bdur +
+                SP_level_strength_adjust(op,caster,type);
+   /* how much woe to inflict :) */
+   effect->stats.dam=SP_PARAMETERS[type].bdam +
+                SP_level_dam_adjust(op,caster,type);
+   if(effect->stats.dam<0) effect->stats.dam = 127;
+   effect->stats.maxhp=effect->count; /*??*/
+   set_owner(effect,op);
+ 
+   /* ok, tell it where to be, and insert! */
+   effect->x=target->x;effect->y=target->y;
+   insert_ob_in_map(effect,op->map);
+ 
+   return 1;
+}
+
