@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_swap_c =
- *    "$Id: swap.c,v 1.10 1994/10/07 10:07:42 master Exp $";
+ *    "$Id: swap.c,v 1.11 1997/03/09 08:20:06 master Exp master $";
  */
 
 /*
@@ -31,6 +31,92 @@
 #endif
 #include <object.h>
 
+/* This writes out information on all the temporary maps.  It is called by
+ * swap_map below.
+ */
+static void write_map_log() 
+{
+    FILE *fp;
+    mapstruct *map;
+    char buf[MAX_BUF];
+    long current_time=time(NULL);
+
+    sprintf(buf,"%s/temp.maps", settings.libdir);
+    if (!(fp=fopen(buf,"w"))) {
+	LOG(llevError,"Could not open %s for writing\n", buf);
+	return;
+    }
+    for (map=first_map; map!=NULL; map=map->next) {
+	if (map->in_memory != MAP_IN_MEMORY)
+/* The format is the same (so recompiling won't really affect this.
+ * However, in one case that element of the structure is available, so
+ * put it in.
+ */
+#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
+	    fprintf(fp,"%s:%s:%ld:%d:%d:%d:%d:%d\n", map->path, map->tmpname,
+		    map->reset_time-current_time, map->lock_items,
+		    map->encounter, map->difficulty, map->do_los,
+		    map->darkness);
+#else
+	    fprintf(fp,"%s:%s:%ld:0:%d:%d:%d:%d\n", map->path, map->tmpname,
+		    map->reset_time-current_time, map->lock_items,
+		    map->encounter, map->difficulty, map->do_los,
+		    map->darkness);
+#endif
+    }
+    fclose(fp);
+}
+
+void read_map_log() 
+{
+    FILE *fp;
+    mapstruct *map;
+    char buf[MAX_BUF],*cp,*cp1;
+    long current_time=time(NULL);
+    int encounter, do_los, darkness;
+
+    sprintf(buf,"%s/temp.maps", settings.libdir);
+    if (!(fp=fopen(buf,"r"))) {
+	LOG(llevDebug,"Could not open %s for reading\n", buf);
+	return;
+    }
+    while (fgets(buf, MAX_BUF, fp)!=NULL) {
+	map=get_linked_map(NULL);
+	/* scanf doesn't work all that great on strings, so we break
+	 * out that manually.  strdup is used for tmpname, since other
+	 * routines will try to free that pointer.
+	 */
+	cp=strchr(buf,':');
+	*cp++='\0';
+	strcpy(map->path, buf);
+	cp1=strchr(cp,':');
+	*cp1++='\0';
+	map->tmpname=strdup_local(cp);
+/* The format is the same (so recompiling won't really affect this.
+ * However, in one case that element of the structure is available, so
+ * put it in.
+ */
+#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
+	sscanf(cp1,"%ld:%d:%d:%hd:%d:%d\n",
+		    &map->reset_time, &map->lock_items,
+		    &encounter, &map->difficulty, &do_los,
+		    &darkness);
+#else
+	sscanf(cp1,"%ld:%d:%d:%hd:%d:%d\n",
+		    &map->reset_time, &map->lock_items,
+		    &encounter, &map->difficulty, &do_los,
+		    &darkness);
+#endif
+	map->reset_time += current_time;
+	map->in_memory=MAP_SWAPPED;
+	map->darkness=darkness;
+	/* The following two are bit fields, so can't be done above */
+	map->encounter=encounter;
+	map->do_los=do_los;
+    }
+    fclose(fp);
+}
+
 void swap_map(mapstruct *map) {
   player *pl;
   if(map->in_memory != MAP_IN_MEMORY) {
@@ -55,6 +141,9 @@ void swap_map(mapstruct *map) {
     delete_map(map);
   } else
     free_map(map,1);
+#ifdef RECYCLE_TMP_MAPS
+  write_map_log();
+#endif
 }
 
 void check_active_maps() {
