@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.29 1995/09/11 03:23:21 master Exp master $";
+ *    "$Id: time.c,v 1.33 1997/03/09 08:20:46 master Exp master $";
  */
 
 /*
@@ -89,10 +89,11 @@ void generate_monster(object *gen) {
     op->y=gen->y+freearr_y[i]+at->clone.y;
     if(head!=NULL)
       op->head=head,prev->more=op;
+    if (RANDOM()%10) generate_artifact(op, gen->map->difficulty);
     insert_ob_in_map(op,gen->map);
     if (QUERY_FLAG(op, FLAG_FREED)) return;
-    if(op->arch->randomitems!=NULL)
-      create_treasure(op->arch->randomitems,op,GT_INVENTORY,
+    if(op->randomitems!=NULL)
+      create_treasure(op->randomitems,op,GT_INVENTORY,
                       gen->map->difficulty,0);
     if(head==NULL)
       head=op;
@@ -130,6 +131,18 @@ void remove_force(object *op) {
   free_object(op);
 }
 
+void remove_blindness(object *op) {
+  if(--op->stats.food > 0)
+    return;
+  CLEAR_FLAG(op, FLAG_APPLIED);
+  if(op->env!=NULL) { 
+     change_abil(op->env,op);
+     fix_player(op->env);
+  }
+  remove_ob(op);
+  free_object(op);
+}
+
 void remove_confusion(object *op) {
   if(--op->stats.food > 0)
     return;
@@ -327,7 +340,14 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
 }
 
 
+/* stop_arrow() - what to do when a non-living flying object
+ * has to stop. Sept 96 - I added in thrown object code in 
+ * here too. -b.t. 
+ * op is arrow, tmp is what is stopping the arrow (can be NULL)
+ */
+
 void stop_arrow(object *op,object *tmp) {
+
   if(wall(op->map,op->x,op->y))
     op->x-=DIRX(op),op->y-=DIRY(op);
   if(wall(op->map, op->x, op->y))
@@ -352,10 +372,27 @@ void stop_arrow(object *op,object *tmp) {
   op->stats.sp = 0;
   op->stats.hp = 0;
   op->face=op->arch->clone.face;
-  if(tmp!=NULL&&tmp->stats.hp>=0) {
+
+  /* this happens for thrown objects, which 'carry' the
+   * real object in inventory. */ 
+  if(op->inv) {
+    object *old=op,*new=op->inv;
+    remove_ob(op->inv);
+    new->map = old->map; 
+    free_object(old);
+    op = new;
+  }
+
+  /* If the missile hit a player, we insert it in their inventory.
+   * However, if the missile is heavy, we don't do so (assume it falls
+   * to the ground after a hit).  What a good value for this is up to
+   * debate - 5000 is 5 kg, so arrows, knives, and other light weapons
+   * stick around.
+   */
+  if(op->weight <= 5000 && tmp!=NULL&&tmp->stats.hp>=0) {
     if(tmp->head != NULL)
       tmp = tmp->head;
-    op = insert_ob_in_ob(op,tmp);
+      op = insert_ob_in_ob(op,tmp);
     if (tmp->type== PLAYER && tmp->contr->eric_server > 0)
       esrv_send_item (tmp, op);
   } else
@@ -363,6 +400,7 @@ void stop_arrow(object *op,object *tmp) {
   op->owner=NULL; /* So that stopped arrows will be saved */
 }
 
+/* now involves thrown objects too */
 void move_arrow(object *op) {
   object *tmp;
 
@@ -373,6 +411,15 @@ void move_arrow(object *op) {
     return;
   }
   remove_ob(op);
+
+  /* we need to stop thrown objects at some point. Like here. */ 
+  if(op->type==THROWN_OBJ) {
+     if(op->last_sp-- < 0) { 
+ 	stop_arrow(op, NULL); 
+	return; 
+     } 
+  }
+
   if(wall(op->map,op->x+DIRX(op),op->y+DIRY(op)))
     if(!QUERY_FLAG(op, FLAG_REFLECTING) || !(RANDOM()%20)) {
       stop_arrow(op,NULL);
@@ -407,12 +454,15 @@ void move_arrow(object *op) {
 	    return;
 	  }
 	}
-      op->face=&new_faces[op->arch->faces[op->direction]];
+      /* many thrown objects *don't* have more than one face */
+      if(op->arch->faces)
+	op->face=&new_faces[op->arch->faces[op->direction]];
   }
   op->x+=DIRX(op),op->y+=DIRY(op);
   tmp=get_map_ob(op->map,op->x,op->y);
   while(tmp!=NULL&&!QUERY_FLAG(tmp, FLAG_ALIVE))
     tmp=tmp->above;
+
   if(tmp==NULL) {
     insert_ob_in_map(op,op->map);
     return;
@@ -420,105 +470,59 @@ void move_arrow(object *op) {
   if (QUERY_FLAG(tmp, FLAG_REFL_MISSILE)){
     int number = op->face->number;
     op->direction=absdir(op->direction+4),op->state=0;
-    number+=4;
-    if(number > op->arch->faces[8])
-      number-=8;
-    op->face = &new_faces[number];
+    if(op->arch->faces) { 
+      number+=4;
+      if(number > op->arch->faces[8])
+        number-=8;
+      op->face = &new_faces[number];
+    }
     insert_ob_in_map(op,op->map);
     return;
   }
   if(attack_ob(tmp,op))
     stop_arrow(op,tmp);
-  else
+  else 
     if(op->direction)
       insert_ob_in_map(op,op->map);
-    else
+    else 
       stop_arrow(op,NULL);
 }
 
-#if 0
-void stop_thrown(object *op,object *tmp) {
-  if(wall(op->map,op->x,op->y))
-    op->x-=DIRX(op),op->y-=DIRY(op);
-  op->direction=0;
-  UNSET_FLY(op);
-  if(op->type!=BOMB) {
-    update_ob_speed(op,0);
-    op->speed_left= -2;
-  }
-  insert_ob_in_map(op,op->map);
-  if(tmp!=NULL&&tmp->stats.hp>=0) { /* Hack to get monster on top */
-    remove_ob(tmp);
-    insert_ob_in_map(tmp,tmp->map);
-  }
-}
-#endif
-
-#if 0 /* This routine is buggy, and removed for now */
-void move_thrown(object *op) {
-  object *tmp;
-  int roll;
-  char buf[MAX_BUF];
-
-  remove_ob(op);
-  if(op->thrown--<1) {
-    stop_thrown(op,NULL);
-    return;
-  }
-  op->x+=DIRX(op),op->y+=DIRY(op);
-  if(out_of_map(op->map,op->x,op->y)) {
-    LOG(llevError,"Thrown object out of map.\n");
-    stop_thrown(op,NULL);
-    remove_ob(op);
-    free_object(op);
-    return;
-  }
-  if(wall(op->map,op->x,op->y)) {
-    stop_thrown(op,NULL);
-    return;
-  }
-  tmp=get_map_ob(op->map,op->x,op->y);
-  while(tmp!=NULL&&!QUERY_FLAG(tmp, FLAG_ALIVE))
-    tmp=tmp->above;
-  if(tmp==NULL) {
-    insert_ob_in_map(op,op->map);
-    return;
-  }
-  roll=RANDOM()%20+1;
-  if(roll==20||tmp->stats.ac>=op->thrownthaco-roll) {
-    sprintf(buf,"%s hits you with %s.",op->owner->name,op->name);
-    new_draw_info(NDI_UNIQUE, 0,tmp,buf);
-    new_draw_info_format(NDI_ALL, 10, NULL,
-	"%s hits %s with %s.",op->owner->name,tmp->name,op->name);
-    hit_player(tmp,RANDOM()%(op->weight>50000?11:op->weight/5000+1),op->owner,
-               AT_PHYSICAL);
-    stop_thrown(op,tmp);
-    return;
-  } else
-  if(op->thrownthaco-roll<5) {
-    new_draw_info_format(NDI_ALL, 10, NULL,
-	"%s's %s hits %s's armour.",op->owner->name,op->name,tmp->name);
-    stop_thrown(op,tmp);
-    return;
-  }
-  insert_ob_in_map(op,op->map);
-}
-#endif
+/* This routine doesnt seem to work for "inanimate" objects that
+ * are being carried, ie a held torch leaps from your hands!. 
+ * Modified this routine to allow held objects. b.t. */
 
 void change_object(object *op) { /* Doesn`t handle linked objs yet */
-  object *tmp;
+  object *tmp,*env;
   int i,j;
+
   if(op->other_arch==NULL) {
     LOG(llevError,"Change object (%s) without other_arch error.\n", op->name);
     return;
   }
+
+  /* In non-living items only change when food value is 0 */
+  if(!QUERY_FLAG(op,FLAG_ALIVE)) {
+	if(op->stats.food-- > 0) return; 
+	else op->stats.food=1; /* so 1 other_arch is made */
+  }
+  env=op->env;
   remove_ob(op);
   for(i=0;i<NROFNEWOBJS(op);i++) {
     tmp=arch_to_object(op->other_arch);
     tmp->stats.hp=op->stats.hp; /* The only variable it keeps. */
-    j=find_first_free_spot(tmp->arch,op->map,op->x,op->y);
-    tmp->x=op->x+freearr_x[j],tmp->y=op->y+freearr_y[j];
-    insert_ob_in_map(tmp,op->map);
+    if(env) {
+        tmp->x=env->x,tmp->y=env->y;
+	insert_ob_in_ob(tmp,env);
+    } else {
+        j=find_first_free_spot(tmp->arch,op->map,op->x,op->y);
+	if (j==-1)  /* No free spot */
+	    free(tmp);
+	else {
+	    tmp->x=op->x+freearr_x[j],tmp->y=op->y+freearr_y[j];
+	    insert_ob_in_map(tmp,op->map);
+	}
+    }
   }
   free_object(op);
 }
@@ -626,8 +630,9 @@ void move_creator(object *op) {
   
 
 int process_object(object *op) {
+
   if(QUERY_FLAG(op, FLAG_MONSTER))
-    if(move_monster(op) || QUERY_FLAG(op, FLAG_FREED))
+    if(move_monster(op) || QUERY_FLAG(op, FLAG_FREED)) 
       return 1;
   if(QUERY_FLAG(op, FLAG_ANIMATE) && op->anim_speed==0) {
     animate_object(op);
@@ -659,6 +664,9 @@ int process_object(object *op) {
   case FORCE:
     remove_force(op);
     return 1;
+  case BLINDNESS:
+    remove_blindness(op);
+    return 0;
   case CONFUSION:
     remove_confusion(op);
     return 0;
@@ -674,6 +682,7 @@ int process_object(object *op) {
   case MMISSILE:
     move_missile(op);
     return 0;
+  case THROWN_OBJ:
   case ARROW:
     move_arrow(op);
     return 0;
@@ -757,6 +766,7 @@ int process_object(object *op) {
     move_player_mover(op);
     return 0;
   }
+
   return 0;
 }
 
